@@ -3,19 +3,22 @@
 
 #include "qquicklabsplatformcolordialog_p.h"
 
+#if QT_DEPRECATED_SINCE(6, 9)
+
 QT_BEGIN_NAMESPACE
 
 /*!
     \qmltype ColorDialog
     \inherits Dialog
-//! \instantiates QQuickLabsPlatformColorDialog
+//! \nativetype QQuickLabsPlatformColorDialog
     \inqmlmodule Qt.labs.platform
     \since 5.8
+    \deprecated [6.9] Use QtQuick.Dialogs::ColorDialog instead.
     \brief A native color dialog.
 
     The ColorDialog type provides a QML API for native platform color dialogs.
 
-    \image qtlabsplatform-colordialog-gtk.png
+    \image {qtlabsplatform-colordialog-gtk.png} {A native color dialog}
 
     To show a color dialog, construct an instance of ColorDialog, set the
     desired properties, and call \l {Dialog::}{open()}. The \l currentColor
@@ -45,13 +48,16 @@ QT_BEGIN_NAMESPACE
     A native platform color dialog is currently available on the following platforms:
 
     \list
-    \li macOS
+    \li iOS
     \li Linux (when running with the GTK+ platform theme)
+    \li macOS
     \endlist
 
     \input includes/widgets.qdocinc 1
 
     \labs
+
+    \sa QtQuick.Dialogs::ColorDialog
 */
 
 QQuickLabsPlatformColorDialog::QQuickLabsPlatformColorDialog(QObject *parent)
@@ -173,3 +179,5 @@ void QQuickLabsPlatformColorDialog::accept()
 QT_END_NAMESPACE
 
 #include "moc_qquicklabsplatformcolordialog_p.cpp"
+
+#endif // QT_DEPRECATED_SINCE(6, 9)
