@@ -3,6 +3,8 @@
 
 #include "qquicklabsplatformfiledialog_p.h"
 
+#if QT_DEPRECATED_SINCE(6, 9)
+
 #include <QtCore/qlist.h>
 
 QT_BEGIN_NAMESPACE
@@ -15,11 +17,12 @@ using namespace Qt::StringLiterals;
 //!     \nativetype QQuickLabsPlatformFileDialog
     \inqmlmodule Qt.labs.platform
     \since 5.8
+    \deprecated [6.9] Use QtQuick.Dialogs::FileDialog instead.
     \brief A native file dialog.
 
     The FileDialog type provides a QML API for native platform file dialogs.
 
-    \image qtlabsplatform-filedialog-gtk.png
+    \image {qtlabsplatform-filedialog-gtk.png} {A native file dialog}
 
     To show a file dialog, construct an instance of FileDialog, set the
     desired properties, and call \l {Dialog::}{open()}. The \l currentFile
@@ -62,7 +65,7 @@ using namespace Qt::StringLiterals;
 
     \labs
 
-    \sa FolderDialog, StandardPaths
+    \sa QtQuick.Dialogs::FileDialog, FolderDialog, StandardPaths
 */
 
 QQuickLabsPlatformFileDialog::QQuickLabsPlatformFileDialog(QObject *parent)
@@ -637,3 +640,5 @@ QString QQuickLabsPlatformFileNameFilter::nameFilter(int index) const
 QT_END_NAMESPACE
 
 #include "moc_qquicklabsplatformfiledialog_p.cpp"
+
+#endif // QT_DEPRECATED_SINCE(6, 9)
