@@ -12,10 +12,6 @@
 #include <QCryptographicHash>
 #include <cmath>
 
-#ifdef CONST
-#undef CONST
-#endif
-
 QT_USE_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -57,56 +53,48 @@ void Object::simplifyRequiredProperties() {
     }
 }
 
-bool Parameter::init(QV4::Compiler::JSUnitGenerator *stringGenerator, const QString &parameterName,
-                     const QString &typeName)
+bool Parameter::initType(
+        QV4::CompiledData::ParameterType *paramType,
+        const QString &typeName, int typeNameIndex,
+        QV4::CompiledData::ParameterType::Flag listFlag)
 {
-    return init(this, stringGenerator, stringGenerator->registerString(parameterName), stringGenerator->registerString(typeName));
-}
-
-bool Parameter::init(QV4::CompiledData::Parameter *param, const QV4::Compiler::JSUnitGenerator *stringGenerator,
-                     int parameterNameIndex, int typeNameIndex)
-{
-    param->nameIndex = parameterNameIndex;
-    return initType(&param->type, stringGenerator, typeNameIndex);
-}
-
-bool Parameter::initType(QV4::CompiledData::ParameterType *paramType, const QV4::Compiler::JSUnitGenerator *stringGenerator, int typeNameIndex)
-{
-    const QString typeName = stringGenerator->stringForIndex(typeNameIndex);
     auto builtinType = stringToBuiltinType(typeName);
-    if (builtinType == QV4::CompiledData::BuiltinType::InvalidBuiltin) {
+    if (builtinType == QV4::CompiledData::CommonType::Invalid) {
         if (typeName.isEmpty()) {
-            paramType->set(false, 0);
+            paramType->set(listFlag, 0);
             return false;
         }
         Q_ASSERT(quint32(typeNameIndex) < (1u << 31));
-        paramType->set(false, typeNameIndex);
+        paramType->set(listFlag, typeNameIndex);
     } else {
         Q_ASSERT(quint32(builtinType) < (1u << 31));
-        paramType->set(true, static_cast<quint32>(builtinType));
+        paramType->set(listFlag | QV4::CompiledData::ParameterType::Common,
+                       static_cast<quint32>(builtinType));
     }
     return true;
 }
 
-QV4::CompiledData::BuiltinType Parameter::stringToBuiltinType(const QString &typeName)
+QV4::CompiledData::CommonType Parameter::stringToBuiltinType(const QString &typeName)
 {
     static const struct TypeNameToType {
         const char *name;
         size_t nameLength;
-        QV4::CompiledData::BuiltinType type;
+        QV4::CompiledData::CommonType type;
     } propTypeNameToTypes[] = {
-        { "int", strlen("int"), QV4::CompiledData::BuiltinType::Int },
-        { "bool", strlen("bool"), QV4::CompiledData::BuiltinType::Bool },
-        { "double", strlen("double"), QV4::CompiledData::BuiltinType::Real },
-        { "real", strlen("real"), QV4::CompiledData::BuiltinType::Real },
-        { "string", strlen("string"), QV4::CompiledData::BuiltinType::String },
-        { "url", strlen("url"), QV4::CompiledData::BuiltinType::Url },
-        { "date", strlen("date"), QV4::CompiledData::BuiltinType::DateTime },
-        { "rect", strlen("rect"), QV4::CompiledData::BuiltinType::Rect },
-        { "point", strlen("point"), QV4::CompiledData::BuiltinType::Point },
-        { "size", strlen("size"), QV4::CompiledData::BuiltinType::Size },
-        { "variant", strlen("variant"), QV4::CompiledData::BuiltinType::Var },
-        { "var", strlen("var"), QV4::CompiledData::BuiltinType::Var }
+        { "void", strlen("void"), QV4::CompiledData::CommonType::Void },
+        { "int", strlen("int"), QV4::CompiledData::CommonType::Int },
+        { "bool", strlen("bool"), QV4::CompiledData::CommonType::Bool },
+        { "double", strlen("double"), QV4::CompiledData::CommonType::Real },
+        { "real", strlen("real"), QV4::CompiledData::CommonType::Real },
+        { "string", strlen("string"), QV4::CompiledData::CommonType::String },
+        { "url", strlen("url"), QV4::CompiledData::CommonType::Url },
+        { "date", strlen("date"), QV4::CompiledData::CommonType::DateTime },
+        { "regexp", strlen("regexp"), QV4::CompiledData::CommonType::RegExp },
+        { "rect", strlen("rect"), QV4::CompiledData::CommonType::Rect },
+        { "point", strlen("point"), QV4::CompiledData::CommonType::Point },
+        { "size", strlen("size"), QV4::CompiledData::CommonType::Size },
+        { "variant", strlen("variant"), QV4::CompiledData::CommonType::Var },
+        { "var", strlen("var"), QV4::CompiledData::CommonType::Var }
     };
     static const int propTypeNameToTypesCount = sizeof(propTypeNameToTypes) /
                                                 sizeof(propTypeNameToTypes[0]);
@@ -117,7 +105,7 @@ QV4::CompiledData::BuiltinType Parameter::stringToBuiltinType(const QString &typ
             return t->type;
         }
     }
-    return QV4::CompiledData::BuiltinType::InvalidBuiltin;
+    return QV4::CompiledData::CommonType::Invalid;
 }
 
 void Object::init(QQmlJS::MemoryPool *pool, int typeNameIndex, int idIndex,
@@ -265,10 +253,11 @@ QString Object::appendAlias(Alias *alias, const QString &aliasName, bool isDefau
 
 void Object::appendFunction(QmlIR::Function *f)
 {
-    Object *target = declarationsOverride;
-    if (!target)
-        target = this;
-    target->functions->append(f);
+    // Unlike properties, a function definition inside a grouped property does not go into
+    // the surrounding object. It's been broken since the Qt 5 era, and the semantics
+    // seems super confusing, so it wouldn't make sense to support that.
+    Q_ASSERT(!declarationsOverride);
+    functions->append(f);
 }
 
 void Object::appendInlineComponent(InlineComponent *ic)
@@ -337,8 +326,8 @@ QStringList Signal::parameterStringList(const QV4::Compiler::StringTableGenerato
     return result;
 }
 
-Document::Document(bool debugMode)
-    : jsModule(debugMode)
+Document::Document(const QString &fileName, const QString &finalUrl, bool debugMode)
+    : jsModule(fileName, finalUrl, debugMode)
     , program(nullptr)
     , jsGenerator(&jsModule)
 {
@@ -371,7 +360,7 @@ void ScriptDirectivesCollector::importModule(const QString &uri, const QString &
     QV4::CompiledData::Import *import = engine->pool()->New<QV4::CompiledData::Import>();
     import->type = QV4::CompiledData::Import::ImportLibrary;
     import->uriIndex = jsGenerator->registerString(uri);
-    import->version = IRBuilder::extractVersion(QStringView(version));
+    import->version = IRBuilder::extractVersion(version);
     import->qualifierIndex = jsGenerator->registerString(module);
     import->location.set(lineNumber, column);
     document->imports << import;
@@ -453,39 +442,6 @@ bool IRBuilder::generateFromQml(const QString &code, const QString &url, Documen
     return errors.isEmpty();
 }
 
-bool IRBuilder::isSignalPropertyName(const QString &name)
-{
-    if (name.length() < 3) return false;
-    if (!name.startsWith(QLatin1String("on"))) return false;
-    int ns = name.length();
-    for (int i = 2; i < ns; ++i) {
-        const QChar curr = name.at(i);
-        if (curr.unicode() == '_') continue;
-        if (curr.isUpper()) return true;
-        return false;
-    }
-    return false; // consists solely of underscores - invalid.
-}
-
-QString IRBuilder::signalNameFromSignalPropertyName(const QString &signalPropertyName)
-{
-    Q_ASSERT(signalPropertyName.startsWith(QLatin1String("on")));
-    QString signalNameCandidate = signalPropertyName;
-    signalNameCandidate.remove(0, 2);
-
-    // Note that the property name could start with any alpha or '_' or '$' character,
-    // so we need to do the lower-casing of the first alpha character.
-    for (int firstAlphaIndex = 0; firstAlphaIndex < signalNameCandidate.size(); ++firstAlphaIndex) {
-        if (signalNameCandidate.at(firstAlphaIndex).isUpper()) {
-            signalNameCandidate[firstAlphaIndex] = signalNameCandidate.at(firstAlphaIndex).toLower();
-            return signalNameCandidate;
-        }
-    }
-
-    Q_UNREACHABLE();
-    return QString();
-}
-
 bool IRBuilder::visit(QQmlJS::AST::UiArrayMemberList *ast)
 {
     return QQmlJS::AST::Visitor::visit(ast);
@@ -603,7 +559,7 @@ bool IRBuilder::visit(QQmlJS::AST::UiArrayBinding *node)
         memberList.append(member);
         member = member->next;
     }
-    for (int i = memberList.count() - 1; i >= 0; --i) {
+    for (int i = memberList.size() - 1; i >= 0; --i) {
         member = memberList.at(i);
         QQmlJS::AST::UiObjectDefinition *def = QQmlJS::AST::cast<QQmlJS::AST::UiObjectDefinition*>(member->member);
 
@@ -731,7 +687,7 @@ bool IRBuilder::visit(QQmlJS::AST::UiImport *node)
 
         // Check for script qualifier clashes
         bool isScript = import->type == QV4::CompiledData::Import::ImportScript;
-        for (int ii = 0; ii < _imports.count(); ++ii) {
+        for (int ii = 0; ii < _imports.size(); ++ii) {
             const QV4::CompiledData::Import *other = _imports.at(ii);
             bool otherIsScript = other->type == QV4::CompiledData::Import::ImportScript;
 
@@ -762,61 +718,222 @@ bool IRBuilder::visit(QQmlJS::AST::UiImport *node)
     return false;
 }
 
+
+template<typename Argument>
+struct PragmaParser
+{
+    static bool run(IRBuilder *builder, QQmlJS::AST::UiPragma *node, Pragma *pragma)
+    {
+        Q_ASSERT(builder);
+        Q_ASSERT(node);
+        Q_ASSERT(pragma);
+
+        if (!isUnique(builder)) {
+            builder->recordError(
+                        node->pragmaToken, QCoreApplication::translate(
+                            "QQmlParser", "Multiple %1 pragmas found").arg(name()));
+            return false;
+        }
+
+        pragma->type = type();
+
+        if (QQmlJS::AST::UiPragmaValueList *bad = assign(pragma, node->values)) {
+            builder->recordError(
+                        node->pragmaToken, QCoreApplication::translate(
+                            "QQmlParser", "Unknown %1 '%2' in pragma").arg(name(), bad->value));
+            return false;
+        }
+
+        return true;
+    }
+
+private:
+    static constexpr Pragma::PragmaType type()
+    {
+        if constexpr (std::is_same_v<Argument, Pragma::ComponentBehaviorValue>) {
+            return Pragma::ComponentBehavior;
+        } else if constexpr (std::is_same_v<Argument, Pragma::ListPropertyAssignBehaviorValue>) {
+            return Pragma::ListPropertyAssignBehavior;
+        } else if constexpr (std::is_same_v<Argument, Pragma::FunctionSignatureBehaviorValue>) {
+            return Pragma::FunctionSignatureBehavior;
+        } else if constexpr (std::is_same_v<Argument, Pragma::NativeMethodBehaviorValue>) {
+            return Pragma::NativeMethodBehavior;
+        } else if constexpr (std::is_same_v<Argument, Pragma::ValueTypeBehaviorValue>) {
+            return Pragma::ValueTypeBehavior;
+        }
+
+        Q_UNREACHABLE_RETURN(Pragma::PragmaType(-1));
+    }
+
+    template<typename F>
+    static QQmlJS::AST::UiPragmaValueList *iterateValues(
+            QQmlJS::AST::UiPragmaValueList *input, F &&process)
+    {
+        for (QQmlJS::AST::UiPragmaValueList *i = input; i; i = i->next) {
+            if (!process(i->value))
+                return i;
+        }
+        return nullptr;
+    }
+
+    static QQmlJS::AST::UiPragmaValueList *assign(
+            Pragma *pragma, QQmlJS::AST::UiPragmaValueList *values)
+    {
+        // We could use QMetaEnum here to make the code more compact,
+        // but it's probably more expensive.
+
+        if constexpr (std::is_same_v<Argument, Pragma::ComponentBehaviorValue>) {
+            return iterateValues(values, [pragma](QStringView value) {
+                if (value == "Unbound"_L1) {
+                    pragma->componentBehavior = Pragma::Unbound;
+                    return true;
+                }
+                if (value == "Bound"_L1) {
+                    pragma->componentBehavior = Pragma::Bound;
+                    return true;
+                }
+                return false;
+            });
+        } else if constexpr (std::is_same_v<Argument, Pragma::ListPropertyAssignBehaviorValue>) {
+            return iterateValues(values, [pragma](QStringView value) {
+                if (value == "Append"_L1) {
+                    pragma->listPropertyAssignBehavior = Pragma::Append;
+                    return true;
+                }
+                if (value == "Replace"_L1) {
+                    pragma->listPropertyAssignBehavior = Pragma::Replace;
+                    return true;
+                }
+                if (value == "ReplaceIfNotDefault"_L1) {
+                    pragma->listPropertyAssignBehavior = Pragma::ReplaceIfNotDefault;
+                    return true;
+                }
+                return false;
+            });
+        } else if constexpr (std::is_same_v<Argument, Pragma::FunctionSignatureBehaviorValue>) {
+            return iterateValues(values, [pragma](QStringView value) {
+                if (value == "Ignored"_L1) {
+                    pragma->functionSignatureBehavior = Pragma::Ignored;
+                    return true;
+                }
+                if (value == "Enforced"_L1) {
+                    pragma->functionSignatureBehavior = Pragma::Enforced;
+                    return true;
+                }
+                return false;
+            });
+        } else if constexpr (std::is_same_v<Argument, Pragma::NativeMethodBehaviorValue>) {
+            return iterateValues(values, [pragma](QStringView value) {
+                if (value == "AcceptThisObject"_L1) {
+                    pragma->nativeMethodBehavior = Pragma::AcceptThisObject;
+                    return true;
+                }
+                if (value == "RejectThisObject"_L1) {
+                    pragma->nativeMethodBehavior = Pragma::RejectThisObject;
+                    return true;
+                }
+                return false;
+            });
+        } else if constexpr (std::is_same_v<Argument, Pragma::ValueTypeBehaviorValue>) {
+            pragma->valueTypeBehavior = Pragma::ValueTypeBehaviorValues().toInt();
+            return iterateValues(values, [pragma](QStringView value) {
+                const auto setFlag = [pragma](Pragma::ValueTypeBehaviorValue flag, bool value) {
+                    pragma->valueTypeBehavior
+                            = Pragma::ValueTypeBehaviorValues(pragma->valueTypeBehavior)
+                                .setFlag(flag, value).toInt();
+                };
+
+                if (value == "Reference"_L1) {
+                    setFlag(Pragma::Copy, false);
+                    return true;
+                }
+                if (value == "Copy"_L1) {
+                    setFlag(Pragma::Copy, true);
+                    return true;
+                }
+
+                if (value == "Inaddressable"_L1) {
+                    setFlag(Pragma::Addressable, false);
+                    return true;
+                }
+                if (value == "Addressable"_L1) {
+                    setFlag(Pragma::Addressable, true);
+                    return true;
+                }
+
+                if (value == "Inassertable"_L1) {
+                    setFlag(Pragma::Assertable, false);
+                    return true;
+                }
+                if (value == "Assertable"_L1) {
+                    setFlag(Pragma::Assertable, true);
+                    return true;
+                }
+
+                return false;
+            });
+        }
+
+        Q_UNREACHABLE_RETURN(nullptr);
+    }
+
+    static bool isUnique(IRBuilder *builder)
+    {
+        for (const Pragma *prev : builder->_pragmas) {
+            if (prev->type == type())
+                return false;
+        }
+        return true;
+    };
+
+    static QLatin1StringView name()
+    {
+        switch (type()) {
+        case Pragma::ListPropertyAssignBehavior:
+            return "list property assign behavior"_L1;
+        case Pragma::ComponentBehavior:
+            return "component behavior"_L1;
+        case Pragma::FunctionSignatureBehavior:
+            return "function signature behavior"_L1;
+        case Pragma::NativeMethodBehavior:
+            return "native method behavior"_L1;
+        case Pragma::ValueTypeBehavior:
+            return "value type behavior"_L1;
+        default:
+            break;
+        }
+        Q_UNREACHABLE_RETURN(QLatin1StringView());
+    }
+};
+
 bool IRBuilder::visit(QQmlJS::AST::UiPragma *node)
 {
     Pragma *pragma = New<Pragma>();
 
     if (!node->name.isNull()) {
-        if (node->name == QStringLiteral("Singleton")) {
+        if (node->name == "Singleton"_L1) {
             pragma->type = Pragma::Singleton;
-        } else if (node->name == QStringLiteral("Strict")) {
+        } else if (node->name == "Strict"_L1) {
             pragma->type = Pragma::Strict;
-        } else if (node->name == QStringLiteral("ComponentBehavior")) {
-            for (const Pragma *prev : _pragmas) {
-                if (prev->type != Pragma::ComponentBehavior)
-                    continue;
-                recordError(node->pragmaToken,
-                            QCoreApplication::translate(
-                                    "QQmlParser", "Multiple component behavior pragmas found"));
+        } else if (node->name == "ComponentBehavior"_L1) {
+            if (!PragmaParser<Pragma::ComponentBehaviorValue>::run(this, node, pragma))
                 return false;
-            }
+        } else if (node->name == "ListPropertyAssignBehavior"_L1) {
+            if (!PragmaParser<Pragma::ListPropertyAssignBehaviorValue>::run(this, node, pragma))
+                return false;
+        } else if (node->name == "FunctionSignatureBehavior"_L1) {
+            if (!PragmaParser<Pragma::FunctionSignatureBehaviorValue>::run(this, node, pragma))
+                return false;
+        } else if (node->name == "NativeMethodBehavior"_L1) {
+            if (!PragmaParser<Pragma::NativeMethodBehaviorValue>::run(this, node, pragma))
+                return false;
+        } else if (node->name == "ValueTypeBehavior"_L1) {
+            if (!PragmaParser<Pragma::ValueTypeBehaviorValue>::run(this, node, pragma))
+                return false;
+        } else if (node->name == "Translator"_L1) {
+            pragma->type = Pragma::Translator;
+            pragma->translationContextIndex = registerString(node->values->value.toString());
 
-            pragma->type = Pragma::ComponentBehavior;
-            if (node->value == QLatin1String("Bound")) {
-                pragma->componentBehavior = Pragma::Bound;
-            } else if (node->value == QLatin1String("Unbound")) {
-                pragma->componentBehavior = Pragma::Unbound;
-            } else {
-                recordError(node->pragmaToken,
-                            QCoreApplication::translate(
-                                    "QQmlParser", "Unknown component behavior '%1' in pragma")
-                                    .arg(node->value));
-                return false;
-            }
-        } else if (node->name == QStringLiteral("ListPropertyAssignBehavior")) {
-            for (const Pragma *prev : _pragmas) {
-                if (prev->type != Pragma::ListPropertyAssignBehavior)
-                    continue;
-                recordError(node->pragmaToken, QCoreApplication::translate(
-                                "QQmlParser",
-                                "Multiple list property assign behavior pragmas found"));
-                return false;
-            }
-
-            pragma->type = Pragma::ListPropertyAssignBehavior;
-            if (node->value == QStringLiteral("Replace")) {
-                pragma->listPropertyAssignBehavior = Pragma::Replace;
-            } else if (node->value == QStringLiteral("ReplaceIfNotDefault")) {
-                pragma->listPropertyAssignBehavior = Pragma::ReplaceIfNotDefault;
-            } else if (node->value == QStringLiteral("Append")) {
-                pragma->listPropertyAssignBehavior = Pragma::Append;
-            } else {
-                recordError(node->pragmaToken, QCoreApplication::translate(
-                                "QQmlParser",
-                                "Unknown list property assign behavior '%1' in pragma")
-                            .arg(node->value));
-                return false;
-            }
         } else {
             recordError(node->pragmaToken, QCoreApplication::translate(
                             "QQmlParser", "Unknown pragma '%1'").arg(node->name));
@@ -910,17 +1027,18 @@ bool IRBuilder::visit(QQmlJS::AST::UiPublicMember *node)
 
         QQmlJS::AST::UiParameterList *p = node->parameters;
         while (p) {
-            const QString memberType = asString(p->type);
-
-            if (memberType.isEmpty()) {
+            if (!p->type) {
                 recordError(node->typeToken, QCoreApplication::translate("QQmlParser","Expected parameter type"));
                 return false;
             }
 
             Parameter *param = New<Parameter>();
-            if (!param->init(jsGenerator, p->name.toString(), memberType)) {
+            param->nameIndex = registerString(p->name.toString());
+            if (!Parameter::initType(
+                        &param->type, [this](const QString &str) { return registerString(str); },
+                        p->type)) {
                 QString errStr = QCoreApplication::translate("QQmlParser","Invalid signal parameter type: ");
-                errStr.append(memberType);
+                errStr.append(p->type->toString());
                 recordError(node->typeToken, errStr);
                 return false;
             }
@@ -950,20 +1068,20 @@ bool IRBuilder::visit(QQmlJS::AST::UiPublicMember *node)
         if (memberType == QLatin1String("alias")) {
             return appendAlias(node);
         } else {
-            const QStringView &name = node->name;
+            QStringView name = node->name;
 
             Property *property = New<Property>();
             property->setIsReadOnly(node->isReadonly());
             property->setIsRequired(node->isRequired());
 
-            const QV4::CompiledData::BuiltinType builtinPropertyType
+            const QV4::CompiledData::CommonType builtinPropertyType
                     = Parameter::stringToBuiltinType(memberType);
-            if (builtinPropertyType != QV4::CompiledData::BuiltinType::InvalidBuiltin)
-                property->setBuiltinType(builtinPropertyType);
+            if (builtinPropertyType != QV4::CompiledData::CommonType::Invalid)
+                property->setCommonType(builtinPropertyType);
             else
-                property->setCustomType(registerString(memberType));
+                property->setTypeNameIndex(registerString(memberType));
 
-            const QStringView &typeModifier = node->typeModifier;
+            QStringView typeModifier = node->typeModifier;
             if (typeModifier == QLatin1String("list")) {
                 property->setIsList(true);
             } else if (!typeModifier.isEmpty()) {
@@ -1011,6 +1129,14 @@ bool IRBuilder::visit(QQmlJS::AST::UiPublicMember *node)
 bool IRBuilder::visit(QQmlJS::AST::UiSourceElement *node)
 {
     if (QQmlJS::AST::FunctionExpression *funDecl = node->sourceElement->asFunctionDefinition()) {
+        if (_object->declarationsOverride) {
+            // See Object::appendFunction() for why.
+            recordError(node->firstSourceLocation(),
+                        QCoreApplication::translate(
+                                "QQmlParser", "Function declaration inside grouped property"));
+            return false;
+        }
+
         CompiledFunctionOrExpression *foe = New<CompiledFunctionOrExpression>();
         foe->node = funDecl;
         foe->parentNode = funDecl;
@@ -1023,8 +1149,11 @@ bool IRBuilder::visit(QQmlJS::AST::UiSourceElement *node)
         f->index = index;
         f->nameIndex = registerString(funDecl->name.toString());
 
-        QString returnTypeName = funDecl->typeAnnotation ? funDecl->typeAnnotation->type->toString() : QString();
-        Parameter::initType(&f->returnType, jsGenerator, registerString(returnTypeName));
+        const auto idGenerator = [this](const QString &str) { return registerString(str); };
+
+        Parameter::initType(
+                    &f->returnType, idGenerator,
+                    funDecl->typeAnnotation ? funDecl->typeAnnotation->type : nullptr);
 
         const QQmlJS::AST::BoundNames formals = funDecl->formals ? funDecl->formals->formals() : QQmlJS::AST::BoundNames();
         int formalsCount = formals.size();
@@ -1032,7 +1161,11 @@ bool IRBuilder::visit(QQmlJS::AST::UiSourceElement *node)
 
         int i = 0;
         for (const auto &arg : formals) {
-            f->formals[i].init(jsGenerator, arg.id, arg.typeName());
+            Parameter *functionParameter = &f->formals[i];
+            functionParameter->nameIndex = registerString(arg.id);
+            Parameter::initType(
+                        &functionParameter->type, idGenerator,
+                        arg.typeAnnotation.isNull() ? nullptr : arg.typeAnnotation->type);
             ++i;
         }
 
@@ -1164,17 +1297,25 @@ void IRBuilder::setBindingValue(QV4::CompiledData::Binding *binding, QQmlJS::AST
 
 void IRBuilder::tryGeneratingTranslationBinding(QStringView base, AST::ArgumentList *args, QV4::CompiledData::Binding *binding)
 {
-    auto registerMainString = [&](QStringView mainString) { return jsGenerator->registerString(mainString.toString()) ; };
-    auto registerCommentString = [&](QStringView commentString) { return jsGenerator->registerString(commentString.toString()); };
-    auto finalizeTranslationData = [&](QV4::CompiledData::Binding::Type type, QV4::CompiledData::TranslationData translationData) {
-        binding->setType(type);
-        if (type == QV4::CompiledData::Binding::Type_Translation || type == QV4::CompiledData::Binding::Type_TranslationById)
-            binding->value.translationDataIndex = jsGenerator->registerTranslation(translationData);
-        else if (type == QV4::CompiledData::Binding::Type_String)
-            binding->stringIndex = translationData.number;
+    const auto registerString = [&](QStringView string) {
+        return jsGenerator->registerString(string.toString()) ;
     };
 
-    tryGeneratingTranslationBindingBase(base, args, registerMainString, registerCommentString, finalizeTranslationData);
+    const auto finalizeTranslationData = [&](
+            QV4::CompiledData::Binding::Type type,
+            QV4::CompiledData::TranslationData translationData) {
+        binding->setType(type);
+        if (type == QV4::CompiledData::Binding::Type_Translation
+                || type == QV4::CompiledData::Binding::Type_TranslationById) {
+            binding->value.translationDataIndex = jsGenerator->registerTranslation(translationData);
+        } else if (type == QV4::CompiledData::Binding::Type_String) {
+            binding->stringIndex = translationData.number;
+        }
+    };
+
+    tryGeneratingTranslationBindingBase(
+                base, args,
+                registerString, registerString, registerString, finalizeTranslationData);
 }
 
 void IRBuilder::appendBinding(QQmlJS::AST::UiQualifiedId *name, QQmlJS::AST::Statement *value, QQmlJS::AST::Node *parentNode)
@@ -1298,13 +1439,13 @@ bool IRBuilder::appendAlias(QQmlJS::AST::UiPublicMember *node)
         COMPILE_EXCEPTION(rhsLoc, tr("Invalid alias reference. An alias reference must be specified as <id>, <id>.<property> or <id>.<value property>.<property>"));
     }
 
-    if (aliasReference.count() < 1 || aliasReference.count() > 3)
+    if (aliasReference.size() < 1 || aliasReference.size() > 3)
         COMPILE_EXCEPTION(rhsLoc, tr("Invalid alias reference. An alias reference must be specified as <id>, <id>.<property> or <id>.<value property>.<property>"));
 
-     alias->idIndex = registerString(aliasReference.first());
+     alias->setIdIndex(registerString(aliasReference.first()));
 
      QString propertyValue = aliasReference.value(1);
-     if (aliasReference.count() == 3)
+     if (aliasReference.size() == 3)
          propertyValue += QLatin1Char('.') + aliasReference.at(2);
      alias->propertyNameIndex = registerString(propertyValue);
 
@@ -1495,7 +1636,7 @@ bool IRBuilder::isStatementNodeScript(QQmlJS::AST::Statement *statement)
 
 bool IRBuilder::isRedundantNullInitializerForPropertyDeclaration(Property *property, QQmlJS::AST::Statement *statement)
 {
-    if (property->isBuiltinType() || property->isList())
+    if (property->isCommonType() || property->isList())
         return false;
     QQmlJS::AST::ExpressionStatement *exprStmt = QQmlJS::AST::cast<QQmlJS::AST::ExpressionStatement *>(statement);
     if (!exprStmt)
@@ -1513,10 +1654,14 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
 
     Unit *jsUnit = nullptr;
 
+    if (!output.javaScriptCompilationUnit)
+        output.javaScriptCompilationUnit.adopt(new QV4::CompiledData::CompilationUnit);
+
     // We may already have unit data if we're loading an ahead-of-time generated cache file.
-    if (output.javaScriptCompilationUnit.data) {
-        jsUnit = const_cast<Unit *>(output.javaScriptCompilationUnit.data);
-        output.javaScriptCompilationUnit.dynamicStrings = output.jsGenerator.stringTable.allStrings();
+    if (output.javaScriptCompilationUnit->unitData()) {
+        jsUnit = const_cast<Unit *>(output.javaScriptCompilationUnit->unitData());
+        output.javaScriptCompilationUnit->dynamicStrings
+                = output.jsGenerator.stringTable.allStrings();
     } else {
         Unit *createdUnit;
         jsUnit = createdUnit = output.jsGenerator.generateUnit();
@@ -1554,6 +1699,44 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
                     break;
                 }
                 break;
+            case Pragma::FunctionSignatureBehavior:
+                switch (p->functionSignatureBehavior) {
+                case Pragma::Enforced:
+                    break;
+                case Pragma::Ignored:
+                    createdUnit->flags |= Unit::FunctionSignaturesIgnored;
+                    break;
+                }
+                break;
+            case Pragma::NativeMethodBehavior:
+                switch (p->nativeMethodBehavior) {
+                case Pragma::AcceptThisObject:
+                    createdUnit->flags |= Unit::NativeMethodsAcceptThisObject;
+                    break;
+                case Pragma::RejectThisObject:
+                    // this is the default;
+                    break;
+                }
+                break;
+            case Pragma::ValueTypeBehavior:
+                if (Pragma::ValueTypeBehaviorValues(p->valueTypeBehavior)
+                        .testFlag(Pragma::Copy)) {
+                    createdUnit->flags |= Unit::ValueTypesCopied;
+                }
+                if (Pragma::ValueTypeBehaviorValues(p->valueTypeBehavior)
+                        .testFlag(Pragma::Addressable)) {
+                    createdUnit->flags |= Unit::ValueTypesAddressable;
+                }
+                if (Pragma::ValueTypeBehaviorValues(p->valueTypeBehavior)
+                            .testFlag(Pragma::Assertable)) {
+                    createdUnit->flags |= Unit::ValueTypesAssertable;
+                }
+                break;
+            case Pragma::Translator:
+                if (createdUnit->translationTableSize)
+                    if (quint32_le *index = createdUnit->translationContextIndex())
+                        *index = p->translationContextIndex;
+                break;
             }
         }
 
@@ -1572,8 +1755,8 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
     // No more new strings after this point, we're calculating offsets.
     output.jsGenerator.stringTable.freeze();
 
-    const uint importSize = uint(sizeof(QV4::CompiledData::Import)) * output.imports.count();
-    const uint objectOffsetTableSize = output.objects.count() * uint(sizeof(quint32));
+    const uint importSize = uint(sizeof(QV4::CompiledData::Import)) * output.imports.size();
+    const uint objectOffsetTableSize = output.objects.size() * uint(sizeof(quint32));
 
     QHash<const Object*, quint32> objectOffsets;
 
@@ -1601,9 +1784,9 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
     memset(data, 0, totalSize);
     QV4::CompiledData::QmlUnit *qmlUnit = reinterpret_cast<QV4::CompiledData::QmlUnit *>(data);
     qmlUnit->offsetToImports = sizeof(*qmlUnit);
-    qmlUnit->nImports = output.imports.count();
+    qmlUnit->nImports = output.imports.size();
     qmlUnit->offsetToObjects = objectOffset;
-    qmlUnit->nObjects = output.objects.count();
+    qmlUnit->nObjects = output.objects.size();
 
     // write imports
     char *importPtr = data + qmlUnit->offsetToImports;
@@ -1615,7 +1798,7 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
 
     // write objects
     quint32_le *objectTable = reinterpret_cast<quint32_le*>(data + qmlUnit->offsetToObjects);
-    for (int i = 0; i < output.objects.count(); ++i) {
+    for (int i = 0; i < output.objects.size(); ++i) {
         const Object *o = output.objects.at(i);
         char * const objectPtr = data + objectOffsets.value(o);
         *objectTable++ = objectOffsets.value(o);
@@ -1755,7 +1938,7 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
         }
     }
 
-    if (!output.javaScriptCompilationUnit.data) {
+    if (!output.javaScriptCompilationUnit->unitData()) {
         // Combine the qml data into the general unit data.
         jsUnit = static_cast<QV4::CompiledData::Unit *>(realloc(jsUnit, jsUnit->unitSize + totalSize));
         jsUnit->offsetToQmlUnit = jsUnit->unitSize;
@@ -1788,8 +1971,8 @@ void QmlUnitGenerator::generate(Document &output, const QV4::CompiledData::Depen
         qDebug() << "    " << totalStringSize << "bytes total strings";
     }
 
-    output.javaScriptCompilationUnit.setUnitData(jsUnit, qmlUnit, output.jsModule.fileName,
-                                                 output.jsModule.finalUrl);
+    output.javaScriptCompilationUnit->setUnitData(
+            jsUnit, qmlUnit, output.jsModule.fileName, output.jsModule.finalUrl);
 }
 
 char *QmlUnitGenerator::writeBindings(char *bindingPtr, const Object *o, BindingFilter filter) const
@@ -1807,8 +1990,10 @@ char *QmlUnitGenerator::writeBindings(char *bindingPtr, const Object *o, Binding
 }
 
 JSCodeGen::JSCodeGen(Document *document, const QSet<QString> &globalNames,
-                     QV4::Compiler::CodegenWarningInterface *interface)
-    : QV4::Compiler::Codegen(&document->jsGenerator, /*strict mode*/ false, interface),
+                     QV4::Compiler::CodegenWarningInterface *iface,
+                     bool storeSourceLocations)
+    : QV4::Compiler::Codegen(&document->jsGenerator, /*strict mode*/ false, iface,
+                             storeSourceLocations),
       document(document)
 {
     m_globalNames = globalNames;
@@ -1817,7 +2002,7 @@ JSCodeGen::JSCodeGen(Document *document, const QSet<QString> &globalNames,
 }
 
 QVector<int> JSCodeGen::generateJSCodeForFunctionsAndBindings(
-        const QList<CompiledFunctionOrExpression> &functions, bool storeSourceLocation)
+        const QList<CompiledFunctionOrExpression> &functions)
 {
     auto qmlName = [&](const CompiledFunctionOrExpression &c) {
         if (c.nameIndex != 0)
@@ -1857,7 +2042,7 @@ QVector<int> JSCodeGen::generateJSCodeForFunctionsAndBindings(
 
     _context = nullptr;
 
-    for (int i = 0; i < functions.count(); ++i) {
+    for (int i = 0; i < functions.size(); ++i) {
         const CompiledFunctionOrExpression &qmlFunction = functions.at(i);
         QQmlJS::AST::Node *node = qmlFunction.node;
         Q_ASSERT(node != document->program);
@@ -1888,14 +2073,14 @@ QVector<int> JSCodeGen::generateJSCodeForFunctionsAndBindings(
         }
 
         int idx = defineFunction(name, function ? function : qmlFunction.parentNode,
-                                 function ? function->formals : nullptr, body, storeSourceLocation);
+                                 function ? function->formals : nullptr, body);
         runtimeFunctionIndices[i] = idx;
     }
 
     return runtimeFunctionIndices;
 }
 
-bool JSCodeGen::generateRuntimeFunctions(QmlIR::Object *object, bool storeSourceLocation)
+bool JSCodeGen::generateRuntimeFunctions(QmlIR::Object *object)
 {
     if (object->functionsAndExpressions->count == 0)
         return true;
@@ -1907,8 +2092,7 @@ bool JSCodeGen::generateRuntimeFunctions(QmlIR::Object *object, bool storeSource
         functionsToCompile << *foe;
     }
 
-    const auto runtimeFunctionIndices =
-            generateJSCodeForFunctionsAndBindings(functionsToCompile, storeSourceLocation);
+    const auto runtimeFunctionIndices = generateJSCodeForFunctionsAndBindings(functionsToCompile);
     if (hasError())
         return false;
 
