@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:jit
 
 #include <QBuffer>
 #include <QFile>
@@ -146,7 +147,9 @@ public:
     void toBoolean(std::function<void(RegisterID)> continuation)
     {
         urshift64(AccumulatorRegister, TrustedImm32(Value::IsIntegerConvertible_Shift), ScratchRegister);
-        auto needsConversion = branch32(NotEqual, TrustedImm32(1), ScratchRegister);
+        auto needsConversion = branch32(
+                    NotEqual, TrustedImm32(Value::IsIntegerConvertible_Value), ScratchRegister);
+
         continuation(AccumulatorRegister);
         Jump done = jump();
 
@@ -164,8 +167,10 @@ public:
 
     void toNumber()
     {
-        urshift64(AccumulatorRegister, TrustedImm32(Value::QuickType_Shift), ScratchRegister);
-        auto isNumber = branch32(GreaterThanOrEqual, ScratchRegister, TrustedImm32(Value::QT_Int));
+        move(TrustedImm64(Value::NumberMask), ScratchRegister);
+        and64(AccumulatorRegister, ScratchRegister);
+        move(TrustedImm64(Value::NumberDiscriminator), ScratchRegister2);
+        auto isNumber = branch64(GreaterThanOrEqual, ScratchRegister, ScratchRegister2);
 
         move(AccumulatorRegister, registerForArg(0));
         callHelper(toNumberHelper);
@@ -246,7 +251,7 @@ public:
     Jump isIntOrBool()
     {
         urshift64(AccumulatorRegister, TrustedImm32(Value::IsIntegerOrBool_Shift), ScratchRegister);
-        return branch32(Equal, TrustedImm32(3), ScratchRegister);
+        return branch32(Equal, TrustedImm32(Value::IsIntegerOrBool_Value), ScratchRegister);
     }
 
     void jumpStrictEqualStackSlotInt(int lhs, int rhs, int offset)
@@ -280,7 +285,7 @@ public:
     void encodeDoubleIntoAccumulator(FPRegisterID src)
     {
         moveDoubleTo64(src, AccumulatorRegister);
-        move(TrustedImm64(Value::NaNEncodeMask), ScratchRegister);
+        move(TrustedImm64(Value::EncodeMask), ScratchRegister);
         xor64(ScratchRegister, AccumulatorRegister);
     }
 
@@ -319,7 +324,8 @@ public:
     Jump unopIntPath(std::function<Jump(void)> fastPath)
     {
         urshift64(AccumulatorRegister, TrustedImm32(Value::IsIntegerConvertible_Shift), ScratchRegister);
-        Jump accNotIntConvertible = branch32(NotEqual, TrustedImm32(1), ScratchRegister);
+        Jump accNotIntConvertible = branch32(
+                    NotEqual, TrustedImm32(Value::IsIntegerConvertible_Value), ScratchRegister);
 
         // both integer
         Jump failure = fastPath();
@@ -449,8 +455,12 @@ public:
 
     void toNumber()
     {
-        urshift32(AccumulatorRegisterTag, TrustedImm32(Value::QuickType_Shift - 32), ScratchRegister);
-        auto isNumber = branch32(GreaterThanOrEqual, ScratchRegister, TrustedImm32(Value::QT_Int));
+        and32(TrustedImm32(Value::NumberMask >> Value::Tag_Shift),
+              AccumulatorRegisterTag, ScratchRegister);
+        auto isNumber = branch32(
+                    GreaterThanOrEqual, ScratchRegister,
+                    TrustedImm32(Value::NumberDiscriminator >> Value::Tag_Shift));
+
 
         if (ArgInRegCount < 2) {
             subPtr(TrustedImm32(2 * PointerSize), StackPointerRegister); // stack alignment
@@ -599,7 +609,7 @@ public:
     Jump isIntOrBool()
     {
         urshift32(AccumulatorRegisterTag, TrustedImm32(Value::IsIntegerOrBool_Shift - 32), ScratchRegister);
-        return branch32(Equal, TrustedImm32(3), ScratchRegister);
+        return branch32(Equal, TrustedImm32(Value::IsIntegerOrBool_Value), ScratchRegister);
     }
 
     void pushValue(ReturnedValue v)
@@ -630,7 +640,8 @@ public:
     {
         urshift32(AccumulatorRegisterTag, TrustedImm32(Value::IsIntegerConvertible_Shift - 32),
                   ScratchRegister);
-        auto needsConversion = branch32(NotEqual, TrustedImm32(1), ScratchRegister);
+        auto needsConversion = branch32(
+                    NotEqual, TrustedImm32(Value::IsIntegerConvertible_Value), ScratchRegister);
         continuation(AccumulatorRegisterValue);
         Jump done = jump();
 
@@ -707,7 +718,7 @@ public:
     void encodeDoubleIntoAccumulator(FPRegisterID src)
     {
         moveDoubleToInts(src, AccumulatorRegisterValue, AccumulatorRegisterTag);
-        xor32(TrustedImm32(Value::NaNEncodeMask >> 32), AccumulatorRegisterTag);
+        xor32(TrustedImm32(Value::EncodeMask >> 32), AccumulatorRegisterTag);
     }
 
     void pushValueAligned(ReturnedValue v)
@@ -873,6 +884,20 @@ void BaselineAssembler::storeLocal(int index, int level)
         --level;
     }
     pasm()->storeAccumulator(Address(PlatformAssembler::ScratchRegister, ctx.locals.offset + offsetof(ValueArray<0>, values) + sizeof(Value)*index));
+    // check if we need a write barrier
+    auto skipBarrier = pasm()->branch8(
+            PlatformAssembler::Equal,
+            PlatformAssembler::Address(PlatformAssembler::EngineRegister,
+                                       offsetof(EngineBase, isGCOngoing)),
+            TrustedImm32(0));
+    saveAccumulatorInFrame();
+    // if so, do a runtime call
+    pasm()->prepareCallWithArgCount(1);
+    pasm()->passAccumulatorAsArg(0);
+    pasm()->callRuntime((void*)Runtime::MarkCustom::call, CallResultDestination::Ignore);
+    loadAccumulatorFromFrame();
+    skipBarrier.link(pasm());
+
 }
 
 void BaselineAssembler::loadString(int stringId)
@@ -893,7 +918,7 @@ void BaselineAssembler::storeHeapObject(int reg)
 void BaselineAssembler::loadImport(int index)
 {
     Address addr = pasm()->loadCompilationUnitPtr(PlatformAssembler::ScratchRegister);
-    addr.offset = offsetof(QV4::CompiledData::CompilationUnitBase, imports);
+    addr.offset = offsetof(QV4::CompilationUnitRuntimeData, imports);
     pasm()->loadPtr(addr, PlatformAssembler::ScratchRegister);
     addr.offset = index * int(sizeof(QV4::Value*));
     pasm()->loadPtr(addr, PlatformAssembler::ScratchRegister);
