@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:jit
 
 #include "qv4baselinejit_p.h"
 #include "qv4baselineassembler_p.h"
@@ -586,7 +587,7 @@ void BaselineJIT::generate_GetIterator(int iterator)
     BASELINEJIT_GENERATE_RUNTIME_CALL(GetIterator, CallResultDestination::InAccumulator);
 }
 
-void BaselineJIT::generate_IteratorNext(int value, int done)
+void BaselineJIT::generate_IteratorNext(int value, int offset)
 {
     as->saveAccumulatorInFrame();
     as->prepareCallWithArgCount(3);
@@ -594,10 +595,10 @@ void BaselineJIT::generate_IteratorNext(int value, int done)
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
     BASELINEJIT_GENERATE_RUNTIME_CALL(IteratorNext, CallResultDestination::InAccumulator);
-    as->storeReg(done);
+    labels.insert(as->jumpTrue(absoluteOffset(offset)));
 }
 
-void BaselineJIT::generate_IteratorNextForYieldStar(int iterator, int object)
+void BaselineJIT::generate_IteratorNextForYieldStar(int iterator, int object, int offset)
 {
     as->saveAccumulatorInFrame();
     as->prepareCallWithArgCount(4);
@@ -606,13 +607,13 @@ void BaselineJIT::generate_IteratorNextForYieldStar(int iterator, int object)
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
     BASELINEJIT_GENERATE_RUNTIME_CALL(IteratorNextForYieldStar, CallResultDestination::InAccumulator);
+    labels.insert(as->jumpTrue(absoluteOffset(offset)));
 }
 
-void BaselineJIT::generate_IteratorClose(int done)
+void BaselineJIT::generate_IteratorClose()
 {
     as->saveAccumulatorInFrame();
-    as->prepareCallWithArgCount(3);
-    as->passJSSlotAsArg(done, 2);
+    as->prepareCallWithArgCount(2);
     as->passAccumulatorAsArg(1);
     as->passEngineAsArg(0);
     BASELINEJIT_GENERATE_RUNTIME_CALL(IteratorClose, CallResultDestination::InAccumulator);
