@@ -6,7 +6,6 @@
 #include <qv4engine_p.h>
 #include <qv4identifierhash_p.h>
 #include "qv4object_p.h"
-#include "qv4identifiertable_p.h"
 #include "qv4value_p.h"
 #include "qv4mm_p.h"
 #include <private/qprimefornumbits_p.h>
@@ -42,34 +41,6 @@ void PropertyHash::addEntry(const PropertyHash::Entry &entry, int classSize)
     ++d->size;
 }
 
-int PropertyHash::removeIdentifier(PropertyKey identifier, int classSize)
-{
-    int val = -1;
-    PropertyHashData *dd = new PropertyHashData(d->numBits);
-    for (int i = 0; i < d->alloc; ++i) {
-        const Entry &e = d->entries[i];
-        if (!e.identifier.isValid() || e.index >= static_cast<unsigned>(classSize))
-            continue;
-        if (e.identifier == identifier) {
-            val = e.index;
-            continue;
-        }
-        uint idx = e.identifier.id() % dd->alloc;
-        while (dd->entries[idx].identifier.isValid()) {
-            ++idx;
-            idx %= dd->alloc;
-        }
-        dd->entries[idx] = e;
-    }
-    dd->size = classSize;
-    if (!--d->refCount)
-        delete d;
-    d = dd;
-
-    Q_ASSERT(val != -1);
-    return val;
-}
-
 void PropertyHash::detach(bool grow, int classSize)
 {
     if (d->refCount == 1 && !grow)
@@ -96,12 +67,11 @@ void PropertyHash::detach(bool grow, int classSize)
 
 SharedInternalClassDataPrivate<PropertyKey>::SharedInternalClassDataPrivate(const SharedInternalClassDataPrivate<PropertyKey> &other)
     : refcount(1),
-      engine(other.engine),
-      data(nullptr)
+      engine(other.engine)
 {
     if (other.alloc()) {
         const uint s = other.size();
-        data = MemberData::allocate(engine, other.alloc(), other.data);
+        data.set(engine, MemberData::allocate(engine, other.alloc(), other.data));
         setSize(s);
     }
 }
@@ -111,7 +81,7 @@ SharedInternalClassDataPrivate<PropertyKey>::SharedInternalClassDataPrivate(cons
     : refcount(1),
       engine(other.engine)
 {
-    data = MemberData::allocate(engine, other.alloc(), nullptr);
+    data.set(engine, MemberData::allocate(engine, other.alloc(), nullptr));
     memcpy(data, other.data, sizeof(Heap::MemberData) - sizeof(Value) + pos*sizeof(Value));
     data->values.size = pos + 1;
     data->values.set(engine, pos, Value::fromReturnedValue(value.id()));
@@ -121,7 +91,7 @@ void SharedInternalClassDataPrivate<PropertyKey>::grow()
 {
     const uint a = alloc() * 2;
     const uint s = size();
-    data = MemberData::allocate(engine, a, data);
+    data.set(engine, MemberData::allocate(engine, a, data));
     setSize(s);
     Q_ASSERT(alloc() >= a);
 }
@@ -151,6 +121,11 @@ PropertyKey SharedInternalClassDataPrivate<PropertyKey>::at(uint i) const
 void SharedInternalClassDataPrivate<PropertyKey>::set(uint i, PropertyKey t)
 {
     Q_ASSERT(data && i < size());
+    QV4::WriteBarrier::markCustom(engine, [&](QV4::MarkStack *stack) {
+        if constexpr (QV4::WriteBarrier::isInsertionBarrier)
+            if (auto string = t.asStringOrSymbol())
+                string->mark(stack);
+    });
     data->values.values[i].rawValueRef() = t.id();
 }
 
@@ -169,11 +144,20 @@ SharedInternalClassDataPrivate<PropertyAttributes>::SharedInternalClassDataPriva
       m_engine(other.m_engine)
 {
     Q_ASSERT(m_size <= m_alloc);
+    Q_ASSERT(m_alloc > 0);
+
     m_engine->memoryManager->changeUnmanagedHeapSizeUsage(m_alloc * sizeof(PropertyAttributes));
-    data = new PropertyAttributes[m_alloc];
-    if (other.data)
-        memcpy(data, other.data, (m_size - 1) * sizeof(PropertyAttributes));
-    data[pos] = value;
+    const PropertyAttributes *source = other.m_alloc > NumAttributesInPointer
+            ? other.m_data
+            : other.m_inlineData;
+    PropertyAttributes *target;
+    if (m_alloc > NumAttributesInPointer)
+        m_data = target = new PropertyAttributes[m_alloc];
+    else
+        target = m_inlineData;
+
+    memcpy(target, source, (m_size - 1) * sizeof(PropertyAttributes));
+    target[pos] = value;
 }
 
 SharedInternalClassDataPrivate<PropertyAttributes>::SharedInternalClassDataPrivate(
@@ -183,12 +167,14 @@ SharedInternalClassDataPrivate<PropertyAttributes>::SharedInternalClassDataPriva
       m_size(other.m_size),
       m_engine(other.m_engine)
 {
-    if (m_alloc) {
-        m_engine->memoryManager->changeUnmanagedHeapSizeUsage(m_alloc * sizeof(PropertyAttributes));
-        data = new PropertyAttributes[m_alloc];
-        memcpy(data, other.data, m_size*sizeof(PropertyAttributes));
+    m_engine->memoryManager->changeUnmanagedHeapSizeUsage(m_alloc * sizeof(PropertyAttributes));
+    if (m_alloc > NumAttributesInPointer) {
+        m_data = new PropertyAttributes[m_alloc];
+        memcpy(m_data, other.m_data, m_size*sizeof(PropertyAttributes));
+    } else if (m_alloc > 0) {
+        memcpy(m_inlineData, other.m_inlineData, m_alloc * sizeof(PropertyAttributes));
     } else {
-        data = nullptr;
+        m_data = nullptr;
     }
 }
 
@@ -196,13 +182,14 @@ SharedInternalClassDataPrivate<PropertyAttributes>::~SharedInternalClassDataPriv
 {
     m_engine->memoryManager->changeUnmanagedHeapSizeUsage(
             -qptrdiff(m_alloc * sizeof(PropertyAttributes)));
-    delete [] data;
+    if (m_alloc > NumAttributesInPointer)
+        delete [] m_data;
 }
 
 void SharedInternalClassDataPrivate<PropertyAttributes>::grow() {
     uint alloc;
     if (!m_alloc) {
-        alloc = 8;
+        alloc = NumAttributesInPointer;
         m_engine->memoryManager->changeUnmanagedHeapSizeUsage(alloc * sizeof(PropertyAttributes));
     } else {
         // yes, signed. We don't want to deal with stuff > 2G
@@ -216,12 +203,16 @@ void SharedInternalClassDataPrivate<PropertyAttributes>::grow() {
                 (alloc - m_alloc) * sizeof(PropertyAttributes));
     }
 
-    auto *n = new PropertyAttributes[alloc];
-    if (data) {
-        memcpy(n, data, m_alloc*sizeof(PropertyAttributes));
-        delete [] data;
+    if (alloc > NumAttributesInPointer) {
+        auto *n = new PropertyAttributes[alloc];
+        if (m_alloc > NumAttributesInPointer) {
+            memcpy(n, m_data, m_alloc * sizeof(PropertyAttributes));
+            delete [] m_data;
+        } else if (m_alloc > 0) {
+            memcpy(n, m_inlineData, m_alloc * sizeof(PropertyAttributes));
+        }
+        m_data = n;
     }
-    data = n;
     m_alloc = alloc;
 }
 
@@ -240,7 +231,7 @@ void InternalClass::init(ExecutionEngine *engine)
     new (&propertyTable) PropertyHash();
     new (&nameMap) SharedInternalClassData<PropertyKey>(engine);
     new (&propertyData) SharedInternalClassData<PropertyAttributes>(engine);
-    new (&transitions) std::vector<Transition>();
+    new (&transitions) QVarLengthArray<Transition, 1>();
 
     this->engine = engine;
     vtable = QV4::InternalClass::staticVTable();
@@ -257,7 +248,7 @@ void InternalClass::init(Heap::InternalClass *other)
     new (&propertyTable) PropertyHash(other->propertyTable);
     new (&nameMap) SharedInternalClassData<PropertyKey>(other->nameMap);
     new (&propertyData) SharedInternalClassData<PropertyAttributes>(other->propertyData);
-    new (&transitions) std::vector<Transition>();
+    new (&transitions) QVarLengthArray<Transition, 1>();
 
     engine = other->engine;
     vtable = other->vtable;
@@ -269,6 +260,11 @@ void InternalClass::init(Heap::InternalClass *other)
     protoId = engine->newProtoId();
 
     internalClass.set(engine, other->internalClass);
+    QV4::WriteBarrier::markCustom(engine, [&](QV4::MarkStack *stack) {
+        if constexpr (QV4::WriteBarrier::isInsertionBarrier) {
+            other->mark(stack);
+        }
+    });
 }
 
 void InternalClass::destroy()
@@ -288,14 +284,20 @@ void InternalClass::destroy()
     propertyTable.~PropertyHash();
     nameMap.~SharedInternalClassData<PropertyKey>();
     propertyData.~SharedInternalClassData<PropertyAttributes>();
-    transitions.~vector<Transition>();
+    transitions.~QVarLengthArray<Transition, 1>();
     engine = nullptr;
     Base::destroy();
 }
 
-QString InternalClass::keyAt(uint index) const
+ReturnedValue InternalClass::keyAt(uint index) const
 {
-    return nameMap.at(index).toQString();
+    PropertyKey key = nameMap.at(index);
+    if (!key.isValid())
+        return Encode::undefined();
+    if (key.isArrayIndex())
+        return Encode(key.asArrayIndex());
+    Q_ASSERT(key.isStringOrSymbol());
+    return key.asStringOrSymbol()->asReturnedValue();
 }
 
 void InternalClass::changeMember(QV4::Object *object, PropertyKey id, PropertyAttributes data, InternalClassEntry *entry)
@@ -309,7 +311,7 @@ void InternalClass::changeMember(QV4::Object *object, PropertyKey id, PropertyAt
 
 InternalClassTransition &InternalClass::lookupOrInsertTransition(const InternalClassTransition &t)
 {
-    std::vector<Transition>::iterator it = std::lower_bound(transitions.begin(), transitions.end(), t);
+    QVarLengthArray<Transition, 1>::iterator it = std::lower_bound(transitions.begin(), transitions.end(), t);
     if (it != transitions.end() && *it == t) {
         return *it;
     } else {
@@ -341,7 +343,7 @@ static Heap::InternalClass *cleanInternalClass(Heap::InternalClass *orig)
 
     // We will generally add quite a few transitions here. We have 255 redundant ones.
     // We can expect at least as many significant ones in addition.
-    std::vector<InternalClassTransition> transitions;
+    QVarLengthArray<InternalClassTransition, 1> transitions;
 
     Scope scope(orig->engine);
     Scoped<QV4::InternalClass> child(scope, orig);
@@ -407,6 +409,9 @@ static Heap::InternalClass *cleanInternalClass(Heap::InternalClass *orig)
         case InternalClassTransition::Frozen:
             child = child->d()->frozen();
             continue;
+        case InternalClassTransition::Locked:
+            child = child->d()->locked();
+            continue;
         default:
             Q_ASSERT(it->flags != 0);
             Q_ASSERT(it->flags < InternalClassTransition::StructureChange);
@@ -443,7 +448,9 @@ Heap::InternalClass *InternalClass::changeMember(
         return t.lookup;
 
     // create a new class and add it to the tree
-    Heap::InternalClass *newClass = engine->newClass(this);
+    Scope scope(engine);
+    Scoped<QV4::InternalClass> scopedNewClass(scope, engine->newClass(this));
+    auto newClass = scopedNewClass->d();
     if (data.isAccessor() && e->setterIndex == UINT_MAX) {
         Q_ASSERT(!propertyData.at(idx).isAccessor());
 
@@ -479,7 +486,12 @@ Heap::InternalClass *InternalClass::changePrototypeImpl(Heap::Object *proto)
         return t.lookup;
 
     // create a new class and add it to the tree
-    Heap::InternalClass *newClass = engine->newClass(this);
+    Scoped<QV4::InternalClass> scopedNewClass(scope, engine->newClass(this));
+    auto newClass = scopedNewClass->d();
+    QV4::WriteBarrier::markCustom(engine, [&](QV4::MarkStack *stack) {
+        if (proto && QV4::WriteBarrier::isInsertionBarrier)
+            proto->mark(stack);
+    });
     newClass->prototype = proto;
 
     t.lookup = newClass;
@@ -498,7 +510,9 @@ Heap::InternalClass *InternalClass::changeVTableImpl(const VTable *vt)
         return t.lookup;
 
     // create a new class and add it to the tree
-    Heap::InternalClass *newClass = engine->newClass(this);
+    Scope scope(engine);
+    Scoped<QV4::InternalClass> scopedNewClass(scope, engine->newClass(this));
+    auto newClass = scopedNewClass->d();
     newClass->vtable = vt;
 
     t.lookup = newClass;
@@ -519,7 +533,9 @@ Heap::InternalClass *InternalClass::nonExtensible()
     if (t.lookup)
         return t.lookup;
 
-    Heap::InternalClass *newClass = engine->newClass(this);
+    Scope scope(engine);
+    Scoped<QV4::InternalClass> scopedNewClass(scope, engine->newClass(this));
+    auto newClass = scopedNewClass->d();
     newClass->flags |= NotExtensible;
 
     t.lookup = newClass;
@@ -527,19 +543,32 @@ Heap::InternalClass *InternalClass::nonExtensible()
     return newClass;
 }
 
+InternalClass *InternalClass::locked()
+{
+    if (isLocked())
+        return this;
+
+    Transition temp = { { PropertyKey::invalid() }, nullptr, Transition::Locked};
+    Transition &t = lookupOrInsertTransition(temp);
+    if (t.lookup)
+        return t.lookup;
+
+    Scope scope(engine);
+    Scoped<QV4::InternalClass> scopedNewClass(scope, engine->newClass(this));
+    auto newClass = scopedNewClass->d();
+    newClass->flags |= Locked;
+
+    t.lookup = newClass;
+    Q_ASSERT(t.lookup);
+    return newClass;
+}
+
 void InternalClass::addMember(QV4::Object *object, PropertyKey id, PropertyAttributes data, InternalClassEntry *entry)
 {
-    Q_ASSERT(id.isStringOrSymbol());
-    if (!data.isEmpty())
-        data.resolve();
-    PropertyHash::Entry *e = object->internalClass()->findEntry(id);
-    if (e) {
-        changeMember(object, id, data, entry);
-        return;
-    }
-
-    Heap::InternalClass *newClass = object->internalClass()->addMemberImpl(id, data, entry);
-    object->setInternalClass(newClass);
+    Heap::InternalClass *oldClass = object->internalClass();
+    Heap::InternalClass *newClass = oldClass->addMember(id, data, entry);
+    if (newClass != oldClass)
+        object->setInternalClass(newClass);
 }
 
 Heap::InternalClass *InternalClass::addMember(PropertyKey identifier, PropertyAttributes data, InternalClassEntry *entry)
@@ -723,7 +752,9 @@ Heap::InternalClass *InternalClass::asProtoClass()
     if (t.lookup)
         return t.lookup;
 
-    Heap::InternalClass *newClass = engine->newClass(this);
+    Scope scope(engine);
+    Scoped<QV4::InternalClass> scopedNewClass(scope, engine->newClass(this));
+    auto newClass = scopedNewClass->d();
     newClass->flags |= UsedAsProto;
 
     t.lookup = newClass;
