@@ -132,6 +132,22 @@ struct Scope {
         return ptr;
     }
 
+    QML_NEARLY_ALWAYS_INLINE Value *construct(int nValues, const ReturnedValue& initialValue) const
+    {
+        Value *ptr = engine->jsAlloca(nValues);
+        for (int i = 0; i < nValues; ++i)
+            ptr[i].setRawValue(initialValue);
+        return ptr;
+    }
+
+    QML_NEARLY_ALWAYS_INLINE Value *construct(int nValues, const Value &initialValue) const
+    {
+        Value *ptr = engine->jsAlloca(nValues);
+        for (int i = 0; i < nValues; ++i)
+            ptr[i] = initialValue;
+        return ptr;
+    }
+
     bool hasException() const {
         return engine->hasException;
     }
@@ -295,8 +311,7 @@ struct Scoped
 
     QML_NEARLY_ALWAYS_INLINE Scoped(const Scope &scope, const Value &v, ConvertType)
     {
-        ptr = scope.alloc<Scope::Uninitialized>();
-        ptr->setRawValue(value_convert<T>(scope.engine, v));
+        ptr = scope.construct(1, value_convert<T>(scope.engine, v));
     }
 
     QML_NEARLY_ALWAYS_INLINE Scoped(const Scope &scope, const Value *v)
@@ -331,8 +346,7 @@ struct Scoped
 
     QML_NEARLY_ALWAYS_INLINE Scoped(const Scope &scope, const ReturnedValue &v, ConvertType)
     {
-        ptr = scope.alloc<Scope::Uninitialized>();
-        ptr->setRawValue(value_convert<T>(scope.engine, QV4::Value::fromReturnedValue(v)));
+        ptr = scope.construct(1, value_convert<T>(scope.engine, QV4::Value::fromReturnedValue(v)));
     }
 
     Scoped<T> &operator=(Heap::Base *o) {
