@@ -19,6 +19,7 @@
 
 QT_BEGIN_NAMESPACE
 using namespace QQmlJS;
+using namespace Qt::StringLiterals;
 
 static inline int regExpFlagFromChar(const QChar &ch)
 {
@@ -64,6 +65,24 @@ QString Lexer::code() const
     return _code;
 }
 
+std::optional<DiagnosticMessage> Lexer::illegalFileLengthError() const
+{
+    Q_ASSERT(_currentOffset >= 0);
+
+    constexpr bool quint32IsBigger = sizeof(qsizetype) <= sizeof(quint32);
+    using BiggerInt = std::conditional_t<quint32IsBigger, quint32, qsizetype>;
+    using SmallerInt = std::conditional_t<!quint32IsBigger, quint32, qsizetype>;
+
+    const BiggerInt codeLength = BiggerInt(_currentOffset) + BiggerInt(_code.size());
+    const BiggerInt maxLength = BiggerInt(std::numeric_limits<SmallerInt>::max());
+    if (codeLength < maxLength)
+        return {};
+
+    constexpr int limit = quint32IsBigger ? 2 : 4;
+    return DiagnosticMessage{ u"File exceeds maximum length (%1GB)."_s.arg(limit), QtCriticalMsg,
+                              SourceLocation{ 0, 1, 1, 1 } };
+}
+
 void Lexer::setCode(const QString &code, int lineno, bool qmlMode,
                     Lexer::CodeContinuation codeContinuation)
 {
@@ -952,7 +971,7 @@ again:
 
 int Lexer::scanString(ScanStringMode mode)
 {
-    QChar quote = (mode == TemplateContinuation) ? QChar(TemplateHead) : QChar(mode);
+    const char16_t quote = mode == TemplateContinuation ? TemplateHead : mode;
     // we actually use T_STRING_LITERAL also for multiline strings, should we want to
     // change that we should set it to:
     //     _state.tokenKind == T_PARTIAL_SINGLE_QUOTE_STRING_LITERAL ||
