@@ -36,7 +36,7 @@ namespace {
     Q_CONSTINIT thread_local int creationDepth = 0;
 }
 
-Q_LOGGING_CATEGORY(lcQmlComponentGeneral, "qt.qml.qmlcomponent")
+Q_STATIC_LOGGING_CATEGORY(lcQmlComponentGeneral, "qt.qml.qmlcomponent")
 
 QT_BEGIN_NAMESPACE
 
@@ -328,8 +328,12 @@ void QQmlComponentPrivate::clear()
         typeData.reset();
     }
 
+    if (loadHelper) {
+        loadHelper->unregisterCallback(this);
+        loadHelper.reset();
+    }
+
     compilationUnit.reset();
-    loadedType = {};
     inlineComponentName.reset();
 }
 
@@ -362,7 +366,7 @@ static void removePendingQPropertyBinding(
     if (QQmlData *ddata = QQmlData::get(o)) {
         const QQmlPropertyData *propData = ddata->propertyCache->property(
             propertyName, o, ddata->outerContext);
-        if (propData && propData->isBindable())
+        if (propData && propData->acceptsQBinding())
             creator->removePendingBinding(o, propData->coreIndex());
         return;
     }
@@ -499,8 +503,10 @@ QQmlComponent::Status QQmlComponent::status() const
         return Loading;
     else if (!d->state.errors.isEmpty())
         return Error;
-    else if (d->engine && (d->compilationUnit || d->loadedType.isValid()))
+    else if (d->engine && (d->compilationUnit || d->loadedType().isValid()))
         return Ready;
+    else if (d->loadHelper)
+        return Loading;
     else
         return Null;
 }
@@ -938,6 +944,11 @@ QObject *QQmlComponent::create(QQmlContext *context)
     there are unset required properties, the object creation fails and returns
     \c nullptr, in which case \l isError() will return \c true.
 
+    If \a context is \nullptr (the default), it will create the instance in the
+    \l {QQmlEngine::rootContext()}{root context} of the engine.
+
+    The ownership of the returned object instance is transferred to the caller.
+
     \sa QQmlComponent::create
     \since 5.14
 */
@@ -1100,7 +1111,8 @@ QObject *QQmlComponentPrivate::beginCreate(QQmlRefPointer<QQmlContextData> conte
 
     QObject *rv = nullptr;
 
-    if (!loadedType.isValid()) {
+    const QQmlType type = loadedType();
+    if (!type.isValid()) {
         enginePriv->referenceScarceResources();
         const QString *icName = inlineComponentName.get();
         state.initCreator(context, compilationUnit, creationContext, icName ? *icName : QString());
@@ -1121,22 +1133,28 @@ QObject *QQmlComponentPrivate::beginCreate(QQmlRefPointer<QQmlContextData> conte
         enginePriv->dereferenceScarceResources();
     } else {
         // TODO: extract into function
-        rv = loadedType.createWithQQmlData();
+        rv = type.createWithQQmlData();
         QQmlPropertyCache::ConstPtr propertyCache = QQmlData::ensurePropertyCache(rv);
-        if (QQmlParserStatus *parserStatus = parserStatusCast(loadedType, rv)) {
+        if (QQmlParserStatus *parserStatus = parserStatusCast(type, rv)) {
             parserStatus->classBegin();
             state.ensureRequiredPropertyStorage(rv);
-        } else if (loadedType.finalizerCast() != -1) {
+        } else if (type.finalizerCast() != -1) {
             state.ensureRequiredPropertyStorage(rv);
         }
 
-        for (int i = 0, propertyCount = propertyCache->propertyCount(); i < propertyCount; ++i) {
-            if (const QQmlPropertyData *propertyData = propertyCache->property(i); propertyData->isRequired()) {
-                state.ensureRequiredPropertyStorage(rv);
-                RequiredPropertyInfo info;
-                info.propertyName = propertyData->name(rv);
-                state.addPendingRequiredProperty(rv, propertyData, info);
+        if (propertyCache) {
+            for (int i = 0, propertyCount = propertyCache->propertyCount(); i < propertyCount; ++i) {
+                if (const QQmlPropertyData *propertyData = propertyCache->property(i); propertyData->isRequired()) {
+                    state.ensureRequiredPropertyStorage(rv);
+                    RequiredPropertyInfo info;
+                    info.propertyName = propertyData->name(rv);
+                    state.addPendingRequiredProperty(rv, propertyData, info);
+                }
             }
+        } else {
+            // we couldn't get a propertyCache from ensurePropertyCache
+            // it is unclear what we can do in that case
+            // ### TOOD: QTBUG-136560
         }
     }
 
@@ -1293,12 +1311,13 @@ void QQmlComponentPrivate::completeCreate()
         }
     }
 
-    if (loadedType.isValid()) {
+    const QQmlType type = loadedType();
+    if (type.isValid()) {
         QObject *rv = state.target();
-        if (QQmlParserStatus *parserStatus = parserStatusCast(loadedType, rv))
+        if (QQmlParserStatus *parserStatus = parserStatusCast(type, rv))
             parserStatus->componentComplete();
 
-        if (const int finalizerCast = loadedType.finalizerCast(); finalizerCast != -1) {
+        if (const int finalizerCast = type.finalizerCast(); finalizerCast != -1) {
             auto *hook = reinterpret_cast<QQmlFinalizerHook *>(
                     reinterpret_cast<char *>(rv) + finalizerCast);
             hook->componentFinalized();
@@ -1377,23 +1396,37 @@ void QQmlComponent::loadFromModule(QAnyStringView uri, QAnyStringView typeName,
                                    QQmlComponent::CompilationMode mode)
 {
     Q_D(QQmlComponent);
-    auto [status, type] = d->prepareLoadFromModule(uri, typeName);
-    d->completeLoadFromModule(uri, typeName, type, status, mode);
+
+    QQmlTypeLoader::Mode typeLoaderMode = QQmlTypeLoader::Synchronous;
+    switch (mode) {
+    case QQmlComponent::PreferSynchronous:
+        typeLoaderMode = QQmlTypeLoader::PreferSynchronous;
+        break;
+    case QQmlComponent::Asynchronous:
+        typeLoaderMode = QQmlTypeLoader::Asynchronous;
+        break;
+    }
+
+    d->prepareLoadFromModule(uri, typeName, typeLoaderMode);
+    if (d->loadHelper->isCompleteOrError())
+        d->completeLoadFromModule(uri, typeName);
+    else
+        d->loadHelper->registerCallback(d);
 }
 
-LoadHelper::ResolveTypeResult QQmlComponentPrivate::prepareLoadFromModule(QAnyStringView uri,
-                                                                          QAnyStringView typeName)
+void QQmlComponentPrivate::prepareLoadFromModule(
+        QAnyStringView uri, QAnyStringView typeName, QQmlTypeLoader::Mode mode)
 {
-    auto enginePriv = QQmlEnginePrivate::get(engine);
-    // LoadHelper must be on the Heap as it derives from QQmlRefCount
-    auto loadHelper = QQml::makeRefPointer<LoadHelper>(&enginePriv->typeLoader, uri);
+    // Don't let any old loadHelper call us back anymore.
+    if (loadHelper)
+        loadHelper->unregisterCallback(this);
 
-    return loadHelper->resolveType(typeName);
+    QQmlTypeLoader *typeLoader = &QQmlEnginePrivate::get(engine)->typeLoader;
+    // LoadHelper must be on the Heap as it derives from QQmlRefCount
+    loadHelper = QQml::makeRefPointer<LoadHelper>(typeLoader, uri, typeName, mode);
 }
 
-void QQmlComponentPrivate::completeLoadFromModule(QAnyStringView uri, QAnyStringView typeName, QQmlType type,
-                                                  LoadHelper::ResolveTypeResult::Status moduleStatus,
-                                                  QQmlComponent::CompilationMode mode)
+void QQmlComponentPrivate::completeLoadFromModule(QAnyStringView uri, QAnyStringView typeName)
 {
     Q_Q(QQmlComponent);
 
@@ -1418,16 +1451,28 @@ void QQmlComponentPrivate::completeLoadFromModule(QAnyStringView uri, QAnyString
         emit q->statusChanged(q->status());
     };
     emitProgressReset();
-    if (moduleStatus == LoadHelper::ResolveTypeResult::NoSuchModule) {
+
+    const QQmlType type = loadHelper->type();
+
+    if (loadHelper->resolveTypeResult() == LoadHelper::ResolveTypeResult::NoSuchModule)  {
         reportError(QLatin1String(R"(No module named "%1" found)").arg(uri.toString()));
     } else if (!type.isValid()) {
         reportError(QLatin1String(R"(Module "%1" contains no type named "%2")")
                     .arg(uri.toString(), typeName.toString()));
     } else if (type.isCreatable()) {
-        clear();
-        loadedType = type;
         emitComplete();
     } else if (type.isComposite()) {
+        QQmlComponent::CompilationMode mode = QQmlComponent::PreferSynchronous;
+        switch (loadHelper->mode()) {
+        case QQmlTypeLoader::Asynchronous:
+            mode = QQmlComponent::Asynchronous;
+            break;
+        case QQmlTypeLoader::PreferSynchronous:
+        case QQmlTypeLoader::Synchronous:
+            mode = QQmlComponent::PreferSynchronous;
+            break;
+        }
+
         // loadUrl takes care of signal emission
         loadUrl(type.sourceUrl(), mode);
     } else if (type.isInlineComponentType()) {
@@ -1469,8 +1514,8 @@ void QQmlComponentPrivate::completeLoadFromModule(QAnyStringView uri, QAnyString
     \a incubator. \a context specifies the context within which to create the object
     instance.
 
-    If \a context is \nullptr (by default), it will create the instance in the
-    engine's \l {QQmlEngine::rootContext()}{root context}.
+    If \a context is \nullptr (the default), it will create the instance in the
+    \l {QQmlEngine::rootContext()}{root context} of the engine.
 
     \a forContext specifies a context that this object creation depends upon.
     If the \a forContext is being created asynchronously, and the
@@ -1513,7 +1558,7 @@ void QQmlComponent::create(QQmlIncubator &incubator, QQmlContext *context, QQmlC
     incubator.clear();
     QExplicitlySharedDataPointer<QQmlIncubatorPrivate> p(incubator.d);
 
-    if (d->loadedType.isValid()) {
+    if (d->loadedType().isValid()) {
         // there isn't really an incubation process for C++ backed types
         // so just create the object and signal that we are ready
 
