@@ -27,6 +27,7 @@ QQmlMetaTypeData::~QQmlMetaTypeData()
     types.clear();
     undeletableTypes.clear();
     qDeleteAll(metaTypeToValueType);
+    clearCompositeMetaTypes();
 }
 
 // This expects a "fresh" QQmlTypePrivate and adopts its reference.
@@ -259,4 +260,18 @@ QQmlPropertyCache::ConstPtr QQmlMetaTypeData::findPropertyCacheInCompositeTypes(
             : propertyCacheForPotentialInlineComponentType(t, iter);
 }
 
+void QQmlMetaTypeData::clearCompositeMetaTypes()
+{
+    for (const auto &compositeMetaType : std::as_const(compositeMetaTypes)) {
+        Q_ASSERT(compositeMetaType.type);
+        QMetaType::unregisterMetaType(QMetaType(compositeMetaType.type));
+        delete compositeMetaType.type;
+
+        Q_ASSERT(compositeMetaType.listType);
+        QMetaType::unregisterMetaType(QMetaType(compositeMetaType.listType));
+        delete compositeMetaType.listType;
+    }
+    compositeMetaTypes.clear();
+}
+
 QT_END_NAMESPACE
