@@ -15,25 +15,26 @@
 // We mean it.
 //
 
-#include <QtQml/qtqmlglobal.h>
-#include <QtQml/qqmlparserstatus.h>
-#include <QtQml/qqmllist.h>
-#include <QtQml/qqmlpropertyvaluesource.h>
+#include <QtQml/qjsprimitivevalue.h>
 #include <QtQml/qjsvalue.h>
+#include <QtQml/qqmllist.h>
+#include <QtQml/qqmlparserstatus.h>
+#include <QtQml/qqmlpropertyvaluesource.h>
+#include <QtQml/qtqmlglobal.h>
 
+#include <QtCore/qdatetime.h>
+#include <QtCore/qdebug.h>
 #include <QtCore/qglobal.h>
-#include <QtCore/qvariant.h>
-#include <QtCore/qurl.h>
+#include <QtCore/qmetacontainer.h>
+#include <QtCore/qmetaobject.h>
 #include <QtCore/qpointer.h>
+#include <QtCore/qurl.h>
+#include <QtCore/qvariant.h>
 #include <QtCore/qversionnumber.h>
 
-#include <QtCore/qmetaobject.h>
-#include <QtCore/qmetacontainer.h>
-#include <QtCore/qdebug.h>
-
 #include <functional>
-#include <type_traits>
 #include <limits>
+#include <type_traits>
 
 QT_BEGIN_NAMESPACE
 
@@ -81,6 +82,10 @@ class QQmlEngine;
 class QQmlCustomParser;
 class QQmlTypeNotAvailable;
 
+class QQmlV4Function;
+using QQmlV4FunctionPtr = QQmlV4Function *;
+using QQmlV4ExecutionEnginePtr = QV4::ExecutionEngine *;
+
 template<class T>
 QQmlCustomParser *qmlCreateCustomParser()
 {
@@ -114,7 +119,7 @@ namespace QQmlPrivate
 #endif
     };
 
-    enum class ConstructionMode
+    enum class SingletonConstructionMode
     {
         None,
         Constructor,
@@ -139,33 +144,45 @@ namespace QQmlPrivate
     };
 
     template<typename T, typename WrapperT>
-    constexpr ConstructionMode constructionMode()
+    constexpr SingletonConstructionMode singletonConstructionMode()
     {
         if constexpr (!std::is_base_of<QObject, T>::value)
-            return ConstructionMode::None;
+            return SingletonConstructionMode::None;
         if constexpr (!std::is_same_v<T, WrapperT> && HasSingletonFactory<T, WrapperT>::value)
-            return ConstructionMode::FactoryWrapper;
+            return SingletonConstructionMode::FactoryWrapper;
         if constexpr (std::is_default_constructible<T>::value)
-            return ConstructionMode::Constructor;
+            return SingletonConstructionMode::Constructor;
         if constexpr (HasSingletonFactory<T>::value)
-            return ConstructionMode::Factory;
+            return SingletonConstructionMode::Factory;
 
-        return ConstructionMode::None;
+        return SingletonConstructionMode::None;
     }
 
+    template<typename>
+    struct QmlMarkerFunction;
+
+    template<typename Ret, typename Class>
+    struct QmlMarkerFunction<Ret (Class::*)()>
+    {
+        using ClassType = Class;
+    };
+
+    template<typename T, typename Marker>
+    using QmlTypeHasMarker = std::is_same<T, typename QmlMarkerFunction<Marker>::ClassType>;
+
     template<typename T>
     void createInto(void *memory, void *) { new (memory) QQmlElement<T>; }
 
-    template<typename T, typename WrapperT, ConstructionMode Mode>
+    template<typename T, typename WrapperT, SingletonConstructionMode Mode>
     QObject *createSingletonInstance(QQmlEngine *q, QJSEngine *j)
     {
         Q_UNUSED(q);
         Q_UNUSED(j);
-        if constexpr (Mode == ConstructionMode::Constructor)
+        if constexpr (Mode == SingletonConstructionMode::Constructor)
             return new T;
-        else if constexpr (Mode == ConstructionMode::Factory)
+        else if constexpr (Mode == SingletonConstructionMode::Factory)
             return T::create(q, j);
-        else if constexpr (Mode == ConstructionMode::FactoryWrapper)
+        else if constexpr (Mode == SingletonConstructionMode::FactoryWrapper)
             return WrapperT::create(q, j);
         else
             return nullptr;
@@ -179,39 +196,43 @@ namespace QQmlPrivate
     using CreateParentFunction = QObject *(*)(QObject *);
     using CreateValueTypeFunction = QVariant (*)(const QJSValue &);
 
-    template<typename T, typename WrapperT = T, ConstructionMode Mode = constructionMode<T, WrapperT>()>
+    template<typename T, typename WrapperT = T,
+             SingletonConstructionMode Mode = singletonConstructionMode<T, WrapperT>()>
     struct Constructors;
 
     template<typename T, typename WrapperT>
-    struct Constructors<T, WrapperT, ConstructionMode::Constructor>
+    struct Constructors<T, WrapperT, SingletonConstructionMode::Constructor>
     {
         static constexpr CreateIntoFunction createInto
                 = QQmlPrivate::createInto<T>;
         static constexpr CreateSingletonFunction createSingletonInstance
-                = QQmlPrivate::createSingletonInstance<T, WrapperT, ConstructionMode::Constructor>;
+                = QQmlPrivate::createSingletonInstance<
+                    T, WrapperT, SingletonConstructionMode::Constructor>;
     };
 
     template<typename T, typename WrapperT>
-    struct Constructors<T, WrapperT, ConstructionMode::None>
+    struct Constructors<T, WrapperT, SingletonConstructionMode::None>
     {
         static constexpr CreateIntoFunction createInto = nullptr;
         static constexpr CreateSingletonFunction createSingletonInstance = nullptr;
     };
 
     template<typename T, typename WrapperT>
-    struct Constructors<T, WrapperT, ConstructionMode::Factory>
+    struct Constructors<T, WrapperT, SingletonConstructionMode::Factory>
     {
         static constexpr CreateIntoFunction createInto = nullptr;
         static constexpr CreateSingletonFunction createSingletonInstance
-                = QQmlPrivate::createSingletonInstance<T, WrapperT, ConstructionMode::Factory>;
+                = QQmlPrivate::createSingletonInstance<
+                    T, WrapperT, SingletonConstructionMode::Factory>;
     };
 
     template<typename T, typename WrapperT>
-    struct Constructors<T, WrapperT, ConstructionMode::FactoryWrapper>
+    struct Constructors<T, WrapperT, SingletonConstructionMode::FactoryWrapper>
     {
         static constexpr CreateIntoFunction createInto = nullptr;
         static constexpr CreateSingletonFunction createSingletonInstance
-                = QQmlPrivate::createSingletonInstance<T, WrapperT, ConstructionMode::FactoryWrapper>;
+                = QQmlPrivate::createSingletonInstance<
+                    T, WrapperT, SingletonConstructionMode::FactoryWrapper>;
     };
 
     template<typename T,
@@ -349,11 +370,13 @@ namespace QQmlPrivate
         struct Properties<Parent, void>
         {
             using Func = QQmlAttachedPropertiesFunc<QObject>;
-            static const QMetaObject *staticMetaObject() { return nullptr; };
-            static Func attachedPropertiesFunc() { return nullptr; };
+            static const QMetaObject *staticMetaObject() { return nullptr; }
+            static Func attachedPropertiesFunc() { return nullptr; }
         };
 
-        using Type = typename OverridableAttachedType<T, typename T::QmlAttachedType>::Type;
+        using Type = typename std::conditional<
+                QmlTypeHasMarker<T, decltype(&T::qt_qmlMarker_attached)>::value,
+                typename OverridableAttachedType<T, typename T::QmlAttachedType>::Type, void>::type;
         using Func = typename Properties<T, Type>::Func;
 
         static const QMetaObject *staticMetaObject()
@@ -413,7 +436,18 @@ namespace QQmlPrivate
     enum AutoParentResult { Parented, IncompatibleObject, IncompatibleParent };
     typedef AutoParentResult (*AutoParentFunction)(QObject *object, QObject *parent);
 
+    enum class ValueTypeCreationMethod { None, Construct, Structured };
+
     struct RegisterType {
+        enum StructVersion: int {
+            Base = 0,
+            FinalizerCast = 1,
+            CreationMethod = 2,
+            CurrentVersion = CreationMethod,
+        };
+
+        bool has(StructVersion v) const { return structVersion >= int(v); }
+
         int structVersion;
 
         QMetaType typeId;
@@ -424,6 +458,7 @@ namespace QQmlPrivate
         void *userdata;
         QString noCreationReason;
 
+        // ### Qt7: Get rid of this. It can be covered by creationMethod below.
         QVariant (*createValueType)(const QJSValue &);
 
         const char *uri;
@@ -445,6 +480,8 @@ namespace QQmlPrivate
 
         QTypeRevision revision;
         int finalizerCast;
+
+        ValueTypeCreationMethod creationMethod;
         // If this is extended ensure "version" is bumped!!!
     };
 
@@ -591,6 +628,7 @@ namespace QQmlPrivate
             qintptr extraData;
         };
 
+        QObject *thisObject() const;
         QQmlEngine *qmlEngine() const;
 
         QJSValue jsMetaType(int index) const;
@@ -600,10 +638,41 @@ namespace QQmlPrivate
         // Run QQmlPropertyCapture::captureProperty() without retrieving the value.
         bool captureLookup(uint index, QObject *object) const;
         bool captureQmlContextPropertyLookup(uint index) const;
+        void captureTranslation() const;
+        QString translationContext() const;
         QMetaType lookupResultMetaType(uint index) const;
         void storeNameSloppy(uint nameIndex, void *value, QMetaType type) const;
         QJSValue javaScriptGlobalProperty(uint nameIndex) const;
 
+        const QLoggingCategory *resolveLoggingCategory(QObject *wrapper, bool *ok) const;
+
+        void writeToConsole(
+                QtMsgType type, const QString &message,
+                const QLoggingCategory *loggingCategory) const;
+
+        QVariant constructValueType(
+                QMetaType resultMetaType, const QMetaObject *resultMetaObject,
+                int ctorIndex, void **args) const;
+#if QT_QML_REMOVED_SINCE(6, 9)
+        QVariant constructValueType(
+                QMetaType resultMetaType, const QMetaObject *resultMetaObject,
+                int ctorIndex, void *ctorArg) const;
+#endif
+
+        // Those are explicit arguments to the Date() ctor, not implicit coercions.
+        QDateTime constructDateTime(double timestamp) const;
+        QDateTime constructDateTime(const QString &string) const;
+        QDateTime constructDateTime(const QJSPrimitiveValue &arg) const
+        {
+            return arg.type() == QJSPrimitiveValue::String
+                    ? constructDateTime(arg.toString())
+                    : constructDateTime(arg.toDouble());
+        }
+
+        QDateTime constructDateTime(
+                double year, double month, double day = 1,
+                double hours = 0, double minutes = 0, double seconds = 0, double msecs = 0) const;
+
         // All of these lookup functions should be used as follows:
         //
         // while (!fooBarLookup(...)) {
@@ -623,15 +692,26 @@ namespace QQmlPrivate
         // exception is present after the initialization there is no way to carry out the lookup and
         // the exception should be propagated. If not, the original lookup can be tried again.
 
+        bool callQmlContextPropertyLookup(uint index, void **args, int argc) const;
+        void initCallQmlContextPropertyLookup(uint index, int relativeMethodIndex) const;
+
+#if QT_QML_REMOVED_SINCE(6, 9)
         bool callQmlContextPropertyLookup(
                 uint index, void **args, const QMetaType *types, int argc) const;
         void initCallQmlContextPropertyLookup(uint index) const;
+#endif
 
         bool loadContextIdLookup(uint index, void *target) const;
         void initLoadContextIdLookup(uint index) const;
 
-        bool callObjectPropertyLookup(uint index, QObject *object,
-                                      void **args, const QMetaType *types, int argc) const;
+        bool callObjectPropertyLookup(uint index, QObject *object, void **args, int argc) const;
+        void initCallObjectPropertyLookup(
+                uint index, QObject *object, int relativeMethodIndex) const;
+        void initCallObjectPropertyLookupAsVariant(uint index, QObject *object) const;
+
+#if QT_QML_REMOVED_SINCE(6, 9)
+        bool callObjectPropertyLookup(
+                uint index, QObject *object, void **args, const QMetaType *types, int argc) const;
         void initCallObjectPropertyLookup(uint index) const;
 
         bool callGlobalLookup(uint index, void **args, const QMetaType *types, int argc) const;
@@ -639,9 +719,17 @@ namespace QQmlPrivate
 
         bool loadGlobalLookup(uint index, void *target, QMetaType type) const;
         void initLoadGlobalLookup(uint index) const;
+#endif
+
+        bool loadGlobalLookup(uint index, void *target) const;
+        void initLoadGlobalLookup(uint index, QMetaType type) const;
 
         bool loadScopeObjectPropertyLookup(uint index, void *target) const;
+        bool writeBackScopeObjectPropertyLookup(uint index, void *source) const;
+        void initLoadScopeObjectPropertyLookup(uint index) const;
+#if QT_QML_REMOVED_SINCE(6, 9)
         void initLoadScopeObjectPropertyLookup(uint index, QMetaType type) const;
+#endif
 
         bool loadSingletonLookup(uint index, void *target) const;
         void initLoadSingletonLookup(uint index, uint importNamespace) const;
@@ -653,29 +741,58 @@ namespace QQmlPrivate
         void initLoadTypeLookup(uint index, uint importNamespace) const;
 
         bool getObjectLookup(uint index, QObject *object, void *target) const;
+        bool writeBackObjectLookup(uint index, QObject *object, void *source) const;
+        void initGetObjectLookup(uint index, QObject *object) const;
+        void initGetObjectLookupAsVariant(uint index, QObject *object) const;
+#if QT_QML_REMOVED_SINCE(6, 9)
         void initGetObjectLookup(uint index, QObject *object, QMetaType type) const;
+#endif
 
         bool getValueLookup(uint index, void *value, void *target) const;
+        bool writeBackValueLookup(uint index, void *value, void *source) const;
+        void initGetValueLookup(uint index, const QMetaObject *metaObject) const;
+#if QT_QML_REMOVED_SINCE(6, 9)
         void initGetValueLookup(uint index, const QMetaObject *metaObject, QMetaType type) const;
+#endif
 
+        bool getEnumLookup(uint index, void *target) const;
+#if QT_QML_REMOVED_SINCE(6, 6)
         bool getEnumLookup(uint index, int *target) const;
+#endif
         void initGetEnumLookup(uint index, const QMetaObject *metaObject,
                                const char *enumerator, const char *enumValue) const;
 
         bool setObjectLookup(uint index, QObject *object, void *value) const;
+        void initSetObjectLookup(uint index, QObject *object) const;
+        void initSetObjectLookupAsVariant(uint index, QObject *object) const;
+#if QT_QML_REMOVED_SINCE(6, 9)
         void initSetObjectLookup(uint index, QObject *object, QMetaType type) const;
+#endif
 
         bool setValueLookup(uint index, void *target, void *value) const;
+        void initSetValueLookup(uint index, const QMetaObject *metaObject) const;
+        void initSetValueLookupAsVariant(uint index, const QMetaObject *metaObject) const;
+#if QT_QML_REMOVED_SINCE(6, 9)
         void initSetValueLookup(uint index, const QMetaObject *metaObject, QMetaType type) const;
+#endif
+
+        bool callValueLookup(uint index, void *target, void **args, int argc) const;
+        void initCallValueLookup(
+                uint index, const QMetaObject *metaObject, int relativeMethodIndex) const;
     };
 
     struct AOTCompiledFunction {
-        qintptr extraData;
-        QMetaType returnType;
-        QList<QMetaType> argumentTypes;
-        void (*functionPtr)(const AOTCompiledContext *context, void *resultPtr, void **arguments);
+        int functionIndex;
+        int numArguments;
+        void (*signature)(QV4::ExecutableCompilationUnit *unit, QMetaType *argTypes);
+        void (*functionPtr)(const AOTCompiledContext *context, void **argv);
     };
 
+#if QT_DEPRECATED_SINCE(6, 6)
+    QT_DEPRECATED_VERSION_X(6, 6, "Use AOTCompiledFunction instead")
+    typedef AOTCompiledFunction TypedFunction;
+#endif
+
     struct CachedQmlUnit {
         const QV4::CompiledData::Unit *qmlData;
         const AOTCompiledFunction *aotCompiledFunctions;
@@ -753,63 +870,20 @@ namespace QQmlPrivate
         const int index = indexOfOwnClassInfo(metaObject, key);
         return (index == -1) ? defaultValue
                              : QTypeRevision::fromEncodedVersion(
-                                   QByteArray(metaObject->classInfo(index).value()).toInt());
+                                   QLatin1StringView(metaObject->classInfo(index).value()).toInt());
     }
 
-    inline QList<QTypeRevision> revisionClassInfos(const QMetaObject *metaObject, const char *key)
-    {
-        QList<QTypeRevision> revisions;
-        for (int index = indexOfOwnClassInfo(metaObject, key); index != -1;
-             index = indexOfOwnClassInfo(metaObject, key, index - 1)) {
-            revisions.push_back(QTypeRevision::fromEncodedVersion(
-                          QByteArray(metaObject->classInfo(index).value()).toInt()));
-        }
-        return revisions;
-    }
+    Q_QML_EXPORT QList<QTypeRevision> revisionClassInfos(const QMetaObject *metaObject, const char *key);
 
     inline bool boolClassInfo(const QMetaObject *metaObject, const char *key,
                               bool defaultValue = false)
     {
         const int index = indexOfOwnClassInfo(metaObject, key);
-        return (index == -1) ? defaultValue
-                             : (QByteArray(metaObject->classInfo(index).value()) == "true");
+        if (index == -1)
+            return defaultValue;
+        return qstrcmp(metaObject->classInfo(index).value(), "true") == 0;
     }
 
-    inline const char *classElementName(const QMetaObject *metaObject)
-    {
-        const char *elementName = classInfo(metaObject, "QML.Element");
-        if (qstrcmp(elementName, "auto") == 0) {
-            const char *strippedClassName = metaObject->className();
-            for (const char *c = strippedClassName; *c != '\0'; c++) {
-                if (*c == ':')
-                    strippedClassName = c + 1;
-            }
-
-            return strippedClassName;
-        }
-        if (qstrcmp(elementName, "anonymous") == 0)
-            return nullptr;
-
-        if (!elementName) {
-            qWarning().nospace() << "Missing QML.Element class info \"" << elementName << "\""
-                                 << " for " << metaObject->className();
-        }
-
-        return elementName;
-    }
-
-    template<typename>
-    struct QmlMarkerFunction;
-
-    template<typename Ret, typename Class>
-    struct QmlMarkerFunction<Ret (Class::*)()>
-    {
-        using ClassType = Class;
-    };
-
-    template<typename T, typename Marker>
-    using QmlTypeHasMarker = std::is_same<T, typename QmlMarkerFunction<Marker>::ClassType>;
-
     template<class T, class = std::void_t<>>
     struct QmlExtended
     {
@@ -851,9 +925,40 @@ namespace QQmlPrivate
     template<class T>
     struct QmlResolved<T, std::void_t<typename T::QmlForeignType>>
     {
-        using Type = typename T::QmlForeignType;
+        using Type = typename std::conditional<
+                QmlTypeHasMarker<T, decltype(&T::qt_qmlMarker_foreign)>::value,
+                typename T::QmlForeignType, T>::type;
     };
 
+    template<class T, class = std::void_t<>>
+    struct QmlUncreatable
+    {
+        static constexpr bool Value = false;
+    };
+
+    template<class T>
+    struct QmlUncreatable<T, std::void_t<typename T::QmlIsUncreatable>>
+    {
+        static constexpr bool Value =
+                QmlTypeHasMarker<T, decltype(&T::qt_qmlMarker_uncreatable)>::value
+                && bool(T::QmlIsUncreatable::yes);
+    };
+
+    template<class T, class = std::void_t<>>
+    struct QmlAnonymous
+    {
+        static constexpr bool Value = false;
+    };
+
+    template<class T>
+    struct QmlAnonymous<T, std::void_t<typename T::QmlIsAnonymous>>
+    {
+        static constexpr bool Value =
+                QmlTypeHasMarker<T, decltype(&T::qt_qmlMarker_anonymous)>::value
+                && bool(T::QmlIsAnonymous::yes);
+    };
+
+
     template<class T, class = std::void_t<>>
     struct QmlSingleton
     {
@@ -863,7 +968,9 @@ namespace QQmlPrivate
     template<class T>
     struct QmlSingleton<T, std::void_t<typename T::QmlIsSingleton>>
     {
-        static constexpr bool Value = bool(T::QmlIsSingleton::yes);
+        static constexpr bool Value =
+                QmlTypeHasMarker<T, decltype(&T::qt_qmlMarker_singleton)>::value
+                && bool(T::QmlIsSingleton::yes);
     };
 
     template<class T, class = std::void_t<>>
@@ -887,7 +994,7 @@ namespace QQmlPrivate
     };
 
     template<class T>
-    struct QmlInterface<T, std::void_t<typename T::QmlIsInterface>>
+    struct QmlInterface<T, std::void_t<typename T::QmlIsInterface, decltype(qobject_interface_iid<T *>())>>
     {
         static constexpr bool Value = bool(T::QmlIsInterface::yes);
     };
@@ -909,11 +1016,15 @@ namespace QQmlPrivate
     {
         static constexpr bool hasAcceptableCtors()
         {
-            return std::is_base_of_v<QObject, T>
-                    || (std::is_default_constructible_v<T> && std::is_copy_constructible_v<T>);
+            if constexpr (!std::is_default_constructible_v<T>)
+                return false;
+            else if constexpr (std::is_base_of_v<QObject, T>)
+                return true;
+            else
+                return std::is_copy_constructible_v<T>;
         }
 
-        static QMetaType self()
+        static constexpr QMetaType self()
         {
             if constexpr (std::is_base_of_v<QObject, T>)
                 return QMetaType::fromType<T*>();
@@ -921,7 +1032,7 @@ namespace QQmlPrivate
                 return QMetaType::fromType<T>();
         }
 
-        static QMetaType list()
+        static constexpr QMetaType list()
         {
             if constexpr (std::is_base_of_v<QObject, T>)
                 return QMetaType::fromType<QQmlListProperty<T>>();
@@ -929,13 +1040,28 @@ namespace QQmlPrivate
                 return QMetaType::fromType<QList<T>>();
         }
 
-        static QMetaSequence sequence()
+        static constexpr QMetaSequence sequence()
         {
             if constexpr (std::is_base_of_v<QObject, T>)
                 return QMetaSequence();
             else
                 return QMetaSequence::fromContainer<QList<T>>();
         }
+
+        static constexpr int size()
+        {
+            return sizeof(T);
+        }
+    };
+
+    template<>
+    struct QmlMetaType<void>
+    {
+        static constexpr bool hasAcceptableCtors() { return true; }
+        static constexpr QMetaType self() { return QMetaType(); }
+        static constexpr QMetaType list() { return QMetaType(); }
+        static constexpr QMetaSequence sequence() { return QMetaSequence(); }
+        static constexpr int size() { return 0; }
     };
 
     template<typename T, typename E, typename WrapperT = T>
@@ -972,27 +1098,11 @@ namespace QQmlPrivate
                                      QVector<int> *qmlTypeIds, const QMetaObject *extension,
                                      bool forceAnonymous = false)
     {
-#if QT_DEPRECATED_SINCE(6, 4)
-        // ### Qt7: Remove the warnings, and leave only the static asserts below.
-        if (!QmlMetaType<T>::hasAcceptableCtors()) {
-            qWarning() << QMetaType::fromType<T>().name()
-                       << "is neither a QObject, nor default- and copy-constructible."
-                       << "You should not use it as a QML type.";
-        }
-        if (!std::is_base_of_v<QObject, T> && QQmlTypeInfo<T>::hasAttachedProperties) {
-            qWarning() << QMetaType::fromType<T>().name()
-                       << "is not a QObject, but has attached properties. This won't work.";
-        }
-#else
-        static_assert(QmlMetaType<T>::hasAcceptableCtors());
-        static_assert(std::is_base_of_v<QObject, T> || !QQmlTypeInfo<T>::hasAttachedProperties);
-#endif
-
         RegisterTypeAndRevisions type = {
             3,
             QmlMetaType<T>::self(),
             QmlMetaType<T>::list(),
-            int(sizeof(T)),
+            QmlMetaType<T>::size(),
             Constructors<T>::createInto,
             nullptr,
             ValueType<T, E>::create,
@@ -1058,7 +1168,7 @@ namespace QQmlPrivate
             /*.alignment=*/ 0,
             /*.size=*/ 0,
             /*.flags=*/ 0,
-            /*.typeId=*/ {},
+            /*.typeId=*/ QBasicAtomicInt(),
             /*.metaObject=*/ metaObjectFunction,
             /*.name=*/ name,
             /*.defaultCtr=*/ nullptr,
@@ -1076,8 +1186,28 @@ namespace QQmlPrivate
 
     Q_QML_EXPORT QObject *qmlExtendedObject(QObject *, int);
 
+    enum QmlRegistrationWarning {
+        UnconstructibleType,
+        UnconstructibleSingleton,
+        NonQObjectWithAtached,
+    };
+
+    Q_QML_EXPORT void qmlRegistrationWarning(QmlRegistrationWarning warning, QMetaType type);
+
+    Q_QML_EXPORT QMetaType compositeMetaType(
+            QV4::ExecutableCompilationUnit *unit, int elementNameId);
+    Q_QML_EXPORT QMetaType compositeMetaType(
+            QV4::ExecutableCompilationUnit *unit, const QString &elementName);
+    Q_QML_EXPORT QMetaType compositeListMetaType(
+            QV4::ExecutableCompilationUnit *unit, int elementNameId);
+    Q_QML_EXPORT QMetaType compositeListMetaType(
+            QV4::ExecutableCompilationUnit *unit, const QString &elementName);
+
 } // namespace QQmlPrivate
 
 QT_END_NAMESPACE
 
+Q_DECLARE_OPAQUE_POINTER(QQmlV4FunctionPtr)
+Q_DECLARE_OPAQUE_POINTER(QQmlV4ExecutionEnginePtr)
+
 #endif // QQMLPRIVATE_H
