@@ -10,13 +10,14 @@
 #include <private/qmetaobject_p.h>
 #include <private/qmetaobjectbuilder_p.h>
 #include <private/qqmlpropertycachemethodarguments_p.h>
+#include <private/qqmlsignalnames_p.h>
 
 #include <private/qv4value_p.h>
 
 #include <QtCore/qdebug.h>
 #include <QtCore/QCryptographicHash>
+#include <QtCore/private/qtools_p.h>
 
-#include <ctype.h> // for toupper
 #include <limits.h>
 #include <algorithm>
 
@@ -53,19 +54,13 @@ QQmlPropertyData::flagsForProperty(const QMetaProperty &p)
     const QMetaType metaType = p.metaType();
     int propType = metaType.id();
     if (p.isEnumType()) {
-        flags.type = QQmlPropertyData::Flags::EnumType;
+        flags.setType(QQmlPropertyData::Flags::EnumType);
     } else if (metaType.flags() & QMetaType::PointerToQObject) {
-        flags.type = QQmlPropertyData::Flags::QObjectDerivedType;
+        flags.setType(QQmlPropertyData::Flags::QObjectDerivedType);
     } else if (propType == QMetaType::QVariant) {
-        flags.type = QQmlPropertyData::Flags::QVariantType;
-    } else if (propType < static_cast<int>(QMetaType::User)) {
-        // nothing to do
-    } else if (propType == qMetaTypeId<QQmlBinding *>()) {
-        flags.type = QQmlPropertyData::Flags::QmlBindingType;
-    } else if (propType == qMetaTypeId<QJSValue>()) {
-        flags.type = QQmlPropertyData::Flags::QJSValueType;
+        flags.setType(QQmlPropertyData::Flags::QVariantType);
     } else if (metaType.flags() & QMetaType::IsQmlList) {
-        flags.type = QQmlPropertyData::Flags::QListType;
+        flags.setType(QQmlPropertyData::Flags::QListType);
     }
 
     return flags;
@@ -85,28 +80,43 @@ void QQmlPropertyData::load(const QMetaProperty &p)
 void QQmlPropertyData::load(const QMetaMethod &m)
 {
     setCoreIndex(m.methodIndex());
-    setArguments(nullptr);
+    m_flags.setType(Flags::FunctionType);
 
-    setPropType(m.returnMetaType());
+    // We need to set the constructor, signal, constant, arguments, V4Function, cloned flags.
+    // These are specific to methods and change with each method.
+    // The same QQmlPropertyData may be loaded with multiple methods in sequence.
 
-    m_flags.type = Flags::FunctionType;
-    if (m.methodType() == QMetaMethod::Signal) {
+    switch (m.methodType()) {
+    case QMetaMethod::Signal:
         m_flags.setIsSignal(true);
-    } else if (m.methodType() == QMetaMethod::Constructor) {
+        m_flags.setIsConstructor(false);
+        setPropType(m.returnMetaType());
+        break;
+    case QMetaMethod::Constructor:
+        m_flags.setIsSignal(false);
         m_flags.setIsConstructor(true);
-        setPropType(QMetaType::fromType<QObject *>());
+        break;
+    default:
+        m_flags.setIsSignal(false);
+        m_flags.setIsConstructor(false);
+        setPropType(m.returnMetaType());
+        break;
     }
+
     m_flags.setIsConstant(m.isConst());
 
     const int paramCount = m.parameterCount();
     if (paramCount) {
         m_flags.setHasArguments(true);
-        if ((paramCount == 1) && (m.parameterMetaType(0) == QMetaType::fromType<QQmlV4Function *>()))
-            m_flags.setIsV4Function(true);
+        m_flags.setIsV4Function(
+                paramCount == 1 &&
+                m.parameterMetaType(0) == QMetaType::fromType<QQmlV4FunctionPtr>());
+    } else {
+        m_flags.setHasArguments(false);
+        m_flags.setIsV4Function(false);
     }
 
-    if (m.attributes() & QMetaMethod::Cloned)
-        m_flags.setIsCloned(true);
+    m_flags.setIsCloned(m.attributes() & QMetaMethod::Cloned);
 
     Q_ASSERT(m.revision() <= std::numeric_limits<quint16>::max());
     setRevision(QTypeRevision::fromEncodedVersion(m.revision()));
@@ -210,15 +220,18 @@ void QQmlPropertyCache::appendProperty(const QString &name, QQmlPropertyData::Fl
     data.setFlags(flags);
     data.setTypeVersion(version);
 
-    const OverrideResult overrideResult = handleOverride(name, &data);
-    if (overrideResult == InvalidOverride)
+    QQmlPropertyData *old = findNamedProperty(name);
+    const OverrideResult overrideResult = handleOverride(name, &data, old);
+    if (overrideResult == InvalidOverride) {
+        // Insert the overridden member once more, to keep the counts in sync
+        propertyIndexCache.append(*old);
         return;
+    }
 
     int index = propertyIndexCache.size();
     propertyIndexCache.append(data);
 
-    setNamedProperty(name, index + propertyOffset(), propertyIndexCache.data() + index,
-                     overrideResult == ValidOverride);
+    setNamedProperty(name, index + propertyOffset(), propertyIndexCache.data() + index);
 }
 
 void QQmlPropertyCache::appendSignal(const QString &name, QQmlPropertyData::Flags flags,
@@ -242,9 +255,16 @@ void QQmlPropertyCache::appendSignal(const QString &name, QQmlPropertyData::Flag
         data.setArguments(args);
     }
 
-    const OverrideResult overrideResult = handleOverride(name, &data);
-    if (overrideResult == InvalidOverride)
+    QQmlPropertyData *old = findNamedProperty(name);
+    const OverrideResult overrideResult = handleOverride(name, &data, old);
+    if (overrideResult == InvalidOverride) {
+        // Insert the overridden member and its signal once more, to keep the counts in sync
+        methodIndexCache.append(*old);
+        handler = *old;
+        handler.m_flags.setIsSignalHandler(true);
+        signalHandlerIndexCache.append(handler);
         return;
+    }
 
     int methodIndex = methodIndexCache.size();
     methodIndexCache.append(data);
@@ -252,14 +272,11 @@ void QQmlPropertyCache::appendSignal(const QString &name, QQmlPropertyData::Flag
     int signalHandlerIndex = signalHandlerIndexCache.size();
     signalHandlerIndexCache.append(handler);
 
-    QString handlerName = QLatin1String("on") + name;
-    handlerName[2] = handlerName.at(2).toUpper();
+    const QString handlerName = QQmlSignalNames::signalNameToHandlerName(name);
 
-    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex,
-                     overrideResult == ValidOverride);
+    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex);
     setNamedProperty(handlerName, signalHandlerIndex + signalOffset(),
-                     signalHandlerIndexCache.data() + signalHandlerIndex,
-                     overrideResult == ValidOverride);
+                     signalHandlerIndexCache.data() + signalHandlerIndex);
 }
 
 void QQmlPropertyCache::appendMethod(const QString &name, QQmlPropertyData::Flags flags,
@@ -273,9 +290,13 @@ void QQmlPropertyCache::appendMethod(const QString &name, QQmlPropertyData::Flag
     data.setPropType(returnType);
     data.setCoreIndex(coreIndex);
     data.setFlags(flags);
-    const OverrideResult overrideResult = handleOverride(name, &data);
-    if (overrideResult == InvalidOverride)
+    QQmlPropertyData *old = findNamedProperty(name);
+    const OverrideResult overrideResult = handleOverride(name, &data, old);
+    if (overrideResult == InvalidOverride) {
+        // Insert the overridden member once more, to keep the counts in sync
+        methodIndexCache.append(*old);
         return;
+    }
 
     QQmlPropertyCacheMethodArguments *args = createArgumentsObject(argumentCount, names);
     new (args->types) QMetaType(returnType);
@@ -286,8 +307,7 @@ void QQmlPropertyCache::appendMethod(const QString &name, QQmlPropertyData::Flag
     int methodIndex = methodIndexCache.size();
     methodIndexCache.append(data);
 
-    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex,
-                     overrideResult == ValidOverride);
+    setNamedProperty(name, methodIndex + methodOffset(), methodIndexCache.data() + methodIndex);
 }
 
 void QQmlPropertyCache::appendEnum(const QString &name, const QVector<QQmlEnumValue> &values)
@@ -358,6 +378,17 @@ QQmlPropertyCache::copyAndAppend(const QMetaObject *metaObject,
     return rv;
 }
 
+static QHashedString signalNameToHandlerName(const QHashedString &methodName)
+{
+    return QQmlSignalNames::signalNameToHandlerName(methodName);
+}
+
+static QHashedString signalNameToHandlerName(const QHashedCStringRef &methodName)
+{
+    return QQmlSignalNames::signalNameToHandlerName(
+            QLatin1StringView{ methodName.constData(), methodName.length() });
+}
+
 void QQmlPropertyCache::append(const QMetaObject *metaObject,
                                QTypeRevision typeVersion,
                                QQmlPropertyData::Flags propertyFlags,
@@ -410,14 +441,14 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
     for (int ii = methodOffset; ii < methodCount; ++ii) {
         if (preventDestruction && (ii == destroyedIdx1 || ii == destroyedIdx2 || ii == deleteLaterIdx))
             continue;
-        QMetaMethod m = metaObject->method(ii);
+        const QMetaMethod &m = metaObject->method(ii);
         if (m.access() == QMetaMethod::Private)
             continue;
 
         // Extract method name
         // It's safe to keep the raw name pointer
         Q_ASSERT(QMetaObjectPrivate::get(metaObject)->revision >= 7);
-        const char *rawName = m.name().constData();
+        const char *rawName = m.nameView().constData();
         const char *cptr = rawName;
         char utf8 = 0;
         while (*cptr) {
@@ -446,49 +477,38 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
 
         QQmlPropertyData *old = nullptr;
 
-        if (utf8) {
-            QHashedString methodName(QString::fromUtf8(rawName, cptr - rawName));
+        const auto doSetNamedProperty = [&](const auto &methodName) {
             if (StringCache::mapped_type *it = stringCache.value(methodName)) {
-                if (handleOverride(methodName, data, (old = it->second)) == InvalidOverride)
-                    continue;
+                if (handleOverride(methodName, data, (old = it->second)) == InvalidOverride) {
+                    *data = *old;
+                    if (sigdata) {
+                        // Keep the signal counts in sync,
+                        // even if the "old" data has no real signal.
+                        *sigdata = *old;
+                        sigdata->m_flags.setIsSignalHandler(true);
+                        ++signalHandlerIndex;
+                    }
+                    return;
+                }
             }
-            setNamedProperty(methodName, ii, data, (old != nullptr));
+
+            setNamedProperty(methodName, ii, data);
 
             if (data->isSignal()) {
-                QHashedString on(QLatin1String("on") % methodName.at(0).toUpper() % QStringView{methodName}.mid(1));
-                setNamedProperty(on, ii, sigdata, (old != nullptr));
+
+                // TODO: Remove this once we can. Signals should not be overridable.
+                if (!utf8)
+                    data->m_flags.setIsOverridableSignal(true);
+
+                setNamedProperty(signalNameToHandlerName(methodName), ii, sigdata);
                 ++signalHandlerIndex;
             }
-        } else {
-            QHashedCStringRef methodName(rawName, cptr - rawName);
-            if (StringCache::mapped_type *it = stringCache.value(methodName)) {
-                if (handleOverride(methodName, data, (old = it->second)) == InvalidOverride)
-                    continue;
-            }
-            setNamedProperty(methodName, ii, data, (old != nullptr));
+        };
 
-            if (data->isSignal()) {
-                int length = methodName.length();
-
-                QVarLengthArray<char, 128> str(length+3);
-                str[0] = 'o';
-                str[1] = 'n';
-                str[2] = toupper(rawName[0]);
-                if (length > 1)
-                    memcpy(&str[3], &rawName[1], length - 1);
-                str[length + 2] = '\0';
-
-                QHashedString on(QString::fromLatin1(str.data()));
-                setNamedProperty(on, ii, data, (old != nullptr));
-                ++signalHandlerIndex;
-            }
-        }
-
-        if (old) {
-            // We only overload methods in the same class, exactly like C++
-            if (old->isFunction() && old->coreIndex() >= methodOffset)
-                data->m_flags.setIsOverload(true);
-        }
+        if (utf8)
+            doSetNamedProperty(QHashedString(QString::fromUtf8(rawName, cptr - rawName)));
+        else
+            doSetNamedProperty(QHashedCStringRef(rawName, cptr - rawName));
     }
 
     int propCount = metaObject->propertyCount();
@@ -524,17 +544,21 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
         if (utf8) {
             QHashedString propName(QString::fromUtf8(str, cptr - str));
             if (StringCache::mapped_type *it = stringCache.value(propName)) {
-                if (handleOverride(propName, data, (old = it->second)) == InvalidOverride)
+                if (handleOverride(propName, data, (old = it->second)) == InvalidOverride) {
+                    *data = *old;
                     continue;
+                }
             }
-            setNamedProperty(propName, ii, data, (old != nullptr));
+            setNamedProperty(propName, ii, data);
         } else {
             QHashedCStringRef propName(str, cptr - str);
             if (StringCache::mapped_type *it = stringCache.value(propName)) {
-                if (handleOverride(propName, data, (old = it->second)) == InvalidOverride)
+                if (handleOverride(propName, data, (old = it->second)) == InvalidOverride) {
+                    *data = *old;
                     continue;
+                }
             }
-            setNamedProperty(propName, ii, data, (old != nullptr));
+            setNamedProperty(propName, ii, data);
         }
 
         bool isGadget = true;
@@ -582,7 +606,6 @@ void QQmlPropertyCache::invalidate(const QMetaObject *metaObject)
     methodIndexCache.clear();
     signalHandlerIndexCache.clear();
 
-    _hasPropertyOverrides = false;
     argumentsCache = nullptr;
 
     int pc = metaObject->propertyCount();
@@ -690,28 +713,9 @@ const QQmlPropertyData *QQmlPropertyCache::findProperty(
     return nullptr;
 }
 
-QString QQmlPropertyData::name(QObject *object) const
-{
-    if (!object)
-        return QString();
 
-    return name(object->metaObject());
-}
 
-QString QQmlPropertyData::name(const QMetaObject *metaObject) const
-{
-    if (!metaObject || coreIndex() == -1)
-        return QString();
 
-    if (isFunction()) {
-        QMetaMethod m = metaObject->method(coreIndex());
-
-        return QString::fromUtf8(m.name().constData());
-    } else {
-        QMetaProperty p = metaObject->property(coreIndex());
-        return QString::fromUtf8(p.name());
-    }
-}
 
 bool QQmlPropertyData::markAsOverrideOf(QQmlPropertyData *predecessor)
 {
@@ -743,13 +747,17 @@ QString QQmlPropertyCache::signalParameterStringForJS(QV4::ExecutionEngine *engi
     const QSet<QString> &illegalNames = engine->illegalNames();
     QString parameters;
 
-    for (int i = 0; i < parameterNameList.size(); ++i) {
+    const qsizetype count = parameterNameList.size();
+    if (count > std::numeric_limits<quint16>::max())
+        *errorString = QCoreApplication::translate("QQmlRewrite", "Signal has an excessive number of parameters: %1").arg(count);
+
+    for (qsizetype i = 0; i < count; ++i) {
         if (i > 0)
             parameters += QLatin1Char(',');
         const QByteArray &param = parameterNameList.at(i);
-        if (param.isEmpty())
+        if (param.isEmpty()) {
             unnamedParameter = true;
-        else if (unnamedParameter) {
+        } else if (unnamedParameter) {
             if (errorString)
                 *errorString = QCoreApplication::translate("QQmlRewrite", "Signal uses unnamed parameter followed by named parameter.");
             return QString();
@@ -976,8 +984,10 @@ void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
     for (StringCache::ConstIterator iter = stringCache.begin(), cend = stringCache.end(); iter != cend; ++iter)
         Insert::in(this, properties, methods, iter, iter.value().second);
 
-    Q_ASSERT(properties.size() == propertyIndexCache.size());
-    Q_ASSERT(methods.size() == methodIndexCache.size());
+    // Any invalid overrides are not linked by name into the properties and methods hashes.
+    // Therefore there can be more properties and methods than present in the hashes.
+    Q_ASSERT(properties.size() <= propertyIndexCache.size());
+    Q_ASSERT(methods.size() <= methodIndexCache.size());
 
     std::sort(properties.begin(), properties.end(), Sort::lt);
     std::sort(methods.begin(), methods.end(), Sort::lt);
@@ -997,7 +1007,7 @@ void QQmlPropertyCache::toMetaObjectBuilder(QMetaObjectBuilder &builder) const
         property.setReadable(true);
         property.setWritable(data->isWritable());
         property.setResettable(data->isResettable());
-        property.setBindable(data->isBindable());
+        property.setBindable(data->notifiesViaBindable());
         property.setAlias(data->isAlias());
     }
 
@@ -1069,14 +1079,18 @@ int visitMethods(const QMetaObject &mo, int methodOffset, int methodCount,
 {
     int fieldsForParameterData = 0;
 
-    bool hasRevisionedMethods = false;
+    bool hasOldStyleRevisionedMethods = false;
 
     for (int i = 0; i < methodCount; ++i) {
         const int handle = methodOffset + i * QMetaObjectPrivate::IntsPerMethod;
 
         const uint flags = mo.d.data[handle + 4];
-        if (flags & MethodRevisioned)
-            hasRevisionedMethods = true;
+        if (flags & MethodRevisioned) {
+            if (mo.d.data[0] < 13)
+                hasOldStyleRevisionedMethods = true;
+            else
+                fieldsForParameterData += 1;    // revision
+        }
 
         visitString(mo.d.data[handle + 0]); // name
         visitString(mo.d.data[handle + 3]); // tag
@@ -1099,7 +1113,7 @@ int visitMethods(const QMetaObject &mo, int methodOffset, int methodCount,
     }
 
     int fieldsForRevisions = 0;
-    if (hasRevisionedMethods)
+    if (hasOldStyleRevisionedMethods)
         fieldsForRevisions = methodCount;
 
     return methodCount * QMetaObjectPrivate::IntsPerMethod
@@ -1189,13 +1203,13 @@ int countMetaObjectFields(const QMetaObject &mo, StringVisitor stringVisitor)
 
 } // anonymous namespace
 
-static_assert(QMetaObjectPrivate::OutputRevision == 10, "Check and adjust determineMetaObjectSizes");
+static_assert(QMetaObjectPrivate::OutputRevision == 13, "Check and adjust determineMetaObjectSizes");
 
 bool QQmlPropertyCache::determineMetaObjectSizes(const QMetaObject &mo, int *fieldCount,
                                                  int *stringCount)
 {
     const QMetaObjectPrivate *priv = reinterpret_cast<const QMetaObjectPrivate*>(mo.d.data);
-    if (priv->revision != 10)
+    if (priv->revision != QMetaObjectPrivate::OutputRevision)
         return false;
 
     uint highestStringIndex = 0;
