@@ -68,11 +68,6 @@ QQmlTypePrivate::~QQmlTypePrivate()
     delete enums.fetchAndStoreAcquire(nullptr);
     delete proxyMetaObjects.fetchAndStoreAcquire(nullptr);
 
-    if (const QtPrivate::QMetaTypeInterface *iface = typeId.iface()) {
-        if (iface->metaObjectFn == &dynamicQmlMetaObject)
-            QQmlMetaType::unregisterInternalCompositeType(typeId, listId);
-    }
-
     switch (regType) {
     case QQmlType::CppType:
         delete extraData.cppTypeData->customParser;
@@ -153,8 +148,9 @@ QQmlType QQmlTypePrivate::resolveCompositeBaseType(QQmlEnginePrivate *engine) co
     if (td.isNull() || !td->isComplete())
         return QQmlType();
     QV4::CompiledData::CompilationUnit *compilationUnit = td->compilationUnit();
-    const QMetaObject *mo = compilationUnit->rootPropertyCache()->firstCppMetaObject();
-    return QQmlMetaType::qmlType(mo);
+    if (const auto cache = compilationUnit->rootPropertyCache())
+        return QQmlMetaType::qmlType(cache->firstCppMetaObject());
+    return QQmlType();
 }
 
 QQmlPropertyCache::ConstPtr QQmlTypePrivate::compositePropertyCache(
@@ -323,9 +319,9 @@ void QQmlTypePrivate::insertEnums(Enums *enums, const QMetaObject *metaObject) c
 
     // Add any enum values defined by this class, overwriting any inherited values
     for (int ii = 0; ii < metaObject->enumeratorCount(); ++ii) {
-        QMetaEnum e = metaObject->enumerator(ii);
-        const bool isScoped = e.isScoped();
-        QStringHash<int> *scoped = isScoped ? new QStringHash<int>() : nullptr;
+        QMetaEnum me = metaObject->enumerator(ii);
+        const bool isScoped = me.isScoped();
+        QStringHash<int> *e = new QStringHash<int>();
 
     // We allow enums in sub-classes to overwrite enums from base-classes, such as
     // ListView.Center (from enum PositionMode) overwriting Item.Center (from enum TransformOrigin).
@@ -333,18 +329,18 @@ void QQmlTypePrivate::insertEnums(Enums *enums, const QMetaObject *metaObject) c
     // anyway, i.e. ListView.Center vs. Item.Center.
         // However if a class defines two enums with the same value, then that must produce a warning
         // because it represents a valid conflict.
-        if (e.enclosingMetaObject() != localMetaObject) {
+        if (me.enclosingMetaObject() != localMetaObject) {
             localEnums.clear();
-            localMetaObject = e.enclosingMetaObject();
+            localMetaObject = me.enclosingMetaObject();
         }
         const bool shouldRegisterUnscoped = !isScoped
                 || (regType == QQmlType::CppType && extraData.cppTypeData->registerEnumClassesUnscoped)
                 || (regType == QQmlType::SingletonType && shouldSingletonAlsoRegisterUnscoped())
         ;
 
-        for (int jj = 0; jj < e.keyCount(); ++jj) {
-            const QString key = QString::fromUtf8(e.key(jj));
-            const int value = e.value(jj);
+        for (int jj = 0; jj < me.keyCount(); ++jj) {
+            const QString key = QString::fromUtf8(me.key(jj));
+            const int value = me.value(jj);
             if (shouldRegisterUnscoped) {
                 if (localEnums.contains(key)) {
                     auto existingEntry = enums->enums.find(key);
@@ -357,13 +353,15 @@ void QQmlTypePrivate::insertEnums(Enums *enums, const QMetaObject *metaObject) c
                 }
                 enums->enums.insert(key, value);
             }
-            if (isScoped)
-                scoped->insert(key, value);
+            e->insert(key, value);
         }
 
         if (isScoped) {
-            enums->scopedEnums << scoped;
-            enums->scopedEnumIndex.insert(QString::fromUtf8(e.name()), enums->scopedEnums.size()-1);
+            enums->scopedEnumIndex.insert(QString::fromUtf8(me.name()), enums->scopedEnums.size());
+            enums->scopedEnums << e;
+        } else {
+            enums->unscopedEnumIndex.insert(QString::fromUtf8(me.name()), enums->unscopedEnums.size());
+            enums->unscopedEnums << e;
         }
     }
 }
@@ -792,12 +790,22 @@ int QQmlType::enumValue(QQmlTypeLoader *typeLoader, const QV4::String *name, boo
 
 int QQmlType::scopedEnumIndex(QQmlTypeLoader *typeLoader, const QV4::String *name, bool *ok) const
 {
-    return QQmlTypePrivate::scopedEnumIndex(d, typeLoader, name, ok);
+    return QQmlTypePrivate::enumIndex<QQmlTypePrivate::Enums::Scoped>(d, typeLoader, name, ok);
 }
 
 int QQmlType::scopedEnumIndex(QQmlTypeLoader *typeLoader, const QString &name, bool *ok) const
 {
-    return QQmlTypePrivate::scopedEnumIndex(d, typeLoader, name, ok);
+    return QQmlTypePrivate::enumIndex<QQmlTypePrivate::Enums::Scoped>(d, typeLoader, name, ok);
+}
+
+int QQmlType::unscopedEnumIndex(QQmlTypeLoader *typeLoader, const QV4::String *name, bool *ok) const
+{
+    return QQmlTypePrivate::enumIndex<QQmlTypePrivate::Enums::Unscoped>(d, typeLoader, name, ok);
+}
+
+int QQmlType::unscopedEnumIndex(QQmlTypeLoader *typeLoader, const QString &name, bool *ok) const
+{
+    return QQmlTypePrivate::enumIndex<QQmlTypePrivate::Enums::Unscoped>(d, typeLoader, name, ok);
 }
 
 int QQmlType::scopedEnumValue(QQmlTypeLoader *typeLoader, int index, const QV4::String *name, bool *ok) const
