@@ -12,6 +12,7 @@
 
 #include <private/qv4object_p.h>
 #include <private/qv4variantobject_p.h>
+#include <private/qv4variantassociationobject_p.h>
 #include <private/qv4functionobject_p.h>
 #include <private/qv4scopedvalue_p.h>
 #include <private/qv4jscall_p.h>
@@ -100,7 +101,7 @@ QQmlVMEResolvedList::~QQmlVMEResolvedList() = default;
 
 void QQmlVMEResolvedList::activateSignal() const
 {
-    m_metaObject->activate(m_metaObject->object, int(m_id + m_metaObject->methodOffset()), nullptr);
+    m_metaObject->activate(m_metaObject->object, int(m_id), nullptr);
 }
 
 void QQmlVMEMetaObject::list_append(QQmlListProperty<QObject> *prop, QObject *o)
@@ -173,7 +174,7 @@ void QQmlVMEVariantQObjectPtr::objectDestroyedImpl(QQmlGuardImpl *guard)
             }
         }
 
-        This->m_target->activate(This->m_target->object, This->m_target->methodOffset() + This->m_index, nullptr);
+        This->m_target->activate(This->m_target->object, This->m_index, nullptr);
     }
 }
 
@@ -211,15 +212,15 @@ void QQmlVMEMetaObjectEndpoint_callback(QQmlNotifierEndpoint *e, void **)
 
 void QQmlVMEMetaObjectEndpoint::tryConnect()
 {
-    Q_ASSERT(metaObject->compiledObject);
     int aliasId = this - metaObject->aliasEndpoints;
 
     if (metaObject.tag() == EndPointIsConnected) {
-        // This is actually notify
-        int sigIdx = metaObject->methodOffset() + aliasId + metaObject->compiledObject->nProperties;
+        // This is actually notify. Aliases are after regular properties in the MetaObject.
+        // So add propCount() to get the signal index.
+        int sigIdx = aliasId + metaObject->propCount();
         metaObject->activate(metaObject->object, sigIdx, nullptr);
-    } else {
-        const QV4::CompiledData::Alias *aliasData = &metaObject->compiledObject->aliasTable()[aliasId];
+    } else if (const QV4::CompiledData::Object *compiledObject = metaObject->findCompiledObject()) {
+        const QV4::CompiledData::Alias *aliasData = &compiledObject->aliasTable()[aliasId];
         if (!aliasData->isObjectAlias()) {
             QQmlRefPointer<QQmlContextData> ctxt = metaObject->ctxt;
             QObject *target = ctxt->idValue(aliasData->targetObjectId());
@@ -392,26 +393,35 @@ bool QQmlInterceptorMetaObject::doIntercept(QMetaObject::Call c, int id, void **
     return false;
 }
 
-static QMetaObject *stringCastMetaObject(QObject *o, const QMetaObject *top)
+static const QMetaObject *stringCastMetaObject(QObject *o, const QMetaObject *top)
 {
     for (const QMetaObject *mo = top; mo; mo = mo->superClass()) {
         if (o->qt_metacast(mo->className()) != nullptr)
-            return const_cast<QMetaObject *>(mo);
+            return mo;
     }
     return nullptr;
 }
 
+#if QT_VERSION >= QT_VERSION_CHECK(7, 0, 0)
+const QMetaObject *QQmlInterceptorMetaObject::toDynamicMetaObject(QObject *o) const
+#else
 QMetaObject *QQmlInterceptorMetaObject::toDynamicMetaObject(QObject *o)
+#endif
 {
     if (!metaObject)
         metaObject = cache->createMetaObject();
 
+    const QMetaObject *mo = nullptr;
     if (Q_UNLIKELY(metaObject.tag() == MetaObjectInvalid))
-        return stringCastMetaObject(o, metaObject->superClass());
+        mo = stringCastMetaObject(o, metaObject->superClass());
+    if (!mo)
+        mo = metaObject.data();
 
-    // ### Qt7: The const_cast is only due to toDynamicMetaObject having the wrong return type.
-    //          It should be const QMetaObject *. Fix this.
-    return const_cast<QMetaObject *>(metaObject.data());
+#if QT_VERSION >= QT_VERSION_CHECK(7, 0, 0)
+    return mo;
+#else
+    return const_cast<QMetaObject *>(mo);
+#endif
 }
 
 QQmlVMEMetaObject::QQmlVMEMetaObject(QV4::ExecutionEngine *engine,
@@ -420,32 +430,40 @@ QQmlVMEMetaObject::QQmlVMEMetaObject(QV4::ExecutionEngine *engine,
     : QQmlInterceptorMetaObject(obj, cache),
       engine(engine),
       ctxt(QQmlData::get(obj, true)->outerContext),
-      aliasEndpoints(nullptr), compilationUnit(qmlCompilationUnit), compiledObject(nullptr)
+      aliasEndpoints(nullptr), compilationUnit(qmlCompilationUnit), qmlObjectId(qmlObjectId)
 {
     Q_ASSERT(engine);
     QQmlData::get(obj)->hasVMEMetaObject = true;
 
-    if (compilationUnit && qmlObjectId >= 0) {
-        compiledObject = compilationUnit->objectAt(qmlObjectId);
-
-        if (compiledObject->nProperties || compiledObject->nFunctions) {
-            uint size = compiledObject->nProperties + compiledObject->nFunctions;
-            if (size) {
-                QV4::Heap::MemberData *data = QV4::MemberData::allocate(engine, size);
-                // we only have a weak reference below; if the VMEMetaObject is already marked
-                // (triggered by the allocate call above)
-                // we therefore might never mark the member data; consequently, mark it now
-                QV4::WriteBarrier::markCustom(engine, [data](QV4::MarkStack *ms) {
-                    data->mark(ms);
-                });
-                propertyAndMethodStorage.set(engine, data);
-                std::fill(data->values.values, data->values.values + data->values.size, QV4::Encode::undefined());
-            }
+    if (const QV4::CompiledData::Object *compiledObject = findCompiledObject()) {
+        numAliases = compiledObject->nAliases;
+        if (const uint size = compiledObject->nProperties + compiledObject->nFunctions) {
+            QV4::Heap::MemberData *data = QV4::MemberData::allocate(engine, size);
+            // we only have a weak reference below; if the VMEMetaObject is already marked
+            // (triggered by the allocate call above)
+            // we therefore might never mark the member data; consequently, mark it now
+            QV4::WriteBarrier::markCustom(engine, [data](QV4::MarkStack *ms) {
+                data->mark(ms);
+            });
+            propertyAndMethodStorage.set(engine, data);
+            std::fill(data->values.values, data->values.values + data->values.size, QV4::Encode::undefined());
 
             // Need JS wrapper to ensure properties/methods are marked.
             ensureQObjectWrapper();
         }
+
+        // Counts retrieved from the property cache should reflect the CompiledObject
+        Q_ASSERT(propCount() == compiledObject->propertyCount());
+        Q_ASSERT(aliasCount() == compiledObject->aliasCount());
+        Q_ASSERT(signalCount() == compiledObject->signalCount()
+                         + compiledObject->propertyCount()
+                         + compiledObject->aliasCount());
+        Q_ASSERT(methodCount() == compiledObject->functionCount());
     }
+
+    // We rely on ordering of properties, alias, signals, and methods.
+    Q_ASSERT(aliasOffset() == propOffset() + propCount());
+    Q_ASSERT(methodOffset() == signalOffset() + signalCount());
 }
 
 QQmlVMEMetaObject::~QQmlVMEMetaObject()
@@ -723,20 +741,12 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
     if (intercept(c, _id, a))
         return -1;
 
-    const int propertyCount = compiledObject ? int(compiledObject->nProperties) : 0;
-    const int aliasCount = compiledObject ? int(compiledObject->nAliases) : 0;
-    const int signalCount = compiledObject ? int(compiledObject->nSignals) : 0;
-    const int methodCount = compiledObject ? int(compiledObject->nFunctions) : 0;
-
     if (c == QMetaObject::ReadProperty || c == QMetaObject::WriteProperty || c == QMetaObject::ResetProperty || c == QMetaObject::BindableProperty) {
         if (id >= propOffset()) {
             id -= propOffset();
 
-            if (id < propertyCount) {
-                // if we reach this point, propertyCount must have been > 0, and thus compiledObject != nullptr
-                Q_ASSERT(compiledObject);
-                const QV4::CompiledData::Property &property = compiledObject->propertyTable()[id];
-                const QV4::CompiledData::CommonType t = property.commonType();
+            const QQmlPropertyData *propertyData = cache->property(_id);
+            if (!propertyData->isAlias()) {
 
                 // the context can be null if accessing var properties from cpp after re-parenting an item.
                 QQmlEnginePrivate *ep = (ctxt.isNull() || ctxt->engine() == nullptr)
@@ -744,7 +754,7 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                         : QQmlEnginePrivate::get(ctxt->engine());
 
                 if (c == QMetaObject::ReadProperty) {
-                    if (property.isList()) {
+                    if (propertyData->isQList()) {
                         // _id because this is an absolute property ID.
                         const QQmlPropertyData *propertyData = cache->property(_id);
                         const QMetaType propType = propertyData->propType();
@@ -796,6 +806,9 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                             qmlWarning(object) << "Cannot find member data";
                         }
                     } else {
+                        const QV4::CompiledData::CommonType t
+                                = QQmlPropertyCacheCreatorBase::propertyTypeForMetaType(
+                                    propertyData->propType());
                         switch (t) {
                         case QV4::CompiledData::CommonType::Void:
                             break;
@@ -879,9 +892,7 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                 } else if (c == QMetaObject::WriteProperty) {
                     bool needActivate = false;
 
-                    if (property.isList()) {
-                        // _id because this is an absolute property ID.
-                        const QQmlPropertyData *propertyData = cache->property(_id);
+                    if (propertyData->isQList()) {
                         const QMetaType propType = propertyData->propType();
 
                         if (propType.flags().testFlag(QMetaType::IsQmlList)) {
@@ -927,6 +938,9 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                             qmlWarning(object) << "Cannot find member data";
                         }
                     } else {
+                        const QV4::CompiledData::CommonType t
+                                = QQmlPropertyCacheCreatorBase::propertyTypeForMetaType(
+                                        propertyData->propType());
                         switch (t) {
                         case QV4::CompiledData::CommonType::Void:
                             break;
@@ -983,7 +997,7 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                             break;
                         case QV4::CompiledData::CommonType::Var:
                             if (ep)
-                                writeProperty(id, *reinterpret_cast<QVariant *>(a[0]));
+                                writeKnownVarProperty(id, *reinterpret_cast<QVariant *>(a[0]));
                             break;
                         case QV4::CompiledData::CommonType::Invalid:
                             if (QV4::MemberData *md = propertyAndMethodStorageAsMemberData()) {
@@ -1026,15 +1040,19 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                     }
 
                     if (needActivate)
-                        activate(object, methodOffset() + id, nullptr);
+                        activate(object, id, nullptr);
                 }
 
                 return -1;
             }
 
-            id -= propertyCount;
+            id -= propCount();
+
+            if (id < aliasCount()) {
+                const QV4::CompiledData::Object *compiledObject = findCompiledObject();
+                if (!compiledObject)
+                    return -1;
 
-            if (id < aliasCount) {
                 const QV4::CompiledData::Alias *aliasData = &compiledObject->aliasTable()[id];
 
                 if (aliasData->hasFlag(QV4::CompiledData::Alias::AliasPointsToPointerObject)
@@ -1052,7 +1070,7 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                 if (!target)
                     return -1;
 
-                connectAlias(id);
+                connectAlias(compiledObject, id);
 
                 if (aliasData->isObjectAlias()) {
                     *reinterpret_cast<QObject **>(a[0]) = target;
@@ -1090,6 +1108,11 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
                     QQmlGadgetPtrWrapper *valueType = QQmlGadgetPtrWrapper::instance(
                                 ctxt->engine(), pd->propType());
                     if (valueType) {
+
+                        // For value type aliases, the core property provides the bindable.
+                        if (c == QMetaObject::BindableProperty)
+                            return QMetaObject::metacall(target, c, coreIndex, a);
+
                         removePendingBinding(target, coreIndex, encodedIndex);
                         valueType->read(target, coreIndex);
                         int rv = QMetaObject::metacall(valueType, c, valueTypePropertyIndex, a);
@@ -1121,18 +1144,16 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
 
     } else if(c == QMetaObject::InvokeMetaMethod) {
 
-        if (id >= methodOffset()) {
+        if (id >= signalOffset()) {
 
-            id -= methodOffset();
-            int plainSignals = signalCount + propertyCount + aliasCount;
-            if (id < plainSignals) {
-                activate(object, _id, a);
+            id -= signalOffset();
+            if (id < signalCount()) {
+                activate(object, id, a);
                 return -1;
             }
 
-            id -= plainSignals;
-
-            if (id < methodCount) {
+            id -= signalCount();
+            if (id < methodCount()) {
                 QQmlEngine *engine = ctxt->engine();
                 if (!engine)
                     return -1; // We can't run the method
@@ -1190,9 +1211,9 @@ int QQmlVMEMetaObject::metaCall(QObject *o, QMetaObject::Call c, int _id, void *
         return object->qt_metacall(c, _id, a);
 }
 
-QV4::ReturnedValue QQmlVMEMetaObject::method(int index) const
+QV4::ReturnedValue QQmlVMEMetaObject::method(int localMethodIndex) const
 {
-    if (ctxt.isNull() || !ctxt->isValid() || !compiledObject) {
+    if (ctxt.isNull() || !ctxt->isValid()) {
         qWarning("QQmlVMEMetaObject: Internal error - attempted to evaluate a function in an invalid context");
         return QV4::Encode::undefined();
     }
@@ -1201,12 +1222,12 @@ QV4::ReturnedValue QQmlVMEMetaObject::method(int index) const
     if (!md)
         return QV4::Encode::undefined();
 
-    return (md->data() + index + compiledObject->nProperties)->asReturnedValue();
+    return (md->data() + localMethodIndex + propCount())->asReturnedValue();
 }
 
 QV4::ReturnedValue QQmlVMEMetaObject::readVarProperty(int id) const
 {
-    Q_ASSERT(compiledObject && compiledObject->propertyTable()[id].commonType() == QV4::CompiledData::CommonType::Var);
+    Q_ASSERT(!findCompiledObject() || findCompiledObject()->propertyTable()[id].commonType() == QV4::CompiledData::CommonType::Var);
 
     QV4::MemberData *md = propertyAndMethodStorageAsMemberData();
     if (md)
@@ -1231,7 +1252,7 @@ QVariant QQmlVMEMetaObject::readPropertyAsVariant(int id) const
 
 void QQmlVMEMetaObject::writeVarProperty(int id, const QV4::Value &value)
 {
-    Q_ASSERT(compiledObject && compiledObject->propertyTable()[id].commonType() == QV4::CompiledData::CommonType::Var);
+    Q_ASSERT(!findCompiledObject() || findCompiledObject()->propertyTable()[id].commonType() == QV4::CompiledData::CommonType::Var);
 
     QV4::MemberData *md = propertyAndMethodStorageAsMemberData();
     if (!md)
@@ -1262,24 +1283,18 @@ void QQmlVMEMetaObject::writeVarProperty(int id, const QV4::Value &value)
         }
         md->set(engine, id, value);
     } else if (const QV4::Sequence *sequence = value.as<QV4::Sequence>()) {
-        QV4::Heap::Sequence *p = sequence->d();
-        if (p->enforcesLocation()) {
-            // If the sequence enforces its location, we don't want it to be updated anymore after
-            // being written to a property.
-            md->set(engine, id, QV4::ReferenceObject::detached(p));
-        } else {
-            // Otherwise, make sure the reference carries some value so that we can still call
-            // toVariant() on it (see note in QV4::SequencePrototype::toVariant).
-            if (!p->hasData())
-                QV4::ReferenceObject::readReference(p);
-            md->set(engine, id, p);
-        }
+        md->set(engine, id, QV4::ReferenceObject::detached(sequence->d()));
     } else if (const QV4::QQmlValueTypeWrapper *wrapper = value.as<QV4::QQmlValueTypeWrapper>()) {
-        // If the value type enforces its location, we don't want it to be updated anymore after
-        // being written to a property.
-        QV4::Heap::QQmlValueTypeWrapper *p = wrapper->d();
-        md->set(engine, id, p->enforcesLocation() ? QV4::ReferenceObject::detached(p) : p);
+        md->set(engine, id, QV4::ReferenceObject::detached(wrapper->d()));
+    } else if (const QV4::DateObject *date = value.as<QV4::DateObject>()) {
+        md->set(engine, id, QV4::ReferenceObject::detached(date->d()));
+    } else if (const QV4::VariantAssociationObject *association = value.as<QV4::VariantAssociationObject>()) {
+        md->set(engine, id, QV4::ReferenceObject::detached(association->d()));
     } else {
+        // TODO: We should have a virtualDetach to reduce the
+        // boilerplate and avoid having to add new cases when a new
+        // reference object is added.
+        Q_ASSERT(!value.as<QV4::ReferenceObject>());
         md->set(engine, id, value);
     }
 
@@ -1287,91 +1302,66 @@ void QQmlVMEMetaObject::writeVarProperty(int id, const QV4::Value &value)
         guard->setGuardedValue(valueObject, this, id);
 
     // Emit change signal as appropriate.
-    activate(object, methodOffset() + id, nullptr);
+    activate(object, id, nullptr);
 }
 
-void QQmlVMEMetaObject::writeProperty(int id, const QVariant &value)
+void QQmlVMEMetaObject::writeKnownVarProperty(int id, const QVariant &value)
 {
-    if (compiledObject && compiledObject->propertyTable()[id].commonType() == QV4::CompiledData::CommonType::Var) {
-        QV4::MemberData *md = propertyAndMethodStorageAsMemberData();
-        if (!md)
-            return;
+    // This can only be called from QQmlVMEMetaObject::metaCall() if we've already determined
+    // that the property type is "var". No need to double-check it here.
+    Q_ASSERT(findCompiledObject() && findCompiledObject()->propertyTable()[id].commonType() == QV4::CompiledData::CommonType::Var);
 
-        // Importantly, if the current value is a scarce resource, we need to ensure that it
-        // gets automatically released by the engine if no other references to it exist.
-        const QV4::VariantObject *oldv = (md->data() + id)->as<QV4::VariantObject>();
-        if (oldv)
-            oldv->removeVmePropertyReference();
+    QV4::MemberData *md = propertyAndMethodStorageAsMemberData();
+    if (!md)
+        return;
 
-        // And, if the new value is a scarce resource, we need to ensure that it does not get
-        // automatically released by the engine until no other references to it exist.
-        QV4::Scope scope(engine);
-        QV4::ScopedValue newv(scope, engine->fromVariant(value));
-        QV4::Scoped<QV4::VariantObject> v(scope, newv);
-        if (!!v)
-            v->addVmePropertyReference();
+    // Importantly, if the current value is a scarce resource, we need to ensure that it
+    // gets automatically released by the engine if no other references to it exist.
+    const QV4::VariantObject *oldv = (md->data() + id)->as<QV4::VariantObject>();
+    if (oldv)
+        oldv->removeVmePropertyReference();
 
-        // Write the value and emit change signal as appropriate.
-        QVariant currentValue = readPropertyAsVariant(id);
-        md->set(engine, id, newv);
-        if ((currentValue.userType() != value.userType() || currentValue != value))
-            activate(object, methodOffset() + id, nullptr);
-    } else {
-        bool needActivate = false;
-        if (value.userType() == QMetaType::QObjectStar) {
-            QObject *o = *(QObject *const *)value.data();
-            needActivate = readPropertyAsQObject(id) != o;  // TODO: still correct?
-            writeProperty(id, o);
-        } else {
-            QV4::MemberData *md = propertyAndMethodStorageAsMemberData();
-            if (md) {
-                const QV4::VariantObject *v = (md->data() + id)->as<QV4::VariantObject>();
-                needActivate = (!v ||
-                                 v->d()->data().userType() != value.userType() ||
-                                 v->d()->data() != value);
-                if (v)
-                    v->removeVmePropertyReference();
-                md->set(engine, id, engine->newVariantObject(value.metaType(), value.constData()));
-                v = static_cast<const QV4::VariantObject *>(md->data() + id);
-                v->addVmePropertyReference();
-            }
-        }
+    // And, if the new value is a scarce resource, we need to ensure that it does not get
+    // automatically released by the engine until no other references to it exist.
+    QV4::Scope scope(engine);
+    QV4::ScopedValue newv(scope, engine->fromVariant(value));
+    QV4::Scoped<QV4::VariantObject> v(scope, newv);
+    if (!!v)
+        v->addVmePropertyReference();
 
-        if (needActivate)
-            activate(object, methodOffset() + id, nullptr);
-    }
+    // Write the value and emit change signal as appropriate.
+    QVariant currentValue = readPropertyAsVariant(id);
+    md->set(engine, id, newv);
+    if ((currentValue.userType() != value.userType() || currentValue != value))
+        activate(object, id, nullptr);
 }
 
 QV4::ReturnedValue QQmlVMEMetaObject::vmeMethod(int index) const
 {
-    if (index < methodOffset()) {
+    if (index < signalOffset()) {
         Q_ASSERT(parentVMEMetaObject());
         return parentVMEMetaObject()->vmeMethod(index);
     }
-    if (!compiledObject)
-        return QV4::Value::undefinedValue().asReturnedValue();
-    const int plainSignals = compiledObject->nSignals + compiledObject->nProperties + compiledObject->nAliases;
-    Q_ASSERT(index >= (methodOffset() + plainSignals) && index < (methodOffset() + plainSignals + int(compiledObject->nFunctions)));
-    return method(index - methodOffset() - plainSignals);
+
+    Q_ASSERT(index >= methodOffset());
+    return method(index - methodOffset());
 }
 
 // Used by debugger
 void QQmlVMEMetaObject::setVmeMethod(int index, const QV4::Value &function)
 {
-    if (index < methodOffset()) {
+    if (index < signalOffset()) {
         Q_ASSERT(parentVMEMetaObject());
         return parentVMEMetaObject()->setVmeMethod(index, function);
     }
-    if (!compiledObject)
-        return;
-    const int plainSignals = compiledObject->nSignals + compiledObject->nProperties + compiledObject->nAliases;
-    Q_ASSERT(index >= (methodOffset() + plainSignals) && index < (methodOffset() + plainSignals + int(compiledObject->nFunctions)));
 
-    int methodIndex = index - methodOffset() - plainSignals;
+    Q_ASSERT(index >= methodOffset());
+
+    const int localMethodIndex = index - methodOffset();
     QV4::MemberData *md = propertyAndMethodStorageAsMemberData();
     if (!md)
         return;
-    md->set(engine, methodIndex + compiledObject->nProperties, function);
+    md->set(engine, localMethodIndex + propCount(), function);
 }
 
 QV4::ReturnedValue QQmlVMEMetaObject::vmeProperty(int index) const
@@ -1412,8 +1402,6 @@ void QQmlVMEMetaObject::mark(QV4::MarkStack *markStack)
 
 bool QQmlVMEMetaObject::aliasTarget(int index, QObject **target, int *coreIndex, int *valueTypeIndex) const
 {
-    Q_ASSERT(compiledObject && (index >= propOffset() + int(compiledObject->nProperties)));
-
     *target = nullptr;
     *coreIndex = -1;
     *valueTypeIndex = -1;
@@ -1421,7 +1409,13 @@ bool QQmlVMEMetaObject::aliasTarget(int index, QObject **target, int *coreIndex,
     if (ctxt.isNull())
         return false;
 
+    const QV4::CompiledData::Object *compiledObject = findCompiledObject();
+    if (!compiledObject)
+        return false;
+
     const int aliasId = index - propOffset() - compiledObject->nProperties;
+    Q_ASSERT(index >= propOffset() + int(compiledObject->nProperties));
+
     const QV4::CompiledData::Alias *aliasData = &compiledObject->aliasTable()[aliasId];
     while (aliasData->isAliasToLocalAlias())
         aliasData = &compiledObject->aliasTable()[aliasData->localAliasIndex];
@@ -1437,7 +1431,7 @@ bool QQmlVMEMetaObject::aliasTarget(int index, QObject **target, int *coreIndex,
     return true;
 }
 
-void QQmlVMEMetaObject::connectAlias(int aliasId)
+void QQmlVMEMetaObject::connectAlias(const QV4::CompiledData::Object *compiledObject, int aliasId)
 {
     Q_ASSERT(compiledObject);
     if (!aliasEndpoints)
@@ -1459,26 +1453,29 @@ void QQmlVMEMetaObject::connectAlias(int aliasId)
 
 void QQmlVMEMetaObject::connectAliasSignal(int index, bool indexInSignalRange)
 {
-    Q_ASSERT(compiledObject);
-    int aliasId = (index - (indexInSignalRange ? signalOffset() : methodOffset())) - compiledObject->nProperties;
-    if (aliasId < 0 || aliasId >= int(compiledObject->nAliases))
-        return;
+    if (const QV4::CompiledData::Object *compiledObject = findCompiledObject()) {
+        const int aliasId = index
+                - (indexInSignalRange ? cache->signalOffset() : signalOffset())
+                - compiledObject->nProperties;
+        if (aliasId < 0 || aliasId >= int(compiledObject->nAliases))
+            return;
 
-    connectAlias(aliasId);
+        connectAlias(compiledObject, aliasId);
+    }
 }
 
 /*! \internal
-    \a index is in the method index range (QMetaMethod::methodIndex()).
+    \a index is in the local signal index.
 */
 void QQmlVMEMetaObject::activate(QObject *object, int index, void **args)
 {
-    QMetaObject::activate(object, signalOffset(), index - methodOffset(), args);
+    QMetaObject::activate(object, cache->signalOffset(), index, args);
 }
 
 QQmlVMEMetaObject *QQmlVMEMetaObject::getForProperty(QObject *o, int coreIndex)
 {
     QQmlVMEMetaObject *vme = QQmlVMEMetaObject::get(o);
-    while (vme && vme->propOffset() > coreIndex)
+    while (vme && vme->cache->propertyOffset() > coreIndex)
         vme = vme->parentVMEMetaObject();
 
     Q_ASSERT(vme);
@@ -1488,7 +1485,7 @@ QQmlVMEMetaObject *QQmlVMEMetaObject::getForProperty(QObject *o, int coreIndex)
 QQmlVMEMetaObject *QQmlVMEMetaObject::getForMethod(QObject *o, int coreIndex)
 {
     QQmlVMEMetaObject *vme = QQmlVMEMetaObject::get(o);
-    while (vme && vme->methodOffset() > coreIndex)
+    while (vme && vme->cache->methodOffset() > coreIndex)
         vme = vme->parentVMEMetaObject();
 
     Q_ASSERT(vme);
@@ -1502,7 +1499,7 @@ QQmlVMEMetaObject *QQmlVMEMetaObject::getForMethod(QObject *o, int coreIndex)
 QQmlVMEMetaObject *QQmlVMEMetaObject::getForSignal(QObject *o, int coreIndex)
 {
     QQmlVMEMetaObject *vme = QQmlVMEMetaObject::get(o);
-    while (vme && vme->signalOffset() > coreIndex)
+    while (vme && vme->cache->signalOffset() > coreIndex)
         vme = vme->parentVMEMetaObject();
 
     Q_ASSERT(vme);
