@@ -8,6 +8,8 @@
 #include <QtCore/qloggingcategory.h>
 #include <QtCore/qfileinfo.h>
 
+#include <QtQml/private/qqmlsignalnames_p.h>
+
 QT_BEGIN_NAMESPACE
 
 using namespace Qt::StringLiterals;
@@ -53,12 +55,15 @@ static QString bindingTypeDescription(QmlIR::Binding::Type type)
 
 void QQmlJSFunctionInitializer::populateSignature(
         const QV4::Compiler::Context *context, QQmlJS::AST::FunctionExpression *ast,
-        QQmlJSCompilePass::Function *function, QQmlJS::DiagnosticMessage *error)
+        QQmlJSCompilePass::Function *function, QList<QQmlJS::DiagnosticMessage> *errors)
 {
     const auto signatureError = [&](const QString &message) {
-        error->type = QtWarningMsg;
-        error->loc = ast->firstSourceLocation();
-        error->message = message;
+        QQmlJS::DiagnosticMessage error;
+        error.type = QtWarningMsg;
+        error.loc = ast->firstSourceLocation();
+        error.message = message;
+        *errors << error;
+        function->isFullyTyped = false;
     };
 
     if (!m_typeResolver->canCallJSFunctions()) {
@@ -71,25 +76,30 @@ void QQmlJSFunctionInitializer::populateSignature(
     if (ast->formals)
         arguments = ast->formals->formals();
 
+    // If the function has no arguments and no return type annotation we assume it's untyped.
+    // You can annotate it to return void to make it typed.
+    // Otherwise we first assume it's typed and reset the flag if we detect a problem.
+    function->isFullyTyped = !arguments.isEmpty() || ast->typeAnnotation;
+
     if (function->argumentTypes.isEmpty()) {
+        bool alreadyWarnedAboutMissingAnnotations = false;
         for (const QQmlJS::AST::BoundName &argument : std::as_const(arguments)) {
             if (argument.typeAnnotation) {
                 if (const auto type = m_typeResolver->typeFromAST(argument.typeAnnotation->type)) {
-                    function->argumentTypes.append(
-                                m_typeResolver->tracked(
-                                    m_typeResolver->globalType(type)));
+                    function->argumentTypes.append(m_typeResolver->namedType(type));
                 } else {
                     function->argumentTypes.append(
-                                m_typeResolver->tracked(
-                                    m_typeResolver->globalType(m_typeResolver->varType())));
+                            m_typeResolver->namedType(m_typeResolver->varType()));
                     signatureError(u"Cannot resolve the argument type %1."_s
                                    .arg(argument.typeAnnotation->type->toString()));
                 }
             } else {
+                if (!alreadyWarnedAboutMissingAnnotations) {
+                    alreadyWarnedAboutMissingAnnotations = true;
+                    signatureError(u"Functions without type annotations won't be compiled"_s);
+                }
                 function->argumentTypes.append(
-                            m_typeResolver->tracked(
-                                m_typeResolver->globalType(m_typeResolver->varType())));
-                signatureError(u"Functions without type annotations won't be compiled"_s);
+                        m_typeResolver->namedType(m_typeResolver->varType()));
             }
         }
     } else {
@@ -97,7 +107,7 @@ void QQmlJSFunctionInitializer::populateSignature(
             const QQmlJS::AST::BoundName &argument = arguments[i];
             if (argument.typeAnnotation) {
                 if (const auto type = m_typeResolver->typeFromAST(argument.typeAnnotation->type)) {
-                    if (!m_typeResolver->registerContains(function->argumentTypes[i], type)) {
+                    if (!function->argumentTypes[i].contains(type)) {
                         signatureError(u"Type annotation %1 on signal handler "
                                         "contradicts signal argument type %2"_s
                                        .arg(argument.typeAnnotation->type->toString(),
@@ -108,10 +118,11 @@ void QQmlJSFunctionInitializer::populateSignature(
         }
     }
 
-    if (!function->returnType) {
+    if (!function->returnType.isValid()) {
         if (ast->typeAnnotation) {
-            function->returnType = m_typeResolver->typeFromAST(ast->typeAnnotation->type);
-            if (!function->returnType)
+            function->returnType = m_typeResolver->namedType(
+                    m_typeResolver->typeFromAST(ast->typeAnnotation->type));
+            if (!function->returnType.isValid())
                 signatureError(u"Cannot resolve return type %1"_s.arg(
                                    QmlIR::IRBuilder::asString(ast->typeAnnotation->type->typeId)));
         }
@@ -119,8 +130,7 @@ void QQmlJSFunctionInitializer::populateSignature(
 
     for (int i = QQmlJSCompilePass::FirstArgument + function->argumentTypes.size();
          i < context->registerCountInFunction; ++i) {
-        function->registerTypes.append(m_typeResolver->tracked(
-                                           m_typeResolver->globalType(m_typeResolver->voidType())));
+        function->registerTypes.append(m_typeResolver->namedType(m_typeResolver->voidType()));
     }
 
     function->addressableScopes = m_typeResolver->objectsById();
@@ -130,13 +140,9 @@ void QQmlJSFunctionInitializer::populateSignature(
 
 static void diagnose(
         const QString &message, QtMsgType type, const QQmlJS::SourceLocation &location,
-        QQmlJS::DiagnosticMessage *error)
+        QList<QQmlJS::DiagnosticMessage> *errors)
 {
-    *error = QQmlJS::DiagnosticMessage{
-        message,
-        type,
-        location
-    };
+    *errors << QQmlJS::DiagnosticMessage{ message, type, location };
 }
 
 QQmlJSCompilePass::Function QQmlJSFunctionInitializer::run(
@@ -144,30 +150,46 @@ QQmlJSCompilePass::Function QQmlJSFunctionInitializer::run(
         const QString &propertyName,
         QQmlJS::AST::Node *astNode,
         const QmlIR::Binding &irBinding,
-        QQmlJS::DiagnosticMessage *error)
+        QList<QQmlJS::DiagnosticMessage> *errors)
 {
     QQmlJS::SourceLocation bindingLocation;
     bindingLocation.startLine = irBinding.location.line();
     bindingLocation.startColumn = irBinding.location.column();
 
     QQmlJSCompilePass::Function function;
-    function.qmlScope = m_scopeType;
+    function.qmlScope = m_typeResolver->registerContentPool()->createType(
+            m_scopeType, QQmlJSRegisterContent::InvalidLookupIndex,
+            QQmlJSRegisterContent::ScopeObject);
 
     if (irBinding.type() != QmlIR::Binding::Type_Script) {
         diagnose(u"Binding is not a script binding, but %1."_s.arg(
                      bindingTypeDescription(QmlIR::Binding::Type(quint32(irBinding.type())))),
-                 QtDebugMsg, bindingLocation, error);
+                 QtDebugMsg, bindingLocation, errors);
     }
 
     function.isProperty = m_objectType->hasProperty(propertyName);
-    if (!function.isProperty && QmlIR::IRBuilder::isSignalPropertyName(propertyName)) {
-        const QString signalName = QmlIR::IRBuilder::signalNameFromSignalPropertyName(propertyName);
+    if (function.isProperty) {
+        const auto property = m_objectType->property(propertyName);
+        if (const QQmlJSScope::ConstPtr propertyType = property.type()) {
+            function.returnType = m_typeResolver->namedType(propertyType->isListProperty()
+                ? m_typeResolver->qObjectListType()
+                : QQmlJSScope::ConstPtr(property.type()));
+        } else {
+            diagnose(u"Cannot resolve property type %1 for binding on %2."_s
+                             .arg(property.typeName(), propertyName),
+                     QtWarningMsg, bindingLocation, errors);
+        }
 
-        if (signalName.endsWith(u"Changed"_s)
-                && m_objectType->hasProperty(signalName.chopped(strlen("Changed")))) {
+        if (!property.bindable().isEmpty() && !property.isPrivate())
+            function.isQPropertyBinding = true;
+    } else if (QQmlSignalNames::isHandlerName(propertyName)) {
+        if (auto actualPropertyName =
+            QQmlSignalNames::changedHandlerNameToPropertyName(propertyName);
+            actualPropertyName && m_objectType->hasProperty(*actualPropertyName)) {
             function.isSignalHandler = true;
         } else {
-            const auto methods = m_objectType->methods(signalName);
+            auto signalName = QQmlSignalNames::handlerNameToSignalName(propertyName);
+            const auto methods = m_objectType->methods(*signalName);
             for (const auto &method : methods) {
                 if (method.isCloned())
                     continue;
@@ -179,53 +201,31 @@ QQmlJSCompilePass::Function QQmlJSFunctionInitializer::run(
                         if (type.isNull()) {
                             diagnose(u"Cannot resolve the argument type %1."_s.arg(
                                              arguments[i].typeName()),
-                                     QtDebugMsg, bindingLocation, error);
+                                     QtDebugMsg, bindingLocation, errors);
                             function.argumentTypes.append(
-                                        m_typeResolver->tracked(
-                                            m_typeResolver->globalType(m_typeResolver->varType())));
+                                    m_typeResolver->namedType(m_typeResolver->varType()));
                         } else {
-                            function.argumentTypes.append(m_typeResolver->tracked(
-                                                              m_typeResolver->globalType(type)));
+                            function.argumentTypes.append(m_typeResolver->namedType(type));
                         }
                     }
                     break;
                 }
             }
-        }
-
-        if (!function.isSignalHandler) {
-            diagnose(u"Could not compile signal handler for %1: The signal does not exist"_s.arg(
-                         signalName),
-                     QtWarningMsg, bindingLocation, error);
-        }
-    }
-
-    if (!function.isSignalHandler) {
-        if (!function.isProperty) {
-            diagnose(u"Could not compile binding for %1: The property does not exist"_s.arg(
-                         propertyName),
-                     QtWarningMsg, bindingLocation, error);
-        }
-
-        const auto property = m_objectType->property(propertyName);
-        if (const QQmlJSScope::ConstPtr propertyType = property.type()) {
-            function.returnType = propertyType->isListProperty()
-                    ? m_typeResolver->qObjectListType()
-                    : propertyType;
-        } else {
-            QString message = u"Cannot resolve property type %1 for binding on %2."_s
-                    .arg(property.typeName(), propertyName);
-            if (m_objectType->isNameDeferred(propertyName)) {
-                // If the property doesn't exist but the name is deferred, then
-                // it's deferred via the presence of immediate names. Those are
-                // most commonly used to enable generalized grouped properties.
-                message += u" You may want use ID-based grouped properties here.";
+            if (!function.isSignalHandler) {
+                diagnose(u"Could not find signal \"%1\"."_s.arg(*signalName),
+                         QtWarningMsg, bindingLocation, errors);
             }
-            diagnose(message, QtWarningMsg, bindingLocation, error);
+        }
+    } else {
+        QString message = u"Could not find property \"%1\"."_s.arg(propertyName);
+        if (m_objectType->isNameDeferred(propertyName)) {
+            // If the property doesn't exist but the name is deferred, then
+            // it's deferred via the presence of immediate names. Those are
+            // most commonly used to enable generalized grouped properties.
+            message += u" You may want use ID-based grouped properties here.";
         }
 
-        if (!property.bindable().isEmpty() && !property.isPrivate())
-            function.isQPropertyBinding = true;
+        diagnose(message, QtWarningMsg, bindingLocation, errors);
     }
 
     QQmlJS::MemoryPool pool;
@@ -242,30 +242,32 @@ QQmlJSCompilePass::Function QQmlJSFunctionInitializer::run(
 
         QString name = u"binding for "_s; // ####
         ast = new (&pool) QQmlJS::AST::FunctionDeclaration(
-                pool.newString(name), /*formals*/ nullptr, body);
+                pool.newString(std::move(name)), /*formals*/ nullptr, body);
         ast->lbraceToken = astNode->firstSourceLocation();
         ast->functionToken = ast->lbraceToken;
         ast->rbraceToken = astNode->lastSourceLocation();
     }
 
-    populateSignature(context, ast, &function, error);
+    populateSignature(context, ast, &function, errors);
     return function;
 }
 
 QQmlJSCompilePass::Function QQmlJSFunctionInitializer::run(
         const QV4::Compiler::Context *context,
         const QString &functionName, QQmlJS::AST::Node *astNode,
-        QQmlJS::DiagnosticMessage *error)
+        QList<QQmlJS::DiagnosticMessage> *errors)
 {
     Q_UNUSED(functionName);
 
     QQmlJSCompilePass::Function function;
-    function.qmlScope = m_scopeType;
+    function.qmlScope = m_typeResolver->registerContentPool()->createType(
+            m_scopeType, QQmlJSRegisterContent::InvalidLookupIndex,
+            QQmlJSRegisterContent::ScopeObject);
 
     auto ast = astNode->asFunctionDefinition();
     Q_ASSERT(ast);
 
-    populateSignature(context, ast, &function, error);
+    populateSignature(context, ast, &function, errors);
     return function;
 }
 
