@@ -60,6 +60,7 @@ const QQmlSA::LoggerWarningId qmlAccessSingleton{ "access-singleton-via-object"
 const QQmlSA::LoggerWarningId qmlTopLevelComponent{ "top-level-component" };
 const QQmlSA::LoggerWarningId qmlUncreatableType{ "uncreatable-type" };
 const QQmlSA::LoggerWarningId qmlMissingEnumEntry{ "missing-enum-entry" };
+const QQmlSA::LoggerWarningId qmlEnumsAreNotTypes{ "enums-are-not-types" };
 
 QQmlJSLogger::QQmlJSLogger()
 {
@@ -183,10 +184,16 @@ const QList<QQmlJS::LoggerCategory> &QQmlJSLogger::defaultCategories()
                 QStringLiteral("Warn if a singleton is accessed via an object"), QtWarningMsg },
         QQmlJS::LoggerCategory{
                 qmlTopLevelComponent.name().toString(), QStringLiteral("TopLevelComponent"),
-                QStringLiteral("Fail when a top level Component are encountered"), QtWarningMsg },
+                QStringLiteral("Warn if a top level Component is encountered"), QtWarningMsg },
         QQmlJS::LoggerCategory{
                 qmlUncreatableType.name().toString(), QStringLiteral("UncreatableType"),
-                QStringLiteral("Warn if uncreatable types are created"), QtWarningMsg }
+                QStringLiteral("Warn if uncreatable types are created"), QtWarningMsg },
+        QQmlJS::LoggerCategory{
+                qmlMissingEnumEntry.name().toString(), QStringLiteral("MissingEnumEntry"),
+                QStringLiteral("Warn about using missing enum values"), QtWarningMsg },
+        QQmlJS::LoggerCategory{
+                qmlEnumsAreNotTypes.name().toString(), QStringLiteral("EnumsAreNotTypes"),
+                QStringLiteral("Warn about the use of enumerations as types."), QtWarningMsg },
     };
 
     return cats;
@@ -254,10 +261,9 @@ void QQmlJSLogger::log(const QString &message, QQmlJS::LoggerWarningId id,
                 (!overrideFileName.isEmpty() ? overrideFileName : m_filePath) + QStringLiteral(":");
 
     if (srcLocation.isValid())
-        prefix += QStringLiteral("%1:%2:").arg(srcLocation.startLine).arg(srcLocation.startColumn);
-
-    if (!prefix.isEmpty())
-        prefix.append(QLatin1Char(' '));
+        prefix += QStringLiteral("%1:%2: ").arg(srcLocation.startLine).arg(srcLocation.startColumn);
+    else if (!prefix.isEmpty())
+        prefix += QStringLiteral(": "); // produce double colon for Qt Creator's issues pane
 
     // Note: we do the clamping to [Info, Critical] range since our logger only
     // supports 3 categories
@@ -274,11 +280,11 @@ void QQmlJSLogger::log(const QString &message, QQmlJS::LoggerWarningId id,
     diagMsg.type = type;
     diagMsg.fixSuggestion = suggestion;
 
-    switch (type) {
-    case QtWarningMsg: m_warnings.push_back(diagMsg); break;
-    case QtCriticalMsg: m_errors.push_back(diagMsg); break;
-    case QtInfoMsg: m_infos.push_back(diagMsg); break;
-    default: break;
+    if (m_inTransaction) {
+        m_pendingMessages.push_back(std::move(diagMsg));
+    } else {
+        countMessage(diagMsg);
+        m_committedMessages.push_back(std::move(diagMsg));
     }
 
     if (srcLocation.length > 0 && !m_code.isEmpty() && showContext)
@@ -286,6 +292,23 @@ void QQmlJSLogger::log(const QString &message, QQmlJS::LoggerWarningId id,
 
     if (suggestion.has_value())
         printFix(suggestion.value());
+
+    if (!m_inTransaction)
+        m_output.flushBuffer();
+}
+
+void QQmlJSLogger::countMessage(const Message &message)
+{
+    switch (message.type) {
+    case QtWarningMsg:
+        ++m_numWarnings;
+        break;
+    case QtCriticalMsg:
+        ++m_numErrors;
+        break;
+    default:
+        break;
+    }
 }
 
 void QQmlJSLogger::processMessages(const QList<QQmlJS::DiagnosticMessage> &messages,
@@ -305,6 +328,52 @@ void QQmlJSLogger::processMessages(const QList<QQmlJS::DiagnosticMessage> &messa
     m_output.write(QStringLiteral("---\n\n"));
 }
 
+/*!
+    \internal
+    Starts a transaction for a compile pass. This buffers all messages until the
+    transaction completes. If you commit the transaction, the messages are printed
+    and added to the list of committed messages. If you roll it back, the logger
+    reverts to the state before the start of the transaction.
+
+    This is useful for compile passes that potentially have to be repeated, such
+    as the type propagator. We don't want to see the same messages logged multiple
+    times.
+ */
+void QQmlJSLogger::startTransaction()
+{
+    Q_ASSERT(!m_inTransaction);
+    m_inTransaction = true;
+}
+
+/*!
+    \internal
+    Commit the current transaction. Print all pending messages, and add them to
+    the list of committed messages. Then, clear the transaction flag.
+ */
+void QQmlJSLogger::commit()
+{
+    Q_ASSERT(m_inTransaction);
+    for (const Message &message : std::as_const(m_pendingMessages))
+        countMessage(message);
+
+    m_committedMessages.append(std::exchange(m_pendingMessages, {}));
+    m_output.flushBuffer();
+    m_inTransaction = false;
+}
+
+/*!
+    \internal
+    Roll back the current transaction and revert the logger to the state before
+    it was started.
+ */
+void QQmlJSLogger::rollback()
+{
+    Q_ASSERT(m_inTransaction);
+    m_pendingMessages.clear();
+    m_output.discardBuffer();
+    m_inTransaction = false;
+}
+
 void QQmlJSLogger::printContext(const QString &overrideFileName,
                                 const QQmlJS::SourceLocation &location)
 {
