@@ -7,7 +7,6 @@
 #include "qqmldomelements_p.h"
 #include "qqmldomexternalitems_p.h"
 #include "qqmldomastdumper_p.h"
-#include "qqmldomattachedinfo_p.h"
 
 #include <QtQml/private/qqmljsastvisitor_p.h>
 #include <QtQml/private/qqmljsast_p.h>
@@ -17,7 +16,7 @@
 
 #include <variant>
 
-static Q_LOGGING_CATEGORY(commentsLog, "qt.qmldom.comments", QtWarningMsg);
+Q_STATIC_LOGGING_CATEGORY(commentsLog, "qt.qmldom.comments", QtWarningMsg);
 
 QT_BEGIN_NAMESPACE
 namespace QQmlJS {
@@ -277,24 +276,35 @@ public:
     FileLocationRegion regionName;
 };
 
+class NodeRef
+{
+public:
+    AST::Node *node = nullptr;
+    CommentAnchor commentAnchor;
+};
+
 // internal class to keep a reference either to an AST::Node* or a region of a DomItem and the
 // size of that region
 class ElementRef
 {
 public:
-    ElementRef(AST::Node *node, quint32 size) : element(node), size(size) { }
-    ElementRef(const Path &path, FileLocationRegion region, quint32 size)
+    ElementRef(AST::Node *node, qsizetype size, CommentAnchor commentAnchor)
+        : element(NodeRef{ node, commentAnchor }), size(size)
+    {
+    }
+    ElementRef(const Path &path, FileLocationRegion region, qsizetype size)
         : element(RegionRef{ path, region }), size(size)
     {
     }
     operator bool() const
     {
-        return (element.index() == 0 && std::get<0>(element)) || element.index() == 1 || size != 0;
+        return (element.index() == 0 && std::get<0>(element).node) || element.index() == 1
+                || size != 0;
     }
     ElementRef() = default;
 
-    std::variant<AST::Node *, RegionRef> element;
-    quint32 size = 0;
+    std::variant<NodeRef, RegionRef> element;
+    qsizetype size = 0;
 };
 
 /*!
@@ -347,21 +357,80 @@ public:
     static const QSet<int> kindsToSkip();
     static bool shouldSkipRegion(const DomItem &item, FileLocationRegion region);
 
+    void addSourceLocations(Node *n, qsizetype start, qsizetype end, CommentAnchor commentAnchor)
+    {
+        if (!starts.contains(start))
+            starts.insert(start, { n, end - start, commentAnchor });
+        if (!ends.contains(end))
+            ends.insert(end, { n, end - start, commentAnchor });
+    }
+    void addSourceLocations(Node *n)
+    {
+        addSourceLocations(n, n->firstSourceLocation().begin(), n->lastSourceLocation().end(),
+                           CommentAnchor{});
+    }
+    void addSourceLocations(Node *n, const SourceLocation &sourceLocation)
+    {
+        if (!sourceLocation.isValid())
+            return;
+        addSourceLocations(n, sourceLocation.begin(), sourceLocation.end(),
+                           CommentAnchor::from(sourceLocation));
+    }
+
     bool preVisit(Node *n) override
     {
         if (!kindsToSkip().contains(n->kind)) {
-            quint32 start = n->firstSourceLocation().begin();
-            quint32 end = n->lastSourceLocation().end();
-            if (!starts.contains(start))
-                starts.insert(start, { n, end - start });
-            if (!ends.contains(end))
-                ends.insert(end, { n, end - start });
+            addSourceLocations(n);
         }
         return true;
     }
 
-    QMap<quint32, ElementRef> starts;
-    QMap<quint32, ElementRef> ends;
+    using VisitAll::visit;
+    bool visit(CaseClause *caseClause) override
+    {
+        // special case: case clauses can have comments attached to their `:` token
+        addSourceLocations(caseClause, caseClause->colonToken);
+        return true;
+    }
+
+    bool visit(ClassDeclaration *classDeclaration) override
+    {
+        // special case: class declarations can have comments attached to their `identifier` token
+        addSourceLocations(classDeclaration, classDeclaration->identifierToken);
+        addSourceLocations(classDeclaration, classDeclaration->lbraceToken);
+        addSourceLocations(classDeclaration, classDeclaration->rbraceToken);
+        return true;
+    }
+
+    bool visit(FormalParameterList *list) override
+    {
+        if (!list->commaToken.isValid())
+            return true;
+
+        // Comments are first attached to some previous element, if possible. Therefore, attach
+        // comments in FormalParameterList to comments so that they don't "jump over commas" during
+        // formatting.
+        addSourceLocations(list, list->commaToken);
+        return true;
+    }
+
+    bool visit(FunctionExpression *fExpr) override
+    {
+        addSourceLocations(fExpr, fExpr->identifierToken);
+        addSourceLocations(fExpr, fExpr->lparenToken);
+        addSourceLocations(fExpr, fExpr->rparenToken);
+        addSourceLocations(fExpr, fExpr->lbraceToken);
+        addSourceLocations(fExpr, fExpr->rbraceToken);
+        return true;
+    }
+
+    bool visit(FunctionDeclaration *fExpr) override
+    {
+        return visit(static_cast<FunctionExpression *>(fExpr));
+    }
+
+    QMap<qsizetype, ElementRef> starts;
+    QMap<qsizetype, ElementRef> ends;
 };
 
 void AstRangesVisitor::addNodeRanges(AST::Node *rootNode)
@@ -381,12 +450,12 @@ void AstRangesVisitor::addItemRanges(
     if (comments) {
         auto regs = itemLocations->info().regions;
         for (auto it = regs.cbegin(), end = regs.cend(); it != end; ++it) {
-            quint32 startI = it.value().begin();
-            quint32 endI = it.value().end();
+            qsizetype startI = it.value().begin();
+            qsizetype endI = it.value().end();
 
             if (!shouldSkipRegion(item, it.key())) {
                 if (!starts.contains(startI))
-                    starts.insert(startI, { currentP, it.key(), quint32(endI - startI) });
+                    starts.insert(startI, { currentP, it.key(), endI - startI });
                 if (!ends.contains(endI))
                     ends.insert(endI, { currentP, it.key(), endI - startI });
             }
@@ -395,9 +464,7 @@ void AstRangesVisitor::addItemRanges(
     {
         auto subMaps = itemLocations->subItems();
         for (auto it = subMaps.begin(), end = subMaps.end(); it != end; ++it) {
-            addItemRanges(item.path(it.key()),
-                          std::static_pointer_cast<AttachedInfoT<FileLocations>>(it.value()),
-                          currentP.path(it.key()));
+            addItemRanges(item.path(it.key()), it.value(), currentP.path(it.key()));
         }
     }
 }
@@ -428,14 +495,6 @@ const QSet<int> AstRangesVisitor::kindsToSkip()
 bool AstRangesVisitor::shouldSkipRegion(const DomItem &item, FileLocationRegion region)
 {
     switch (item.internalKind()) {
-    case DomType::EnumDecl: {
-        return (region == FileLocationRegion::IdentifierRegion)
-                || (region == FileLocationRegion::EnumKeywordRegion);
-    }
-    case DomType::EnumItem: {
-        return (region == FileLocationRegion::IdentifierRegion)
-                || (region == FileLocationRegion::EnumValueRegion);
-    }
     case DomType::QmlObject: {
         return (region == FileLocationRegion::RightBraceRegion
                 || region == FileLocationRegion::LeftBraceRegion);
@@ -452,7 +511,7 @@ bool AstRangesVisitor::shouldSkipRegion(const DomItem &item, FileLocationRegion
 class CommentLinker
 {
 public:
-    CommentLinker(QStringView code, ElementRef &commentedElement, const AstRangesVisitor &ranges, quint32 &lastPostCommentPostEnd,
+    CommentLinker(QStringView code, ElementRef &commentedElement, const AstRangesVisitor &ranges, qsizetype &lastPostCommentPostEnd,
                   const SourceLocation &commentLocation)
         : m_code{ code },
           m_commentedElement{ commentedElement },
@@ -481,18 +540,16 @@ public:
     [[nodiscard]] Comment createComment() const
     {
         const auto [preSpacesIndex, postSpacesIndex, preNewlineCount] = m_spaces;
-        return Comment{ m_code.mid(preSpacesIndex, quint32(postSpacesIndex) - preSpacesIndex),
-                        m_commentLocation,
-                        static_cast<int>(preNewlineCount),
-                        m_commentType};
+        return Comment{ m_code.mid(preSpacesIndex, postSpacesIndex - preSpacesIndex),
+                        m_commentLocation, static_cast<int>(preNewlineCount), m_commentType };
     }
 
 private:
     struct SpaceTrace
     {
-        quint32 iPre;
+        qsizetype iPre;
         qsizetype iPost;
-        int preNewline;
+        qsizetype preNewline;
     };
 
     /*! \internal
@@ -505,9 +562,9 @@ private:
     */
     [[nodiscard]] SpaceTrace findSpacesAroundComment() const
     {
-        quint32 iPre = m_commentLocation.begin();
-        int preNewline = 0;
-        int postNewline = 0;
+        qsizetype iPre = m_commentLocation.begin();
+        qsizetype preNewline = 0;
+        qsizetype postNewline = 0;
         QStringView commentStartStr;
         while (iPre > 0) {
             QChar c = m_code.at(iPre - 1);
@@ -522,7 +579,7 @@ private:
             } else if (c == QLatin1Char('\n') || c == QLatin1Char('\r')) {
                 preNewline = 1;
                 // possibly add an empty line if it was there (but never more than one)
-                int i = iPre - 1;
+                qsizetype i = iPre - 1;
                 if (c == QLatin1Char('\n') && i > 0 && m_code.at(i - 1) == QLatin1Char('\r'))
                     --i;
                 while (i > 0 && m_code.at(--i).isSpace()) {
@@ -589,7 +646,7 @@ private:
                 // expression (think a + //comment\n b  ==> a // comment\n + b), in this
                 // case attaching as preComment of iStart (b in the example) should be
                 // preferred as it is safe
-                quint32 i = m_spaces.iPre;
+                qsizetype i = m_spaces.iPre;
                 while (i != 0 && m_code.at(--i).isSpace())
                     ;
                 if (i <= preEnd.key() || i < m_lastPostCommentPostEnd
@@ -634,6 +691,10 @@ private:
         } else {
             --preStart;
         }
+        if (m_endElement == m_ranges.ends.end()) {
+            m_commentedElement = preStart.value();
+            return;
+        }
         // we are inside a node, actually inside both n1 and n2 (which might be the same)
         // add to pre of the smallest between n1 and n2.
         // This is needed because if there are multiple nodes starting/ending at the same
@@ -648,7 +709,7 @@ private:
 private:
     QStringView m_code;
     ElementRef &m_commentedElement;
-    quint32 &m_lastPostCommentPostEnd;
+    qsizetype &m_lastPostCommentPostEnd;
     Comment::CommentType m_commentType = Comment::Pre;
     const AstRangesVisitor &m_ranges;
     const SourceLocation &m_commentLocation;
@@ -667,12 +728,8 @@ bool AstComments::iterateDirectSubpaths(const DomItem &self, DirectVisitor visit
 {
     // TODO: QTBUG-123645
     // Revert this commit to reproduce crash with tst_qmldomitem::doNotCrashAtAstComments
-    QList<Comment> pre;
-    QList<Comment> post;
-    for (const auto &commentedElement : commentedElements().values()) {
-        pre.append(commentedElement.preComments());
-        post.append(commentedElement.postComments());
-    }
+    auto [pre, post] = collectPreAndPostComments();
+
     if (!pre.isEmpty())
         self.dvWrapField(visitor, Fields::preComments, pre);
     if (!post.isEmpty())
@@ -714,7 +771,7 @@ void CommentCollector::collectComments(
     ranges.addItemRanges(m_rootItem.item(), m_fileLocations, Path());
     ranges.addNodeRanges(rootNode);
     QStringView code = engine->code();
-    quint32 lastPostCommentPostEnd = 0;
+    qsizetype lastPostCommentPostEnd = 0;
     for (const SourceLocation &commentLocation : engine->comments()) {
         // collect whitespace before and after cLoc -> iPre..iPost contains whitespace,
         // do not add newline before, but add the one after
@@ -730,15 +787,17 @@ void CommentCollector::collectComments(
                 elementToBeLinked.element = RegionRef{ Path(), MainRegion };
                 elementToBeLinked.size = FileLocations::region(m_fileLocations, MainRegion).length;
             } else if (rootNode) {
-                elementToBeLinked.element = rootNode;
+                elementToBeLinked.element = NodeRef{ rootNode, CommentAnchor{} };
                 elementToBeLinked.size = rootNode->lastSourceLocation().end() - rootNode->firstSourceLocation().begin();
             }
         }
 
-        if (const auto *const commentNode = std::get_if<AST::Node *>(&elementToBeLinked.element)) {
-            auto &commentedElement = astComments->commentedElements()[*commentNode];
-            commentedElement.addComment(comment);
-        } else if (const auto * const regionRef = std::get_if<RegionRef>(&elementToBeLinked.element)) {
+        if (const auto *const commentNode = std::get_if<NodeRef>(&elementToBeLinked.element)) {
+            auto commentedElement = astComments->ensureCommentForNode(commentNode->node,
+                                                                      commentNode->commentAnchor);
+            commentedElement->addComment(comment);
+        } else if (const auto *const regionRef =
+                           std::get_if<RegionRef>(&elementToBeLinked.element)) {
             DomItem currentItem = m_rootItem.item().path(regionRef->path);
             MutableDomItem regionComments = currentItem.field(Fields::comments);
             if (auto *regionCommentsPtr = regionComments.mutableAs<RegionComments>()) {
@@ -746,9 +805,8 @@ void CommentCollector::collectComments(
 
                 // update file locations with the comment region
                 const auto base = FileLocations::treeOf(currentItem);
-                const auto fileLocations =
-                        FileLocations::ensure(base, Path::Field(Fields::comments).path(commentPath),
-                                              AttachedInfo::PathType::Relative);
+                const auto fileLocations = FileLocations::ensure(
+                        base, Path::Field(Fields::comments).path(commentPath));
 
                 FileLocations::addRegion(fileLocations, MainRegion,
                                          comment.info().sourceLocation());
