@@ -331,7 +331,7 @@ void Import::writeOut(const DomItem &self, OutWriter &ow) const
     // check for an empty line before the import, and preserve it
     int preNewlines = 0;
 
-    const FileLocations::Tree elLoc = FileLocations::findAttachedInfo(self).foundTree;
+    const FileLocations::Tree elLoc = FileLocations::treeOf(self);
 
     quint32 start = elLoc->info().fullRegion.offset;
     if (size_t(code.size()) >= start) {
@@ -353,7 +353,7 @@ void Import::writeOut(const DomItem &self, OutWriter &ow) const
     if (uri.isModule()) {
         QString vString = version.stringValue();
         if (!vString.isEmpty())
-            ow.ensureSpace().write(vString);
+            ow.ensureSpace().writeRegion(VersionRegion, vString);
     }
     if (!importId.isEmpty()) {
         ow.ensureSpace().writeRegion(AsTokenRegion).ensureSpace().writeRegion(IdNameRegion,
@@ -713,24 +713,8 @@ MutableDomItem QmlObject::addMethod(
     return self.owner().path(p);
 }
 
-void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTarget) const
+void QmlObject::writeOutId(const DomItem &self, OutWriter &ow) const
 {
-    const quint32 posOfNewElements = std::numeric_limits<quint32>::max();
-    bool isRootObject = pathFromOwner().length() == 5
-            && pathFromOwner()[0] == Path::Field(Fields::components)
-            && pathFromOwner()[3] == Path::Field(Fields::objects);
-    QString code;
-    DomItem owner = self.owner();
-    if (std::shared_ptr<QmlFile> qmlFilePtr = self.ownerAs<QmlFile>())
-        code = qmlFilePtr->code();
-    ow.writeRegion(IdentifierRegion, name());
-    if (!onTarget.isEmpty()) {
-        ow.ensureSpace().writeRegion(OnTokenRegion).ensureSpace().writeRegion(OnTargetRegion,
-                                                                              onTarget);
-    }
-    ow.writeRegion(LeftBraceRegion, u" {");
-    int baseIndent = ow.increaseIndent();
-    int spacerId = 0;
     if (!idStr().isEmpty()) { // *always* put id first
         DomItem myId = self.component().field(Fields::ids).key(idStr()).index(0);
         if (myId)
@@ -749,152 +733,152 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
             ow.ensureNewline(1);
         }
     }
-    quint32 counter = ow.counter();
-    DomItem component;
-    if (isRootObject)
-        component = self.containingObject();
+}
+
+QList<QPair<SourceLocation, DomItem>> QmlObject::orderOfAttributes(const DomItem &self,
+                                                                   const DomItem &component) const
+{
     auto startLoc = [&](const FileLocations::Tree &l) {
         if (l)
             return l->info().fullRegion;
         return SourceLocation(posOfNewElements, 0, 0, 0);
     };
-    if (ow.lineWriter.options().attributesSequence
-        == LineWriterOptions::AttributesSequence::Preserve) {
-        QList<QPair<SourceLocation, DomItem>> attribs;
-        AttachedInfoLookupResult<FileLocations::Tree> objLoc =
-                FileLocations::findAttachedInfo(self);
-        FileLocations::Tree componentLoc;
-        if (isRootObject && objLoc.foundTree)
-            componentLoc = objLoc.foundTree->parent()->parent();
-        auto addMMap
-                = [&attribs, &startLoc](const DomItem &base, const FileLocations::Tree &baseLoc) {
-            if (!base)
-                return;
-            const auto values = base.values();
-            for (const auto &els : values) {
-                FileLocations::Tree elsLoc =
-                        FileLocations::find(baseLoc, els.pathFromOwner().last());
-                const auto elsValues = els.values();
-                for (const auto &el : elsValues) {
-                    FileLocations::Tree elLoc =
-                            FileLocations::find(elsLoc, el.pathFromOwner().last());
-                    attribs.append(std::make_pair(startLoc(elLoc), el));
-                }
-            }
-        };
-        auto addMyMMap = [this, &objLoc, &self, &addMMap](QStringView fieldName) {
-            DomItem base = this->field(self, fieldName);
-            addMMap(base, FileLocations::find(objLoc.foundTree, base.pathFromOwner().last()));
-        };
-        auto addSingleLevel
-                = [&attribs, &startLoc](const DomItem &base, const FileLocations::Tree &baseLoc) {
-            if (!base)
-                return;
-            const auto baseValues = base.values();
-            for (const auto &el : baseValues) {
-                FileLocations::Tree elLoc = FileLocations::find(baseLoc, el.pathFromOwner().last());
+
+    QList<QPair<SourceLocation, DomItem>> attribs;
+    const auto objLocPtr = FileLocations::treeOf(self);
+    FileLocations::Tree componentLoc;
+    if (component && objLocPtr)
+        componentLoc = objLocPtr->parent()->parent();
+    auto addMMap = [&attribs, &startLoc](const DomItem &base, const FileLocations::Tree &baseLoc) {
+        if (!base)
+            return;
+        const auto values = base.values();
+        for (const auto &els : values) {
+            FileLocations::Tree elsLoc = FileLocations::find(baseLoc, els.pathFromOwner().last());
+            const auto elsValues = els.values();
+            for (const auto &el : elsValues) {
+                FileLocations::Tree elLoc = FileLocations::find(elsLoc, el.pathFromOwner().last());
                 attribs.append(std::make_pair(startLoc(elLoc), el));
             }
-        };
-        if (isRootObject) {
-            DomItem enums = component.field(Fields::enumerations);
-            addMMap(enums, FileLocations::find(componentLoc, enums.pathFromOwner().last()));
         }
-        addMyMMap(Fields::propertyDefs);
-        addMyMMap(Fields::bindings);
-        addMyMMap(Fields::methods);
-        DomItem children = field(self, Fields::children);
-        addSingleLevel(children,
-                       FileLocations::find(objLoc.foundTree, children.pathFromOwner().last()));
-        if (isRootObject) {
-            DomItem subCs = component.field(Fields::subComponents);
-            for (const DomItem &c : subCs.values()) {
-                AttachedInfoLookupResult<FileLocations::Tree> subLoc =
-                        FileLocations::findAttachedInfo(c);
-                Q_ASSERT(subLoc.foundTree);
-                attribs.append(std::make_pair(startLoc(subLoc.foundTree), c));
-            }
+    };
+    auto addMyMMap = [this, &objLocPtr, &self, &addMMap](QStringView fieldName) {
+        DomItem base = this->field(self, fieldName);
+        addMMap(base, FileLocations::find(objLocPtr, base.pathFromOwner().last()));
+    };
+    auto addSingleLevel = [&attribs, &startLoc](const DomItem &base,
+                                                const FileLocations::Tree &baseLoc) {
+        if (!base)
+            return;
+        const auto baseValues = base.values();
+        for (const auto &el : baseValues) {
+            FileLocations::Tree elLoc = FileLocations::find(baseLoc, el.pathFromOwner().last());
+            attribs.append(std::make_pair(startLoc(elLoc), el));
         }
-        std::stable_sort(attribs.begin(), attribs.end(),
-                         [](const std::pair<SourceLocation, DomItem> &el1,
-                            const std::pair<SourceLocation, DomItem> &el2) {
-                             if (el1.first.offset < el2.first.offset)
-                                 return true;
-                             if (el1.first.offset > el2.first.offset)
-                                 return false;
-                             int i = int(el1.second.internalKind())
-                                     - int(el2.second.internalKind());
-                             return i < 0;
-                         });
-        qsizetype iAttr = 0;
-        while (iAttr != attribs.size()) {
-            auto &el = attribs[iAttr++];
-            // check for an empty line before the current element, and preserve it
-            int preNewlines = 0;
-            quint32 start = el.first.offset;
-            if (start != posOfNewElements && size_t(code.size()) >= start) {
-                while (start != 0) {
-                    QChar c = code.at(--start);
-                    if (c == u'\n') {
-                        if (++preNewlines == 2)
-                            break;
-                    } else if (!c.isSpace())
+    };
+    if (component) {
+        DomItem enums = component.field(Fields::enumerations);
+        addMMap(enums, FileLocations::find(componentLoc, enums.pathFromOwner().last()));
+    }
+    addMyMMap(Fields::propertyDefs);
+    addMyMMap(Fields::bindings);
+    addMyMMap(Fields::methods);
+    DomItem children = field(self, Fields::children);
+    addSingleLevel(children, FileLocations::find(objLocPtr, children.pathFromOwner().last()));
+    if (component) {
+        DomItem subCs = component.field(Fields::subComponents);
+        for (const DomItem &c : subCs.values()) {
+            const auto subLocPtr = FileLocations::treeOf(c);
+            Q_ASSERT(subLocPtr);
+            attribs.append(std::make_pair(startLoc(subLocPtr), c));
+        }
+    }
+    std::stable_sort(attribs.begin(), attribs.end(),
+                     [](const std::pair<SourceLocation, DomItem> &el1,
+                        const std::pair<SourceLocation, DomItem> &el2) {
+                         if (el1.first.offset < el2.first.offset)
+                             return true;
+                         if (el1.first.offset > el2.first.offset)
+                             return false;
+                         int i = int(el1.second.internalKind()) - int(el2.second.internalKind());
+                         return i < 0;
+                     });
+    return attribs;
+}
+
+void QmlObject::writeOutAttributes(const DomItem &self, OutWriter &ow, const DomItem &component,
+                                   const QString &code) const
+{
+    const QList<QPair<SourceLocation, DomItem>> attribs = orderOfAttributes(self, component);
+    qsizetype iAttr = 0;
+    while (iAttr != attribs.size()) {
+        auto &el = attribs[iAttr++];
+        // check for an empty line before the current element, and preserve it
+        int preNewlines = 0;
+        quint32 start = el.first.offset;
+        if (start != posOfNewElements && size_t(code.size()) >= start) {
+            while (start != 0) {
+                QChar c = code.at(--start);
+                if (c == u'\n') {
+                    if (++preNewlines == 2)
                         break;
-                }
+                } else if (!c.isSpace())
+                    break;
             }
-            if (preNewlines == 0)
-                ++preNewlines;
-            ow.ensureNewline(preNewlines);
-            if (el.second.internalKind() == DomType::PropertyDefinition && iAttr != attribs.size()
-                && el.first.offset != ~quint32(0)) {
-                DomItem b;
-                auto &bPair = attribs[iAttr];
-                if (bPair.second.internalKind() == DomType::Binding
-                    && bPair.first.begin() < el.first.end()
-                    && bPair.second.name() == el.second.name()) {
-                    b = bPair.second;
-                    ++iAttr;
-                    b.writeOutPre(ow);
-                }
-                el.second.writeOut(ow);
-                if (b) {
-                    ow.write(u": ");
-                    if (const Binding *bPtr = b.as<Binding>())
-                        bPtr->writeOutValue(b, ow);
-                    else {
-                        qWarning() << "Internal error casting binding to Binding in"
-                                   << b.canonicalPath();
-                        ow.writeRegion(LeftBraceRegion).writeRegion(RightBraceRegion);
-                    }
-                    b.writeOutPost(ow);
-                }
-            } else {
-                el.second.writeOut(ow);
-            }
-            ow.ensureNewline();
         }
-        ow.decreaseIndent(1, baseIndent);
-        ow.writeRegion(RightBraceRegion);
-
-        return;
+        if (preNewlines == 0)
+            ++preNewlines;
+        ow.ensureNewline(preNewlines);
+        if (el.second.internalKind() == DomType::PropertyDefinition && iAttr != attribs.size()
+            && el.first.offset != ~quint32(0)) {
+            DomItem b;
+            auto &bPair = attribs[iAttr];
+            if (bPair.second.internalKind() == DomType::Binding
+                && bPair.first.begin() < el.first.end()
+                && bPair.second.name() == el.second.name()) {
+                b = bPair.second;
+                ++iAttr;
+                b.writeOutPre(ow);
+            }
+            el.second.writeOut(ow);
+            if (b) {
+                ow.writeRegion(ColonTokenRegion);
+                ow.ensureSpace();
+                if (const Binding *bPtr = b.as<Binding>())
+                    bPtr->writeOutValue(b, ow);
+                else {
+                    qWarning() << "Internal error casting binding to Binding in"
+                               << b.canonicalPath();
+                    ow.writeRegion(LeftBraceRegion).writeRegion(RightBraceRegion);
+                }
+                b.writeOutPost(ow);
+            }
+        } else {
+            el.second.writeOut(ow);
+        }
+        ow.ensureNewline();
     }
-    DomItem bindings = field(self, Fields::bindings);
+}
+
+void QmlObject::writeOutSortedEnumerations(const DomItem &component, OutWriter &ow) const
+{
+    const auto descs = component.field(Fields::enumerations).values();
+    for (const auto &enumDescs : descs) {
+        const auto values = enumDescs.values();
+        for (const auto &enumDesc : values) {
+            ow.ensureNewline(1);
+            enumDesc.writeOut(ow);
+            ow.ensureNewline(1);
+        }
+    }
+}
+
+void QmlObject::writeOutSortedPropertyDefinition(const DomItem &self, OutWriter &ow,
+                                                 QSet<QString> &mergedDefBinding) const
+{
     DomItem propertyDefs = field(self, Fields::propertyDefs);
+    DomItem bindings = field(self, Fields::bindings);
 
-    if (isRootObject) {
-        const auto descs = component.field(Fields::enumerations).values();
-        for (const auto &enumDescs : descs) {
-            const auto values = enumDescs.values();
-            for (const auto &enumDesc : values) {
-                ow.ensureNewline(1);
-                enumDesc.writeOut(ow);
-                ow.ensureNewline(1);
-            }
-        }
-    }
-    if (counter != ow.counter() || !idStr().isEmpty())
-        spacerId = ow.addNewlinesAutospacerCallback(2);
-    QSet<QString> mergedDefBinding;
     for (const QString &defName : propertyDefs.sortedKeys()) {
         const auto pDefs = propertyDefs.key(defName).values();
         for (const auto &pDef : pDefs) {
@@ -911,13 +895,11 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
                             b = el;
                             break;
                         case BindingValueKind::Array:
-                            if (!pDefPtr->isDefaultMember
-                                && pDefPtr->isParametricType())
+                            if (!pDefPtr->isDefaultMember && pDefPtr->isParametricType())
                                 b = el;
                             break;
                         case BindingValueKind::Object:
-                            if (!pDefPtr->isDefaultMember
-                                && !pDefPtr->isParametricType())
+                            if (!pDefPtr->isDefaultMember && !pDefPtr->isParametricType())
                                 b = el;
                             break;
                         case BindingValueKind::Empty:
@@ -933,7 +915,7 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
             }
             pDef.writeOut(ow);
             if (b) {
-                ow.write(u":");
+                ow.writeRegion(ColonTokenRegion);
                 ow.ensureSpace();
                 if (const Binding *bPtr = b.as<Binding>())
                     bPtr->writeOutValue(b, ow);
@@ -946,9 +928,12 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
             }
         }
     }
-    ow.removeTextAddCallback(spacerId);
+}
+
+static std::pair<QList<DomItem>, QList<DomItem>> splitSignalsAndMethods(const DomItem &methods)
+{
     QList<DomItem> signalList, methodList;
-    const auto fields = field(self, Fields::methods).values();
+    const auto fields = methods.values();
     for (const auto &ms : fields) {
         const auto values = ms.values();
         for (const auto &m : values) {
@@ -959,27 +944,12 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
                 methodList.append(m);
         }
     }
-    if (counter != ow.counter())
-        spacerId = ow.addNewlinesAutospacerCallback(2);
-    for (const auto &sig : std::as_const(signalList)) {
-        ow.ensureNewline();
-        sig.writeOut(ow);
-        ow.ensureNewline();
-    }
-    ow.removeTextAddCallback(spacerId);
-    if (counter != ow.counter())
-        spacerId = ow.addNewlinesAutospacerCallback(2);
-    bool first = true;
-    for (const auto &method : std::as_const(methodList)) {
-        if (!first && ow.lineWriter.options().functionsSpacing) {
-            ow.newline();
-        }
-        ow.ensureNewline();
-        first = false;
-        method.writeOut(ow);
-        ow.ensureNewline();
-    }
-    ow.removeTextAddCallback(spacerId);
+    return std::make_pair(signalList, methodList);
+}
+
+static std::tuple<QList<DomItem>, QList<DomItem>, QList<DomItem>>
+splitBindings(const DomItem &bindings, const QSet<QString> &mergedDefBinding)
+{
     QList<DomItem> normalBindings, signalHandlers, delayedBindings;
     for (const auto &bName : bindings.sortedKeys()) {
         bool skipFirstNormal = mergedDefBinding.contains(bName);
@@ -1001,24 +971,77 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
                 normalBindings.append(b);
         }
     }
+    return std::make_tuple(normalBindings, signalHandlers, delayedBindings);
+}
+
+void QmlObject::writeOutSortedAttributes(const DomItem &self, OutWriter &ow,
+                                         const DomItem &component) const
+{
+    int spacerId = 0;
+    quint32 counter = ow.counter();
+
+    if (component)
+        writeOutSortedEnumerations(component, ow);
+
+    if (counter != ow.counter() || !idStr().isEmpty())
+        spacerId = ow.addNewlinesAutospacerCallback(2);
+
+    QSet<QString> mergedDefBinding;
+    writeOutSortedPropertyDefinition(self, ow, mergedDefBinding);
+
+    ow.removeTextAddCallback(spacerId);
+    if (counter != ow.counter())
+        spacerId = ow.addNewlinesAutospacerCallback(2);
+
+    const auto [signalList, methodList] = splitSignalsAndMethods(field(self, Fields::methods));
+    for (const auto &sig : std::as_const(signalList)) {
+        ow.ensureNewline();
+        sig.writeOut(ow);
+        ow.ensureNewline();
+    }
+
+    ow.removeTextAddCallback(spacerId);
+    if (counter != ow.counter())
+        spacerId = ow.addNewlinesAutospacerCallback(2);
+
+    bool first = true;
+    for (const auto &method : std::as_const(methodList)) {
+        if (!first && ow.lineWriter.options().functionsSpacing) {
+            ow.newline();
+        }
+        ow.ensureNewline();
+        first = false;
+        method.writeOut(ow);
+        ow.ensureNewline();
+    }
+    ow.removeTextAddCallback(spacerId);
+
+    DomItem bindings = field(self, Fields::bindings);
+    const auto [normalBindings, signalHandlers, delayedBindings] =
+            splitBindings(bindings, mergedDefBinding);
+
     if (counter != ow.counter())
         spacerId = ow.addNewlinesAutospacerCallback(2);
     for (const auto &b : std::as_const(normalBindings))
         b.writeOut(ow);
     ow.removeTextAddCallback(spacerId);
+
     if (counter != ow.counter())
         spacerId = ow.addNewlinesAutospacerCallback(2);
     for (const auto &b : std::as_const(delayedBindings))
         b.writeOut(ow);
     ow.removeTextAddCallback(spacerId);
+
     if (counter != ow.counter())
         spacerId = ow.addNewlinesAutospacerCallback(2);
     for (const auto &b : std::as_const(signalHandlers))
         b.writeOut(ow);
     ow.removeTextAddCallback(spacerId);
+
     if (counter != ow.counter())
         spacerId = ow.addNewlinesAutospacerCallback(2);
     first = true;
+
     const auto values = field(self, Fields::children).values();
     for (const auto &c : values) {
         if (!first && ow.lineWriter.options().objectsSpacing) {
@@ -1029,7 +1052,8 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
         c.writeOut(ow);
     }
     ow.removeTextAddCallback(spacerId);
-    if (isRootObject) {
+
+    if (component) {
         // we are a root object, possibly add components
         DomItem subComps = component.field(Fields::subComponents);
         if (counter != ow.counter())
@@ -1041,9 +1065,40 @@ void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTa
         }
         ow.removeTextAddCallback(spacerId);
     }
+    ow.ensureNewline();
+}
 
+void QmlObject::writeOut(const DomItem &self, OutWriter &ow, const QString &onTarget) const
+{
+    bool isRootObject = pathFromOwner().length() == 5
+            && pathFromOwner()[0] == Path::Field(Fields::components)
+            && pathFromOwner()[3] == Path::Field(Fields::objects);
+    ow.writeRegion(IdentifierRegion, name());
+    if (!onTarget.isEmpty()) {
+        ow.ensureSpace().writeRegion(OnTokenRegion).ensureSpace().writeRegion(OnTargetRegion,
+                                                                              onTarget);
+    }
+    ow.ensureSpace();
+    ow.writeRegion(LeftBraceRegion);
+    int baseIndent = ow.increaseIndent();
+
+    // *always* put id first
+    writeOutId(self, ow);
+
+    DomItem component;
+    if (isRootObject)
+        component = self.containingObject();
+    if (ow.lineWriter.options().attributesSequence
+        == LineWriterOptions::AttributesSequence::Preserve) {
+        QString code;
+        if (std::shared_ptr<QmlFile> qmlFilePtr = self.ownerAs<QmlFile>())
+            code = qmlFilePtr->code();
+        writeOutAttributes(self, ow, component, code);
+    } else {
+        writeOutSortedAttributes(self, ow, component);
+    }
     ow.decreaseIndent(1, baseIndent);
-    ow.ensureNewline().writeRegion(RightBraceRegion);
+    ow.writeRegion(RightBraceRegion);
 }
 
 Binding::Binding(const QString &name) : Binding(name, std::unique_ptr<BindingValue>()) { }
@@ -1240,7 +1295,8 @@ void Binding::writeOutValue(const DomItem &self, OutWriter &lw) const
     switch (valueKind()) {
     case BindingValueKind::Empty:
         qCWarning(writeOutLog()) << "Writing of empty binding " << name();
-        lw.write(u"{}");
+        lw.writeRegion(LeftBraceRegion);
+        lw.writeRegion(RightBraceRegion);
         break;
     case BindingValueKind::Array:
         if (const List *vPtr = v.as<List>()) {
@@ -1356,10 +1412,8 @@ void EnumDecl::writeOut(const DomItem &self, OutWriter &ow) const
             .writeRegion(LeftBraceRegion);
     int iLevel = ow.increaseIndent(1);
     const auto values = self.field(Fields::values).values();
-    for (const auto &value : values) {
-        ow.ensureNewline();
+    for (const auto &value : values)
         value.writeOut(ow);
-    }
     ow.decreaseIndent(1, iLevel);
     ow.ensureNewline().writeRegion(RightBraceRegion);
 }
@@ -1642,20 +1696,17 @@ bool ScriptExpression::iterateDirectSubpaths(const DomItem &self, DirectVisitor
 class FirstNodeVisitor : public VisitAll
 {
 public:
-    quint32 minStart = 0;
-    quint32 maxEnd = std::numeric_limits<quint32>::max();
+    qsizetype minStart = 0;
+    qsizetype maxEnd = std::numeric_limits<qint32>::max(); // see also Lexer::checkFileLength().
     AST::Node *firstNodeInRange = nullptr;
 
-    FirstNodeVisitor(quint32 minStart = 0, quint32 maxEnd = std::numeric_limits<quint32>::max())
-        : minStart(minStart), maxEnd(maxEnd)
-    {
-    }
+    FirstNodeVisitor(qsizetype minStart, qsizetype maxEnd) : minStart(minStart), maxEnd(maxEnd) { }
 
     bool preVisit(AST::Node *n) override
     {
         if (!VisitAll::uiKinds().contains(n->kind)) {
-            quint32 start = n->firstSourceLocation().begin();
-            quint32 end = n->lastSourceLocation().end();
+            qsizetype start = n->firstSourceLocation().begin();
+            qsizetype end = n->lastSourceLocation().end();
             if (!firstNodeInRange && minStart <= start && end <= maxEnd && start < end)
                 firstNodeInRange = n;
         }
@@ -1663,7 +1714,7 @@ public:
     }
 };
 
-AST::Node *firstNodeInRange(AST::Node *n, quint32 minStart = 0, quint32 maxEnd = ~quint32(0))
+AST::Node *firstNodeInRange(AST::Node *n, qsizetype minStart = 0, qsizetype maxEnd = std::numeric_limits<qint32>::max())
 {
     FirstNodeVisitor visitor(minStart, maxEnd);
     AST::Node::accept(n, &visitor);
@@ -1772,29 +1823,8 @@ QString ScriptExpression::astRelocatableDump() const
     });
 }
 
-void ScriptExpression::writeOut(const DomItem &self, OutWriter &lw) const
+void ScriptExpression::writeOut(const DomItem &, OutWriter &lw) const
 {
-    OutWriter *ow = &lw;
-
-    std::optional<PendingSourceLocationId> codeLoc;
-    if (lw.lineWriter.options().updateOptions & LineWriterOptions::Update::Expressions)
-        codeLoc = lw.lineWriter.startSourceLocation([this, self, ow](SourceLocation myLoc) mutable {
-            QStringView reformattedCode =
-                    QStringView(ow->writtenStr).mid(myLoc.offset, myLoc.length);
-            if (reformattedCode != code()) {
-                // If some reformatting of the expression took place,
-                // it will be saved as an intermediate step.
-                // then it will be used to restore writtenOut fileItem
-                // in the OutWriter::restoreWrittenFile
-
-                //Interestingly enough, this copyWithUpdatedCode will
-                //instantiate Engine and Parser and will parse "reformattedCode"
-                //because it calls ScriptExpression::setCode function
-                std::shared_ptr<ScriptExpression> copy =
-                        copyWithUpdatedCode(self, reformattedCode.toString());
-                ow->addReformattedScriptExpression(self.canonicalPath(), copy);
-            }
-        });
     reformatAst(
             lw, m_astComments,
             [this](SourceLocation astL) {
@@ -1802,8 +1832,6 @@ void ScriptExpression::writeOut(const DomItem &self, OutWriter &lw) const
                 return this->code().mid(l.offset, l.length);
             },
             ast());
-    if (codeLoc)
-        lw.lineWriter.endSourceLocation(*codeLoc);
 }
 
 SourceLocation ScriptExpression::globalLocation(const DomItem &self) const
@@ -1875,7 +1903,7 @@ QString MethodInfo::preCode(const DomItem &self) const
         if (first) {
             first = false;
         } else {
-            ow.write(u",");
+            ow.writeRegion(CommaTokenRegion);
             ow.ensureSpace();
         }
         ow.write(mp.value->code());
@@ -1892,63 +1920,80 @@ QString MethodInfo::postCode(const DomItem &) const
     return QLatin1String("\n}\n");
 }
 
+void MethodInfo::writeOutArguments(const DomItem &self, OutWriter &ow) const
+{
+    if (parameters.isEmpty() && methodType == MethodType::Signal)
+        return;
+
+    ow.writeRegion(LeftParenthesisRegion);
+    bool first = true;
+    for (const DomItem &arg : self.field(Fields::parameters).values()) {
+        if (first)
+            first = false;
+        else
+            ow.writeRegion(CommaTokenRegion).ensureSpace();
+        arg.writeOut(ow);
+    }
+    ow.writeRegion(RightParenthesisRegion);
+}
+
+void MethodInfo::writeOutReturnType(OutWriter &ow) const
+{
+    if (typeName.isEmpty())
+        return;
+
+    ow.writeRegion(ColonTokenRegion);
+    ow.ensureSpace();
+    ow.writeRegion(TypeIdentifierRegion, typeName);
+}
+
+void MethodInfo::writeOutBody(const DomItem &self, OutWriter &ow) const
+{
+    ow.ensureSpace().writeRegion(LeftBraceRegion);
+    int baseIndent = ow.increaseIndent();
+    if (DomItem b = self.field(Fields::body)) {
+        ow.ensureNewline();
+        b.writeOut(ow);
+    }
+    ow.decreaseIndent(1, baseIndent);
+    ow.ensureNewline().writeRegion(RightBraceRegion);
+}
+
 void MethodInfo::writeOut(const DomItem &self, OutWriter &ow) const
 {
-    switch (methodType) {
-    case MethodType::Signal: {
-        if (body)
-            qCWarning(domLog) << "signal should not have a body in" << self.canonicalPath();
-        ow.writeRegion(SignalKeywordRegion).ensureSpace().writeRegion(IdentifierRegion, name);
-        if (parameters.isEmpty())
-            return;
-        bool first = true;
-        ow.writeRegion(LeftParenthesisRegion);
-        int baseIndent = ow.increaseIndent();
-        for (const DomItem &arg : self.field(Fields::parameters).values()) {
-            if (first)
-                first = false;
-            else
-                ow.write(u", ");
-
-            if (const MethodParameter *argPtr = arg.as<MethodParameter>()) {
-                if (argPtr->typeAnnotationStyle == MethodParameter::TypeAnnotationStyle::Prefix)
-                    argPtr->writeOutSignal(arg, ow);
-                else
-                    argPtr->writeOut(arg, ow);
-            } else
-                qCWarning(domLog) << "failed to cast to MethodParameter";
-        }
-        ow.writeRegion(RightParenthesisRegion);
-        ow.decreaseIndent(1, baseIndent);
-        return;
-    } break;
-    case MethodType::Method: {
-        ow.writeRegion(FunctionKeywordRegion).ensureSpace().writeRegion(IdentifierRegion, name);
-        bool first = true;
-        ow.writeRegion(LeftParenthesisRegion);
-        for (const DomItem &arg : self.field(Fields::parameters).values()) {
-            if (first)
-                first = false;
-            else
-                ow.write(u", ");
-            arg.writeOut(ow);
-        }
-        ow.writeRegion(RightParenthesisRegion);
-        if (!typeName.isEmpty()) {
-            ow.writeRegion(ColonTokenRegion);
-            ow.ensureSpace();
-            ow.writeRegion(TypeIdentifierRegion, typeName);
-        }
-        ow.ensureSpace().writeRegion(LeftBraceRegion);
-        int baseIndent = ow.increaseIndent();
-        if (DomItem b = self.field(Fields::body)) {
-            ow.ensureNewline();
-            b.writeOut(ow);
-        }
-        ow.decreaseIndent(1, baseIndent);
-        ow.ensureNewline().writeRegion(RightBraceRegion);
-    } break;
+    if (methodType == MethodType::Signal) {
+        ow.writeRegion(SignalKeywordRegion).ensureSpace();
+    } else {
+        ow.writeRegion(FunctionKeywordRegion).ensureSpace();
     }
+    ow.writeRegion(IdentifierRegion, name);
+    writeOutArguments(self, ow);
+    if (methodType == MethodType::Signal) {
+        // signal doesn't have returnType or body
+        return;
+    }
+    writeOutReturnType(ow);
+    writeOutBody(self, ow);
+}
+
+QString MethodInfo::signature(const DomItem &self) const
+{
+    QString resultStr;
+    QTextStream res(&resultStr);
+    LineWriter lw([&res](QStringView s) { res << s; }, QLatin1String("*testStream*"));
+    OutWriter ow(lw);
+    ow.indentNextlines = true;
+    ow.skipComments = true;
+
+    ow.itemStart(self);
+    writeOutArguments(self, ow);
+
+    writeOutReturnType(ow);
+
+    ow.itemEnd(self);
+    lw.eof(false);
+    res.flush();
+    return resultStr.simplified();
 }
 
 bool MethodParameter::iterateDirectSubpaths(const DomItem &self, DirectVisitor visitor) const
@@ -1977,6 +2022,11 @@ bool MethodParameter::iterateDirectSubpaths(const DomItem &self, DirectVisitor v
 
 void MethodParameter::writeOut(const DomItem &self, OutWriter &ow) const
 {
+    if (typeAnnotationStyle == MethodParameter::TypeAnnotationStyle::Prefix) {
+        writeOutSignal(self, ow);
+        return;
+    }
+
     if (!name.isEmpty()) {
         if (isRestElement)
             ow.writeRegion(EllipsisTokenRegion);
@@ -2034,17 +2084,17 @@ bool EnumItem::iterateDirectSubpaths(const DomItem &self, DirectVisitor visitor)
 
 void EnumItem::writeOut(const DomItem &self, OutWriter &ow) const
 {
+    index_type myIndex = self.pathFromOwner().last().headIndex();
+    if (myIndex != 0)
+        ow.writeRegion(CommaTokenRegion);
     ow.ensureNewline();
     ow.writeRegion(IdentifierRegion, name());
-    index_type myIndex = self.pathFromOwner().last().headIndex();
     if (m_valueKind == ValueKind::ExplicitValue) {
         QString v = QString::number(value(), 'f', 0);
         if (abs(value() - v.toDouble()) > 1.e-10)
             v = QString::number(value());
         ow.ensureSpace().writeRegion(EqualTokenRegion).ensureSpace().writeRegion(EnumValueRegion, v);
     }
-    if (myIndex >= 0 && self.container().indexes() != myIndex + 1)
-        ow.writeRegion(CommaTokenRegion);
 }
 
 QmlUri QmlUri::fromString(const QString &str)
