@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:interprocess-communication
 
 #include "qquickdrag_p.h"
 #include "qquickdrag_p_p.h"
@@ -9,7 +10,7 @@
 #include <private/qquickitem_p.h>
 #include <QtQuick/private/qquickevents_p_p.h>
 #include <private/qquickitemchangelistener_p.h>
-#include <private/qquickpixmapcache_p.h>
+#include <private/qquickpixmap_p.h>
 #include <private/qv4scopedvalue_p.h>
 #include <QtCore/qbuffer.h>
 #include <QtCore/qmimedata.h>
@@ -30,7 +31,7 @@ using namespace Qt::StringLiterals;
 
 /*!
     \qmltype Drag
-    \instantiates QQuickDrag
+    \nativetype QQuickDrag
     \inqmlmodule QtQuick
     \ingroup qtquick-input
     \brief For specifying drag and drop events for moved Items.
@@ -296,7 +297,7 @@ QObject *QQuickDragAttached::target() const
 }
 
 /*!
-    \qmlattachedproperty QPointF QtQuick::Drag::hotSpot
+    \qmlattachedproperty point QtQuick::Drag::hotSpot
 
     This property holds the drag position relative to the top left of the item.
 
@@ -325,7 +326,7 @@ void QQuickDragAttached::setHotSpot(const QPointF &hotSpot)
 }
 
 /*!
-    \qmlattachedproperty QUrl QtQuick::Drag::imageSource
+    \qmlattachedproperty url QtQuick::Drag::imageSource
     \since 5.8
 
     This property holds the URL of the image which will be used to represent
@@ -354,13 +355,66 @@ void QQuickDragAttached::setImageSource(const QUrl &url)
         if (url.isEmpty()) {
             d->pixmapLoader.clear();
         } else {
-            d->pixmapLoader.load(qmlEngine(parent()), url);
+            d->loadPixmap();
         }
 
         Q_EMIT imageSourceChanged();
     }
 }
 
+/*!
+    \qmlattachedproperty size QtQuick::Drag::imageSourceSize
+    \since 6.8
+
+    This property holds the size of the image that will be used to represent
+    the data during the drag and drop operation. Changing this property after
+    the drag operation has started will have no effect.
+
+    This property sets the maximum number of pixels stored for the loaded
+    image so that large images do not use more memory than necessary.
+    See \l {QtQuick::Image::sourceSize}{Image.sourceSize} for more details.
+
+    The example below shows an SVG image rendered at one size, and re-renders
+    it at a different size for the drag image:
+
+    \snippet qml/externalDragScaledImage.qml 0
+
+    \sa imageSource, Item::grabToImage()
+*/
+
+QSize QQuickDragAttached::imageSourceSize() const
+{
+    Q_D(const QQuickDragAttached);
+    int width = d->imageSourceSize.width();
+    int height = d->imageSourceSize.height();
+    // If width or height is invalid, check whether the size is valid from the loaded image.
+    // If it ends up 0x0 though, leave it as an invalid QSize instead (-1 x -1).
+    if (width == -1) {
+        width = d->pixmapLoader.width();
+        if (!width)
+            width = -1;
+    }
+    if (height == -1) {
+        height = d->pixmapLoader.height();
+        if (!height)
+            height = -1;
+    }
+    return QSize(width, height);
+}
+
+void QQuickDragAttached::setImageSourceSize(const QSize &size)
+{
+    Q_D(QQuickDragAttached);
+    if (d->imageSourceSize != size) {
+        d->imageSourceSize = size;
+
+        if (!d->imageSource.isEmpty())
+            d->loadPixmap();
+
+        Q_EMIT imageSourceSizeChanged();
+    }
+}
+
 /*!
     \qmlattachedproperty stringlist QtQuick::Drag::keys
 
@@ -538,7 +592,7 @@ void QQuickDragAttachedPrivate::start(Qt::DropActions supportedActions)
     property for the started sequence.
 */
 
-void QQuickDragAttached::start(QQmlV4Function *args)
+void QQuickDragAttached::start(QQmlV4FunctionPtr args)
 {
     Q_D(QQuickDragAttached);
     if (d->inEvent) {
@@ -686,11 +740,18 @@ QMimeData *QQuickDragAttachedPrivate::createMimeData() const
             } else if (mimeType == u"text/html"_s) {
                 mimeData->setHtml(text);
             } else if (mimeType == u"text/uri-list"_s) {
-                const QUrl url(text);
-                if (url.isValid())
-                    mimeData->setUrls({url});
-                else
-                    qmlWarning(q) << text << " is not a valid URI";
+                QList<QUrl> urls;
+                // parse and split according to RFC2483
+                const auto lines = text.split(u"\r\n"_s, Qt::SkipEmptyParts);
+                for (const auto &line : lines) {
+                    const QUrl url(line);
+                    if (url.isValid())
+                        urls.push_back(url);
+                    else
+                        qmlWarning(q) << line << " is not a valid URI";
+
+                }
+                mimeData->setUrls(urls);
             } else if (mimeType.startsWith(u"text/"_s)) {
                 if (qsizetype charsetIdx = mimeType.lastIndexOf(u";charset="_s); charsetIdx != -1) {
                     charsetIdx += sizeof(";charset=") - 1;
@@ -701,7 +762,7 @@ QMimeData *QQuickDragAttachedPrivate::createMimeData() const
                     else
                         qmlWarning(q) << "Don't know how to encode text as " << mimeType;
                 } else {
-                    mimeData->setData(mimeType, text.toUtf8().constData());
+                    mimeData->setData(mimeType, text.toUtf8());
                 }
             } else {
                 mimeData->setData(mimeType, text.toUtf8());
@@ -729,6 +790,10 @@ QMimeData *QQuickDragAttachedPrivate::createMimeData() const
                     mimeData->setUrls(urls);
             }
             break;
+        case QMetaType::QColor:
+            if (mimeType == u"application/x-color"_s)
+                mimeData->setColorData(value);
+            break;
         case QMetaType::QImage:
             if (const QByteArray mimeTypeUtf8 = mimeType.toUtf8();
                 QImageWriter::supportedMimeTypes().contains(mimeTypeUtf8)) {
@@ -760,6 +825,17 @@ QMimeData *QQuickDragAttachedPrivate::createMimeData() const
     return mimeData;
 }
 
+void QQuickDragAttachedPrivate::loadPixmap()
+{
+    Q_Q(QQuickDragAttached);
+
+    QUrl loadUrl = imageSource;
+    const QQmlContext *context = qmlContext(q->parent());
+    if (context)
+        loadUrl = context->resolvedUrl(imageSource);
+    pixmapLoader.load(context ? context->engine() : nullptr, loadUrl, QRect(), q->imageSourceSize());
+}
+
 Qt::DropAction QQuickDragAttachedPrivate::startDrag(Qt::DropActions supportedActions)
 {
     Q_Q(QQuickDragAttached);
@@ -803,7 +879,7 @@ Qt::DropAction QQuickDragAttachedPrivate::startDrag(Qt::DropActions supportedAct
     property for the started sequence.
 */
 
-void QQuickDragAttached::startDrag(QQmlV4Function *args)
+void QQuickDragAttached::startDrag(QQmlV4FunctionPtr args)
 {
     Q_D(QQuickDragAttached);
 
