@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtGui module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:critical reason:data-parser
 
 #include "qquicktextcontrol_p.h"
 #include "qquicktextcontrol_p_p.h"
@@ -72,12 +37,12 @@
 #include <qmetaobject.h>
 
 #include <private/qqmlglobal_p.h>
+#include <private/qquickdeliveryagent_p_p.h>
 
 // ### these should come from QStyleHints
 const int textCursorWidth = 1;
 
 QT_BEGIN_NAMESPACE
-Q_DECLARE_LOGGING_CATEGORY(lcHoverTrace)
 
 // could go into QTextCursor...
 static QTextLine currentTextLine(const QTextCursor &cursor)
@@ -115,6 +80,9 @@ QQuickTextControlPrivate::QQuickTextControlPrivate()
       hasImState(false),
       cursorRectangleChanged(false),
       hoveredMarker(false),
+      selectByTouchDrag(false),
+      imSelectionAfterPress(false),
+      beingEdited(false),
       lastSelectionStart(-1),
       lastSelectionEnd(-1)
 {}
@@ -302,7 +270,7 @@ void QQuickTextControlPrivate::setContent(Qt::TextFormat format, const QString &
     const int oldCursorPos = cursor.position();
 
     // avoid multiple textChanged() signals being emitted
-    qmlobject_disconnect(doc, QTextDocument, SIGNAL(contentsChanged()), q, QQuickTextControl, SIGNAL(textChanged()));
+    QObject::disconnect(doc, &QTextDocument::contentsChanged, q, &QQuickTextControl::textChanged);
 
     if (!text.isEmpty()) {
         // clear 'our' cursor for insertion to prevent
@@ -342,7 +310,7 @@ void QQuickTextControlPrivate::setContent(Qt::TextFormat format, const QString &
     }
     cursor.setCharFormat(charFormatForInsertion);
 
-    qmlobject_connect(doc, QTextDocument, SIGNAL(contentsChanged()), q, QQuickTextControl, SIGNAL(textChanged()));
+    QObject::connect(doc, &QTextDocument::contentsChanged, q, &QQuickTextControl::textChanged);
     emit q->textChanged();
     doc->setUndoRedoEnabled(previousUndoRedoState);
     _q_updateCurrentCharFormatAndSelection();
@@ -617,25 +585,7 @@ void QQuickTextControl::clear()
 QQuickTextControl::QQuickTextControl(QTextDocument *doc, QObject *parent)
     : QInputControl(TextEdit, *new QQuickTextControlPrivate, parent)
 {
-    Q_D(QQuickTextControl);
-    Q_ASSERT(doc);
-
-    QAbstractTextDocumentLayout *layout = doc->documentLayout();
-    qmlobject_connect(layout, QAbstractTextDocumentLayout, SIGNAL(update(QRectF)), this, QQuickTextControl, SIGNAL(updateRequest()));
-    qmlobject_connect(layout, QAbstractTextDocumentLayout, SIGNAL(updateBlock(QTextBlock)), this, QQuickTextControl, SIGNAL(updateRequest()));
-    qmlobject_connect(doc, QTextDocument, SIGNAL(contentsChanged()), this, QQuickTextControl, SIGNAL(textChanged()));
-    qmlobject_connect(doc, QTextDocument, SIGNAL(contentsChanged()), this, QQuickTextControl, SLOT(_q_updateCurrentCharFormatAndSelection()));
-    qmlobject_connect(doc, QTextDocument, SIGNAL(cursorPositionChanged(QTextCursor)), this, QQuickTextControl, SLOT(_q_updateCursorPosChanged(QTextCursor)));
-    connect(doc, &QTextDocument::contentsChange, this, &QQuickTextControl::contentsChange);
-
-    layout->setProperty("cursorWidth", textCursorWidth);
-
-    d->doc = doc;
-    d->cursor = QTextCursor(doc);
-    d->lastCharFormat = d->cursor.charFormat();
-    doc->setPageSize(QSizeF(0, 0));
-    doc->setModified(false);
-    doc->setUndoRedoEnabled(true);
+    setDocument(doc);
 }
 
 QQuickTextControl::~QQuickTextControl()
@@ -648,6 +598,37 @@ QTextDocument *QQuickTextControl::document() const
     return d->doc;
 }
 
+void QQuickTextControl::setDocument(QTextDocument *doc)
+{
+    Q_D(QQuickTextControl);
+    if (!doc || d->doc == doc)
+        return;
+
+    if (d->doc) {
+        QAbstractTextDocumentLayout *oldLayout = d->doc->documentLayout();
+        disconnect(oldLayout, nullptr, this, nullptr);
+        disconnect(d->doc, nullptr, this, nullptr);
+    }
+
+    d->doc = doc;
+    d->cursor = QTextCursor(doc);
+    d->lastCharFormat = d->cursor.charFormat();
+
+    QAbstractTextDocumentLayout *layout = doc->documentLayout();
+    connect(layout, &QAbstractTextDocumentLayout::update, this, &QQuickTextControl::updateRequest);
+    connect(layout, &QAbstractTextDocumentLayout::updateBlock, this, &QQuickTextControl::updateRequest);
+    connect(doc, &QTextDocument::contentsChanged, this, [d]() {
+        d->_q_updateCurrentCharFormatAndSelection();
+    });
+    connect(doc, &QTextDocument::contentsChanged, this, &QQuickTextControl::textChanged);
+    connect(doc, &QTextDocument::cursorPositionChanged, this, [d](const QTextCursor &cursor) {
+        d->_q_updateCursorPosChanged(cursor);
+    });
+    connect(doc, &QTextDocument::contentsChange, this, &QQuickTextControl::contentsChange);
+    if (auto *qtdlayout = qobject_cast<QTextDocumentLayout *>(layout))
+        qtdlayout->setCursorWidth(textCursorWidth);
+}
+
 void QQuickTextControl::updateCursorRectangle(bool force)
 {
     Q_D(QQuickTextControl);
@@ -775,9 +756,8 @@ void QQuickTextControl::processEvent(QEvent *e, const QTransform &transform)
 
         case QEvent::ShortcutOverride:
             if (d->interactionFlags & Qt::TextEditable) {
-                QKeyEvent* ke = static_cast<QKeyEvent *>(e);
-                if (isCommonTextEditShortcut(ke))
-                    ke->accept();
+                QKeyEvent *ke = static_cast<QKeyEvent *>(e);
+                ke->setAccepted(isCommonTextEditShortcut(ke));
             }
             break;
         default:
@@ -790,6 +770,12 @@ bool QQuickTextControl::event(QEvent *e)
     return QObject::event(e);
 }
 
+bool QQuickTextControl::isBeingEdited()
+{
+    Q_D(QQuickTextControl);
+    return d->beingEdited;
+}
+
 void QQuickTextControl::timerEvent(QTimerEvent *e)
 {
     Q_D(QQuickTextControl);
@@ -828,6 +814,7 @@ void QQuickTextControlPrivate::keyPressEvent(QKeyEvent *e)
 {
     Q_Q(QQuickTextControl);
 
+    QScopedValueRollback<bool> rollbackBeingEdited(beingEdited, true);
     if (e->key() == Qt::Key_Back) {
          e->ignore();
          return;
@@ -997,7 +984,7 @@ QRectF QQuickTextControlPrivate::rectForPosition(int position) const
         if (relativePos == preeditPos)
             relativePos += preeditCursor;
         else if (relativePos > preeditPos)
-            relativePos += layout->preeditAreaText().length();
+            relativePos += layout->preeditAreaText().size();
     }
 #endif
     QTextLine line = layout->lineForTextPosition(relativePos);
@@ -1027,6 +1014,7 @@ void QQuickTextControlPrivate::mousePressEvent(QMouseEvent *e, const QPointF &po
 
     mousePressed = (interactionFlags & Qt::TextSelectableByMouse) && (e->button() & Qt::LeftButton);
     mousePressPos = pos.toPoint();
+    imSelectionAfterPress = false;
 
     if (sendMouseEventToInputContext(e, pos))
         return;
@@ -1040,6 +1028,9 @@ void QQuickTextControlPrivate::mousePressEvent(QMouseEvent *e, const QPointF &po
             cursor.clearSelection();
         }
     }
+
+    if (!selectByTouchDrag && !QQuickDeliveryAgentPrivate::isEventFromMouseOrTouchpad(e))
+        return;
     if (interactionFlags & Qt::TextEditable)
         blockWithMarkerUnderMousePress = q->blockWithMarkerAt(pos);
     if (e->button() & Qt::MiddleButton) {
@@ -1110,6 +1101,9 @@ void QQuickTextControlPrivate::mousePressEvent(QMouseEvent *e, const QPointF &po
 
 void QQuickTextControlPrivate::mouseMoveEvent(QMouseEvent *e, const QPointF &mousePos)
 {
+    if (!selectByTouchDrag && !QQuickDeliveryAgentPrivate::isEventFromMouseOrTouchpad(e))
+        return;
+
     Q_Q(QQuickTextControl);
 
     if ((e->buttons() & Qt::LeftButton)) {
@@ -1190,6 +1184,7 @@ void QQuickTextControlPrivate::mouseReleaseEvent(QMouseEvent *e, const QPointF &
 
     const QTextCursor oldSelection = cursor;
     const int oldCursorPos = cursor.position();
+    const bool isMouse = QQuickDeliveryAgentPrivate::isEventFromMouseOrTouchpad(e);
 
     if (mousePressed) {
         mousePressed = false;
@@ -1205,6 +1200,8 @@ void QQuickTextControlPrivate::mouseReleaseEvent(QMouseEvent *e, const QPointF &
             q->insertFromMimeData(md);
 #endif
     }
+    if (!isMouse && !selectByTouchDrag && !imSelectionAfterPress && interactionFlags.testFlag(Qt::TextEditable))
+        setCursorPosition(pos);
 
     repaintOldAndNewSelection(oldSelection);
 
@@ -1213,7 +1210,8 @@ void QQuickTextControlPrivate::mouseReleaseEvent(QMouseEvent *e, const QPointF &
         q->updateCursorRectangle(true);
     }
 
-    if ((interactionFlags & Qt::TextEditable) && (e->button() & Qt::LeftButton) && blockWithMarkerUnderMousePress.isValid()) {
+    if ((isMouse || selectByTouchDrag) && interactionFlags.testFlag(Qt::TextEditable) &&
+            (e->button() & Qt::LeftButton) && blockWithMarkerUnderMousePress.isValid()) {
         QTextBlock block = q->blockWithMarkerAt(pos);
         if (block == blockWithMarkerUnderMousePress) {
             auto fmt = block.blockFormat();
@@ -1251,7 +1249,8 @@ void QQuickTextControlPrivate::mouseDoubleClickEvent(QMouseEvent *e, const QPoin
 {
     Q_Q(QQuickTextControl);
 
-    if (e->button() == Qt::LeftButton && (interactionFlags & Qt::TextSelectableByMouse)) {
+    if (e->button() == Qt::LeftButton && (interactionFlags & Qt::TextSelectableByMouse)
+            && (selectByTouchDrag || QQuickDeliveryAgentPrivate::isEventFromMouseOrTouchpad(e))) {
 #if QT_CONFIG(im)
         commitPreedit();
 #endif
@@ -1295,7 +1294,7 @@ bool QQuickTextControlPrivate::sendMouseEventToInputContext(QMouseEvent *e, cons
         QTextLayout *layout = cursor.block().layout();
         int cursorPos = q->hitTest(pos, Qt::FuzzyHit) - cursor.position();
 
-        if (cursorPos >= 0 && cursorPos <= layout->preeditAreaText().length()) {
+        if (cursorPos >= 0 && cursorPos <= layout->preeditAreaText().size()) {
             if (e->type() == QEvent::MouseButtonRelease) {
                 QGuiApplication::inputMethod()->invokeAction(QInputMethod::Click, cursorPos);
             }
@@ -1346,6 +1345,8 @@ void QQuickTextControlPrivate::inputMethodEvent(QInputMethodEvent *e)
         for (int i = 0; i < e->attributes().size(); ++i) {
             const QInputMethodEvent::Attribute &a = e->attributes().at(i);
             if (a.type == QInputMethodEvent::Selection) {
+                if (mousePressed)
+                    imSelectionAfterPress = true;
                 QTextCursor oldCursor = cursor;
                 int blockStart = a.start + cursor.block().position();
                 cursor.setPosition(blockStart, QTextCursor::MoveAnchor);
@@ -1367,7 +1368,7 @@ void QQuickTextControlPrivate::inputMethodEvent(QInputMethodEvent *e)
             emit q->preeditTextChanged();
         }
         QVector<QTextLayout::FormatRange> overrides;
-        preeditCursor = e->preeditString().length();
+        preeditCursor = e->preeditString().size();
         hasImState = !e->preeditString().isEmpty();
         cursorVisible = true;
         for (int i = 0; i < e->attributes().size(); ++i) {
@@ -1440,7 +1441,7 @@ QVariant QQuickTextControl::inputMethodQuery(Qt::InputMethodQuery property, cons
         QTextCursor tmpCursor = d->cursor;
         int localPos = d->cursor.position() - block.position();
         QString result = block.text().mid(localPos);
-        while (result.length() < maxLength) {
+        while (result.size() < maxLength) {
             int currentBlock = tmpCursor.blockNumber();
             tmpCursor.movePosition(QTextCursor::NextBlock);
             if (tmpCursor.blockNumber() == currentBlock)
@@ -1645,6 +1646,14 @@ void QQuickTextControl::setWordSelectionEnabled(bool enabled)
     d->wordSelectionEnabled = enabled;
 }
 
+#if QT_VERSION < QT_VERSION_CHECK(7, 0, 0)
+void QQuickTextControl::setTouchDragSelectionEnabled(bool enabled)
+{
+    Q_D(QQuickTextControl);
+    d->selectByTouchDrag = enabled;
+}
+#endif
+
 QMimeData *QQuickTextControl::createMimeDataFromSelection() const
 {
     Q_D(const QQuickTextControl);
