@@ -5,6 +5,7 @@
 
 #include <QtCore/qobject.h>
 #include <QtQml/qqmlcontext.h>
+#include <QtQuick/private/qquicktaphandler_p.h>
 
 #include <QtQmlModels/private/qqmltreemodeltotablemodel_p_p.h>
 
@@ -19,7 +20,7 @@
     A TreeView has a \l model that defines the data to be displayed, and a
     \l delegate that defines how the data should be displayed.
 
-    TreeView inherits \l TableView. This means that even if the model
+    TreeView inherits \l TableView. This means that, despite the model
     has a parent-child tree structure, TreeView is internally using a
     proxy model that converts that structure into a flat table
     model that can be rendered by TableView. Each node in the tree ends up
@@ -28,23 +29,57 @@
     parent-child depth in the model, it will end up looking like a tree, even
     if it's technically still just a flat list of items.
 
-    To allow for maximum flexibility, TreeView itself will not position the delegate
-    items into a tree structure. This burden is placed on the delegate.
-    \l {Qt Quick Controls} offers a ready-made TreeViewDelegate that can be
-    used for this, which has the advantage that it works out-of-the-box and
-    renders a tree which follows the style of the platform where the application runs.
+    \section2 Declare a TreeView
 
-    Even if TreeViewDelegate is customizable, there might be situations
+    TreeView is a data bound control, so it cannot show anything without a
+    data model. You cannot declare tree nodes in QML.
+
+    When you declare a TreeView, you need to specify:
+
+    \list
+      \li \b{A data model}. TreeView can work with data models that derive from
+        \l QAbstractItemModel.
+      \li \b{A delegate}. A delegate is a template that specifies how the tree
+        nodes are displayed in the UI.
+    \endlist
+
+    \qml
+    TreeView {
+        // The model needs to be a QAbstractItemModel
+        model: myTreeModel
+        // You can set a custom delegate or use a built-in TreeViewDelegate
+        delegate: TreeViewDelegate {}
+    }
+    \endqml
+
+    \section2 Creating a data model
+
+    A TreeView only accepts a model that inherits \l QAbstractItemModel.
+
+    For information on how to create and use a custom tree model, see the
+    example: \l {Qt Quick Controls - Table of Contents}.
+
+    \section2 Customize tree nodes
+
+    For better flexibility, TreeView itself doesn't position the delegate items
+    into a tree structure. This burden is placed on the delegate.
+    \l {Qt Quick Controls} offers a ready-made \l TreeViewDelegate that can be
+    used for this, which has the advantage that it works out-of-the-box and
+    renders a tree which follows the style of the platform where the application
+    runs.
+
+    Even though \l TreeViewDelegate is customizable, there might be situations
     where you want to render the tree in a different way, or ensure that
     the delegate ends up as minimal as possible, perhaps for performance reasons.
     Creating your own delegate from scratch is easy, since TreeView offers
     a set of properties that can be used to position and render each node
     in the tree correctly.
 
-    An example of a custom delegate is shown below:
+    An example of a custom delegate with an animating indicator is shown below:
 
     \snippet qml/treeview/qml-customdelegate.qml 0
 
+
     The properties that are marked as \c required will be filled in by
     TreeView, and are similar to attached properties. By marking them as
     required, the delegate indirectly informs TreeView that it should take
@@ -67,7 +102,7 @@
         \li \c {required property bool expanded}
                 - Is \c true if the model item drawn by the delegate is expanded
                 in the view.
-        \li \c {required property int hasChildren}
+        \li \c {required property bool hasChildren}
                 - Is \c true if the model item drawn by the delegate has children
                 in the model.
         \li \c {required property int depth}
@@ -78,7 +113,26 @@
 
     See also \l {Required Properties}.
 
-    \note A TreeView only accepts a model that inherits \l QAbstractItemModel.
+    \section2 End-user interaction
+
+    By default, TreeView \l {toggleExpanded()}{toggles} the expanded state
+    of a row when you double tap on it. Since this is in conflict with
+    double tapping to edit a cell, TreeView sets \l {TableView::}{editTriggers} to
+    \c TableView.EditKeyPressed by default (which is different from TableView,
+    which uses \c {TableView.EditKeyPressed | TableView.DoubleTapped}.
+    If you change \l {TableView::}{editTriggers} to also contain \c TableView.DoubleTapped,
+    toggling the expanded state with a double tap will be disabled.
+
+*/
+
+/*!
+    \qmlproperty QModelIndex QtQuick::TreeView::rootIndex
+    \since 6.6
+
+    This property holds the model index of the root item in the tree.
+    By default, this is the same as the root index in the model, but you can
+    set it to be a child index instead, to show only a branch of the tree.
+    Set it to \c undefined to show the whole model.
 */
 
 /*!
@@ -259,9 +313,6 @@ void QQuickTreeViewPrivate::setModelImpl(const QVariant &newModel)
 {
     Q_Q(QQuickTreeView);
 
-    if (newModel == m_assignedModel)
-        return;
-
     m_assignedModel = newModel;
     QVariant effectiveModel = m_assignedModel;
     if (effectiveModel.userType() == qMetaTypeId<QJSValue>())
@@ -328,55 +379,39 @@ void QQuickTreeViewPrivate::updateSelection(const QRect &oldSelection, const QRe
 {
     Q_Q(QQuickTreeView);
 
-    const QRect oldRect = oldSelection.normalized();
-    const QRect newRect = newSelection.normalized();
-
     if (oldSelection == newSelection)
         return;
 
-    // Select the rows inside newRect that doesn't overlap with oldRect
-    for (int row = newRect.y(); row <= newRect.y() + newRect.height(); ++row) {
-        if (oldRect.y() != -1 && oldRect.y() <= row && row <= oldRect.y() + oldRect.height())
-            continue;
-        const QModelIndex startIndex = q->modelIndex(newRect.x(), row);
-        const QModelIndex endIndex = q->modelIndex(newRect.x() + newRect.width(), row);
-        selectionModel->select(QItemSelection(startIndex, endIndex), QItemSelectionModel::Select);
+    QItemSelection select;
+    QItemSelection deselect;
+
+    // Because each row can have a different parent, we need to create separate QItemSelections
+    // per row. But all the cells in a given row have the same parent, so they can be combined.
+    // As a result, the final QItemSelection can end up more fragmented compared to a selection
+    // in QQuickTableView, where all cells have the same parent. In the end, if TreeView has
+    // a lot of columns and the selection mode is "SelectCells", using the mouse to adjust
+    // a selection containing a _large_ number of columns can be slow.
+    const QRect cells = newSelection.normalized();
+    for (int row = cells.y(); row <= cells.y() + cells.height(); ++row) {
+        const QModelIndex startIndex = q->index(row, cells.x());
+        const QModelIndex endIndex = q->index(row, cells.x() + cells.width());
+        select.merge(QItemSelection(startIndex, endIndex), QItemSelectionModel::Select);
     }
 
-    if (oldRect.x() != -1) {
-        // Since oldRect is valid, this update is a continuation of an already existing selection!
+    const QModelIndexList indexes = selectionModel->selection().indexes();
+    for (const QModelIndex &index : indexes) {
+        if (!select.contains(index) && !existingSelection.contains(index))
+            deselect.merge(QItemSelection(index, index), QItemSelectionModel::Select);
+    }
 
-        // Select the columns inside newRect that don't overlap with oldRect
-        for (int column = newRect.x(); column <= newRect.x() + newRect.width(); ++column) {
-            if (oldRect.x() <= column && column <= oldRect.x() + oldRect.width())
-                continue;
-            for (int row = newRect.y(); row <= newRect.y() + newRect.height(); ++row)
-                selectionModel->select(q->modelIndex(column, row), QItemSelectionModel::Select);
-        }
-
-        // Unselect the rows inside oldRect that don't overlap with newRect
-        for (int row = oldRect.y(); row <= oldRect.y() + oldRect.height(); ++row) {
-            if (newRect.y() <= row && row <= newRect.y() + newRect.height())
-                continue;
-            const QModelIndex startIndex = q->modelIndex(oldRect.x(), row);
-            const QModelIndex endIndex = q->modelIndex(oldRect.x() + oldRect.width(), row);
-            selectionModel->select(QItemSelection(startIndex, endIndex), QItemSelectionModel::Deselect);
-        }
-
-        // Unselect the columns inside oldRect that don't overlap with newRect
-        for (int column = oldRect.x(); column <= oldRect.x() + oldRect.width(); ++column) {
-            if (newRect.x() <= column && column <= newRect.x() + newRect.width())
-                continue;
-            // Since we're not allowed to call select/unselect on the selectionModel with
-            // indices from different parents, and since indicies from different parents are
-            // expected when working with trees, we need to unselect the indices in the column
-            // one by one, rather than the whole column in one go. This, however, can cause a
-            // lot of selection fragments in the selectionModel, which eventually can hurt
-            // performance. But large selections containing a lot of columns is not normally
-            // the case for a treeview, so accept this potential corner case for now.
-            for (int row = newRect.y(); row <= newRect.y() + newRect.height(); ++row)
-                selectionModel->select(q->modelIndex(column, row), QItemSelectionModel::Deselect);
-        }
+    if (selectionFlag == QItemSelectionModel::Select) {
+        selectionModel->select(deselect, QItemSelectionModel::Deselect);
+        selectionModel->select(select, QItemSelectionModel::Select);
+    } else {
+        QItemSelection oldSelection = existingSelection;
+        oldSelection.merge(select, QItemSelectionModel::Deselect);
+        selectionModel->select(oldSelection, QItemSelectionModel::Select);
+        selectionModel->select(select, QItemSelectionModel::Deselect);
     }
 }
 
@@ -386,6 +421,7 @@ QQuickTreeView::QQuickTreeView(QQuickItem *parent)
     Q_D(QQuickTreeView);
 
     setSelectionBehavior(SelectRows);
+    setEditTriggers(EditKeyPressed);
 
     // Note: QQuickTableView will only ever see the table model m_treeModelToTableModel, and
     // never the actual tree model that is assigned to us by the application.
@@ -393,12 +429,45 @@ QQuickTreeView::QQuickTreeView(QQuickItem *parent)
     d->QQuickTableViewPrivate::setModelImpl(modelAsVariant);
     QObjectPrivate::connect(&d->m_treeModelToTableModel, &QAbstractItemModel::dataChanged,
                             d, &QQuickTreeViewPrivate::dataChangedCallback);
+    QObject::connect(&d->m_treeModelToTableModel, &QQmlTreeModelToTableModel::rootIndexChanged,
+                     this, &QQuickTreeView::rootIndexChanged);
+
+    auto tapHandler = new QQuickTapHandler(this);
+    tapHandler->setAcceptedModifiers(Qt::NoModifier);
+    connect(tapHandler, &QQuickTapHandler::doubleTapped, [this, tapHandler]{
+        if (!pointerNavigationEnabled())
+            return;
+        if (editTriggers() & DoubleTapped)
+            return;
+
+        const int row = cellAtPosition(tapHandler->point().pressPosition()).y();
+        toggleExpanded(row);
+    });
 }
 
 QQuickTreeView::~QQuickTreeView()
 {
 }
 
+QModelIndex QQuickTreeView::rootIndex() const
+{
+    return d_func()->m_treeModelToTableModel.rootIndex();
+}
+
+void QQuickTreeView::setRootIndex(const QModelIndex &index)
+{
+    Q_D(QQuickTreeView);
+    d->m_treeModelToTableModel.setRootIndex(index);
+    positionViewAtCell({0, 0}, QQuickTableView::AlignTop | QQuickTableView::AlignLeft);
+}
+
+void QQuickTreeView::resetRootIndex()
+{
+    Q_D(QQuickTreeView);
+    d->m_treeModelToTableModel.resetRootIndex();
+    positionViewAtCell({0, 0}, QQuickTableView::AlignTop | QQuickTableView::AlignLeft);
+}
+
 int QQuickTreeView::depth(int row) const
 {
     Q_D(const QQuickTreeView);
@@ -588,10 +657,25 @@ QPoint QQuickTreeView::cellAtIndex(const QModelIndex &index) const
     return QPoint(tableIndex.column(), tableIndex.row());
 }
 
-QModelIndex QQuickTreeView::modelIndex(int column, int row) const
+#if QT_DEPRECATED_SINCE(6, 4)
+QModelIndex QQuickTreeView::modelIndex(int row, int column) const
 {
-    return modelIndex({column, row});
+    static const bool compat6_4 = qEnvironmentVariable("QT_QUICK_TABLEVIEW_COMPAT_VERSION") == QStringLiteral("6.4");
+    if (compat6_4) {
+        // XXX Qt 7: Remove this compatibility path here and in QQuickTableView.
+        // In Qt 6.4.0 and 6.4.1, a source incompatible change led to row and column
+        // being documented to be specified in the opposite order.
+        // QT_QUICK_TABLEVIEW_COMPAT_VERSION can therefore be set to force tableview
+        // to continue accepting calls to modelIndex(column, row).
+        return modelIndex({row, column});
+    } else {
+        qmlWarning(this) << "modelIndex(row, column) is deprecated. "
+                            "Use index(row, column) instead. For more information, see "
+                            "https://doc.qt.io/qt-6/qml-qtquick-tableview-obsolete.html";
+        return modelIndex({column, row});
+    }
 }
+#endif
 
 void QQuickTreeView::keyPressEvent(QKeyEvent *event)
 {
@@ -616,8 +700,10 @@ void QQuickTreeView::keyPressEvent(QKeyEvent *event)
         break;
     }
 
-    if (!event->isAccepted())
+    if (!event->isAccepted()) {
+        event->accept();
         QQuickTableView::keyPressEvent(event);
+    }
 }
 
 QT_END_NAMESPACE
