@@ -35,7 +35,7 @@ QT_BEGIN_NAMESPACE
 
 /*!
     \qmltype Animation
-    \instantiates QQuickAbstractAnimation
+    \nativetype QQuickAbstractAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \brief Is the base of all QML animations.
@@ -130,7 +130,8 @@ void QQuickAbstractAnimationPrivate::commence()
     QQmlProperties properties;
 
     auto *newInstance = q->transition(actions, properties, QQuickAbstractAnimation::Forward);
-    Q_ASSERT(newInstance != animationInstance);
+    // transition can return a nullptr; that's the only allowed case were old and new have the same value
+    Q_ASSERT(newInstance != animationInstance || !newInstance);
     delete animationInstance;
     animationInstance = newInstance;
 
@@ -165,6 +166,27 @@ QQmlProperty QQuickAbstractAnimationPrivate::createProperty(QObject *obj, const
     return prop;
 }
 
+void QQuickAbstractAnimationPrivate::animationGroupDirty()
+{
+    Q_ASSERT(group != nullptr);
+    if (!componentComplete)
+        return;
+
+    auto *animGroupPriv = static_cast<QQuickAnimationGroupPrivate *>(QQuickAnimationGroupPrivate::get(group));
+    if (animGroupPriv->running && !animGroupPriv->animationDirty) {
+        animGroupPriv->animationDirty = true;
+
+        if (animGroupPriv->animationInstance && group->currentTime() == 0) {
+            // restart if the animation didn't proceed yet.
+            animGroupPriv->restartFromCurrentLoop();
+        }
+    }
+
+    // check the animationGroup is one of another animationGroup members
+    if (animGroupPriv->group)
+        animGroupPriv->animationGroupDirty();
+}
+
 /*!
     \qmlsignal QtQuick::Animation::started()
 
@@ -269,6 +291,11 @@ void QQuickAbstractAnimation::setRunning(bool r)
         // Therefore, the state of d->running will in that case be different than r if we are back in
         // the root stack frame of the recursive calls to setRunning()
         emit runningChanged(d->running);
+    } else if (d->animationInstance) {
+        // If there was a recursive call, make sure the d->running is set correctly
+        d->running = d->animationInstance->isRunning();
+    } else {
+        d->running = r;
     }
 }
 
@@ -643,7 +670,7 @@ QQuickAbstractAnimation::ThreadingModel QQuickAbstractAnimation::threadingModel(
 
 /*!
     \qmltype PauseAnimation
-    \instantiates QQuickPauseAnimation
+    \nativetype QQuickPauseAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \inherits Animation
@@ -696,6 +723,8 @@ void QQuickPauseAnimation::setDuration(int duration)
         return;
     d->duration = duration;
     emit durationChanged(duration);
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 QAbstractAnimationJob* QQuickPauseAnimation::transition(QQuickStateActions &actions,
@@ -714,7 +743,7 @@ QAbstractAnimationJob* QQuickPauseAnimation::transition(QQuickStateActions &acti
 
 /*!
     \qmltype ColorAnimation
-    \instantiates QQuickColorAnimation
+    \nativetype QQuickColorAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-properties
     \inherits PropertyAnimation
@@ -872,7 +901,7 @@ void QActionAnimation::debugAnimation(QDebug d) const
 
 /*!
     \qmltype ScriptAction
-    \instantiates QQuickScriptAction
+    \nativetype QQuickScriptAction
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \inherits Animation
@@ -1015,7 +1044,7 @@ QAbstractAnimationJob* QQuickScriptAction::transition(QQuickStateActions &action
 
 /*!
     \qmltype PropertyAction
-    \instantiates QQuickPropertyAction
+    \nativetype QQuickPropertyAction
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \inherits Animation
@@ -1052,7 +1081,7 @@ QAbstractAnimationJob* QQuickScriptAction::transition(QQuickStateActions &action
     PropertyAction object) so that the rotation animation begins with the
     correct transform origin.
 
-    \sa {Animation and Transitions in Qt Quick}, {Qt QML}
+    \sa {Animation and Transitions in Qt Quick}, {Qt Qml}
 */
 QQuickPropertyAction::QQuickPropertyAction(QObject *parent)
 : QQuickAbstractAnimation(*(new QQuickPropertyActionPrivate), parent)
@@ -1076,6 +1105,8 @@ void QQuickPropertyAction::setTargetObject(QObject *o)
         return;
     d->target = o;
     emit targetChanged();
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 QString QQuickPropertyAction::property() const
@@ -1091,6 +1122,8 @@ void QQuickPropertyAction::setProperty(const QString &n)
         return;
     d->propertyName = n;
     emit propertyChanged();
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 /*!
@@ -1121,6 +1154,8 @@ void QQuickPropertyAction::setProperties(const QString &p)
         return;
     d->properties = p;
     emit propertiesChanged(p);
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 QQmlListProperty<QObject> QQuickPropertyAction::targets()
@@ -1159,7 +1194,7 @@ QVariant QQuickPropertyAction::value() const
 void QQuickPropertyAction::setValue(const QVariant &v)
 {
     Q_D(QQuickPropertyAction);
-    if (d->value.isNull || d->value != v) {
+    if (!d->value.isValid() || d->value != v) {
         d->value = v;
         emit valueChanged(v);
     }
@@ -1276,7 +1311,7 @@ QAbstractAnimationJob* QQuickPropertyAction::transition(QQuickStateActions &acti
 
 /*!
     \qmltype NumberAnimation
-    \instantiates QQuickNumberAnimation
+    \nativetype QQuickNumberAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-properties
     \inherits PropertyAnimation
@@ -1389,7 +1424,7 @@ void QQuickNumberAnimation::setTo(qreal t)
 
 /*!
     \qmltype Vector3dAnimation
-    \instantiates QQuickVector3dAnimation
+    \nativetype QQuickVector3dAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-properties
     \inherits PropertyAnimation
@@ -1466,7 +1501,7 @@ void QQuickVector3dAnimation::setTo(QVector3D t)
 
 /*!
     \qmltype RotationAnimation
-    \instantiates QQuickRotationAnimation
+    \nativetype QQuickRotationAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-properties
     \inherits PropertyAnimation
@@ -1620,14 +1655,16 @@ void QQuickRotationAnimation::setTo(qreal t)
 
     Possible values are:
 
-    \list
-    \li RotationAnimation.Numerical (default) - Rotate by linearly interpolating between the two numbers.
-           A rotation from 10 to 350 will rotate 340 degrees clockwise.
-    \li RotationAnimation.Clockwise - Rotate clockwise between the two values
-    \li RotationAnimation.Counterclockwise - Rotate counterclockwise between the two values
-    \li RotationAnimation.Shortest - Rotate in the direction that produces the shortest animation path.
-           A rotation from 10 to 350 will rotate 20 degrees counterclockwise.
-    \endlist
+    \value RotationAnimation.Numerical
+        (default) Rotate by linearly interpolating between the two numbers.
+        A rotation from \c 10 to \c 350 will rotate 340 degrees clockwise.
+    \value RotationAnimation.Clockwise
+        Rotate clockwise between the two values
+    \value RotationAnimation.Counterclockwise
+        Rotate counterclockwise between the two values
+    \value RotationAnimation.Shortest
+        Rotate in the direction that produces the shortest animation path.
+        A rotation from \c 10 to \c 350 will rotate \c 20 degrees counterclockwise.
 */
 QQuickRotationAnimation::RotationDirection QQuickRotationAnimation::direction() const
 {
@@ -1720,6 +1757,36 @@ void QQuickAnimationGroupPrivate::removeLast_animation(QQmlListProperty<QQuickAb
         q->d_func()->animations.last()->setGroup(nullptr);
 }
 
+void QQuickAnimationGroupPrivate::restartFromCurrentLoop()
+{
+    Q_Q(QQuickAnimationGroup);
+    if (!animationDirty)
+        return;
+
+    animationDirty = false;
+
+    Q_ASSERT(animationInstance);
+    const int currentLoop = animationInstance->currentLoop();
+
+    QSignalBlocker signalBlocker(q);
+    q->stop();
+    q->start();
+
+    Q_ASSERT(animationInstance);
+    // Restarting adjusts animationInstance's loopCount
+    // Since we just want to start it from this loop,
+    // it will be restored again.
+    if (loopCount != -1)
+        animationInstance->setLoopCount(loopCount - currentLoop);
+}
+
+void QQuickAnimationGroupPrivate::animationCurrentLoopChanged(QAbstractAnimationJob *)
+{
+    if (!animationDirty)
+        return;
+    restartFromCurrentLoop();
+}
+
 QQuickAnimationGroup::~QQuickAnimationGroup()
 {
     Q_D(QQuickAnimationGroup);
@@ -1743,7 +1810,7 @@ QQmlListProperty<QQuickAbstractAnimation> QQuickAnimationGroup::animations()
 
 /*!
     \qmltype SequentialAnimation
-    \instantiates QQuickSequentialAnimation
+    \nativetype QQuickSequentialAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \inherits Animation
@@ -1837,7 +1904,7 @@ QAbstractAnimationJob* QQuickSequentialAnimation::transition(QQuickStateActions
 
 /*!
     \qmltype ParallelAnimation
-    \instantiates QQuickParallelAnimation
+    \nativetype QQuickParallelAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \inherits Animation
@@ -2026,7 +2093,7 @@ void QQuickBulkValueAnimator::debugAnimation(QDebug d) const
 
 /*!
     \qmltype PropertyAnimation
-    \instantiates QQuickPropertyAnimation
+    \nativetype QQuickPropertyAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-animation-properties
     \inherits Animation
@@ -2084,7 +2151,7 @@ void QQuickBulkValueAnimator::debugAnimation(QDebug d) const
     Note that PropertyAnimation inherits the abstract \l Animation type.
     This includes additional properties and methods for controlling the animation.
 
-    \section1 Modifying Properties Duration Animations
+    \section1 Modifying running animations
 
     Since Qt 6.4, it is possible to set the \l from, \l to, \l duration, and
     \l easing properties on a top-level animation while it is running. The
@@ -2133,6 +2200,8 @@ void QQuickPropertyAnimation::setDuration(int duration)
     if (d->componentComplete && d->running)
         d->ourPropertiesDirty = true;
     emit durationChanged(duration);
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 /*!
@@ -2162,6 +2231,8 @@ void QQuickPropertyAnimation::setFrom(const QVariant &f)
     if (d->componentComplete && d->running)
         d->ourPropertiesDirty = true;
     emit fromChanged();
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 /*!
@@ -2191,6 +2262,8 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
     if (d->componentComplete && d->running)
         d->ourPropertiesDirty = true;
     emit toChanged();
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 /*!
@@ -2423,6 +2496,8 @@ void QQuickPropertyAnimation::setEasing(const QEasingCurve &e)
     if (d->componentComplete && d->running)
         d->ourPropertiesDirty = true;
     emit easingChanged(e);
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 QObject *QQuickPropertyAnimation::target() const
@@ -2438,6 +2513,8 @@ void QQuickPropertyAnimation::setTargetObject(QObject *o)
         return;
     d->target = o;
     emit targetChanged();
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 QString QQuickPropertyAnimation::property() const
@@ -2453,6 +2530,8 @@ void QQuickPropertyAnimation::setProperty(const QString &n)
         return;
     d->propertyName = n;
     emit propertyChanged();
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 QString QQuickPropertyAnimation::properties() const
@@ -2469,6 +2548,8 @@ void QQuickPropertyAnimation::setProperties(const QString &prop)
 
     d->properties = prop;
     emit propertiesChanged(prop);
+    if (d->group)
+        d->animationGroupDirty();
 }
 
 /*!
@@ -2574,7 +2655,38 @@ void QQuickPropertyAnimation::setProperties(const QString &prop)
 QQmlListProperty<QObject> QQuickPropertyAnimation::targets()
 {
     Q_D(QQuickPropertyAnimation);
-    return QQmlListProperty<QObject>(this, &(d->targets));
+    using ListPtr = QList<QPointer<QObject>> *;
+    using LP = QQmlListProperty<QObject>;
+    LP::AppendFunction appendFn = [](LP *prop, QObject *value)
+    {
+        static_cast<ListPtr>(prop->data)->append(value);
+    };
+    LP::CountFunction countFn = [](LP *prop)
+    {
+        return static_cast<ListPtr>(prop->data)->size();
+    };
+
+    LP::AtFunction atFn = [](LP *prop, qsizetype index) -> QObject *
+    {
+        return static_cast<ListPtr>(prop->data)->at(index);
+    };
+
+    LP::ClearFunction clearFN = [](LP *prop)
+    {
+        return static_cast<ListPtr>(prop->data)->clear();
+    };
+
+    LP::ReplaceFunction replaceFn = [](LP *prop, qsizetype index, QObject *value)
+    {
+        static_cast<ListPtr>(prop->data)->replace(index, value);
+    };
+
+    LP::RemoveLastFunction removeLastFn = [](LP *prop)
+    {
+        static_cast<ListPtr>(prop->data)->removeLast();
+    };
+
+    return QQmlListProperty<QObject>(this, &(d->targets), appendFn, countFn, atFn, clearFN, replaceFn, removeLastFn);
 }
 
 /*!
@@ -2646,7 +2758,7 @@ QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateA
     if (!d->propertyName.isEmpty())
         props << d->propertyName;
 
-    QList<QObject*> targets = d->targets;
+    QList<QPointer<QObject>> targets = d->targets;
     if (d->target)
         targets.append(d->target);
 
@@ -2675,10 +2787,14 @@ QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateA
 
         for (int i = 0; i < props.size(); ++i) {
             for (int j = 0; j < targets.size(); ++j) {
+                const auto& guarded = targets.at(j);
+                if (guarded.isNull())
+                    continue;
+                QObject *target = guarded.get();
                 QQuickStateAction myAction;
                 QString errorMessage;
                 const QString &propertyName = props.at(i);
-                myAction.property = d->createProperty(targets.at(j), propertyName, this, &errorMessage);
+                myAction.property = d->createProperty(target, propertyName, this, &errorMessage);
                 if (myAction.property.isValid()) {
                     if (usingDefaultProperties)
                         successfullyCreatedDefaultProperty = true;
@@ -2711,36 +2827,37 @@ QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateA
         }
     }
 
-    if (!hasExplicit)
-    for (int ii = 0; ii < actions.size(); ++ii) {
-        QQuickStateAction &action = actions[ii];
+    if (!hasExplicit) {
+        for (int ii = 0; ii < actions.size(); ++ii) {
+            QQuickStateAction &action = actions[ii];
 
-        QObject *obj = action.property.object();
-        QString propertyName = action.property.name();
-        QObject *sObj = action.specifiedObject;
-        QString sPropertyName = action.specifiedProperty;
-        bool same = (obj == sObj);
+            QObject *obj = action.property.object();
+            QString propertyName = action.property.name();
+            QObject *sObj = action.specifiedObject;
+            QString sPropertyName = action.specifiedProperty;
+            bool same = (obj == sObj);
 
-        if ((targets.isEmpty() || targets.contains(obj) || (!same && targets.contains(sObj))) &&
-           (!d->exclude.contains(obj)) && (same || (!d->exclude.contains(sObj))) &&
-           (props.contains(propertyName) || (!same && props.contains(sPropertyName))
-               || (useType && action.property.propertyType() == d->interpolatorType))) {
-            QQuickStateAction myAction = action;
+            if ((targets.isEmpty() || targets.contains(obj) || (!same && targets.contains(sObj))) &&
+               (!d->exclude.contains(obj)) && (same || (!d->exclude.contains(sObj))) &&
+               (props.contains(propertyName) || (!same && props.contains(sPropertyName))
+                   || (useType && action.property.propertyType() == d->interpolatorType))) {
+                QQuickStateAction myAction = action;
 
-            if (d->fromIsDefined)
-                myAction.fromValue = d->from;
-            else
-                myAction.fromValue = QVariant();
-            if (d->toIsDefined)
-                myAction.toValue = d->to;
+                if (d->fromIsDefined)
+                    myAction.fromValue = d->from;
+                else
+                    myAction.fromValue = QVariant();
+                if (d->toIsDefined)
+                    myAction.toValue = d->to;
 
-            d->convertVariant(myAction.fromValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
-            d->convertVariant(myAction.toValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
+                d->convertVariant(myAction.fromValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
+                d->convertVariant(myAction.toValue, d->interpolatorType ? QMetaType(d->interpolatorType) : myAction.property.propertyMetaType());
 
-            modified << action.property;
+                modified << action.property;
 
-            newActions << myAction;
-            action.fromValue = myAction.toValue;
+                newActions << myAction;
+                action.fromValue = myAction.toValue;
+            }
         }
     }
     return newActions;
