@@ -7,7 +7,9 @@
 #include "qquickanimator_p.h"
 #include "qquickanimator_p_p.h"
 #include <private/qquickitem_p.h>
+#if QT_CONFIG(quick_shadereffect)
 #include <private/qquickshadereffect_p.h>
+#endif
 #include <private/qanimationgroupjob_p.h>
 
 #include <qcoreapplication.h>
@@ -358,11 +360,12 @@ void QQuickTransformAnimatorJob::Helper::sync()
         wasSynced = true;
     }
 
+    // We update the node before checking on dirty, as the node might have changed without the animator running
+    node = d->itemNode();
+
     if (dirty == 0)
         return;
 
-    node = d->itemNode();
-
     if (dirty & QQuickItemPrivate::Position) {
         dx = item->x();
         dy = item->y();
@@ -574,6 +577,7 @@ void QQuickOpacityAnimatorJob::updateCurrentTime(int time)
     m_opacityNode->setOpacity(m_value);
 }
 
+#if QT_CONFIG(quick_shadereffect)
 QQuickUniformAnimatorJob::QQuickUniformAnimatorJob()
 {
     m_isUniform = true;
@@ -588,7 +592,7 @@ void QQuickUniformAnimatorJob::setTarget(QQuickItem *target)
 
 void QQuickUniformAnimatorJob::updateCurrentTime(int time)
 {
-    if (!m_effect)
+    if (!m_effect || m_target != m_effect)
         return;
 
     m_value = m_from + (m_to - m_from) * progress(time);
@@ -613,8 +617,9 @@ void QQuickUniformAnimatorJob::postSync()
 
 void QQuickUniformAnimatorJob::invalidate()
 {
-    m_effect = nullptr;
+
 }
+#endif
 
 QT_END_NAMESPACE
 
