@@ -4,7 +4,7 @@
 #include "qquickimageprovider.h"
 
 #include "qquickimageprovider_p.h"
-#include "qquickpixmapcache_p.h"
+#include "qquickpixmap_p.h"
 #include <QtQuick/private/qsgcontext_p.h>
 #include <private/qqmlglobal_p.h>
 #include <QtGui/qcolorspace.h>
@@ -259,6 +259,8 @@ void QQuickImageResponse::cancel()
     Note that the example registers the provider via a \l{QQmlEngineExtensionPlugin}{plugin}
     instead of registering it in the application \c main() function as shown above.
 
+    It is possible to provide \l {High Resolution Versions of Images}{"@nx" high DPI syntax}.
+
 
     \section2 Asynchronous Image Loading
 
@@ -476,6 +478,7 @@ public:
     }
 
     QColorSpace targetColorSpace;
+    QRectF sourceClipRect;
     QQuickImageProviderOptions::AutoTransform autoTransform = QQuickImageProviderOptions::UsePluginDefaultTransform;
     bool preserveAspectRatioCrop = false;
     bool preserveAspectRatioFit = false;
@@ -582,6 +585,19 @@ void QQuickImageProviderOptions::setTargetColorSpace(const QColorSpace &colorSpa
     d->targetColorSpace = colorSpace;
 }
 
+/*!
+    Returns the requested source clip rect.
+*/
+QRectF QQuickImageProviderOptions::sourceClipRect() const
+{
+    return d->sourceClipRect;
+}
+
+void QQuickImageProviderOptions::setSourceClipRect(const QRectF &rect)
+{
+    d->sourceClipRect = rect;
+}
+
 QQuickImageProviderWithOptions::QQuickImageProviderWithOptions(ImageType type, Flags flags)
  : QQuickAsyncImageProvider()
 {
