@@ -12,7 +12,7 @@
 
 /*!
     \qmltype Layout
-    //! \instantiates QQuickLayoutAttached
+    //! \nativetype QQuickLayoutAttached
     \inqmlmodule QtQuick.Layouts
     \ingroup layouts
     \brief Provides attached properties for items pushed onto a \l GridLayout,
@@ -80,15 +80,21 @@ QQuickLayoutAttached::QQuickLayoutAttached(QObject *parent)
       m_fillHeight(false),
       m_isFillWidthSet(false),
       m_isFillHeightSet(false),
+      m_isUseDefaultSizePolicySet(false),
+      m_useDefaultSizePolicy(QQuickLayout::SizePolicyExplicit),
       m_isMinimumWidthSet(false),
       m_isMinimumHeightSet(false),
       m_isMaximumWidthSet(false),
       m_isMaximumHeightSet(false),
       m_changesNotificationEnabled(true),
+      m_isMarginsSet(false),
       m_isLeftMarginSet(false),
       m_isTopMarginSet(false),
       m_isRightMarginSet(false),
-      m_isBottomMarginSet(false)
+      m_isBottomMarginSet(false),
+      m_isAlignmentSet(false),
+      m_horizontalStretch(-1),
+      m_verticalStretch(-1)
 {
 
 }
@@ -299,15 +305,16 @@ void QQuickLayoutAttached::setMaximumImplicitSize(const QSizeF &sz)
     If this property is \c true, the item will be as wide as possible while respecting
     the given constraints. If the property is \c false, the item will have a fixed width
     set to the preferred width.
-    The default is \c false, except for layouts themselves, which default to \c true.
+    The default depends on implicit (built-in) size policy of item.
 
     \sa fillHeight
 */
 void QQuickLayoutAttached::setFillWidth(bool fill)
 {
+    bool oldFillWidth = fillWidth();
     m_isFillWidthSet = true;
-    if (m_fillWidth != fill) {
-        m_fillWidth = fill;
+    m_fillWidth = fill;
+    if (oldFillWidth != fill) {
         invalidateItem();
         emit fillWidthChanged();
     }
@@ -319,20 +326,45 @@ void QQuickLayoutAttached::setFillWidth(bool fill)
     If this property is \c true, the item will be as tall as possible while respecting
     the given constraints. If the property is \c false, the item will have a fixed height
     set to the preferred height.
-    The default is \c false, except for layouts themselves, which default to \c true.
+    The default depends on implicit (built-in) size policy of the item.
 
     \sa fillWidth
 */
 void QQuickLayoutAttached::setFillHeight(bool fill)
 {
+    bool oldFillHeight = fillHeight();
     m_isFillHeightSet = true;
-    if (m_fillHeight != fill) {
-        m_fillHeight = fill;
+    m_fillHeight = fill;
+    if (oldFillHeight != fill) {
         invalidateItem();
         emit fillHeightChanged();
     }
 }
 
+/*!
+    \qmlattachedproperty enumeration Layout::useDefaultSizePolicy
+    \since 6.8
+
+    This property allows the user to configure the layout size policy at the component
+    level.
+
+    The default value will be inherited by querying the application attribute
+    \l Qt::AA_QtQuickUseDefaultSizePolicy. You can use this property to override that value.
+
+    \value Layout.SizePolicyImplicit
+        The item in the layout uses implicit or built-in size policy
+    \value Layout.SizePolicyExplicit
+        The item in the layout doesn't use implicit size policies.
+*/
+void QQuickLayoutAttached::setUseDefaultSizePolicy(QQuickLayout::SizePolicy sizePolicy)
+{
+    m_isUseDefaultSizePolicySet = true;
+    if (m_useDefaultSizePolicy != sizePolicy) {
+        m_useDefaultSizePolicy = sizePolicy;
+        emit useDefaultSizePolicyChanged();
+    }
+}
+
 /*!
     \qmlattachedproperty int Layout::row
 
@@ -401,6 +433,7 @@ void QQuickLayoutAttached::setColumn(int column)
 */
 void QQuickLayoutAttached::setAlignment(Qt::Alignment align)
 {
+    m_isAlignmentSet = true;
     if (align != m_alignment) {
         m_alignment = align;
         if (QQuickLayout *layout = parentLayout()) {
@@ -411,6 +444,80 @@ void QQuickLayoutAttached::setAlignment(Qt::Alignment align)
     }
 }
 
+/*!
+    \qmlattachedproperty int Layout::horizontalStretchFactor
+
+    This property allows you to specify the horizontal stretch factor. By default, two identical
+    items arranged in a linear layout will have the same size, but if the first item has a
+    stretch factor of 1 and the second item has a stretch factor of 2, the first item will \e
+    aim to get 1/3 of the available space, and the second will \e aim to get 2/3 of the available
+    space. Note that, whether they become exactly 1/3 and 2/3 of the available space depends on
+    their size hints. This is because when e.g a horizontal layout is shown in its minimum width
+    all its child items will consequently also have their minimum width.
+
+    Likewise, when a horizontal layout has its preferred width, all child items will have their
+    preferred widths, and when a horizontal layout has its maximum width, all child items will have
+    their maximum widths. This strategy is applied regardless of what the individual stretch
+    factors are. As a consequence of this, stretch factors will only determine the growth rate of
+    child items \e between the preferredWidth and maximumWidth range.
+
+    The default value is \c -1, which means that no stretch factor is applied.
+
+    \note This requires that Layout::fillWidth is set to true
+
+    \since Qt 6.5
+
+    \sa verticalStretchFactor
+*/
+void QQuickLayoutAttached::setHorizontalStretchFactor(int factor)
+{
+    if (factor != m_horizontalStretch) {
+        m_horizontalStretch = factor;
+        if (QQuickLayout *layout = parentLayout()) {
+            layout->setStretchFactor(item(), factor, Qt::Horizontal);
+            invalidateItem();
+        }
+        emit horizontalStretchFactorChanged();
+    }
+}
+
+/*!
+    \qmlattachedproperty int Layout::verticalStretchFactor
+
+    This property allows you to specify the vertical stretch factor. By default, two identical
+    items arranged in a linear layout will have the same size, but if the first item has a
+    stretch factor of 1 and the second item has a stretch factor of 2, the first item will \e
+    aim to get 1/3 of the available space, and the second will \e aim to get 2/3 of the available
+    space. Note that, whether they become exactly 1/3 and 2/3 of the available space depends on
+    their size hints. This is because when e.g a vertical layout is shown in its minimum height
+    all its child items will consequently also have their minimum height.
+
+    Likewise, when a vertical layout has its preferred height, all child items will have their
+    preferred heights, and when a vertical layout has its maximum height, all child items will have
+    their maximum heights. This strategy is applied regardless of what the individual stretch
+    factors are. As a consequence of this, stretch factors will only determine the growth rate of
+    child items \e between the preferredHeight and maximumHeight range.
+
+    The default value is \c -1, which means that no stretch factor is applied.
+
+    \note This requires that Layout::fillHeight is set to true
+
+    \since Qt 6.5
+
+    \sa horizontalStretchFactor
+*/
+void QQuickLayoutAttached::setVerticalStretchFactor(int factor)
+{
+    if (factor != m_verticalStretch) {
+        m_verticalStretch = factor;
+        if (QQuickLayout *layout = parentLayout()) {
+            layout->setStretchFactor(item(), factor, Qt::Vertical);
+            invalidateItem();
+        }
+        emit verticalStretchFactorChanged();
+    }
+}
+
 /*!
     \qmlattachedproperty real Layout::margins
 
@@ -442,6 +549,7 @@ void QQuickLayoutAttached::setAlignment(Qt::Alignment align)
 */
 void QQuickLayoutAttached::setMargins(qreal m)
 {
+    m_isMarginsSet = true;
     if (m == m_defaultMargins)
         return;
 
@@ -651,7 +759,7 @@ QQuickLayout *QQuickLayoutAttached::parentLayout() const
         parentItem = parentItem->parentItem();
         return qobject_cast<QQuickLayout *>(parentItem);
     } else {
-        qmlWarning(parent()) << "Layout must be attached to Item elements";
+        qmlWarning(parent()) << "Layout attached property must be attached to an object deriving from Item";
     }
     return nullptr;
 }
@@ -753,19 +861,16 @@ void QQuickLayout::invalidate(QQuickItem * /*childItem*/)
     d->m_dirtyArrangement = true;
 
     if (!qobject_cast<QQuickLayout *>(parentItem())) {
+        polish();
 
-        if (m_inUpdatePolish)
-            ++m_polishInsideUpdatePolish;
-        else
-            m_polishInsideUpdatePolish = 0;
-
-        if (m_polishInsideUpdatePolish <= 2) {
-            // allow at most two consecutive loops in order to respond to height-for-width
-            // (e.g QQuickText changes implicitHeight when its width gets changed)
-            qCDebug(lcQuickLayouts) << "QQuickLayout::invalidate(), polish()";
-            polish();
+        if (m_inUpdatePolish) {
+            if (++m_polishInsideUpdatePolish > 2)
+                // allow at most two consecutive loops in order to respond to height-for-width
+                // (e.g QQuickText changes implicitHeight when its width gets changed)
+                qCDebug(lcQuickLayouts) << "Layout polish loop detected for " << this
+                            << ". The polish request will still be scheduled.";
         } else {
-            qmlWarning(this) << "Qt Quick Layouts: Polish loop detected. Aborting after two iterations.";
+            m_polishInsideUpdatePolish = 0;
         }
     }
 }
@@ -844,7 +949,8 @@ void QQuickLayout::geometryChange(const QRectF &newGeometry, const QRectF &oldGe
 {
     Q_D(QQuickLayout);
     QQuickItem::geometryChange(newGeometry, oldGeometry);
-    if (d->m_disableRearrange || !isReady() || !newGeometry.isValid())
+    if ((invalidated() && !qobject_cast<QQuickLayout *>(parentItem())) ||
+        d->m_disableRearrange || !isReady())
         return;
 
     qCDebug(lcQuickLayouts) << "QQuickLayout::geometryChange" << newGeometry << oldGeometry;
@@ -1174,21 +1280,33 @@ void QQuickLayout::effectiveSizeHints_helper(QQuickItem *item, QSizeF *cachedSiz
  */
 QLayoutPolicy::Policy QQuickLayout::effectiveSizePolicy_helper(QQuickItem *item, Qt::Orientation orientation, QQuickLayoutAttached *info)
 {
-    bool fillExtent = false;
+    QLayoutPolicy::Policy pol{QLayoutPolicy::Fixed};
     bool isSet = false;
     if (info) {
         if (orientation == Qt::Horizontal) {
             isSet = info->isFillWidthSet();
-            if (isSet) fillExtent = info->fillWidth();
+            if (isSet && info->fillWidth())
+                pol = QLayoutPolicy::Preferred;
         } else {
             isSet = info->isFillHeightSet();
-            if (isSet) fillExtent = info->fillHeight();
+            if (isSet && info->fillHeight())
+                pol = QLayoutPolicy::Preferred;
+        }
+    }
+    if (!isSet && item) {
+        auto effectiveUseDefaultSizePolicy = [info]() {
+            return info ? info->useDefaultSizePolicy() == QQuickLayout::SizePolicyImplicit
+                        : QGuiApplication::testAttribute(Qt::AA_QtQuickUseDefaultSizePolicy);
+        };
+        if (qobject_cast<QQuickLayout*>(item)) {
+            pol = QLayoutPolicy::Preferred;
+        } else if (effectiveUseDefaultSizePolicy()) {
+            QLayoutPolicy sizePolicy = QQuickItemPrivate::get(item)->sizePolicy();
+            pol = (orientation == Qt::Horizontal) ? sizePolicy.horizontalPolicy() : sizePolicy.verticalPolicy();
         }
     }
-    if (!isSet && qobject_cast<QQuickLayout*>(item))
-        fillExtent = true;
-    return fillExtent ? QLayoutPolicy::Preferred : QLayoutPolicy::Fixed;
 
+    return pol;
 }
 
 void QQuickLayout::_q_dumpLayoutTree() const
