@@ -11,7 +11,7 @@
 
 /*!
     \qmltype RowLayout
-    //! \instantiates QQuickRowLayout
+    //! \nativetype QQuickRowLayout
     \inherits Item
     \inqmlmodule QtQuick.Layouts
     \ingroup layouts
@@ -71,7 +71,7 @@
 
 /*!
     \qmltype ColumnLayout
-    //! \instantiates QQuickColumnLayout
+    //! \nativetype QQuickColumnLayout
     \inherits Item
     \inqmlmodule QtQuick.Layouts
     \ingroup layouts
@@ -130,7 +130,7 @@
 
 /*!
     \qmltype GridLayout
-    //! \instantiates QQuickGridLayout
+    //! \nativetype QQuickGridLayout
     \inherits Item
     \inqmlmodule QtQuick.Layouts
     \ingroup layouts
@@ -247,10 +247,8 @@ QSizeF QQuickGridLayoutBase::sizeHint(Qt::SizeHint whichSizeHint) const
 
     Possible values:
 
-    \list
-    \li Qt.LeftToRight (default) - Items are laid out from left to right.
-    \li Qt.RightToLeft - Items are laid out from right to left.
-    \endlist
+    \value Qt.LeftToRight   (default) Items are laid out from left to right.
+    \value Qt.RightToLeft   Items are laid out from right to left.
 
     \sa RowLayout::layoutDirection, ColumnLayout::layoutDirection
 */
@@ -284,6 +282,12 @@ void QQuickGridLayoutBase::setAlignment(QQuickItem *item, Qt::Alignment alignmen
     maybeSubscribeToBaseLineOffsetChanges(item);
 }
 
+void QQuickGridLayoutBase::setStretchFactor(QQuickItem *item, int stretchFactor, Qt::Orientation orient)
+{
+    Q_D(QQuickGridLayoutBase);
+    d->engine.setStretchFactor(item, stretchFactor, orient);
+}
+
 QQuickGridLayoutBase::~QQuickGridLayoutBase()
 {
     Q_D(QQuickGridLayoutBase);
@@ -357,26 +361,24 @@ void QQuickGridLayoutBase::invalidate(QQuickItem *childItem)
     if (!isReady())
         return;
     qCDebug(lcQuickLayouts) << "QQuickGridLayoutBase::invalidate()" << this << ", invalidated:" << invalidated();
-    if (invalidated()) {
-        return;
-    }
-    qCDebug(lcQuickLayouts) << "d->m_rearranging:" << d->m_rearranging;
-    if (d->m_rearranging) {
-        d->m_invalidateAfterRearrange << childItem;
-        return;
+    if (childItem) {
+        if (d->m_rearranging) {
+            if (!d->m_invalidateAfterRearrange.contains(childItem))
+                d->m_invalidateAfterRearrange << childItem;
+            return;
+        }
+        if (QQuickGridLayoutItem *layoutItem = d->engine.findLayoutItem(childItem)) {
+            layoutItem->invalidate();
+        }
     }
 
-    if (childItem) {
-        if (QQuickGridLayoutItem *layoutItem = d->engine.findLayoutItem(childItem))
-            layoutItem->invalidate();
-    }
     // invalidate engine
     d->engine.invalidate();
 
     qCDebug(lcQuickLayouts) << "calling QQuickLayout::invalidate();";
     QQuickLayout::invalidate();
 
-    if (QQuickLayout *parentLayout = qobject_cast<QQuickLayout *>(parentItem()))
+    if (auto *parentLayout = qobject_cast<QQuickLayout *>(parentItem()))
         parentLayout->invalidate(this);
     qCDebug(lcQuickLayouts) << "QQuickGridLayoutBase::invalidate() LEAVING" << this;
 }
@@ -439,7 +441,7 @@ void QQuickGridLayoutBase::itemVisibilityChanged(QQuickItem *item)
 void QQuickGridLayoutBase::rearrange(const QSizeF &size)
 {
     Q_D(QQuickGridLayoutBase);
-    if (!isReady())
+    if (!isReady() || !size.isValid())
         return;
 
     qCDebug(lcQuickLayouts) << "QQuickGridLayoutBase::rearrange" << d->m_recurRearrangeCounter << this;
@@ -449,10 +451,15 @@ void QQuickGridLayoutBase::rearrange(const QSizeF &size)
     if (d->m_recurRearrangeCounter++ == 2) {
         // allow a recursive depth of two in order to respond to height-for-width
         // (e.g QQuickText changes implicitHeight when its width gets changed)
-        qWarning() << "Qt Quick Layouts: Detected recursive rearrange. Aborting after two iterations.";
+        qmlWarning(this)
+                << "Qt Quick Layouts: Detected recursive rearrange. Aborting after two iterations.";
         return;
     }
 
+    // Should normally not be needed, but there might be an incoming window resize event that we
+    // will process before we process updatePolish()
+    ensureLayoutItemsUpdated(QQuickLayout::ApplySizeHints | QQuickLayout::Recursive);
+
     d->m_rearranging = true;
     qCDebug(lcQuickLayouts) << objectName() << "QQuickGridLayoutBase::rearrange()" << size;
     Qt::LayoutDirection visualDir = effectiveLayoutDirection();
@@ -471,8 +478,8 @@ void QQuickGridLayoutBase::rearrange(const QSizeF &size)
     d->engine.setGeometries(QRectF(QPointF(0,0), size), d->styleInfo);
     d->m_rearranging = false;
 
-    for (QQuickItem *invalid : std::as_const(d->m_invalidateAfterRearrange))
-        invalidate(invalid);
+    for (auto childItem : std::as_const(d->m_invalidateAfterRearrange))
+        invalidate(childItem);
     d->m_invalidateAfterRearrange.clear();
 }
 
@@ -589,12 +596,10 @@ void QQuickGridLayout::setRows(int rows)
 
     Possible values are:
 
-    \list
-    \li GridLayout.LeftToRight (default) - Items are positioned next to
-       each other, then wrapped to the next line.
-    \li GridLayout.TopToBottom - Items are positioned next to each
-       other from top to bottom, then wrapped to the next column.
-    \endlist
+    \value GridLayout.LeftToRight
+        (default) Items are positioned next to each other, then wrapped to the next line.
+    \value GridLayout.TopToBottom
+        Items are positioned next to each other from top to bottom, then wrapped to the next column.
 
     \sa rows
     \sa columns
@@ -616,6 +621,69 @@ void QQuickGridLayout::setFlow(QQuickGridLayout::Flow flow)
     emit flowChanged();
 }
 
+/*!
+    \qmlproperty bool GridLayout::uniformCellWidths
+    \since QtQuick.Layouts 6.6
+
+    If this property is set to \c true, the layout will force all cells to have
+    a uniform width. The layout aims to respect
+    \l{Layout::minimumWidth}{Layout.minimumWidth},
+    \l{Layout::preferredWidth}{Layout.preferredWidth} and
+    \l{Layout::maximumWidth}{Layout.maximumWidth} in this mode but might make
+    compromisses to fullfill the requirements of all items.
+
+    Default value is \c false.
+
+    \sa GridLayout::uniformCellHeights, RowLayout::uniformCellSizes, ColumnLayout::uniformCellSizes
+*/
+bool QQuickGridLayout::uniformCellWidths() const
+{
+    Q_D(const QQuickGridLayout);
+    return d->engine.uniformCellWidths();
+}
+
+void QQuickGridLayout::setUniformCellWidths(bool uniformCellWidths)
+{
+    Q_D(QQuickGridLayout);
+    if (d->engine.uniformCellWidths() == uniformCellWidths)
+        return;
+    d->engine.setUniformCellWidths(uniformCellWidths);
+    invalidate();
+    emit uniformCellWidthsChanged();
+}
+
+/*!
+    \qmlproperty bool GridLayout::uniformCellHeights
+    \since QtQuick.Layouts 6.6
+
+    If this property is set to \c true, the layout will force all cells to have an
+    uniform Height. The layout aims to respect
+    \l{Layout::minimumHeight}{Layout.minimumHeight},
+    \l{Layout::preferredHeight}{Layout.preferredHeight} and
+    \l{Layout::maximumHeight}{Layout.maximumHeight} in this mode but might make
+    compromisses to fullfill the requirements of all items.
+
+    Default value is \c false.
+
+    \sa GridLayout::uniformCellWidths, RowLayout::uniformCellSizes, ColumnLayout::uniformCellSizes
+*/
+bool QQuickGridLayout::uniformCellHeights() const
+{
+    Q_D(const QQuickGridLayout);
+    return d->engine.uniformCellHeights();
+}
+
+void QQuickGridLayout::setUniformCellHeights(bool uniformCellHeights)
+{
+    Q_D(QQuickGridLayout);
+    if (d->engine.uniformCellHeights() == uniformCellHeights)
+        return;
+    d->engine.setUniformCellHeights(uniformCellHeights);
+    invalidate();
+    emit uniformCellHeightsChanged();
+}
+
+
 void QQuickGridLayout::insertLayoutItems()
 {
     Q_D(QQuickGridLayout);
@@ -642,6 +710,8 @@ void QQuickGridLayout::insertLayoutItems()
         QQuickLayoutAttached *info = attachedLayoutObject(child, false);
 
         Qt::Alignment alignment;
+        int hStretch = -1;
+        int vStretch = -1;
         int row = -1;
         int column = -1;
         int span[2] = {1,1};
@@ -681,6 +751,12 @@ void QQuickGridLayout::insertLayoutItems()
                 return;
             }
             alignment = info->alignment();
+            hStretch = info->horizontalStretchFactor();
+            if (hStretch >= 0 && !info->fillWidth())
+                qmlWarning(child) << "horizontalStretchFactor requires fillWidth to also be set to true";
+            vStretch = info->verticalStretchFactor();
+            if (vStretch >= 0 && !info->fillHeight())
+                qmlWarning(child) << "verticalStretchFactor requires fillHeight to also be set to true";
         }
 
         Q_ASSERT(columnSpan >= 1);
@@ -732,6 +808,11 @@ void QQuickGridLayout::insertLayoutItems()
         column = nextColumn;
         row = nextRow;
         QQuickGridLayoutItem *layoutItem = new QQuickGridLayoutItem(child, row, column, rowSpan, columnSpan, alignment);
+        if (hStretch >= 0)
+            layoutItem->setStretchFactor(hStretch, Qt::Horizontal);
+        if (vStretch >= 0)
+            layoutItem->setStretchFactor(vStretch, Qt::Vertical);
+
         d->engine.insertItem(layoutItem, -1);
     }
 }
@@ -757,10 +838,8 @@ QQuickLinearLayout::QQuickLinearLayout(Qt::Orientation orientation,
 
     Possible values:
 
-    \list
-    \li Qt.LeftToRight (default) - Items are laid out from left to right.
-    \li Qt.RightToLeft - Items are laid out from right to left
-    \endlist
+    \value Qt.LeftToRight   (default) Items are laid out from left to right.
+    \value Qt.RightToLeft   Items are laid out from right to left
 
     \sa GridLayout::layoutDirection, ColumnLayout::layoutDirection
 */
@@ -774,14 +853,49 @@ QQuickLinearLayout::QQuickLinearLayout(Qt::Orientation orientation,
 
     Possible values:
 
-    \list
-    \li Qt.LeftToRight (default) - Items are laid out from left to right.
-    \li Qt.RightToLeft - Items are laid out from right to left
-    \endlist
+    \value Qt.LeftToRight   (default) Items are laid out from left to right.
+    \value Qt.RightToLeft   Items are laid out from right to left
 
     \sa GridLayout::layoutDirection, RowLayout::layoutDirection
 */
 
+/*!
+    \qmlproperty bool RowLayout::uniformCellSizes
+    \since QtQuick.Layouts 6.6
+
+    If this property is set to \c true, the layout will force all cells to have
+    a uniform size.
+
+    \sa GridLayout::uniformCellWidths, GridLayout::uniformCellHeights, ColumnLayout::uniformCellSizes
+*/
+/*!
+    \qmlproperty bool ColumnLayout::uniformCellSizes
+    \since QtQuick.Layouts 6.6
+
+    If this property is set to \c true, the layout will force all cells to have
+    a uniform size.
+
+    \sa GridLayout::uniformCellWidths, GridLayout::uniformCellHeights, RowLayout::uniformCellSizes
+*/
+bool QQuickLinearLayout::uniformCellSizes() const
+{
+    Q_D(const QQuickLinearLayout);
+    Q_ASSERT(d->engine.uniformCellWidths() == d->engine.uniformCellHeights());
+    return d->engine.uniformCellWidths();
+}
+
+void QQuickLinearLayout::setUniformCellSizes(bool uniformCellSizes)
+{
+    Q_D(QQuickLinearLayout);
+    Q_ASSERT(d->engine.uniformCellWidths() == d->engine.uniformCellHeights());
+    if (d->engine.uniformCellHeights() == uniformCellSizes)
+        return;
+    d->engine.setUniformCellWidths(uniformCellSizes);
+    d->engine.setUniformCellHeights(uniformCellSizes);
+    invalidate();
+    emit uniformCellSizesChanged();
+}
+
 
 /*!
     \qmlproperty real RowLayout::spacing
@@ -827,8 +941,17 @@ void QQuickLinearLayout::insertLayoutItems()
         QQuickLayoutAttached *info = attachedLayoutObject(child, false);
 
         Qt::Alignment alignment;
-        if (info)
+        int hStretch = -1;
+        int vStretch = -1;
+        bool fillWidth = false;
+        bool fillHeight = false;
+        if (info) {
             alignment = info->alignment();
+            hStretch = info->horizontalStretchFactor();
+            vStretch = info->verticalStretchFactor();
+            fillWidth = info->fillWidth();
+            fillHeight = info->fillHeight();
+        }
 
         const int index = d->engine.rowCount(d->orientation);
         d->engine.insertRow(index, d->orientation);
@@ -838,6 +961,17 @@ void QQuickLinearLayout::insertLayoutItems()
         if (d->orientation == Qt::Vertical)
             qSwap(gridRow, gridColumn);
         QQuickGridLayoutItem *layoutItem = new QQuickGridLayoutItem(child, gridRow, gridColumn, 1, 1, alignment);
+
+        if (hStretch >= 0) {
+            if (!fillWidth)
+                qmlWarning(child) << "horizontalStretchFactor requires fillWidth to also be set to true";
+            layoutItem->setStretchFactor(hStretch, Qt::Horizontal);
+        }
+        if (vStretch >= 0) {
+            if (!fillHeight)
+                qmlWarning(child) << "verticalStretchFactor requires fillHeight to also be set to true";
+            layoutItem->setStretchFactor(vStretch, Qt::Vertical);
+        }
         d->engine.insertItem(layoutItem, index);
     }
 }
