@@ -15,6 +15,9 @@
 #include <QtCore/qfile.h>
 #include <QtCore/qtextstream.h>
 #include <QtGui/qpixmapcache.h>
+#include <QtGui/qpa/qplatformintegration.h>
+#include <QtGui/private/qguiapplication_p.h>
+#include <QtGui/qstylehints.h>
 #include <private/qmath_p.h>
 #include <qmath.h>
 #include <QtGui/qpainterpath.h>
@@ -25,6 +28,7 @@
 #include <private/qguiapplication_p.h>
 #include <private/qhighdpiscaling_p.h>
 #include <qpa/qplatformnativeinterface.h>
+#include <QtQuickTemplates2/private/qquicktheme_p.h>
 
 #if 0 && QT_CONFIG(animation)
 //#include <private/qstyleanimation_p.h>
@@ -179,79 +183,21 @@ QWindowsStyle::QWindowsStyle(QWindowsStylePrivate &dd) : QCommonStyle(dd)
 {
 }
 
+void QWindowsStyle::timerEvent(QTimerEvent* event)
+{
+    // Update palette in style object through palette timer timeout and this timer
+    // will be triggered during ApplicationPaletteChange event
+    if (event->timerId() == paletteTimer.timerId()) {
+        paletteTimer.stop();
+        refreshPalette();
+    }
+}
 
 /*! Destroys the QWindowsStyle object. */
 QWindowsStyle::~QWindowsStyle()
 {
 }
 
-#ifdef Q_OS_WIN
-static inline QRgb colorref2qrgb(COLORREF col)
-{
-    return qRgb(GetRValue(col), GetGValue(col), GetBValue(col));
-}
-#endif
-#if 0
-/*! \reimp */
-void QWindowsStyle::polish(QApplication *app)
-{
-    QCommonStyle::polish(app);
-    QWindowsStylePrivate *d = const_cast<QWindowsStylePrivate*>(d_func());
-    // We only need the overhead when shortcuts are sometimes hidden
-    if (!proxy()->styleHint(SH_UnderlineShortcut, nullptr) && app)
-        app->installEventFilter(this);
-
-    const auto &palette = QGuiApplication::palette();
-    d->activeGradientCaptionColor = palette.highlight().color();
-    d->activeCaptionColor = d->activeGradientCaptionColor;
-    d->inactiveGradientCaptionColor = palette.dark().color();
-    d->inactiveCaptionColor = d->inactiveGradientCaptionColor;
-    d->inactiveCaptionText = palette.window().color();
-
-#if defined(Q_OS_WIN) && !defined(Q_OS_WINRT) //fetch native title bar colors
-    if (app->desktopSettingsAware()){
-        DWORD activeCaption = GetSysColor(COLOR_ACTIVECAPTION);
-        DWORD gradientActiveCaption = GetSysColor(COLOR_GRADIENTACTIVECAPTION);
-        DWORD inactiveCaption = GetSysColor(COLOR_INACTIVECAPTION);
-        DWORD gradientInactiveCaption = GetSysColor(COLOR_GRADIENTINACTIVECAPTION);
-        DWORD inactiveCaptionText = GetSysColor(COLOR_INACTIVECAPTIONTEXT);
-        d->activeCaptionColor = colorref2qrgb(activeCaption);
-        d->activeGradientCaptionColor = colorref2qrgb(gradientActiveCaption);
-        d->inactiveCaptionColor = colorref2qrgb(inactiveCaption);
-        d->inactiveGradientCaptionColor = colorref2qrgb(gradientInactiveCaption);
-        d->inactiveCaptionText = colorref2qrgb(inactiveCaptionText);
-    }
-#endif
-}
-
-/*! \reimp */
-void QWindowsStyle::unpolish(QApplication *app)
-{
-    QCommonStyle::unpolish(app);
-    app->removeEventFilter(this);
-}
-
-/*! \reimp */
-void QWindowsStyle::polish(QWidget *widget)
-{
-    QCommonStyle::polish(widget);
-}
-
-/*! \reimp */
-void QWindowsStyle::unpolish(QWidget *widget)
-{
-    QCommonStyle::unpolish(widget);
-}
-
-/*!
-  \reimp
-*/
-void QWindowsStyle::polish(QPalette &pal)
-{
-    QCommonStyle::polish(pal);
-}
-#endif
-
 int QWindowsStylePrivate::pixelMetricFromSystemDp(QStyle::PixelMetric pm, const QStyleOption *opt)
 {
 #if defined(Q_OS_WIN) && !defined(Q_OS_WINRT)
@@ -810,7 +756,7 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
             points[4] = { points[3].x(),                points[3].y() - 2 * scalev };
             points[5] = { points[4].x() - 4 * scaleh,   points[4].y() + 4 * scalev };
             p->setPen(QPen(opt->palette.text().color(), 0));
-            p->setBrush(opt->palette.text().color());
+            p->setBrush(opt->palette.text());
             p->drawPolygon(points, 6);
         }
         if (doRestore)
@@ -869,26 +815,26 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
 
             p->setClipRegion(QRegion(topLeftPol));
             p->setPen(opt->palette.dark().color());
-            p->setBrush(opt->palette.dark().color());
+            p->setBrush(opt->palette.dark());
             p->drawPath(path1);
             p->setPen(opt->palette.shadow().color());
-            p->setBrush(opt->palette.shadow().color());
+            p->setBrush(opt->palette.shadow());
             p->drawPath(path2);
 
             p->setClipRegion(QRegion(bottomRightPol));
             p->setPen(opt->palette.light().color());
-            p->setBrush(opt->palette.light().color());
+            p->setBrush(opt->palette.light());
             p->drawPath(path1);
             p->setPen(opt->palette.midlight().color());
-            p->setBrush(opt->palette.midlight().color());
+            p->setBrush(opt->palette.midlight());
             p->drawPath(path2);
 
-            QColor fillColor = ((opt->state & State_Sunken) || !(opt->state & State_Enabled)) ?
-                                opt->palette.button().color() : opt->palette.base().color();
+            const QBrush fill = ((opt->state & State_Sunken) || !(opt->state & State_Enabled))
+                              ? opt->palette.button() : opt->palette.base();
 
             p->setClipping(false);
-            p->setPen(fillColor);
-            p->setBrush(fillColor);
+            p->setPen(fill.color());
+            p->setBrush(fill);
             p->drawPath(path3);
 
             if (opt->state & State_On) {
@@ -906,14 +852,14 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
             if (frame->lineWidth == 2 || pe == PE_Frame) {
                 QPalette popupPal = frame->palette;
                 if (pe == PE_FrameMenu) {
-                    popupPal.setColor(QPalette::Light, frame->palette.window().color());
-                    popupPal.setColor(QPalette::Midlight, frame->palette.light().color());
+                    popupPal.setBrush(QPalette::Light, frame->palette.window());
+                    popupPal.setBrush(QPalette::Midlight, frame->palette.light());
                 }
                 if (pe == PE_Frame && (frame->state & State_Raised))
                     qDrawWinButton(p, frame->rect, popupPal, frame->state & State_Sunken);
                 else if (pe == PE_Frame && (frame->state & State_Sunken))
                 {
-                    popupPal.setColor(QPalette::Midlight, frame->palette.window().color());
+                    popupPal.setBrush(QPalette::Midlight, frame->palette.window());
                     qDrawWinPanel(p, frame->rect, popupPal, frame->state & State_Sunken);
                 }
                 else
@@ -923,8 +869,8 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
             }
         } else {
             QPalette popupPal = opt->palette;
-            popupPal.setColor(QPalette::Light, opt->palette.window().color());
-            popupPal.setColor(QPalette::Midlight, opt->palette.light().color());
+            popupPal.setBrush(QPalette::Light, opt->palette.window());
+            popupPal.setBrush(QPalette::Midlight, opt->palette.light());
             qDrawWinPanel(p, opt->rect, popupPal, opt->state & State_Sunken);
         }
         break;
@@ -951,8 +897,8 @@ void QWindowsStyle::drawPrimitive(PrimitiveElement pe, const QStyleOption *opt,
         break; }
     case PE_FrameWindow: {
          QPalette popupPal = opt->palette;
-         popupPal.setColor(QPalette::Light, opt->palette.window().color());
-         popupPal.setColor(QPalette::Midlight, opt->palette.light().color());
+         popupPal.setBrush(QPalette::Light, opt->palette.window());
+         popupPal.setBrush(QPalette::Midlight, opt->palette.light());
          qDrawWinPanel(p, opt->rect, popupPal, opt->state & State_Sunken);
         break; }
 #if 0 && QT_CONFIG(dockwidget)
@@ -1438,7 +1384,7 @@ void QWindowsStyle::drawControl(ControlElement ce, const QStyleOption *opt, QPai
             if (!(buttonOpt.state & State_Sunken))
                 buttonOpt.state |= State_Raised;
             QPalette pal(opt->palette);
-            pal.setColor(QPalette::Button, opt->palette.light().color());
+            pal.setBrush(QPalette::Button, opt->palette.light());
             pal.setColor(QPalette::Light, opt->palette.button().color());
             qDrawWinButton(p, opt->rect, pal, opt->state & (State_Sunken | State_On),
                            &opt->palette.brush(QPalette::Button));
@@ -2338,6 +2284,26 @@ QSize QWindowsStyle::sizeFromContents(ContentsType ct, const QStyleOption *opt,
     return sz;
 }
 
+void QWindowsStyle::refreshPalette()
+{
+    // Update system palette in the theme (quick)
+    // Since windows style doesn't support dark appearance,
+    // light palette will always be set in the quick theme
+    QPalette pal;
+    using QWindowsApplication = QNativeInterface::Private::QWindowsApplication;
+    if (auto nativeWindowsApp = dynamic_cast<QWindowsApplication *>(QGuiApplicationPrivate::platformIntegration()))
+        nativeWindowsApp->populateLightSystemPalette(pal);
+    QQuickTheme::instance()->setPalette(QQuickTheme::System, pal);
+}
+
+void QWindowsStyle::polish()
+{
+    // The timer used here compresses ApplicationPaletteChange event and get triggered once
+    // this event has been propagated for all items.
+    if (!paletteTimer.isActive())
+        paletteTimer.start(0, this);
+}
+
 /*!
     \reimp
 */
