@@ -8,6 +8,7 @@
 #include <QtGui/qguiapplication.h>
 #include "qquicklabel_p.h"
 #include "qquicklabel_p_p.h"
+#include "qquicktemplatesutils_p.h"
 #include "qquicktextarea_p.h"
 #include "qquicktextarea_p_p.h"
 #include "qquicktextfield_p.h"
@@ -15,6 +16,7 @@
 #include "qquickpopup_p.h"
 #include "qquickpopupitem_p_p.h"
 #include "qquickapplicationwindow_p.h"
+#include "qquickapplicationwindow_p_p.h"
 #include "qquickdeferredexecute_p_p.h"
 #include "qquickcontentitem_p.h"
 
@@ -29,7 +31,7 @@ Q_LOGGING_CATEGORY(lcItemManagement, "qt.quick.controls.control.itemmanagement")
 /*!
     \qmltype Control
     \inherits Item
-//!     \instantiates QQuickControl
+//!     \nativetype QQuickControl
     \inqmlmodule QtQuick.Controls
     \since 5.7
     \brief Abstract base type providing functionality common to all controls.
@@ -585,22 +587,13 @@ void QQuickControlPrivate::updateFontRecur(QQuickItem *item, const QFont &font)
 
 QLocale QQuickControlPrivate::calcLocale(const QQuickItem *item)
 {
-    const QQuickItem *p = item;
-    while (p) {
+    for (const QQuickItem *p = item; p; p = p->parentItem())
         if (const QQuickControl *control = qobject_cast<const QQuickControl *>(p))
             return control->locale();
 
-        QVariant v = p->property("locale");
-        if (v.isValid() && v.userType() == QMetaType::QLocale)
-            return v.toLocale();
-
-        p = p->parentItem();
-    }
-
-    if (item) {
+    if (item)
         if (QQuickApplicationWindow *window = qobject_cast<QQuickApplicationWindow *>(item->window()))
             return window->locale();
-    }
 
     return QLocale();
 }
@@ -702,12 +695,23 @@ bool QQuickControlPrivate::calcHoverEnabled(const QQuickItem *item)
         if (qobject_cast<const QQuickPopupItem *>(p))
             break;
 
-        if (const QQuickControl *control = qobject_cast<const QQuickControl *>(p))
-            return control->isHoverEnabled();
+        auto *applicationWindow = qobject_cast<QQuickApplicationWindow *>(p->window());
+        if (applicationWindow) {
+            const auto *applicationWindowPrivate = QQuickApplicationWindowPrivate::get(applicationWindow);
+            if (p == applicationWindowPrivate->control) {
+                // Don't let the next check get hit, because it will return
+                // false since it's a plain QQuickControl. Instead, skip to the
+                // global flags.
+                break;
+            }
+        }
 
-        QVariant v = p->property("hoverEnabled");
-        if (v.isValid() && v.userType() == QMetaType::Bool)
-            return v.toBool();
+        if (QQuickTemplatesUtils::isInteractiveControlType(p)) {
+            const QVariant hoverEnabledProperty = p->property("hoverEnabled");
+            Q_ASSERT(hoverEnabledProperty.isValid());
+            Q_ASSERT(hoverEnabledProperty.userType() == QMetaType::Bool);
+            return hoverEnabledProperty.toBool();
+        }
 
         p = p->parentItem();
     }
@@ -911,6 +915,19 @@ void QQuickControlPrivate::itemFocusChanged(QQuickItem *item, Qt::FocusReason re
         setLastFocusChangeReason(reason);
 }
 
+bool QQuickControlPrivate::setLastFocusChangeReason(Qt::FocusReason reason)
+{
+    Q_Q(QQuickControl);
+    Qt::FocusReason oldReason = static_cast<Qt::FocusReason>(focusReason);
+    const auto focusReasonChanged = QQuickItemPrivate::setLastFocusChangeReason(reason);
+    if (focusReasonChanged)
+        emit q->focusReasonChanged();
+    if (isKeyFocusReason(oldReason) != isKeyFocusReason(reason))
+        emit q->visualFocusChanged();
+
+    return focusReasonChanged;
+}
+
 QQuickControl::QQuickControl(QQuickItem *parent)
     : QQuickItem(*(new QQuickControlPrivate), parent)
 {
@@ -1353,12 +1370,15 @@ bool QQuickControl::isMirrored() const
     return d->isMirrored();
 }
 
+// ### Qt 7: replace with signal parameter: QTBUG-117596
 /*!
     \qmlproperty enumeration QtQuick.Controls::Control::focusReason
-    \readonly
 
     This property holds the reason of the last focus change.
 
+    The value of this property is modified by Qt whenever focus is transferred,
+    and you should never have to set this property yourself.
+
     \note This property does not indicate whether the item has \l {Item::activeFocus}
         {active focus}, but the reason why the item either gained or lost focus.
 
@@ -1384,11 +1404,7 @@ Qt::FocusReason QQuickControl::focusReason() const
 void QQuickControl::setFocusReason(Qt::FocusReason reason)
 {
     Q_D(QQuickControl);
-    Qt::FocusReason oldReason = static_cast<Qt::FocusReason>(d->focusReason);
     d->setLastFocusChangeReason(reason);
-    emit focusReasonChanged();
-    if (isKeyFocusReason(oldReason) != isKeyFocusReason(reason))
-        emit visualFocusChanged();
 }
 
 /*!
@@ -1403,7 +1419,7 @@ void QQuickControl::setFocusReason(Qt::FocusReason reason)
     \l Item::activeFocus. This ensures that key focus is only visualized when
     interacting with keys - not when interacting via touch or mouse.
 
-    \sa Item::focusReason, Item::activeFocus
+    \sa focusReason, Item::activeFocus
 */
 bool QQuickControl::hasVisualFocus() const
 {
@@ -1448,7 +1464,7 @@ void QQuickControl::setHovered(bool hovered)
     \qmlproperty bool QtQuick.Controls::Control::hoverEnabled
 
     This property determines whether the control accepts hover events. The default value
-    is \c Qt.styleHints.useHoverEffects.
+    is \c Application.styleHints.useHoverEffects.
 
     Setting this property propagates the value to all child controls that do not have
     \c hoverEnabled explicitly set.
@@ -1741,6 +1757,23 @@ void QQuickControl::resetVerticalPadding()
 qreal QQuickControl::implicitContentWidth() const
 {
     Q_D(const QQuickControl);
+
+    if (auto *safeArea = static_cast<QQuickSafeArea*>(
+        qmlAttachedPropertiesObject<QQuickSafeArea>(this, false))) {
+        // If the control's padding is tied to the safe area we may in
+        // some cases end up with a binding loop if the implicit size
+        // moves the control further into the non-safe area. Detect this
+        // and break the binding loop by returning a constrained content
+        // size based on an earlier known good implicit size.
+        static constexpr auto kLastKnownGoodImplicitWidth = "_q_lastKnownGoodImplicitWidth";
+        if (safeArea->detectedPossibleBindingLoop) {
+            const auto lastImplicitWidth = safeArea->property(kLastKnownGoodImplicitWidth).value<int>();
+            return lastImplicitWidth - leftPadding() - rightPadding();
+        } else {
+            safeArea->setProperty(kLastKnownGoodImplicitWidth, implicitWidth());
+        }
+    }
+
     return d->implicitContentWidth;
 }
 
@@ -1769,6 +1802,23 @@ qreal QQuickControl::implicitContentWidth() const
 qreal QQuickControl::implicitContentHeight() const
 {
     Q_D(const QQuickControl);
+
+    if (auto *safeArea = static_cast<QQuickSafeArea*>(
+        qmlAttachedPropertiesObject<QQuickSafeArea>(this, false))) {
+        // If the control's padding is tied to the safe area we may in
+        // some cases end up with a binding loop if the implicit size
+        // moves the control further into the non-safe area. Detect this
+        // and break the binding loop by returning a constrained content
+        // size based on an earlier known good implicit size.
+        static constexpr auto kLastKnownGoodImplicitHeight = "_q_lastKnownGoodImplicitHeight";
+        if (safeArea->detectedPossibleBindingLoop) {
+            const auto lastImplicitHeight = safeArea->property(kLastKnownGoodImplicitHeight).value<int>();
+            return lastImplicitHeight - topPadding() - bottomPadding();
+        } else {
+            safeArea->setProperty(kLastKnownGoodImplicitHeight, implicitHeight());
+        }
+    }
+
     return d->implicitContentHeight;
 }
 
@@ -1969,22 +2019,12 @@ QFont QQuickControl::defaultFont() const
 
 void QQuickControl::focusInEvent(QFocusEvent *event)
 {
-    Q_D(QQuickControl);
-    Qt::FocusReason oldReason = static_cast<Qt::FocusReason>(d->focusReason);
     QQuickItem::focusInEvent(event);
-    Qt::FocusReason reason = event->reason();
-    if (isKeyFocusReason(oldReason) != isKeyFocusReason(reason))
-        emit visualFocusChanged();
 }
 
 void QQuickControl::focusOutEvent(QFocusEvent *event)
 {
-    Q_D(QQuickControl);
-    Qt::FocusReason oldReason = static_cast<Qt::FocusReason>(d->focusReason);
     QQuickItem::focusOutEvent(event);
-    Qt::FocusReason reason = event->reason();
-    if (isKeyFocusReason(oldReason) != isKeyFocusReason(reason))
-        emit visualFocusChanged();
 }
 
 #if QT_CONFIG(quicktemplates2_hover)
