@@ -14,6 +14,8 @@
 #include <QtQuick/private/qquicktransition_p.h>
 #include <QtQuickTemplates2/private/qquickoverlay_p.h>
 
+#include <algorithm>
+
 QT_BEGIN_NAMESPACE
 
 /*!
@@ -343,8 +345,10 @@ bool QQuickDrawerPrivate::grabMouse(QQuickItem *item, QMouseEvent *event)
     bool overThreshold = false;
     Qt::Edge effEdge = effectiveEdge();
     if (position > 0 || dragMargin > 0) {
-        const bool xOverThreshold = QQuickWindowPrivate::dragOverThreshold(movePoint.x() - pressPoint.x(), Qt::XAxis, event, threshold);
-        const bool yOverThreshold = QQuickWindowPrivate::dragOverThreshold(movePoint.y() - pressPoint.y(), Qt::YAxis, event, threshold);
+        const bool xOverThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(movePoint.x() - pressPoint.x(),
+                                                                    Qt::XAxis, event, threshold);
+        const bool yOverThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(movePoint.y() - pressPoint.y(),
+                                                                    Qt::YAxis, event, threshold);
         if (effEdge == Qt::LeftEdge || effEdge == Qt::RightEdge)
             overThreshold = xOverThreshold && !yOverThreshold;
         else
@@ -396,8 +400,10 @@ bool QQuickDrawerPrivate::grabTouch(QQuickItem *item, QTouchEvent *event)
         const int threshold = qMax(20, QGuiApplication::styleHints()->startDragDistance() + 5);
         const Qt::Edge effEdge = effectiveEdge();
         if (position > 0 || dragMargin > 0) {
-            const bool xOverThreshold = QQuickWindowPrivate::dragOverThreshold(movePoint.x() - pressPoint.x(), Qt::XAxis, &point, threshold);
-            const bool yOverThreshold = QQuickWindowPrivate::dragOverThreshold(movePoint.y() - pressPoint.y(), Qt::YAxis, &point, threshold);
+            const bool xOverThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(movePoint.x() - pressPoint.x(),
+                                                                    Qt::XAxis, point, threshold);
+            const bool yOverThreshold = QQuickDeliveryAgentPrivate::dragOverThreshold(movePoint.y() - pressPoint.y(),
+                                                                    Qt::YAxis, point, threshold);
             if (effEdge == Qt::LeftEdge || effEdge == Qt::RightEdge)
                 overThreshold = xOverThreshold && !yOverThreshold;
             else
@@ -640,6 +646,12 @@ QQuickDrawer::QQuickDrawer(QObject *parent)
 
     setFocus(true);
     setModal(true);
+
+    QQuickItemPrivate::get(d->popupItem)->isTabFence = isModal();
+    connect(this, &QQuickPopup::modalChanged, this, [this] {
+        QQuickItemPrivate::get(d_func()->popupItem)->isTabFence = isModal();
+    });
+
     setFiltersChildMouseEvents(true);
     setClosePolicy(CloseOnEscape | CloseOnReleaseOutside);
 }
@@ -716,7 +728,7 @@ qreal QQuickDrawer::position() const
 void QQuickDrawer::setPosition(qreal position)
 {
     Q_D(QQuickDrawer);
-    position = qBound<qreal>(0.0, position, 1.0);
+    position = std::clamp(position, qreal(0.0), qreal(1.0));
     if (qFuzzyCompare(d->position, position))
         return;
 
