@@ -1,5 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "testhttpserver_p.h"
 #include <QTcpSocket>
@@ -158,12 +158,10 @@ bool TestHTTPServer::wait(const QUrl &expect, const QUrl &reply, const QUrl &bod
     bool headers_done = false;
     while (!(line = expectFile.readLine()).isEmpty()) {
         line.replace('\r', "");
-        if (line.at(0) == '\n') {
-            headers_done = true;
-            continue;
-        }
         if (headers_done) {
             m_waitData.body.append(line);
+        } else if (line.at(0) == '\n') {
+            headers_done = true;
         } else if (line.endsWith("{{Ignore}}\n")) {
             m_waitData.headerPrefixes.append(line.left(line.size() - strlen("{{Ignore}}\n")));
         } else {
