@@ -1,5 +1,5 @@
 // Copyright (C) 2021 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef NEIGHBOR_H
 #define NEIGHBOR_H
@@ -15,4 +15,11 @@ public:
     Neighbor(QObject *parent = nullptr);
 };
 
+class Absent : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    QML_UNAVAILABLE
+};
+
 #endif // NEIGHBOR_H
