@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include "debugutil_p.h"
 #include "qqmldebugprocess_p.h"
@@ -77,20 +52,20 @@ private:
 void QQmlProfilerTestClient::startTrace(qint64 timestamp, const QList<int> &engineIds)
 {
     types.append(QQmlProfilerEventType(Event, MaximumRangeType, StartTrace));
-    asynchronousMessages.append(QQmlProfilerEvent(timestamp, types.length() - 1,
+    asynchronousMessages.append(QQmlProfilerEvent(timestamp, types.size() - 1,
                                                   engineIds.toVector()));
 }
 
 void QQmlProfilerTestClient::endTrace(qint64 timestamp, const QList<int> &engineIds)
 {
     types.append(QQmlProfilerEventType(Event, MaximumRangeType, EndTrace));
-    asynchronousMessages.append(QQmlProfilerEvent(timestamp, types.length() - 1,
+    asynchronousMessages.append(QQmlProfilerEvent(timestamp, types.size() - 1,
                                                   engineIds.toVector()));
 }
 
 int QQmlProfilerTestClient::numLoadedEventTypes() const
 {
-    return types.length();
+    return types.size();
 }
 
 void QQmlProfilerTestClient::addEventType(const QQmlProfilerEventType &type)
@@ -101,11 +76,11 @@ void QQmlProfilerTestClient::addEventType(const QQmlProfilerEventType &type)
 void QQmlProfilerTestClient::addEvent(const QQmlProfilerEvent &event)
 {
     const int typeIndex = event.typeIndex();
-    QVERIFY(typeIndex < types.length());
+    QVERIFY(typeIndex < types.size());
 
     const QQmlProfilerEventType &type = types[typeIndex];
 
-    QVERIFY(event.timestamp() >= lastTimestamp);
+    const qint64 oldTimestamp = lastTimestamp;
     lastTimestamp = event.timestamp();
 
     switch (type.message()) {
@@ -174,6 +149,8 @@ void QQmlProfilerTestClient::addEvent(const QQmlProfilerEvent &event)
         }
         break;
     }
+
+    QCOMPARE_GE(lastTimestamp, oldTimestamp);
 }
 
 class tst_QQmlProfilerService : public QQmlDebugTest
@@ -297,14 +274,14 @@ void tst_QQmlProfilerService::checkTraceReceived()
 
     // must end with "EndTrace"
     expected = QQmlProfilerEventType(Event, MaximumRangeType, EndTrace);
-    VERIFY(MessageListAsynchronous, m_client->asynchronousMessages.length() - 1, expected,
+    VERIFY(MessageListAsynchronous, m_client->asynchronousMessages.size() - 1, expected,
            CheckMessageType | CheckDetailType, numbers);
 }
 
 void tst_QQmlProfilerService::checkJsHeap()
 {
     QVERIFY(m_client);
-    QVERIFY2(m_client->jsHeapMessages.count() > 0, "no JavaScript heap messages received");
+    QVERIFY2(m_client->jsHeapMessages.size() > 0, "no JavaScript heap messages received");
 
     bool seen_alloc = false;
     bool seen_small = false;
@@ -312,7 +289,7 @@ void tst_QQmlProfilerService::checkJsHeap()
     qint64 allocated = 0;
     qint64 used = 0;
     qint64 lastTimestamp = -1;
-    foreach (const QQmlProfilerEvent &message, m_client->jsHeapMessages) {
+    for (const QQmlProfilerEvent &message : std::as_const(m_client->jsHeapMessages)) {
         const auto amount = message.number<qint64>(0);
         const QQmlProfilerEventType &type = m_client->types.at(message.typeIndex());
         switch (type.detailType()) {
@@ -382,9 +359,9 @@ bool tst_QQmlProfilerService::verify(tst_QQmlProfilerService::MessageListType ty
         return false;
     }
 
-    if (target->length() <= expectedPosition) {
+    if (target->size() <= expectedPosition) {
         qWarning() << "Not enough events. expected position:" << expectedPosition
-                   << "length:" << target->length();
+                   << "length:" << target->size();
         return false;
     }
 
@@ -463,9 +440,9 @@ bool tst_QQmlProfilerService::verify(tst_QQmlProfilerService::MessageListType ty
         }
 
         return true;
-    } while (++position < target->length() && target->at(position).timestamp() == timestamp);
+    } while (++position < target->size() && target->at(position).timestamp() == timestamp);
 
-    foreach (const QString &message, warnings)
+    for (const QString &message : std::as_const(warnings))
         qWarning() << message.toLocal8Bit().constData();
 
     return false;
@@ -492,34 +469,34 @@ void tst_QQmlProfilerService::cleanup()
     };
 
     if (m_client && QTest::currentTestFailed()) {
-        qDebug() << "QML Messages:" << m_client->qmlMessages.count();
+        qDebug() << "QML Messages:" << m_client->qmlMessages.size();
         int i = 0;
-        for (const QQmlProfilerEvent &data : qAsConst(m_client->qmlMessages))
+        for (const QQmlProfilerEvent &data : std::as_const(m_client->qmlMessages))
             log(data, i++);
 
         qDebug() << " ";
-        qDebug() << "JavaScript Messages:" << m_client->javascriptMessages.count();
+        qDebug() << "JavaScript Messages:" << m_client->javascriptMessages.size();
         i = 0;
 
-        for (const QQmlProfilerEvent &data : qAsConst(m_client->javascriptMessages))
+        for (const QQmlProfilerEvent &data : std::as_const(m_client->javascriptMessages))
             log(data, i++);
 
         qDebug() << " ";
-        qDebug() << "Asynchronous Messages:" << m_client->asynchronousMessages.count();
+        qDebug() << "Asynchronous Messages:" << m_client->asynchronousMessages.size();
         i = 0;
-        for (const QQmlProfilerEvent &data : qAsConst(m_client->asynchronousMessages))
+        for (const QQmlProfilerEvent &data : std::as_const(m_client->asynchronousMessages))
             log(data, i++);
 
         qDebug() << " ";
-        qDebug() << "Pixmap Cache Messages:" << m_client->pixmapMessages.count();
+        qDebug() << "Pixmap Cache Messages:" << m_client->pixmapMessages.size();
         i = 0;
-        for (const QQmlProfilerEvent &data : qAsConst(m_client->pixmapMessages))
+        for (const QQmlProfilerEvent &data : std::as_const(m_client->pixmapMessages))
             log(data, i++);
 
         qDebug() << " ";
-        qDebug() << "Javascript Heap Messages:" << m_client->jsHeapMessages.count();
+        qDebug() << "Javascript Heap Messages:" << m_client->jsHeapMessages.size();
         i = 0;
-        for (const QQmlProfilerEvent &data : qAsConst(m_client->jsHeapMessages))
+        for (const QQmlProfilerEvent &data : std::as_const(m_client->jsHeapMessages))
             log(data, i++);
 
         qDebug() << " ";
@@ -605,7 +582,7 @@ void tst_QQmlProfilerService::scenegraphData()
     QCOMPARE(connectTo(true, "scenegraphTest.qml"), ConnectSuccess);
 
     while (!m_process->output().contains(QLatin1String("tick")))
-        QVERIFY(QQmlDebugTest::waitForSignal(m_process, SIGNAL(readyReadStandardOutput())));
+        QVERIFY(QQmlDebugTest::waitForSignal(m_process, SIGNAL(readyReadStandardOutput()), 30000));
     m_client->client->setRecording(false);
 
     checkTraceReceived();
@@ -619,7 +596,7 @@ void tst_QQmlProfilerService::scenegraphData()
     // interleaved. Also, events could carry the same time stamps and be sorted in an unexpected way
     // if the clocks are acting up.
     qint64 renderFrameTime = -1;
-    foreach (const QQmlProfilerEvent &msg, m_client->asynchronousMessages) {
+    for (const QQmlProfilerEvent &msg : std::as_const(m_client->asynchronousMessages)) {
         const QQmlProfilerEventType &type = m_client->types.at(msg.typeIndex());
         if (type.detailType() == SceneGraphRendererFrame) {
             renderFrameTime = msg.timestamp();
@@ -702,9 +679,9 @@ void tst_QQmlProfilerService::flushInterval()
     QCOMPARE(connectTo(true, "timer.qml", true, 1), ConnectSuccess);
 
     // Make sure we get multiple messages
-    QTRY_VERIFY(m_client->qmlMessages.length() > 0);
-    QVERIFY(m_client->qmlMessages.length() < 100);
-    QTRY_VERIFY(m_client->qmlMessages.length() > 100);
+    QTRY_VERIFY(m_client->qmlMessages.size() > 0);
+    QVERIFY(m_client->qmlMessages.size() < 100);
+    QTRY_VERIFY(m_client->qmlMessages.size() > 100);
 
     m_client->client->setRecording(false);
     checkTraceReceived();
@@ -808,7 +785,7 @@ void tst_QQmlProfilerService::multiEngine()
     QTRY_COMPARE(m_process->state(), QProcess::NotRunning);
     QCOMPARE(m_process->exitStatus(), QProcess::NormalExit);
 
-    QCOMPARE(spy.count(), 1);
+    QCOMPARE(spy.size(), 1);
 }
 
 void tst_QQmlProfilerService::batchOverflow()
