@@ -3,10 +3,10 @@
 
 /*---
 esid: sec-array.prototype.reduce
-es5id: 15.4.4.21-3-5
+es5id: 15.4.4.21-3-10
 description: >
-    Array.prototype.reduce - value of 'length' is a number (value is
-    -0)
+    Array.prototype.reduce - value of 'length' is number primitive
+    (value is NaN)
 ---*/
 
 var accessed = false;
@@ -17,8 +17,8 @@ function callbackfn(prevVal, curVal, idx, obj) {
 }
 
 var obj = {
-  0: 11,
-  length: -0
+  0: 9,
+  length: NaN
 };
 
 assert.sameValue(Array.prototype.reduce.call(obj, callbackfn, 1), 1, 'Array.prototype.reduce.call(obj, callbackfn, 1)');
