@@ -2,12 +2,10 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.2.3.12-3-16
-description: >
-    Object.isFrozen returns false for all built-in objects
-    (Date.prototype)
+es5id: 15.2.3.12-3-19
+description: Object.isFrozen returns false for all built-in objects (Error)
 ---*/
 
-var b = Object.isFrozen(Date.prototype);
+var b = Object.isFrozen(Error);
 
 assert.sameValue(b, false, 'b');
