@@ -2,16 +2,18 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 14.1-4gs
+es5id: 14.1-5gs
 description: >
-    StrictMode - a Use Strict Directive followed by a strict mode
-    violation
+    StrictMode - a Use Strict Directive embedded in a directive
+    prologue followed by a strict mode violation
 negative:
   phase: parse
   type: SyntaxError
 flags: [raw]
 ---*/
 
+"a";
 "use strict";
+"c";
 throw new Error("This code should not execute");
 eval = 42;
