@@ -2,12 +2,8 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-esid: sec-literals-regular-expression-literals
+esid: prod-RegularExpressionBackslashSequence
 info: |
-  RegularExpressionChar ::
-    RegularExpressionBackslashSequence
-
-
   RegularExpressionBackslashSequence ::
     \ RegularExpressionNonTerminator
 
@@ -21,16 +17,16 @@ info: |
     <PS>
 
 description: >
-  A regular expression may not contain a <CR> as a SourceCharacter, evaluated
+  A regular expression may not contain a <LF> as a SourceCharacter, evaluated
 ---*/
 
 //CHECK#1
 try {
-   eval("/a\u000D/").source;
-   $ERROR('#1.1: RegularExpressionChar :: Carriage Retur is incorrect. Actual: ' + (eval("/a\u000D/").source));
+   eval("/a\\\u000D/").source;
+   $ERROR('#1.1: RegularExpressionChar :: BackslashSequence :: \\Carriage Return is incorrect. Actual: ' + (eval("/a\\\u000D/").source));
 }
 catch (e) {
   if ((e instanceof SyntaxError) !== true) {
-     $ERROR('#1.2: RegularExpressionChar :: Carriage Retur is incorrect. Actual: ' + (e));
+     $ERROR('#1.2: RegularExpressionChar :: BackslashSequence :: \\Carriage Return is incorrect. Actual: ' + (e));
   }
 }
