@@ -2,11 +2,11 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.1-4
-description: "12.1 - block '{ StatementListopt };' is not allowed: if-else"
+es5id: 12.1-3
+description: "12.1 - block '{ StatementListopt };' is not allowed: try-finally"
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-            eval("if{};else{}");
+            eval("try{};finally{}");
 });
