@@ -2,12 +2,14 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 12.2.1-35-s
-description: "'for(var eval = 42 in ...) {...}' throws SyntaxError in strict mode"
+es5id: 12.2.1-37-s
+description: >
+    'for(var arguments = 42 in ...) {...}' throws SyntaxError in
+    strict mode
 flags: [onlyStrict]
 ---*/
 
 
 assert.throws(SyntaxError, function() {
-    eval('for (var eval = 42 in null) {};');
+    eval('for (var arguments = 42 in null) {};');
 });
