@@ -36,36 +36,27 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '7.7.2.js';
+gTestfile = '7.5-1.js';
 
 /**
-   File Name:          7.7.2.js
-   ECMA Section:       7.7.2 Boolean Literals
-
-   Description:        BooleanLiteral::
-   true
-   false
-
-   The value of the Boolean literal true is a value of the
-   Boolean type, namely true.
-
-   The value of the Boolean literal false is a value of the
-   Boolean type, namely false.
+   File Name:          7.5-1.js
+   ECMA Section:       7.5 Identifiers
+   Description:        Identifiers are of unlimited length
+   - can contain letters, a decimal digit, _, or $
+   - the first character cannot be a decimal digit
+   - identifiers are case sensitive
 
    Author:             christine@netscape.com
-   Date:               16 september 1997
+   Date:               11 september 1997
 */
-
-var SECTION = "7.7.2";
+var SECTION = "7.5-1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Boolean Literals";
+var TITLE   = "Identifiers";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-// StringLiteral:: "" and ''
-
-new TestCase( SECTION, "true",     Boolean(true),     true );
-new TestCase( SECTION, "false",    Boolean(false),    false );
+new TestCase( SECTION,    "var $123 = 5",      5,       eval("var $123 = 5;$123") );
+new TestCase( SECTION,    "var _123 = 5",      5,       eval("var _123 = 5;_123") );
 
 test();
