@@ -1,5 +1,5 @@
 // Copyright (C) 2022 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef OBJECTWITHMETOD_H
 #define OBJECTWITHMETOD_H
@@ -29,12 +29,19 @@ public:
     QProperty<int> theThing;
     QBindable<int> theThingBindable() { return QBindable<int>(&theThing); }
 
-    Q_INVOKABLE void overloaded(QQmlV4Function *) { setObjectName(QStringLiteral("javaScript")); }
+    // The meta methods are populated back to front.
+    // The V4Function flag should not bleed into the others in either case.
+
+    Q_INVOKABLE void overloaded(QQmlV4FunctionPtr) { setObjectName(QStringLiteral("javaScript")); }
     Q_INVOKABLE void overloaded(double) { setObjectName(QStringLiteral("double")); }
     Q_INVOKABLE void overloaded(const QString &) { setObjectName(QStringLiteral("string")); }
 
     Q_INVOKABLE void foo(const QString &bla) { setObjectName(bla); }
     Q_INVOKABLE void foo(ObjectWithMethod *) { setObjectName(QStringLiteral("ObjectWithMethod")); }
+
+    Q_INVOKABLE void overloaded2(double) { setObjectName(QStringLiteral("double")); }
+    Q_INVOKABLE void overloaded2(const QString &) { setObjectName(QStringLiteral("string")); }
+    Q_INVOKABLE void overloaded2(QQmlV4FunctionPtr) { setObjectName(QStringLiteral("javaScript")); }
 };
 
 class OverriddenObjectName : public ObjectWithMethod
@@ -72,8 +79,8 @@ class ObjectWithStringListMethod : public QObject
 public:
     explicit ObjectWithStringListMethod(QObject *parent = nullptr) : QObject(parent)
     {
-        m_names.append("One");
-        m_names.append("Two");
+        m_names.append(QStringLiteral("One"));
+        m_names.append(QStringLiteral("Two"));
     }
 
     Q_INVOKABLE QStringList names() const { return m_names; }
