@@ -1,12 +1,12 @@
 // Copyright (C) 2022 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef LINTPLUGIN_H
 #define LINTPLUGIN_H
 
 #include <QtPlugin>
 #include <QtCore/qobject.h>
-#include <QtQmlCompiler/private/qqmlsa_p.h>
+#include <QtQmlCompiler/qqmlsa.h>
 
 class LintPlugin : public QObject, public QQmlSA::LintPlugin
 {
