@@ -3,6 +3,7 @@
 
 #include <qtest.h>
 #include <private/qtqmlmodelsglobal_p.h>
+#include <private/qtquick-config_p.h> // For feature definitions
 
 using namespace Qt::StringLiterals;
 
@@ -27,8 +28,12 @@ private slots:
     void qtQuickIncludes();
     void enumerations();
     void methods();
+#if QT_CONFIG(quick_tableview)
     void properties();
+#if QT_CONFIG(quick_gridview)
     void ids();
+#endif
+#endif
     void importNamespace();
     void deferredProperties();
     void gradients(); // QTBUG-102560
