@@ -1,14 +1,22 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-#include <qtest.h>
-#include <QtQml/qqmlengine.h>
-#include <QtQml/qqmlcomponent.h>
-#include <QtQml/private/qqmlbind_p.h>
-#include <QtQml/private/qqmlcomponentattached_p.h>
-#include <QtQuick/private/qquickrectangle_p.h>
-#include <QtQuickTestUtils/private/qmlutils_p.h>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
 #include "WithBindableProperties.h"
 
+#include <private/qmlutils_p.h>
+#include <private/qqmlanybinding_p.h>
+#include <private/qqmlbind_p.h>
+#include <private/qqmlcomponentattached_p.h>
+#include <private/qqmlinstantiator_p.h>
+#include <private/qqmlpropertytopropertybinding_p.h>
+#include <private/qquickrectangle_p.h>
+
+#include <QtTest/qtest.h>
+
+#include <QtQml/qqmlcomponent.h>
+#include <QtQml/qqmlengine.h>
+#include <QtQml/qqmlproperty.h>
+
 class tst_qqmlbinding : public QQmlDataTest
 {
     Q_OBJECT
@@ -25,6 +33,7 @@ private slots:
     void restoreBindingJSValue();
     void restoreBindingWithLoop();
     void restoreBindingWithoutCrash();
+    void restoreBindingWhenDestroyed();
     void deletedObject();
     void warningOnUnknownProperty();
     void warningOnReadOnlyProperty();
@@ -32,11 +41,20 @@ private slots:
     void disabledOnReadonlyProperty();
     void delayed();
     void bindingOverwriting();
+    void bindingInDeadContext();
     void bindToQmlComponent();
     void bindingDoesNoWeirdConversion();
     void bindNaNToInt();
     void intOverflow();
     void generalizedGroupedProperties();
+    void localSignalHandler();
+    void whenEvaluatedEarlyEnough();
+    void propertiesAttachedToBindingItself();
+    void toggleEnableProperlyRemembersValues();
+    void qQmlPropertyToPropertyBinding();
+    void delayedBindingDestruction();
+    void deleteStashedObject();
+    void multiValueTypeBinding();
 
 private:
     QQmlEngine engine;
@@ -45,6 +63,9 @@ private:
 tst_qqmlbinding::tst_qqmlbinding()
     : QQmlDataTest(QT_QMLTEST_DATADIR)
 {
+#ifdef QML_DISABLE_INTERNAL_DEFERRED_PROPERTIES
+    qputenv("QML_DISABLE_INTERNAL_DEFERRED_PROPERTIES", "1");
+#endif
 }
 
 void tst_qqmlbinding::binding()
@@ -147,8 +168,11 @@ void tst_qqmlbinding::restoreBindingValue()
 {
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("restoreBinding2.qml"));
-    QScopedPointer<QQuickRectangle> rect(qobject_cast<QQuickRectangle*>(c.create()));
-    QVERIFY(!rect.isNull());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(o.data());
+    QVERIFY(rect);
 
     auto myItem = qobject_cast<QQuickRectangle*>(rect->findChild<QQuickRectangle*>("myItem"));
     QVERIFY(myItem != nullptr);
@@ -171,8 +195,11 @@ void tst_qqmlbinding::restoreBindingVarValue()
 {
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("restoreBinding3.qml"));
-    QScopedPointer<QQuickRectangle> rect(qobject_cast<QQuickRectangle*>(c.create()));
-    QVERIFY(!rect.isNull());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(o.data());
+    QVERIFY(rect);
 
     auto myItem = qobject_cast<QQuickRectangle*>(rect->findChild<QQuickRectangle*>("myItem"));
     QVERIFY(myItem != nullptr);
@@ -195,8 +222,11 @@ void tst_qqmlbinding::restoreBindingJSValue()
 {
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("restoreBinding4.qml"));
-    QScopedPointer<QQuickRectangle> rect(qobject_cast<QQuickRectangle*>(c.create()));
-    QVERIFY(!rect.isNull());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QQuickRectangle *rect = qobject_cast<QQuickRectangle*>(o.data());
+    QVERIFY(rect);
 
     auto myItem = qobject_cast<QQuickRectangle*>(rect->findChild<QQuickRectangle*>("myItem"));
     QVERIFY(myItem != nullptr);
@@ -286,6 +316,25 @@ void tst_qqmlbinding::restoreBindingWithoutCrash()
     QCOMPARE(myItem->x(), qreal(100-49));
 }
 
+void tst_qqmlbinding::restoreBindingWhenDestroyed()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("bindingRestoredWhenDestroyed.qml"));
+    QScopedPointer<QObject> root {c.create()};
+    QVERIFY2(root, qUtf8Printable(c.errorString()));
+    QCOMPARE(root->property("text").toString(), u"original");
+    QCOMPARE(root->property("i").toInt(), 42);
+
+    root->setProperty("toggle", true);
+    // QTRY_COMPARE as loader is async
+    QTRY_COMPARE(root->property("text").toString(), u"changed");
+    QCOMPARE(root->property("i").toInt(), 100);
+
+    root->setProperty("toggle", false);
+    QTRY_COMPARE(root->property("text").toString(), u"original");
+    QCOMPARE(root->property("i").toInt(), 100);
+}
+
 //QTBUG-20692
 void tst_qqmlbinding::deletedObject()
 {
@@ -355,10 +404,6 @@ void tst_qqmlbinding::disabledOnReadonlyProperty()
 
 void tst_qqmlbinding::delayed()
 {
-#ifdef Q_OS_ANDROID
-    QSKIP("This test crashes on Android. QTBUG-103310");
-#endif
-
     QQmlEngine engine;
     QQmlComponent c(&engine, testFileUrl("delayed.qml"));
     QScopedPointer<QQuickItem> item {qobject_cast<QQuickItem*>(c.create())};
@@ -445,9 +490,49 @@ void tst_qqmlbinding::bindingOverwriting()
     QQmlComponent c(&engine, testFileUrl("bindingOverwriting.qml"));
     QScopedPointer<QQuickItem> item {qobject_cast<QQuickItem*>(c.create())};
     QVERIFY(item);
+    QCOMPARE(messageHandler.messages().size(), 2);
+
+    QQmlComponent c2(&engine, testFileUrl("bindingOverwriting2.qml"));
+    QScopedPointer<QObject> o(c2.create());
+    QVERIFY(o);
+    QTRY_COMPARE(o->property("i").toInt(), 123);
+    QCOMPARE(messageHandler.messages().size(), 3);
 
     QLoggingCategory::setFilterRules(QString());
-    QCOMPARE(messageHandler.messages().size(), 2);
+}
+
+void tst_qqmlbinding::bindingInDeadContext()
+{
+    // We manually control the deletion order of the objects here.
+    // This is what some of our views also do. One way to prevent
+    // the engine from deleting objects is to parent them to the
+    // application.
+
+    QScopedPointer<QObject> o;
+    QScopedPointer<QObject> inner1;
+    {
+        QScopedPointer<QObject> inner2;
+
+        QQmlEngine engine;
+        QQmlComponent c(&engine, testFileUrl("bindingInDeadContext.qml"));
+
+        QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+        o.reset(c.create());
+        QVERIFY(!o.isNull());
+        o->setParent(QCoreApplication::instance());
+
+        inner1.reset(o->property("inner1").value<QObject *>());
+        QVERIFY(inner1);
+        inner1->setParent(QCoreApplication::instance());
+
+        inner2.reset(o->property("inner2").value<QObject *>());
+        QVERIFY(inner2);
+        inner2->setParent(QCoreApplication::instance());
+    }
+
+    // The objectName binding did not get re-evaluated when inner2 died
+    // because the engine was gone already.
+    QCOMPARE(inner1->objectName(), "aa");
 }
 
 void tst_qqmlbinding::bindToQmlComponent()
@@ -572,6 +657,242 @@ void tst_qqmlbinding::generalizedGroupedProperties()
     QCOMPARE(rootAttached->objectName(), QString());
 }
 
+void tst_qqmlbinding::localSignalHandler()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("bindingWithHandler.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    o->setProperty("input", QStringLiteral("abc"));
+    QCOMPARE(o->property("output").toString(), QStringLiteral("abc"));
+}
+
+void tst_qqmlbinding::whenEvaluatedEarlyEnough()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("whenEvaluatedEarlyEnough.qml"));
+    QTest::failOnWarning(QRegularExpression(".*"));
+    std::unique_ptr<QObject> root { c.create() };
+    root->setProperty("toggle", false); // should not cause warnings
+    // until "when" is actually true
+    QTest::ignoreMessage(QtMsgType::QtWarningMsg,
+                         QRegularExpression(".*QML Binding: Property 'i' does not exist on Item.*"));
+    root->setProperty("forceEnable", true);
+}
+
+void tst_qqmlbinding::propertiesAttachedToBindingItself()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("propertiesAttachedToBindingItself.qml"));
+    QTest::failOnWarning(QRegularExpression(".*"));
+    std::unique_ptr<QObject> root { c.create() };
+    QVERIFY2(root, qPrintable(c.errorString()));
+    // 0 => everything broken; 1 => normal attached properties broken;
+    // 2 => Component.onCompleted broken, 3 => everything works
+    QTRY_COMPARE(root->property("check").toInt(), 3);
+}
+
+void tst_qqmlbinding::toggleEnableProperlyRemembersValues()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("toggleEnableProperlyRemembersValues.qml"));
+    std::unique_ptr<QObject> root { c.create() };
+    QVERIFY2(root, qPrintable(c.errorString()));
+    for (int i = 0; i < 3; ++i) {
+        {
+            QJSManagedValue arr(root->property("arr"), &e);
+            QJSManagedValue func(root->property("func"), &e);
+            QCOMPARE(arr.property("length").toInt(), 2);
+            QCOMPARE(func.call().toInt(), 1);
+        }
+        root->setProperty("enabled", true);
+        {
+            QJSManagedValue arr(root->property("arr"), &e);
+            QJSManagedValue func(root->property("func"), &e);
+            QCOMPARE(arr.property("length").toInt(), 3);
+            QCOMPARE(func.call().toInt(), 2);
+        }
+        root->setProperty("enabled", false);
+    }
+}
+
+class PropertyToPropertyObject : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(QRectF a READ a WRITE setA NOTIFY aChanged)
+    Q_PROPERTY(QRectF b READ b BINDABLE bindableB)
+
+    Q_PROPERTY(qreal top READ top WRITE setTop NOTIFY topChanged)
+    Q_PROPERTY(qreal bottom READ bottom BINDABLE bindableBottom)
+    Q_PROPERTY(qreal left READ left WRITE setLeft NOTIFY leftChanged)
+    Q_PROPERTY(qreal right READ right BINDABLE bindableRight)
+public:
+
+    QRectF a() const { return m_a; }
+    void setA(const QRectF &a)
+    {
+        if (m_a == a)
+            return;
+        m_a = a;
+        emit aChanged();
+    }
+
+    QRectF b() const { return m_b.value(); }
+    QBindable<QRectF> bindableB() { return QBindable<QRectF>(&m_b); }
+
+    qreal top() const { return m_top; }
+    void setTop(qreal top)
+    {
+        if (qFuzzyCompare(m_top, top))
+            return;
+        m_top = top;
+        emit topChanged();
+    }
+
+    qreal bottom() const { return m_bottom; }
+    QBindable<qreal> bindableBottom() { return QBindable<qreal>(&m_bottom); }
+
+    qreal left() const { return m_left; }
+    void setLeft(qreal left)
+    {
+        if (qFuzzyCompare(m_left, left))
+            return;
+        m_left = left;
+        emit leftChanged();
+    }
+
+    qreal right() const { return m_right; }
+    QBindable<qreal> bindableRight() { return QBindable<qreal>(&m_right); }
+
+signals:
+    void aChanged();
+    void topChanged();
+    void leftChanged();
+
+private:
+    QRectF m_a;
+    QProperty<QRectF> m_b;
+    qreal m_top = 0;
+    QProperty<qreal> m_bottom;
+    qreal m_left = 0;
+    QProperty<qreal> m_right;
+};
+
+void tst_qqmlbinding::qQmlPropertyToPropertyBinding()
+{
+    using namespace Qt::StringLiterals;
+
+    QScopedPointer<PropertyToPropertyObject> target(new PropertyToPropertyObject);
+    QScopedPointer<PropertyToPropertyObject> source(new PropertyToPropertyObject);
+
+    auto installPropertyBinding
+            = [&](QObject *targetObject, const QString &targetPropertyName,
+                  QObject *sourceObject, const QString &sourcePropertyName) {
+        QQmlProperty targetProperty(targetObject, targetPropertyName);
+        QQmlProperty sourceProperty(sourceObject, sourcePropertyName);
+        QQmlAnyBinding binding;
+
+        binding = new QQmlPropertyToPropertyBinding(
+                &engine,
+                sourceObject, QQmlPropertyPrivate::get(sourceProperty)->encodedIndex(),
+                targetObject, targetProperty.index());
+        binding.installOn(targetProperty);
+    };
+
+    installPropertyBinding(target.data(), "left"_L1,   source.data(), "a.left"_L1);
+    installPropertyBinding(target.data(), "top"_L1,    source.data(), "b.top"_L1);
+
+    // TODO: We cannot install a QQmlPropertyToPropertyBinding on bindable properties
+    //       This is because it derives from QQmlAbstractBinding which is only meant for
+    //       non-bindable properties.
+
+    // installPropertyBinding(target.data(), "right"_L1,  source.data(), "a.right"_L1);
+    // installPropertyBinding(target.data(), "bottom"_L1, source.data(), "b.bottom"_L1);
+
+    QCOMPARE(target->top(), 0);
+    QCOMPARE(target->bottom(), 0);
+    QCOMPARE(target->left(), 0);
+    QCOMPARE(target->right(), 0);
+
+    source->setA(QRectF(11, 22, 33, 44));
+    QCOMPARE(target->top(), 0);
+    QCOMPARE(target->bottom(), 0);
+    QCOMPARE(target->left(), 11);
+    // QCOMPARE(target->right(), 33);
+
+    source->bindableB().setValue(QRectF(55, 66, 77, 88));
+    QCOMPARE(target->top(), 66);
+    // QCOMPARE(target->bottom(), 88);
+    QCOMPARE(target->left(), 11);
+    // QCOMPARE(target->right(), 33);
+}
+
+void tst_qqmlbinding::delayedBindingDestruction()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("delayedBindingDestruction.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(object);
+    QVERIFY(object->objectName().isEmpty());
+    QVERIFY(object->property("result").toString().isEmpty());
+
+    const auto verifyDelegate = [&](const QString &expected) {
+        QQmlInstantiator *instantiator
+                = object->property("instantiator").value<QQmlInstantiator *>();
+        QVERIFY(instantiator);
+        QCOMPARE(instantiator->object()->objectName(), expected);
+    };
+
+    QMetaObject::invokeMethod(object.data(), "toggle");
+    QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
+    QCoreApplication::processEvents();
+    QCOMPARE(object->objectName(), QLatin1String("bar"));
+    verifyDelegate(QLatin1String("bar"));
+
+    QMetaObject::invokeMethod(object.data(), "toggle");
+    QCoreApplication::sendPostedEvents(nullptr, QEvent::DeferredDelete);
+    QCoreApplication::processEvents();
+    QCOMPARE(object->objectName(), QLatin1String("foo"));
+    verifyDelegate(QLatin1String("foo"));
+}
+
+void tst_qqmlbinding::deleteStashedObject()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("deleteStashedObject.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    QTest::ignoreMessage(QtDebugMsg, "alive");
+    QTest::ignoreMessage(QtDebugMsg, "destroy");
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(object);
+    QVERIFY(object->property("page").value<QObject *>() != nullptr);
+
+    QTest::ignoreMessage(QtDebugMsg, "dead");
+    QTest::ignoreMessage(QtDebugMsg, "before");
+    QTest::ignoreMessage(QtDebugMsg, "after");
+    QTRY_VERIFY(object->property("page").value<QObject *>() == nullptr);
+}
+
+void tst_qqmlbinding::multiValueTypeBinding()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("multiValueTypeBinding.qml"));
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(object);
+
+    QObject *label = object->property("label").value<QObject *>();
+    QVERIFY(label);
+
+    QRectF rect = label->property("rect").toRectF();
+    QCOMPARE(rect.x(), 12);
+    QCOMPARE(rect.y(), 24);
+    QCOMPARE(rect.width(), 9);
+}
+
 QTEST_MAIN(tst_qqmlbinding)
 
 #include "tst_qqmlbinding.moc"
