@@ -85,6 +85,10 @@ private slots:
     void objectListArgumentMethod();
     void variantListQJsonConversion();
     void attachedObjectOfUnregistered();
+    void dropCUOnEngineShutdown();
+    void multiLoadedJavaScriptModule();
+    void metaObjectOfScriptCU();
+    void registerModule();
 
 public slots:
     QObject *createAQObjectForOwnershipTest ()
@@ -1815,6 +1819,77 @@ void tst_qqmlengine::attachedObjectOfUnregistered()
     QVERIFY(c != a);
 }
 
+void tst_qqmlengine::dropCUOnEngineShutdown()
+{
+    QScopedPointer<QObject> o;
+    const QUrl url("qrc:/some/Type.qml");
+    {
+        QQmlEngine e;
+        QQmlComponent c(&e);
+        c.setData("import QtQml\nQtObject {}", url);
+        QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+        o.reset(c.create());
+        QVERIFY(!o.isNull());
+        QVERIFY(QByteArray(o->metaObject()->className()).startsWith("Type_QMLTYPE_"));
+        QVERIFY(QQmlMetaType::qmlType(url).isValid());
+        QVERIFY(!QQmlMetaType::obtainCompilationUnit(url).isNull());
+    }
+
+    // The object is still there (don't do this at home).
+    QVERIFY(!o.isNull());
+
+    // QMetaObject is still ok.
+    QVERIFY(QByteArray(o->metaObject()->className()).startsWith("Type_QMLTYPE_"));
+
+    // But the QQmlType is gone.
+    QVERIFY(!QQmlMetaType::qmlType(url).isValid());
+
+    // And also the CU.
+    QVERIFY(QQmlMetaType::obtainCompilationUnit(url).isNull());
+}
+
+void tst_qqmlengine::multiLoadedJavaScriptModule()
+{
+    QQmlEngine e;
+    QQmlComponent c(&e, testFileUrl("multiLoaded.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+
+    QTest::ignoreMessage(QtCriticalMsg, "In a1 - DATA_IN_A1");
+    QTest::ignoreMessage(QtCriticalMsg, "In B1 DATA_IN_A1");
+    QTest::ignoreMessage(QtCriticalMsg, "In B1 DATA_IN_A1");
+
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+}
+
+void tst_qqmlengine::metaObjectOfScriptCU()
+{
+    QQmlEngine engine;
+    engine.addImportPath(dataDirectory());
+    QQmlComponent c(&engine);
+    c.setData(R"(
+        import JS
+        import QtQml
+        QtObject {
+            objectName: ColorUtils.withOpacity('blue', 0.5)
+        })", QUrl());
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QCOMPARE(o->objectName(), "blue/0.5");
+
+    // Don't crash when retrieving the (possibly non-existent) metaobject for the script.
+    for (int i = QMetaType::User + 1; QMetaType::isRegistered(i); ++i)
+        QMetaType(i).metaObject();
+}
+
+void tst_qqmlengine::registerModule()
+{
+    // Make sure that registerModule() doesn't crash when invoked via QQmlEngine
+    QQmlEngine engine;
+    QVERIFY(engine.registerModule("magic", QJSValue(63)));
+}
+
 QTEST_MAIN(tst_qqmlengine)
 
 #include "tst_qqmlengine.moc"
