@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #ifndef Q_TEST_MODEL_H
 #define Q_TEST_MODEL_H
@@ -19,6 +19,11 @@ public:
     TestModel(int _rows, int _cols, QObject *parent = nullptr): QAbstractItemModel(parent),
        fetched(false), rows(_rows), cols(_cols), levels(INT_MAX), wrongIndex(false) { init(); }
 
+    ~TestModel()
+    {
+        delete tree;
+    }
+
     void init() {
         decorationsEnabled = false;
         alternateChildlessRows = true;
@@ -251,6 +256,8 @@ public:
     mutable bool wrongIndex;
 
     struct Node {
+        Q_DISABLE_COPY_MOVE(Node)
+
         Node *parent;
         QVector<Node *> children;
 
@@ -261,8 +268,7 @@ public:
 
         ~Node()
         {
-            foreach (Node *n, children)
-               delete n;
+            qDeleteAll(children);
         }
 
         void addRows(int row, int count)
