@@ -3074,6 +3074,42 @@ private:
     std::vector<std::vector<int>> m_list;
 };
 
+class VariantAssociationProvider : public QObject {
+
+    Q_OBJECT
+    Q_PROPERTY(QVariantMap variantMap READ getMap WRITE setMap)
+    Q_PROPERTY(QVariantHash variantHash READ getHash WRITE setHash)
+
+    QML_ELEMENT
+
+    QVariantMap m_variantMap;
+    QVariantHash m_variantHash;
+
+public:
+    VariantAssociationProvider(QObject* parent = nullptr) : QObject(parent)
+    {}
+
+    QVariantMap getMap() { return m_variantMap; }
+    void setMap(const QVariantMap& map) { m_variantMap = map; }
+
+    QVariantHash getHash() { return m_variantHash; }
+    void setHash(const QVariantHash& hash) { m_variantHash = hash; }
+
+    Q_INVOKABLE QVariantList getListOfMap() const {
+        return QVariantList{
+            QVariantMap{},
+            QVariantMap{ { "email", "Alice Smith"} }
+        };
+    }
+
+    Q_INVOKABLE QVariantList getListOfHash() const {
+        return QVariantList{
+            QVariantHash{},
+            QVariantHash{ { "email", "Alice Smith"} }
+        };
+    }
+};
+
 namespace YepNamespaceA {
 class YepAttached : public QObject
 {
@@ -3114,4 +3150,78 @@ public:
 };
 } // namespace YepNamespaceA
 
+class BindablePoint : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(QPointF point BINDABLE bindablePoint READ default WRITE default)
+public:
+    BindablePoint(QObject *parent = nullptr) : QObject(parent), m_point(QPointF(101, 102)) {}
+
+    QBindable<QPointF> bindablePoint() { return QBindable<QPointF>(&m_point); }
+
+private:
+    QProperty<QPointF> m_point;
+};
+
+class LargeRevisionBase : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(int c MEMBER m_c CONSTANT)
+    Q_PROPERTY(int d MEMBER m_d CONSTANT)
+public:
+    int m_c = 11;
+    int m_d = 12;
+};
+
+class LargeRevision : public LargeRevisionBase
+{
+    Q_OBJECT
+    QML_ELEMENT
+    Q_PROPERTY(int a MEMBER m_a CONSTANT REVISION(1, 12))
+    Q_PROPERTY(int b MEMBER m_b CONSTANT REVISION(12, 12))
+    Q_PROPERTY(int c MEMBER m_c CONSTANT REVISION(1, 12))
+    Q_PROPERTY(int d MEMBER m_d CONSTANT REVISION(12, 12))
+public:
+    int m_a = 13;
+    int m_b = 14;
+    int m_c = 15;
+    int m_d = 16;
+};
+
+class CppEnum : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    enum class Scoped { S1, S2, S3 = 5, S4 = 5 };
+    Q_ENUM(Scoped)
+    enum Unscoped { U1, U2, U3 = 5, U4 = 5 };
+    Q_ENUM(Unscoped)
+};
+
+namespace EnumNamespace {
+Q_NAMESPACE
+QML_ELEMENT
+
+enum Unscoped { U1, U2, U3 = 5, U4 = 5 };
+Q_ENUM_NS(Unscoped)
+enum class Scoped { S1, S2, S3 = 5, S4 = 5 };
+Q_ENUM_NS(Scoped)
+} // namespace EnumNamespace
+
+class ConflictingEnums : public QObject
+{
+    Q_OBJECT
+    QML_ELEMENT
+
+public:
+    enum class E1 { A = 1 };
+    Q_ENUM(E1);
+    enum E2 { A = 2 };
+    Q_ENUM(E2);
+};
+
 #endif // TESTTYPES_H
