@@ -5,17 +5,14 @@
 #include <QtQml/qqml.h>
 #include <QDebug>
 
-class MyPluginType : public QObject
+class MyPluginTypeV2_1 : public QObject
 {
     Q_OBJECT
     Q_PROPERTY(int value READ value WRITE setValue)
     Q_PROPERTY(int valueOnlyIn2 READ value WRITE setValue)
 
 public:
-    MyPluginType(QObject *parent=nullptr) : QObject(parent)
-    {
-        qWarning("import2.1 worked");
-    }
+    MyPluginTypeV2_1(QObject *parent = nullptr) : QObject(parent) { qWarning("import2.1 worked"); }
 
     int value() const { return v; }
     void setValue(int i) { v = i; }
@@ -24,22 +21,18 @@ private:
     int v;
 };
 
-
-class MyPlugin : public QQmlExtensionPlugin
+class MyPluginV2_1 : public QQmlExtensionPlugin
 {
     Q_OBJECT
     Q_PLUGIN_METADATA(IID QQmlExtensionInterface_iid)
 
 public:
-    MyPlugin()
-    {
-        qWarning("plugin2.1 created");
-    }
+    MyPluginV2_1() { qWarning("plugin2.1 created"); }
 
     void registerTypes(const char *uri) override
     {
         Q_ASSERT(QLatin1String(uri) == "org.qtproject.AutoTestQmlPluginType");
-        qmlRegisterType<MyPluginType>(uri, 2, 1, "MyPluginType");
+        qmlRegisterType<MyPluginTypeV2_1>(uri, 2, 1, "MyPluginType");
     }
 };
 
