@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
 #include <qtest.h>
 #include <qdir.h>
 #include <QtQml/qqmlengine.h>
@@ -16,11 +17,38 @@
 
 #include <QtQuickShapes/private/qquickshapesglobal_p.h>
 
-#if defined(Q_OS_MAC)
+#if defined(Q_OS_DARWIN)
 // For _PC_CASE_SENSITIVE
 #include <unistd.h>
 #endif
 
+#ifdef QT_STATIC
+Q_IMPORT_QML_PLUGIN(MyPlugin);
+Q_IMPORT_QML_PLUGIN(MyPluginV2);
+Q_IMPORT_QML_PLUGIN(MyPluginV2_1);
+Q_IMPORT_QML_PLUGIN(MyPluginV2_2);
+Q_IMPORT_QML_PLUGIN(MyChildPlugin);
+Q_IMPORT_QML_PLUGIN(MyChildPluginV2);
+Q_IMPORT_QML_PLUGIN(MyChildPluginV2_1);
+
+Q_IMPORT_QML_PLUGIN(MyMixedPlugin);
+Q_IMPORT_QML_PLUGIN(QtQuick2Plugin);
+
+Q_IMPORT_QML_PLUGIN(MyPluginWithQml);
+Q_IMPORT_QML_PLUGIN(MyMixedPluginVersion);
+Q_IMPORT_QML_PLUGIN(MyPluginNested);
+
+Q_IMPORT_QML_PLUGIN(MyPluginStrict);
+Q_IMPORT_QML_PLUGIN(MyPluginStrictV2);
+Q_IMPORT_QML_PLUGIN(MyPluginNonStrict);
+Q_IMPORT_QML_PLUGIN(MyPluginPreemptive);
+Q_IMPORT_QML_PLUGIN(MyPluginPreemptedStrict);
+Q_IMPORT_QML_PLUGIN(MyPluginInvalidNamespace);
+Q_IMPORT_QML_PLUGIN(MyPluginInvalidFirstCommand);
+
+Q_IMPORT_QML_PLUGIN(MyPluginSingleton);
+#endif
+
 #include <QtQuickTestUtils/private/testhttpserver_p.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 
@@ -29,6 +57,7 @@
 // This occurs because type registration is process-global, while the test
 // cases should really be run in proper per-process isolation.
 
+
 class tst_qqmlmoduleplugin : public QQmlDataTest
 {
     Q_OBJECT
@@ -56,9 +85,6 @@ private slots:
     void importStrictModule_data();
     void importProtectedModule();
     void importVersionedModule();
-    void importsChildPlugin();
-    void importsChildPlugin2();
-    void importsChildPlugin21();
     void parallelPluginImport();
     void multiSingleton();
     void optionalPlugin();
@@ -130,7 +156,9 @@ void registerStaticPlugin(const char *uri)
     PluginType::metaData.append(char(QT_VERSION_MAJOR));
     PluginType::metaData.append(char(QT_VERSION_MINOR));
     PluginType::metaData.append(char(qPluginArchRequirements()));
+#if QT_CONFIG(cborstreamwriter)
     PluginType::metaData.append(QCborValue(QCborMap::fromJsonObject(md)).toCbor());
+#endif
 
     auto rawMetaDataFunctor = []() -> QPluginMetaData {
         return {reinterpret_cast<const uchar *>(PluginType::metaData.constData()), size_t(PluginType::metaData.size())};
@@ -203,17 +231,31 @@ void tst_qqmlmoduleplugin::importsPlugin()
 
     QQmlEngine engine;
     engine.addImportPath(m_importsDirectory);
-    QTest::ignoreMessage(QtWarningMsg, qPrintable(QString("plugin%1 created").arg(suffix)));
-    QTest::ignoreMessage(QtWarningMsg, qPrintable(QString("import%1 worked").arg(suffix)));
-    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType' does not contain a module identifier directive - it cannot be protected from external registrations.");
+#ifndef QT_STATIC
+    const auto childPrefix =
+            qmlFile.startsWith("child") ? QLatin1String("child ") : QLatin1String("");
+    QTest::ignoreMessage(QtWarningMsg,
+                         qPrintable(QString("%1plugin%2 created").arg(childPrefix, suffix)));
+    QTest::ignoreMessage(QtWarningMsg,
+                         qPrintable(QString("%1import%2 worked").arg(childPrefix, suffix)));
+    const auto moduleURIsuffix =
+            childPrefix.empty() ? QLatin1String("") : QLatin1String(".ChildPlugin");
+    QTest::ignoreMessage(
+            QtWarningMsg,
+            qPrintable(QString("Module 'org.qtproject.AutoTestQmlPluginType%1' does not contain a "
+                               "module "
+                               "identifier directive - it cannot be protected from external "
+                               "registrations.")
+                               .arg(moduleURIsuffix)));
+#endif
     QQmlComponent component(&engine, testFileUrl(qmlFile));
-    foreach (QQmlError err, component.errors())
+    const auto errors = component.errors();
+    for (const QQmlError &err : errors)
         qWarning() << err;
     VERIFY_ERRORS(0);
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
+    std::unique_ptr<QObject> object { component.create() };
+    QVERIFY(object.get() != nullptr);
     QCOMPARE(object->property("value").toInt(),123);
-    delete object;
 }
 
 void tst_qqmlmoduleplugin::importsPlugin_data()
@@ -225,6 +267,10 @@ void tst_qqmlmoduleplugin::importsPlugin_data()
     QTest::newRow("2.0") << "2" << "works2.qml";
     QTest::newRow("2.1") << "2.1" << "works21.qml";
     QTest::newRow("2.2") << "2.2" << "works22.qml";
+
+    QTest::newRow("c1.0") << "" << "child.qml";
+    QTest::newRow("c2.0") << "2" << "child2.qml";
+    QTest::newRow("c2.1") << "2.1" << "child21.qml";
 }
 
 void tst_qqmlmoduleplugin::incorrectPluginCase()
@@ -239,20 +285,18 @@ void tst_qqmlmoduleplugin::incorrectPluginCase()
 
     QString expectedError = QLatin1String("module \"org.qtproject.WrongCase\" plugin \"PluGin\" not found");
 
-#if defined(Q_OS_MAC) || defined(Q_OS_WIN32)
-    bool caseSensitive = true;
-#if defined(Q_OS_MAC)
-    int res = pathconf(QDir::currentPath().toLatin1().constData(), _PC_CASE_SENSITIVE);
-    if (res == -1)
-        QSKIP("Could not establish case sensitivity of file system");
-    caseSensitive = res != 0 && res != -1;
-#ifdef QT_DEBUG
+#ifndef QT_STATIC
+#if defined(Q_OS_DARWIN) || defined(Q_OS_WIN32)
+#if defined(Q_OS_DARWIN)
+    bool caseSensitive =
+            pathconf(QDir::currentPath().toLatin1().constData(), _PC_CASE_SENSITIVE) == 1;
+#if QT_CONFIG(debug) && !QT_CONFIG(framework)
     QString libname = "libPluGin_debug.dylib";
 #else
     QString libname = "libPluGin.dylib";
 #endif
 #elif defined(Q_OS_WIN32)
-    caseSensitive = false;
+    bool caseSensitive = false;
     QString libname = "PluGin.dll";
 #endif
     if (!caseSensitive) {
@@ -261,7 +305,7 @@ void tst_qqmlmoduleplugin::incorrectPluginCase()
             + QLatin1Char('"');
     }
 #endif
-
+#endif
     QCOMPARE(errors.at(0).description(), expectedError);
 }
 
@@ -282,12 +326,12 @@ void tst_qqmlmoduleplugin::importPluginWithQmlFile()
     QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestPluginWithQmlFile' does not contain a module identifier directive - it cannot be protected from external registrations.");
 
     QQmlComponent component(&engine, testFileUrl(QStringLiteral("pluginWithQmlFile.qml")));
-    foreach (QQmlError err, component.errors())
+    const auto errors = component.errors();
+    for (const QQmlError &err : errors)
         qWarning() << err;
     VERIFY_ERRORS(0);
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
-    delete object;
+    std::unique_ptr<QObject> object { component.create() };
+    QVERIFY(object.get() != nullptr);
 }
 
 void tst_qqmlmoduleplugin::remoteImportWithQuotedUrl()
@@ -300,12 +344,12 @@ void tst_qqmlmoduleplugin::remoteImportWithQuotedUrl()
     component.setData(qml.toUtf8(), QUrl());
 
     QTRY_COMPARE(component.status(), QQmlComponent::Ready);
-    QObject *object = component.create();
+    std::unique_ptr<QObject> object { component.create() };
     QCOMPARE(object->property("width").toInt(), 300);
-    QVERIFY(object != nullptr);
-    delete object;
+    QVERIFY(object.get() != nullptr);
 
-    foreach (QQmlError err, component.errors())
+    const auto errors = component.errors();
+    for (const QQmlError &err : errors)
         qWarning() << err;
     VERIFY_ERRORS(0);
 }
@@ -323,12 +367,12 @@ void tst_qqmlmoduleplugin::remoteImportWithUnquotedUri()
 
 
     QTRY_COMPARE(component.status(), QQmlComponent::Ready);
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
+    std::unique_ptr<QObject> object { component.create() };
+    QVERIFY(object.get() != nullptr);
     QCOMPARE(object->property("width").toInt(), 300);
-    delete object;
 
-    foreach (QQmlError err, component.errors())
+    const auto errors = component.errors();
+    for (const QQmlError &err : errors)
         qWarning() << err;
     VERIFY_ERRORS(0);
 }
@@ -368,20 +412,18 @@ void tst_qqmlmoduleplugin::importsMixedQmlCppPlugin()
     {
     QQmlComponent component(&engine, testFileUrl(QStringLiteral("importsMixedQmlCppPlugin.qml")));
 
-    QObject *o = component.create();
-    QVERIFY2(o != nullptr, msgComponentError(component, &engine));
+    std::unique_ptr<QObject> o { component.create() };
+    QVERIFY2(o.get() != nullptr, msgComponentError(component, &engine));
     QCOMPARE(o->property("test").toBool(), true);
-    delete o;
     }
 
     {
     QQmlComponent component(&engine, testFileUrl(QStringLiteral("importsMixedQmlCppPlugin.2.qml")));
 
-    QObject *o = component.create();
-    QVERIFY2(o != nullptr, msgComponentError(component, &engine));
+    std::unique_ptr<QObject> o { component.create() };
+    QVERIFY2(o.get() != nullptr, msgComponentError(component, &engine));
     QCOMPARE(o->property("test").toBool(), true);
     QCOMPARE(o->property("test2").toBool(), true);
-    delete o;
     }
 
 
@@ -444,27 +486,30 @@ void tst_qqmlmoduleplugin::implicitQmldir()
     QList<QQmlError> errors = component.errors();
     VERIFY_ERRORS(errorFileName.toLatin1().constData());
     QTest::ignoreMessage(QtWarningMsg, "QQmlComponent: Component is not ready");
-    QObject *obj = component.create();
-    QVERIFY(!obj);
-    delete obj;
+    std::unique_ptr<QObject> obj { component.create() };
+    QVERIFY(!obj.get());
 }
 
 void tst_qqmlmoduleplugin::importsNested_data()
 {
     QTest::addColumn<QString>("file");
     QTest::addColumn<QString>("errorFile");
+    QTest::addColumn<bool>("expectGreeting");
 
-    // Note: no other test case should import the plugin used for this test, or the
-    // wrong order test will pass spuriously
-    QTest::newRow("wrongOrder") << "importsNested.1.qml" << "importsNested.1.errors.txt";
-    QTest::newRow("missingImport") << "importsNested.3.qml" << "importsNested.3.errors.txt";
-    QTest::newRow("invalidVersion") << "importsNested.4.qml" << "importsNested.4.errors.txt";
-    QTest::newRow("correctOrder") << "importsNested.2.qml" << QString();
+    // NB: The order is wrong in the sense that it tries to load the "Nested" URI first, which
+    //     is not visible in the file system. However, in the process of loading the non-nested
+    //     URI it can discover the types for the nested one and insert them in the right place.
+    //     This used to be an error but doesn't have to be.
+    QTest::newRow("wrongOrder") << "importsNested.1.qml" << QString() << true;
+    QTest::newRow("missingImport") << "importsNested.3.qml" << "importsNested.3.errors.txt" << false;
+    QTest::newRow("invalidVersion") << "importsNested.4.qml" << "importsNested.4.errors.txt" << false;
+    QTest::newRow("correctOrder") << "importsNested.2.qml" << QString() << false;
 }
 void tst_qqmlmoduleplugin::importsNested()
 {
     QFETCH(QString, file);
     QFETCH(QString, errorFile);
+    QFETCH(bool, expectGreeting);
 
     // Note: because imports are cached between test case data rows (and the plugins remain loaded),
     // these tests should really be run in new instances of the app...
@@ -481,17 +526,21 @@ void tst_qqmlmoduleplugin::importsNested()
         QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlNestedPluginType' does not contain a module identifier directive - it cannot be protected from external registrations.");
 
     QQmlComponent component(&engine, testFile(file));
-    QObject *obj = component.create();
+
+    if (expectGreeting)
+        QTest::ignoreMessage(QtDebugMsg, "Hello");
+
+    std::unique_ptr<QObject> obj { component.create() };
 
     if (errorFile.isEmpty()) {
         if (qgetenv("DEBUG") != "" && !component.errors().isEmpty())
             qWarning() << "Unexpected Errors:" << component.errors();
-        QVERIFY(obj);
-        delete obj;
+        QVERIFY(obj.get());
+        obj.reset();
     } else {
         QList<QQmlError> errors = component.errors();
         VERIFY_ERRORS(errorFile.toLatin1().constData());
-        QVERIFY(!obj);
+        QVERIFY(!obj.get());
     }
 }
 
@@ -605,17 +654,24 @@ void tst_qqmlmoduleplugin::importStrictModule_data()
         << QString();
 
     QTest::newRow("non-strict clash")
-        << "import org.qtproject.NonstrictModule 1.0\n"
-           "MyPluginType {}"
-        << "Module 'org.qtproject.NonstrictModule' does not contain a module identifier directive - it cannot be protected from external registrations."
-        << ":1:1: Cannot install element 'MyPluginType' into protected module 'org.qtproject.StrictModule' version '1'";
+            << "import QtQml\n"
+               "import org.qtproject.StrictModule 1.0 as S\n"
+               "import org.qtproject.NonstrictModule 1.0 as N\n"
+               "S.MyPluginType { Component.onCompleted: "
+               "Qt.createComponent(\"org.qtproject.NonstrictModule\", \"MyPluginType\") }"
+            << "Module 'org.qtproject.NonstrictModule' does not contain a module identifier "
+               "directive - it cannot be protected from external registrations."
+            << ":3:1: Cannot install element 'MyPluginType' into protected module "
+               "'org.qtproject.StrictModule' version '1'";
 
     QTest::newRow("non-strict preemption")
-        << "import org.qtproject.PreemptiveModule 1.0\n"
-           "import org.qtproject.PreemptedStrictModule 1.0\n"
-           "MyPluginType {}"
-        << "Module 'org.qtproject.PreemptiveModule' does not contain a module identifier directive - it cannot be protected from external registrations."
-        << ":2:1: Namespace 'org.qtproject.PreemptedStrictModule' has already been used for type registration";
+            << "import org.qtproject.PreemptiveModule 1.0\n"
+               "import org.qtproject.PreemptedStrictModule 1.0\n"
+               "MyPluginType {}"
+            << "Module 'org.qtproject.PreemptiveModule' does not contain a module identifier "
+               "directive - it cannot be protected from external registrations."
+            << ":2:1: Namespace 'org.qtproject.PreemptedStrictModule' has already been used for "
+               "type registration";
 
     QTest::newRow("invalid namespace")
         << "import org.qtproject.InvalidNamespaceModule 1.0\n"
@@ -676,57 +732,6 @@ void tst_qqmlmoduleplugin::importVersionedModule()
     QCOMPARE(component.errorString(), QString("%1:1 module \"org.qtproject.VersionedModule\" version 1.2 is not installed\n").arg(url.toString()));
 }
 
-void tst_qqmlmoduleplugin::importsChildPlugin()
-{
-    QQmlEngine engine;
-    engine.addImportPath(m_importsDirectory);
-    QTest::ignoreMessage(QtWarningMsg, "child plugin created");
-    QTest::ignoreMessage(QtWarningMsg, "child import worked");
-    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType.ChildPlugin' does not contain a module identifier directive - it cannot be protected from external registrations.");
-    QQmlComponent component(&engine, testFileUrl(QStringLiteral("child.qml")));
-    foreach (QQmlError err, component.errors())
-        qWarning() << err;
-    VERIFY_ERRORS(0);
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
-    QCOMPARE(object->property("value").toInt(),123);
-    delete object;
-}
-
-void tst_qqmlmoduleplugin::importsChildPlugin2()
-{
-    QQmlEngine engine;
-    engine.addImportPath(m_importsDirectory);
-    QTest::ignoreMessage(QtWarningMsg, "child plugin2 created");
-    QTest::ignoreMessage(QtWarningMsg, "child import2 worked");
-    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType.ChildPlugin' does not contain a module identifier directive - it cannot be protected from external registrations.");
-    QQmlComponent component(&engine, testFileUrl(QStringLiteral("child2.qml")));
-    foreach (QQmlError err, component.errors())
-        qWarning() << err;
-    VERIFY_ERRORS(0);
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
-    QCOMPARE(object->property("value").toInt(),123);
-    delete object;
-}
-
-void tst_qqmlmoduleplugin::importsChildPlugin21()
-{
-    QQmlEngine engine;
-    engine.addImportPath(m_importsDirectory);
-    QTest::ignoreMessage(QtWarningMsg, "child plugin2.1 created");
-    QTest::ignoreMessage(QtWarningMsg, "child import2.1 worked");
-    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType.ChildPlugin' does not contain a module identifier directive - it cannot be protected from external registrations.");
-    QQmlComponent component(&engine, testFileUrl(QStringLiteral("child21.qml")));
-    foreach (QQmlError err, component.errors())
-        qWarning() << err;
-    VERIFY_ERRORS(0);
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
-    QCOMPARE(object->property("value").toInt(),123);
-    delete object;
-}
-
 void tst_qqmlmoduleplugin::parallelPluginImport()
 {
     QMutexLocker locker(&PluginThatWaits::initializeEngineEntered);
@@ -780,10 +785,9 @@ void tst_qqmlmoduleplugin::multiSingleton()
     engine.addImportPath(m_importsDirectory);
     QQmlComponent component(&engine, testFileUrl("multiSingleton.qml"));
     QVERIFY2(component.isReady(), qPrintable(component.errorString()));
-    QObject *object = component.create();
-    QVERIFY(object != nullptr);
+    std::unique_ptr<QObject> object { component.create() };
+    QVERIFY(object.get() != nullptr);
     QCOMPARE(obj.objectName(), QLatin1String("first"));
-    delete object;
 }
 
 void tst_qqmlmoduleplugin::optionalPlugin()
