@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 #include <qtest.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQml/qqmlengine.h>
@@ -23,6 +23,7 @@ private slots:
 
     void insert();
     void insertMany();
+    void insertDuplicate();
     void operatorInsert();
     void operatorValue();
     void clear();
@@ -191,6 +192,34 @@ void tst_QQmlPropertyMap::insertMany()
     QCOMPARE(map.value(QStringLiteral("key1")).toInt(), 100);
 }
 
+void tst_QQmlPropertyMap::insertDuplicate()
+{
+    QHash<QString, QVariant> values;
+    values.insert(QLatin1String("key2"), 200);
+    values.insert(QLatin1String("key1"), "Hello World");
+
+    auto expectedCount = values.count();
+
+    QQmlPropertyMap map;
+    map.insert(values);
+    QCOMPARE(map.keys().size(), expectedCount);
+
+    map.insert(QStringLiteral("key2"), 24);
+    QCOMPARE(map.keys().size(), expectedCount);
+    QCOMPARE(map.value(QStringLiteral("key2")).toInt(), 24);
+
+    map.insert(QString(), QVariant("Empty1"));
+    QCOMPARE(map.keys().size(), ++expectedCount);
+    map.insert(QString(), QVariant("Empty2"));
+    QCOMPARE(map.keys().size(), expectedCount);
+
+    QHash<QString, QVariant> emptyKeyMap;
+    emptyKeyMap.insert(QString(), 200);
+    emptyKeyMap.insert(QString(), 400);
+    map.insert(emptyKeyMap);
+    QCOMPARE(map.keys().size(), expectedCount);
+}
+
 void tst_QQmlPropertyMap::operatorInsert()
 {
     QQmlPropertyMap map;
@@ -317,9 +346,8 @@ void tst_QQmlPropertyMap::controlledWrite()
     component.setData(qmlSource, QUrl::fromLocalFile(""));
     QVERIFY(component.isReady());
 
-    QObject *obj = component.create();
-    QVERIFY(obj);
-    delete obj;
+    std::unique_ptr<QObject> obj { component.create() };
+    QVERIFY(obj.get());
 
     QCOMPARE(map.value(QLatin1String("key1")), QVariant("HELLO WORLD"));
     QCOMPARE(map.value(QLatin1String("key2")), QVariant("Goodbye"));
@@ -335,8 +363,7 @@ void tst_QQmlPropertyMap::crashBug()
 
     QQmlComponent c(&engine);
     c.setData("import QtQuick 2.0\nBinding { target: map; property: \"myProp\"; value: 10 + 23 }",QUrl());
-    QObject *obj = c.create(&context);
-    delete obj;
+    std::unique_ptr<QObject> obj { c.create(&context) };
 }
 
 void tst_QQmlPropertyMap::QTBUG_17868()
@@ -349,10 +376,9 @@ void tst_QQmlPropertyMap::QTBUG_17868()
     map.insert("key", 1);
     QQmlComponent c(&engine);
     c.setData("import QtQuick 2.0\nItem {property bool error:false; Component.onCompleted: {try{ map.keys(); }catch(e) {error=true;}}}",QUrl());
-    QObject *obj = c.create(&context);
-    QVERIFY(obj);
+    std::unique_ptr<QObject> obj { c.create(&context) };
+    QVERIFY(obj.get());
     QVERIFY(!obj->property("error").toBool());
-    delete obj;
 
 }
 
@@ -405,17 +431,14 @@ void tst_QQmlPropertyMap::QTBUG_31226()
               "  Timer { interval: 5; running: true; onTriggered: { myProp = qmlPropertyMap.greeting; } }\n"
               "}",
               QUrl());
-    QObject *obj = c.create(&context);
-    QVERIFY(obj);
+    std::unique_ptr<QObject> obj { c.create(&context) };
+    QVERIFY(obj.get());
 
     QQmlPropertyMap *qmlPropertyMap = obj->findChild<QQmlPropertyMap*>(QString("qmlPropertyMap"));
     QVERIFY(qmlPropertyMap);
 
     qmlPropertyMap->insert("greeting", QString("Hello world!"));
     QTRY_COMPARE(obj->property("myProp").toString(), QString("Hello world!"));
-
-    delete obj;
-
 }
 
 void tst_QQmlPropertyMap::QTBUG_29836()
@@ -432,13 +455,10 @@ void tst_QQmlPropertyMap::QTBUG_29836()
               "  Timer { interval: 5; running: true; onTriggered: enhancedMap.testSlot() }\n"
               "}",
               QUrl());
-    QObject *obj = c.create(&context);
-    QVERIFY(obj);
+    std::unique_ptr<QObject> obj { c.create(&context) };
+    QVERIFY(obj.get());
 
     QTRY_COMPARE(map.testSlotCalled(), true);
-
-    delete obj;
-
 }
 
 void tst_QQmlPropertyMap::QTBUG_35233()
