@@ -4,6 +4,7 @@
 #include "tst_qmlls_modules.h"
 #include "QtQmlLS/private/qqmllsutils_p.h"
 #include "QtQmlLS/private/qqmlsemantictokens_p.h"
+#include "QtQmlLS/private/documentsymbolutils_p.h"
 #include <algorithm>
 #include <memory>
 #include <optional>
@@ -393,6 +394,31 @@ void tst_qmlls_modules::buildDir()
                                    QStringList({ u"QtQuick"_s, u"vector4d"_s })));
 }
 
+void tst_qmlls_modules::inMemoryEnumCompletion()
+{
+    ignoreDiagnostics();
+    const QString filePath = u"completions/SimpleItem.qml"_s;
+    const auto uri = openFile(filePath);
+    QVERIFY(uri);
+
+    // add in memory enum that does not exist on disk
+    TextDocumentContentChangeEvent change;
+    change.range = Range{ Position{ 3, 0 }, Position{ 3, 0 } };
+    change.text = "enum HelloEnum { Hello, World, Kitty, Qt }\nproperty var myP: SimpleItem.H";
+
+    DidChangeTextDocumentParams didChange;
+    didChange.textDocument.uri = *uri;
+    didChange.textDocument.version = 2;
+    didChange.contentChanges.append(change);
+    m_protocol->notifyDidChangeTextDocument(didChange);
+
+    QTEST_CHECKED(checkCompletions(*uri, 4, 29,
+                                   ExpectedCompletions({
+                                           { u"Hello"_s, CompletionItemKind::EnumMember },
+                                   }),
+                                   QStringList({ u"QtQuick"_s, u"vector4d"_s })));
+}
+
 void tst_qmlls_modules::automaticSemicolonInsertionForCompletions_data()
 {
     QTest::addColumn<QString>("filePath");
@@ -653,9 +679,12 @@ void tst_qmlls_modules::findUsages_data()
 
     // line and character start at 1!
     const QList<QLspSpecification::Location> sumUsages = {
-        locationFrom(jsIdentifierUsagesUri, jsIdentifierUsagesContent, 8, 13, strlen("sum")),
-        locationFrom(jsIdentifierUsagesUri, jsIdentifierUsagesContent, 10, 13, strlen("sum")),
-        locationFrom(jsIdentifierUsagesUri, jsIdentifierUsagesContent, 10, 19, strlen("sum")),
+        locationFrom(jsIdentifierUsagesUri, jsIdentifierUsagesContent, 8, 13,
+                     static_cast<quint32>(strlen("sum"))),
+        locationFrom(jsIdentifierUsagesUri, jsIdentifierUsagesContent, 10, 13,
+                     static_cast<quint32>(strlen("sum"))),
+        locationFrom(jsIdentifierUsagesUri, jsIdentifierUsagesContent, 10, 19,
+                     static_cast<quint32>(strlen("sum"))),
     };
     QVERIFY(sumUsages.front().uri.startsWith("file://"_ba));
 
@@ -787,6 +816,10 @@ void tst_qmlls_modules::documentFormatting_data()
     QTest::newRow("leading-and-trailing-blanklines")
             << testFile("formatting/blanklines.qml")
             << testFile("formatting/blanklines.formatted.qml");
+
+    QTest::newRow("read-qmlformat-ini")
+            << directory.filePath(u"settings/Example1.qml"_s)
+            << directory.filePath(u"settings/Example1.formatted_mac_cr.qml"_s);
 }
 
 void tst_qmlls_modules::documentFormatting()
@@ -837,6 +870,14 @@ void tst_qmlls_modules::documentFormatting()
             QCOMPARE(textEdit.range.start.character, 0);
             QCOMPARE(textEdit.range.end.line, lineCount(originalFile));
             QCOMPARE(textEdit.range.end.character, 0);
+
+            QEXPECT_FAIL("noSuperfluousSpaceInsertions.fail_id.qml",
+                         "Not all cases have been covered yet (QTBUG-133315, QTBUG-123386)",
+                         Continue);
+            QEXPECT_FAIL("noSuperfluousSpaceInsertions.fail_parameters.qml",
+                         "Not all cases have been covered yet (QTBUG-133315, QTBUG-123386)",
+                         Continue);
+
             QCOMPARE(textEdit.newText, file.readAll());
         }
     };
@@ -888,13 +929,16 @@ void tst_qmlls_modules::renameUsages_data()
                         OptionalVersionedTextDocumentIdentifier{ { jsIdentifierUsagesUri } },
                         {
                                 TextEdit{
-                                        rangeFrom(jsIdentifierUsagesContent, 8, 13, strlen("sum")),
+                                        rangeFrom(jsIdentifierUsagesContent, 8, 13,
+                                                  static_cast<quint32>(strlen("sum"))),
                                         "specialSum" },
                                 TextEdit{
-                                        rangeFrom(jsIdentifierUsagesContent, 10, 13, strlen("sum")),
+                                        rangeFrom(jsIdentifierUsagesContent, 10, 13,
+                                                  static_cast<quint32>(strlen("sum"))),
                                         "specialSum" },
                                 TextEdit{
-                                        rangeFrom(jsIdentifierUsagesContent, 10, 19, strlen("sum")),
+                                        rangeFrom(jsIdentifierUsagesContent, 10, 19,
+                                                  static_cast<quint32>(strlen("sum"))),
                                         "specialSum" },
                         } },
         }
@@ -932,7 +976,7 @@ void tst_qmlls_modules::renameUsages_data()
                             OptionalVersionedTextDocumentIdentifier{ { renameUsagesUri } },
                             {
                                     TextEdit{ rangeFrom(renamingContent, 4, 5,
-                                                        strlen("RenameMe")),
+                                                        static_cast<quint32>(strlen("RenameMe"))),
                                               "HelloWorld" },
                             } },
                     RenameFile{ "rename", renameMeUri, newFileUri } }
@@ -950,7 +994,7 @@ void tst_qmlls_modules::renameUsages_data()
                             OptionalVersionedTextDocumentIdentifier{ { renameUsagesUri } },
                             {
                                     TextEdit{ rangeFrom(renamingContent, 5, 5,
-                                                        strlen("RenameMe2")),
+                                                        static_cast<quint32>(strlen("RenameMe2"))),
                                               "HelloWorld" },
                             } },
                     RenameFile{ "rename", renameMe2Uri, newFileUri2 } }
@@ -1130,7 +1174,7 @@ struct EditingRecorder
 
 static constexpr int characterAfter(const char *line)
 {
-    return std::char_traits<char>::length(line) + 1;
+    return static_cast<int>(std::char_traits<char>::length(line) + 1);
 }
 
 static EditingRecorder propertyTypoScenario(const QByteArray &fileUri)
@@ -1168,6 +1212,23 @@ static EditingRecorder propertyTypoScenario(const QByteArray &fileUri)
     return propertyTypo;
 }
 
+static EditingRecorder inMemoryEnumScenario(const QByteArray &fileUri)
+{
+    EditingRecorder scenario;
+    scenario.lastFileUri = fileUri;
+
+    const QString enumWithTypo =
+            u"enum Hello { World, Qt, Kitty }\nproperty var xxx: SimpleItem.Q"_s;
+    // expect an error because of the missing "t"
+    scenario.changeText(5, 1, 5, 1, enumWithTypo);
+    scenario.setCurrentExpectedDiagnostic("Member \"Q\" not found");
+
+    // fix the typo and expect no more error, despite the enum not existing on the file on disk
+    scenario.changeText(6, 31, 6, 31, u"t"_s);
+
+    return scenario;
+}
+
 void tst_qmlls_modules::linting_data()
 {
     QTest::addColumn<QString>("filePath");
@@ -1176,6 +1237,10 @@ void tst_qmlls_modules::linting_data()
     QTest::addRow("property-typo")
             << u"linting/SimpleItem.qml"_s
             << propertyTypoScenario(testFileUrl(u"linting/SimpleItem.qml"_s).toEncoded());
+
+    QTest::addRow("in-memory-enum")
+            << u"linting/SimpleItem.qml"_s
+            << inMemoryEnumScenario(testFileUrl(u"linting/SimpleItem.qml"_s).toEncoded());
 }
 
 void tst_qmlls_modules::linting()
@@ -1647,15 +1712,13 @@ static QQmlJS::Dom::DomItem fileObject(const QString &filePath)
     if (!f.open(QIODevice::ReadOnly))
         return file;
     QString code = f.readAll();
-    QQmlJS::Dom::DomCreationOptions options;
-    options.setFlag(QQmlJS::Dom::DomCreationOption::WithScriptExpressions);
-    options.setFlag(QQmlJS::Dom::DomCreationOption::WithSemanticAnalysis);
-    options.setFlag(QQmlJS::Dom::DomCreationOption::WithRecovery);
 
     QStringList dirs = {QLibraryInfo::path(QLibraryInfo::Qml2ImportsPath)};
-    auto envPtr = QQmlJS::Dom::DomEnvironment::create(dirs,
+    auto envPtr = QQmlJS::Dom::DomEnvironment::create(
+            dirs,
             QQmlJS::Dom::DomEnvironment::Option::SingleThreaded
-                    | QQmlJS::Dom::DomEnvironment::Option::NoDependencies, options);
+                    | QQmlJS::Dom::DomEnvironment::Option::NoDependencies,
+            QQmlJS::Dom::Extended);
     envPtr->loadBuiltins();
     envPtr->loadFile(QQmlJS::Dom::FileToLoad::fromMemory(envPtr, filePath, code),
                         [&file](QQmlJS::Dom::Path, const QQmlJS::Dom::DomItem &, const QQmlJS::Dom::DomItem &newIt) {
@@ -1837,4 +1900,231 @@ void tst_qmlls_modules::semanticHighlightingDelta()
     }
 }
 
+static bool compareRanges(const Range &lhs, const Range &rhs)
+{
+    return lhs.start.line == rhs.start.line && lhs.start.character == rhs.start.character
+            && lhs.end.line == rhs.end.line && lhs.end.character == rhs.end.character;
+}
+
+static void debugPrint(const Range &range)
+{
+    qDebug() << '{' << range.start.line << ',' << range.start.character << '}' << ',' << '{'
+             << range.end.line << ',' << range.end.character << '}';
+}
+
+static bool compareDocumentSymbols(const DocumentSymbol &current, const DocumentSymbol &expected)
+{
+    // For details about comparison see comment in documentSymbolsExpectedResult
+
+    if (current.name.isEmpty() || current.kind == SymbolKind{})
+        return false;
+
+    if (current.tags != expected.tags || current.deprecated != expected.deprecated) {
+        return false;
+    }
+
+    using namespace QQmlJS::Dom;
+    QmlObject qmlObj;
+    DomItem qmlObjectItem = DomItem(DomEnvironment::create({})).copy(&qmlObj);
+    const auto qmlObjectKind = DocumentSymbolUtils::symbolKindOf(qmlObjectItem);
+    if (current.kind == qmlObjectKind) {
+        if (current.detail != expected.detail) {
+            qDebug() << "Current " << current.detail << " Expected " << expected.detail;
+            return false;
+        }
+    } else {
+        if (expected.detail.has_value() != current.detail.has_value()) {
+            qDebug() << current.name;
+            return false;
+        }
+    }
+    if (!compareRanges(current.range, expected.range)) {
+        qDebug() << "Current Range: ";
+        debugPrint(current.range);
+        qDebug() << " Expected ";
+        debugPrint(expected.range);
+    }
+    if (!compareRanges(current.selectionRange, expected.selectionRange)) {
+        qDebug() << "Current Selection Range: ";
+        debugPrint(current.selectionRange);
+        qDebug() << " Expected ";
+        debugPrint(expected.selectionRange);
+    }
+    // Compare children
+    if (current.children.has_value() != expected.children.has_value()) {
+        return false;
+    }
+    if (!current.children.has_value()) {
+        return true;
+    }
+    return std::equal(current.children->cbegin(), current.children->cend(),
+                      expected.children->cbegin(), expected.children->cend(),
+                      compareDocumentSymbols);
+}
+
+static bool compareDocumentSymbolsLists(const QList<DocumentSymbol> &current,
+                                        const QList<DocumentSymbol> &expected)
+{
+    return std::equal(current.cbegin(), current.cend(), expected.cbegin(), expected.cend(),
+                      compareDocumentSymbols);
+}
+
+static QList<DocumentSymbol> documentSymbolsExpectedResult()
+{
+    // What essentially needs to be compared / verified are:
+    // - ranges
+    // - there is no duplication of symbols for properties bound at definition
+    // - tags, deprecated
+    // - other detail-s, name, kind are not empty or default when they need to be present*
+    //
+    //* The last one is a bit of a short-cut**, relying that
+    // "correctness" of the assigned data is covered by unit-tests (tst_document_symbol_utils).
+    // However it still needs to be tested, that the relevant data is indeed being
+    // assigned when it should.
+    // ** I find this short-cut fair, because the feature is not so critical
+
+    // TODO(QTBUG-128277)
+
+    using namespace QQmlJS::Dom;
+    DocumentSymbol propDefA;
+    propDefA.range = { { 4, 4 }, { 4, 18 } };
+    propDefA.selectionRange = { { 4, 17 }, { 4, 18 } };
+
+    // Simple bindings---------------------------------------------------
+    DocumentSymbol bindingA;
+    bindingA.range = { { 5, 4 }, { 5, 19 } };
+    bindingA.selectionRange = { { 5, 4 }, { 5, 5 } };
+    bindingA.detail = QByteArray{};
+
+    DocumentSymbol bindingB;
+    bindingB.range = bindingB.selectionRange = { { 6, 4 }, { 6, 22 } };
+    bindingB.detail = QByteArray{};
+
+    DocumentSymbol bindingC;
+    bindingC.range = bindingC.selectionRange = { { 7, 4 }, { 7, 47 } };
+    bindingC.detail = QByteArray{};
+
+    // list<Rectangle> binding-------------------------------------------
+    DocumentSymbol bindingColorRed;
+    bindingColorRed.range = { { 11, 20 }, { 11, 32 } };
+    bindingColorRed.selectionRange = { { 11, 20 }, { 11, 25 } };
+    bindingColorRed.detail = QByteArray{};
+
+    DocumentSymbol redRect;
+    redRect.range = redRect.selectionRange = { { 11, 8 }, { 11, 34 } };
+    redRect.children = { bindingColorRed };
+
+    DocumentSymbol bindingColorBlue;
+    bindingColorBlue.range = { { 12, 20 }, { 12, 33 } };
+    bindingColorBlue.selectionRange = { { 12, 20 }, { 12, 25 } };
+    bindingColorBlue.detail = QByteArray{};
+
+    DocumentSymbol blueRect;
+    blueRect.range = blueRect.selectionRange = { { 12, 8 }, { 12, 34 } };
+    blueRect.children = { bindingColorBlue };
+
+    DocumentSymbol bindingD;
+    bindingD.range = bindingD.selectionRange = { { 10, 29 }, { 13, 5 } };
+    bindingD.children = { redRect, blueRect };
+
+    // Binding : Item{}-------------------------------------------------
+    DocumentSymbol itemE;
+    itemE.range = itemE.selectionRange = { { 14, 20 }, { 14, 26 } };
+    DocumentSymbol bindingE;
+    bindingE.range = { { 14, 17 }, { 14, 26 } };
+    bindingE.selectionRange = { { 14, 17 }, { 14, 18 } };
+    bindingE.children = { itemE };
+
+    // Methods and Signals----------------------------------------------
+    DocumentSymbol f;
+    f.range = { { 17, 4 }, { 17, 18 } };
+    f.selectionRange = { { 17, 13 }, { 17, 14 } };
+    f.detail = QByteArray{};
+
+    DocumentSymbol signalG;
+    signalG.range = { { 18, 4 }, { 18, 20 } };
+    signalG.selectionRange = { { 18, 11 }, { 18, 12 } };
+    signalG.detail = QByteArray{};
+
+    DocumentSymbol h;
+    h.range = { { 19, 4 }, { 19, 39 } };
+    h.selectionRange = { { 19, 13 }, { 19, 14 } };
+    h.detail = QByteArray{};
+
+    // Child obj & hierarchy----------------------------------------------
+    DocumentSymbol inlCmpRootObj;
+    inlCmpRootObj.detail = QString("root").toUtf8();
+    inlCmpRootObj.range = { { 23, 26 }, { 23, 33 } };
+    inlCmpRootObj.selectionRange = { { 23, 26 }, { 23, 30 } };
+
+    DocumentSymbol inlCmp;
+    inlCmp.range = { { 23, 8 }, { 23, 33 } };
+    inlCmp.selectionRange = { { 23, 18 }, { 23, 24 } };
+    inlCmp.children = { inlCmpRootObj };
+
+    // Button and Enum----------------------------------------------
+    DocumentSymbol enumMember;
+    enumMember.range = enumMember.selectionRange = { { 27, 31 }, { 27, 36 } };
+    enumMember.detail = QByteArray{};
+    DocumentSymbol enumDecl;
+    enumDecl.range = { { 27, 12 }, { 27, 38 } };
+    enumDecl.selectionRange = { { 27, 17 }, { 27, 28 } };
+    enumDecl.children = { enumMember };
+
+    DocumentSymbol buttonId;
+    // TODO(QTBUG-128277) Why there is no IdentifierRegion for this???
+    buttonId.range = buttonId.selectionRange = { { 25, 12 }, { 25, 22 } };
+    buttonId.detail = QByteArray{};
+
+    DocumentSymbol button;
+    button.detail = QString("button").toUtf8();
+    button.range = { { 24, 8 }, { 28, 9 } };
+    button.selectionRange = { { 24, 8 }, { 24, 14 } };
+    button.children = { buttonId, enumDecl };
+
+    DocumentSymbol childObj;
+    childObj.range = { { 22, 4 }, { 29, 5 } };
+    childObj.selectionRange = { { 22, 4 }, { 22, 13 } };
+    childObj.children = {
+        button, inlCmp
+    }; // during rearrangement cmp is pushed down only after button is added
+
+    DocumentSymbol rootItem;
+    rootItem.detail = QString("root").toUtf8();
+    rootItem.range = { { 2, 0 }, { 30, 1 } };
+    rootItem.selectionRange = { { 2, 0 }, { 2, 4 } };
+    rootItem.children = { propDefA, bindingA, bindingB, bindingC, bindingD,
+                          bindingE, f,        signalG,  h,        childObj };
+
+    return { rootItem };
+}
+
+void tst_qmlls_modules::documentSymbols()
+{
+    QString filePath("documentSymbol/DocumentSymbolTest.qml");
+    const auto uri = openFile(filePath);
+    QVERIFY(uri);
+    QLspSpecification::DocumentSymbolParams params;
+    params.textDocument.uri = *uri;
+
+    std::shared_ptr<bool> didFinish = std::make_shared<bool>(false);
+    const auto cleanup = [didFinish]() { *didFinish = true; };
+
+    auto &&responseHandler = [&](auto res) {
+        QScopeGuard callAtExit(cleanup);
+        const auto result = std::get_if<QList<DocumentSymbol>>(&res);
+        QVERIFY(result);
+        auto expectedResult = documentSymbolsExpectedResult();
+        compareDocumentSymbolsLists(*result, expectedResult);
+    };
+
+    auto &&errorHandler = [&](auto &error) {
+        QScopeGuard callAtExit(cleanup);
+        ProtocolBase::defaultResponseErrorHandler(error);
+    };
+
+    m_protocol->requestDocumentSymbol(params, std::move(responseHandler), std::move(errorHandler));
+    QTRY_VERIFY_WITH_TIMEOUT(*didFinish, 10000);
+}
+
 QTEST_MAIN(tst_qmlls_modules)
