@@ -1,28 +1,33 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
-#include <QtTest/QtTest>
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
+
+#include <private/qanimationgroupjob_p.h>
+#include <private/qmlutils_p.h>
+
+#include <private/qqmllistmodel_p.h>
+#include <private/qqmltimer_p.h>
+
+#include <private/qquickanimation_p_p.h>
+#include <private/qquickanimatorjob_p.h>
+#include <private/qquickflickable_p.h>
+#include <private/qquickframeanimation_p.h>
+#include <private/qquickitem_p.h>
+#include <private/qquickitemanimation_p.h>
+#include <private/qquickpathinterpolator_p.h>
+#include <private/qquickrectangle_p.h>
+#include <private/qquicktransition_p.h>
+
+#include <QtQuick/qquickview.h>
+
+#include <QtTest/qtest.h>
+#include <QtTest/qsignalspy.h>
+
+#include <QtQuickTestUtils/private/visualtestutils_p.h>
+
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlcomponent.h>
-#include <QtQuick/qquickview.h>
-#include <QtQml/private/qqmltimer_p.h>
-#include <QtQmlModels/private/qqmllistmodel_p.h>
-#include <QtQml/private/qanimationgroupjob_p.h>
-#include <QtQuick/private/qquickrectangle_p.h>
-#include <QtQuick/private/qquickitemanimation_p.h>
-#include <QtQuick/private/qquickitemanimation_p_p.h>
-#include <QtQuick/private/qquicktransition_p.h>
-#include <QtQuick/private/qquickanimation_p.h>
-#include <QtQuick/private/qquickanimatorjob_p.h>
-#include <QtQuick/private/qquickpathinterpolator_p.h>
-#include <QtQuick/private/qquickitem_p.h>
-#include <QtQuick/private/qquicklistview_p.h>
-#include <QtQuick/private/qquickframeanimation_p.h>
-#include <QEasingCurve>
 
-#include <limits.h>
-#include <math.h>
-
-#include <QtQuickTestUtils/private/qmlutils_p.h>
+#include <QtCore/qeasingcurve.h>
 
 class tst_qquickanimations : public QQmlDataTest
 {
@@ -98,6 +103,11 @@ private slots:
     void infiniteLoopsWithoutFrom();
     void frameAnimation1();
     void frameAnimation2();
+    void restartAnimationGroupWhenDirty();
+    void restartNestedAnimationGroupWhenDirty();
+    void targetsDeletedNotRemoved();
+    void alwaysRunToEndSetFalseRestartBug();
+    void animationInstantiator();
 };
 
 #define QTIMED_COMPARE(lhs, rhs) do { \
@@ -1293,7 +1303,7 @@ void tst_qquickanimations::startStopSignals()
 
     root->setProperty("alwaysRunToEnd", true);
 
-    timer.restart();
+    timer.start();
     QMetaObject::invokeMethod(root, "start");
 
     QCOMPARE(root->property("startedCount").toInt(), 3);
@@ -1844,8 +1854,7 @@ void tst_qquickanimations::fastFlickingBug()
 
 void tst_qquickanimations::opacityAnimationFromZero()
 {
-    if (QGuiApplication::platformName() == QLatin1String("minimal"))
-        QSKIP("Skipping due to grabWindow not functional on minimal platforms");
+    SKIP_IF_NO_WINDOW_GRAB;
 
     // not easy to verify this in threaded render loop
     // since it's difficult to capture the first frame when scene graph
@@ -2212,6 +2221,131 @@ void tst_qquickanimations::frameAnimation2()
     QVERIFY(frameAnimation->currentFrame() > 3);
 }
 
+//QTBUG-110589
+void tst_qquickanimations::restartAnimationGroupWhenDirty()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("restartAnimationGroupWhenDirty.qml"));
+    QScopedPointer<QObject> obj(c.create());
+    auto *root = qobject_cast<QQuickRectangle*>(obj.data());
+    QVERIFY2(root, qPrintable(c.errorString()));
+
+    QQuickSequentialAnimation *seqAnim0 = root->findChild<QQuickSequentialAnimation*>("seqAnim0");
+    QVERIFY(seqAnim0);
+    QQuickRectangle *target0 = root->findChild<QQuickRectangle*>("target0");
+    QVERIFY(target0);
+    QQuickParallelAnimation *parAnim0 = root->findChild<QQuickParallelAnimation*>("parAnim0");
+    QVERIFY(parAnim0);
+    QQuickRectangle *target1 = root->findChild<QQuickRectangle*>("target1");
+    QVERIFY(target1);
+
+    QTRY_VERIFY(seqAnim0->isPaused());
+    QTRY_VERIFY(parAnim0->isPaused());
+    QTRY_VERIFY(target0->x() > 140);
+    QTRY_VERIFY(target1->x() > 140);
+    seqAnim0->resume();
+    parAnim0->resume();
+    QTRY_VERIFY(target0->property("onFinishedCalled").value<bool>());
+    QTRY_VERIFY(target1->property("onFinishedCalled").value<bool>());
+    QTRY_COMPARE(target0->x(), 140);
+    QTRY_COMPARE(target1->x(), 140);
+}
+
+//QTBUG-95840
+void tst_qquickanimations::restartNestedAnimationGroupWhenDirty()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("restartNestedAnimationGroupWhenDirty.qml"));
+    QScopedPointer<QObject> obj(c.create());
+    auto *root = qobject_cast<QQuickRectangle*>(obj.data());
+    QVERIFY2(root, qPrintable(c.errorString()));
+
+    QQuickSequentialAnimation *seqAnim0 = root->findChild<QQuickSequentialAnimation*>("seqAnim0");
+    QVERIFY(seqAnim0);
+    QQuickRectangle *target0 = root->findChild<QQuickRectangle*>("target0");
+    QVERIFY(target0);
+    QQuickParallelAnimation *parAnim0 = root->findChild<QQuickParallelAnimation*>("parAnim0");
+    QVERIFY(parAnim0);
+    QQuickRectangle *target1 = root->findChild<QQuickRectangle*>("target1");
+    QVERIFY(target1);
+
+    QTRY_VERIFY(seqAnim0->isPaused());
+    QTRY_VERIFY(parAnim0->isPaused());
+    QTRY_VERIFY(target0->x() > 140);
+    QTRY_VERIFY(target1->x() > 140);
+    seqAnim0->resume();
+    parAnim0->resume();
+    QTRY_VERIFY(target0->property("onFinishedCalled").value<bool>());
+    QTRY_VERIFY(target1->property("onFinishedCalled").value<bool>());
+    QTRY_COMPARE(target0->x(), 140);
+    QTRY_COMPARE(target1->x(), 140);
+}
+
+void tst_qquickanimations::targetsDeletedNotRemoved()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("targetsDeletedWithoutRemoval.qml"));
+    QScopedPointer<QObject> obj(component.create());
+    QVERIFY2(obj.get(), qPrintable(component.errorString()));
+    {
+        QQmlListReference ref(obj.get(), "targets");
+        QVERIFY(ref.isValid());
+        QCOMPARE(ref.size(), 1);
+        QTRY_COMPARE(ref.at(0), nullptr);
+    }
+    {
+        QQmlListReference ref(obj.get(), "animTargets");
+        QVERIFY(ref.isValid());
+        QCOMPARE(ref.size(), 1);
+        QCOMPARE(ref.at(0), nullptr);
+    }
+}
+
+//QTBUG-125224
+void tst_qquickanimations::alwaysRunToEndSetFalseRestartBug()
+{
+    QQuickRectangle rect;
+    QQuickSequentialAnimation sequential;
+    QQuickPropertyAnimation beginAnim;
+    QQuickPropertyAnimation endAnim;
+
+    beginAnim.setTargetObject(&rect);
+    beginAnim.setProperty("x");
+    beginAnim.setTo(200);
+    beginAnim.setDuration(1000);
+
+    endAnim.setTargetObject(&rect);
+    endAnim.setProperty("x");
+    endAnim.setFrom(200);
+    endAnim.setDuration(1000);
+
+    beginAnim.setGroup(&sequential);
+    endAnim.setGroup(&sequential);
+
+    sequential.setLoops(-1);
+    sequential.setAlwaysRunToEnd(true);
+
+    QCOMPARE(sequential.loops(), -1);
+    QVERIFY(sequential.alwaysRunToEnd());
+    sequential.start();
+    sequential.stop();
+    sequential.setAlwaysRunToEnd(false);
+    sequential.start();
+    QCOMPARE(sequential.isRunning(), true);
+    sequential.stop();
+    QCOMPARE(sequential.isRunning(), false);
+}
+
+void tst_qquickanimations::animationInstantiator()
+{
+    QQmlEngine engine;
+    QQmlComponent c(&engine, testFileUrl("animationInstantiator.qml"));
+    QVERIFY2(c.isReady(), qPrintable(c.errorString()));
+    QScopedPointer<QObject> o(c.create());
+    QVERIFY(!o.isNull());
+    QTRY_VERIFY(o->property("v").toInt() > 10);
+}
+
 QTEST_MAIN(tst_qquickanimations)
 
 #include "tst_qquickanimations.moc"
