@@ -1,5 +1,5 @@
 // Copyright (C) 2016 The Qt Company Ltd.
-// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 #include <qtest.h>
 #include <QTextDocument>
 #include <QTcpServer>
@@ -45,6 +45,7 @@ private slots:
     void sciSource();
     void sciSource_data();
     void invalidSciFile();
+    void nonExistingSciFile();
     void validSciFiles_data();
     void validSciFiles();
     void pendingRemoteRequest();
@@ -111,13 +112,6 @@ void tst_qquickborderimage::imageSource()
     QFETCH(bool, remote);
     QFETCH(QString, error);
 
-#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
-    if (qstrcmp(QTest::currentDataTag(), "remote") == 0
-        || qstrcmp(QTest::currentDataTag(), "remote not found") == 0) {
-        QSKIP("Remote tests cause occasional hangs in the CI system -- QTBUG-45655");
-    }
-#endif
-
     TestHTTPServer server;
     if (remote) {
         QVERIFY2(server.listen(), qPrintable(server.errorString()));
@@ -342,6 +336,21 @@ void tst_qquickborderimage::invalidSciFile()
     delete obj;
 }
 
+void tst_qquickborderimage::nonExistingSciFile()
+{
+    const QString componentStr = "import QtQuick 2.0\nBorderImage { source: \"" + testFileUrl("this_file_does_not_exist.sci").toString() +"\"; width: 300; height: 300 }";
+    QQmlComponent component(&engine);
+    component.setData(componentStr.toLatin1(), QUrl::fromLocalFile(""));
+    std::unique_ptr<QObject> obj(component.create());
+    auto bimage = qobject_cast<QQuickBorderImage *>(obj.get());
+    QVERIFY(bimage != nullptr);
+    QCOMPARE(bimage->width(), 300.);
+    QCOMPARE(bimage->height(), 300.);
+    QCOMPARE(bimage->status(), QQuickImageBase::Error);
+    QCOMPARE(bimage->horizontalTileMode(), QQuickBorderImage::Stretch);
+    QCOMPARE(bimage->verticalTileMode(), QQuickBorderImage::Stretch);
+}
+
 void tst_qquickborderimage::validSciFiles_data()
 {
     QTest::addColumn<QString>("source");
@@ -566,8 +575,7 @@ void tst_qquickborderimage::progressAndStatusChanges()
 #if QT_CONFIG(opengl)
 void tst_qquickborderimage::borderImageMesh()
 {
-    if (QGuiApplication::platformName() == QLatin1String("minimal"))
-        QSKIP("Skipping due to grabWindow not functional on minimal platforms");
+    SKIP_IF_NO_WINDOW_GRAB;
 
     QQuickView *window = new QQuickView;
 
@@ -599,8 +607,7 @@ void tst_qquickborderimage::multiFrame_data()
 
 void tst_qquickborderimage::multiFrame()
 {
-    if (QGuiApplication::platformName() == QLatin1String("minimal"))
-        QSKIP("Skipping due to grabWindow not functional on minimal platforms");
+    SKIP_IF_NO_WINDOW_GRAB;
 
     QFETCH(QString, qmlfile);
     QFETCH(bool, asynchronous);
