@@ -1,13 +1,15 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
+#include <QtQmlModels/private/qqmlobjectmodel_p.h>
 #include <QtQuick/qquickview.h>
 #include <QtQuick/private/qquickitemview_p_p.h>
 #include <QtQuick/private/qquicklistview_p.h>
 #include <QtQuickTest/QtQuickTest>
 #include <QStringListModel>
 #include <QQmlApplicationEngine>
+#include <QtQml/QQmlComponent>
 
 #include <QtQuickTestUtils/private/viewtestutils_p.h>
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
@@ -18,6 +20,8 @@ Q_LOGGING_CATEGORY(lcTests, "qt.quick.tests")
 using namespace QQuickViewTestUtils;
 using namespace QQuickVisualTestUtils;
 
+static const int oneSecondInMs = 1000;
+
 class tst_QQuickListView2 : public QQmlDataTest
 {
     Q_OBJECT
@@ -35,6 +39,7 @@ private slots:
     void delegateModelRefresh();
     void wheelSnap();
     void wheelSnap_data();
+    void nestedWheelSnap();
 
     void sectionsNoOverlap();
     void metaSequenceAsModel();
@@ -67,10 +72,17 @@ private slots:
     void bindingDirectlyOnPositionInHeaderAndFooterDelegates();
 
     void clearObjectListModel();
+    void gadgetModelSections();
+    void visibleBoundToCountGreaterThanZero();
+
+    void setDelegateAfterModel();
+
+    void removeAndDestroyObjectModelItem_data();
+    void removeAndDestroyObjectModelItem();
 
 private:
     void flickWithTouch(QQuickWindow *window, const QPoint &from, const QPoint &to);
-    QScopedPointer<QPointingDevice> touchDevice = QScopedPointer<QPointingDevice>(QTest::createTouchDevice());
+    std::unique_ptr<QPointingDevice> touchscreen{QTest::createTouchDevice()};
 };
 
 tst_QQuickListView2::tst_QQuickListView2()
@@ -458,6 +470,7 @@ void tst_QQuickListView2::tapDelegateDuringFlicking() // QTBUG-103832
     flickWithTouch(&window, {100, 400}, {100, 100});
     QTRY_VERIFY(listView->contentY() > 501); // let it flick some distance
     QVERIFY(listView->isFlicking()); // we want to test the case when it's still moving while we tap
+    QVERIFY(listView->isMoving());
     // @y = 400 we pressed the 4th delegate; started flicking, and the press was canceled
     QCOMPARE(listView->property("pressedDelegates").toList().first(), 4);
     // At first glance one would expect MouseArea and Button would be consistent about this;
@@ -472,10 +485,20 @@ void tst_QQuickListView2::tapDelegateDuringFlicking() // QTBUG-103832
     QCOMPARE(listView->property("releasedDelegates").toList().size(), 0);
 
     // press a delegate during flicking (at y > 501 + 100, so likely delegate 6)
-    QTest::touchEvent(&window, touchDevice.data()).press(0, {100, 100});
+    QTest::touchEvent(&window, touchscreen.get()).press(0, {100, 100});
     QQuickTouchUtils::flush(&window);
-    QTest::touchEvent(&window, touchDevice.data()).release(0, {100, 100});
+    // The press will stop listView from flicking, but it will still be "moving", which
+    // means that the user is still interacting with it
+    QVERIFY(!listView->isFlicking());
+    QVERIFY(!listView->isDragging());
+    QVERIFY(listView->isMoving());
+    QTest::touchEvent(&window, touchscreen.get()).release(0, {100, 100});
     QQuickTouchUtils::flush(&window);
+    // Releasing while "flicking" is false will stop the flicking
+    // session, and set "moving" to false
+    QVERIFY(!listView->isMoving());
+    QVERIFY(!listView->isFlicking());
+    QVERIFY(!listView->isDragging());
 
     const QVariantList pressedDelegates = listView->property("pressedDelegates").toList();
     const QVariantList releasedDelegates = listView->property("releasedDelegates").toList();
@@ -487,11 +510,13 @@ void tst_QQuickListView2::tapDelegateDuringFlicking() // QTBUG-103832
     qCDebug(lcTests) << "tapped" << tappedDelegates;
     qCDebug(lcTests) << "canceled" << canceledDelegates;
 
-    // which delegate received the second press, during flicking?
+    // Since the flickable was already moving (that is, the user was interacting with
+    // it), the second tap was only used to stop the flicking. Hence, no delegates
+    // received any pointer events.
     const int lastPressed = pressedDelegates.last().toInt();
-    QVERIFY(lastPressed > 5);
-    QCOMPARE(releasedDelegates.last(), lastPressed);
-    QCOMPARE(tappedDelegates.last(), lastPressed);
+    QCOMPARE(lastPressed, 4); // Nothing changed since the beginning
+    QVERIFY(releasedDelegates.isEmpty());
+    QVERIFY(tappedDelegates.isEmpty());
     QCOMPARE(canceledDelegates.size(), expectCanceled ? 1 : 0); // only the first press was canceled, not the second
 }
 
@@ -537,38 +562,38 @@ void tst_QQuickListView2::flickDuringFlicking() // QTBUG-103832
 
     // flick again during flicking, and make sure that it doesn't jump back to the first delegate,
     // but flicks incrementally further from the position at that time
-    QTest::touchEvent(&window, touchDevice.data()).press(0, {100, 400});
+    QTest::touchEvent(&window, touchscreen.get()).press(0, {100, 400});
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "second press: contentY" << posBeforeSecondFlick << "->" << listView->contentY();
     qCDebug(lcTests) << "pressed delegates" << listView->property("pressedDelegates").toList();
     QVERIFY(listView->contentY() >= posBeforeSecondFlick);
 
     QTest::qWait(20);
-    QTest::touchEvent(&window, touchDevice.data()).move(0, {100, 300});
+    QTest::touchEvent(&window, touchscreen.get()).move(0, {100, 300});
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "first move after second press: contentY" << posBeforeSecondFlick << "->" << listView->contentY();
     QVERIFY(listView->contentY() >= posBeforeSecondFlick);
 
     QTest::qWait(20);
-    QTest::touchEvent(&window, touchDevice.data()).move(0, {100, 200});
+    QTest::touchEvent(&window, touchscreen.get()).move(0, {100, 200});
     QQuickTouchUtils::flush(&window);
     qCDebug(lcTests) << "second move after second press: contentY" << posBeforeSecondFlick << "->" << listView->contentY();
     QVERIFY(listView->contentY() >= posBeforeSecondFlick + 100);
 
-    QTest::touchEvent(&window, touchDevice.data()).release(0, {100, 100});
+    QTest::touchEvent(&window, touchscreen.get()).release(0, {100, 100});
 }
 
 void tst_QQuickListView2::flickWithTouch(QQuickWindow *window, const QPoint &from, const QPoint &to)
 {
-    QTest::touchEvent(window, touchDevice.data()).press(0, from, window);
+    QTest::touchEvent(window, touchscreen.get()).press(0, from, window);
     QQuickTouchUtils::flush(window);
 
     QPoint diff = to - from;
     for (int i = 1; i <= 8; ++i) {
-        QTest::touchEvent(window, touchDevice.data()).move(0, from + i * diff / 8, window);
+        QTest::touchEvent(window, touchscreen.get()).move(0, from + i * diff / 8, window);
         QQuickTouchUtils::flush(window);
     }
-    QTest::touchEvent(window, touchDevice.data()).release(0, to, window);
+    QTest::touchEvent(window, touchscreen.get()).release(0, to, window);
     QQuickTouchUtils::flush(window);
 }
 
@@ -583,7 +608,7 @@ void tst_QQuickListView2::singletonModelLifetime()
 {
     // this does not really test any functionality of listview, but we do not have a good way
     // to unit test QQmlAdaptorModel in isolation.
-    qmlRegisterSingletonType<SingletonModel>("test", 1, 0, "SingletonModel",
+    qmlRegisterSingletonType<SingletonModel>("SingletonModelLifeTimeTest", 1, 0, "SingletonModel",
             [](QQmlEngine* , QJSEngine*) -> QObject* { return new SingletonModel; });
 
     QQmlApplicationEngine engine(testFile("singletonModelLifetime.qml"));
@@ -870,6 +895,67 @@ void tst_QQuickListView2::wheelSnap_data()
             << 210.0;
 }
 
+void tst_QQuickListView2::nestedWheelSnap()
+{
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl("nestedSnap.qml")));
+
+    quint64 timestamp = 10;
+    auto sendWheelEvent = [&timestamp, &window](const QPoint &pixelDelta, Qt::ScrollPhase phase) {
+        const QPoint pos(100, 100);
+        QWheelEvent event(pos, window.mapToGlobal(pos), pixelDelta, pixelDelta, Qt::NoButton,
+                          Qt::NoModifier, phase, false, Qt::MouseEventSynthesizedBySystem);
+        event.setAccepted(false);
+        event.setTimestamp(timestamp);
+        QGuiApplication::sendEvent(&window, &event);
+        timestamp += 50;
+    };
+
+    QQuickListView *outerListView = qobject_cast<QQuickListView *>(window.rootObject());
+    QTRY_VERIFY(outerListView);
+    QSignalSpy outerCurrentIndexSpy(outerListView, &QQuickListView::currentIndexChanged);
+    int movingAtIndex = -1;
+
+    // send horizontal pixel-delta wheel events with phases; confirm that ListView hits the next item boundary
+    sendWheelEvent({}, Qt::ScrollBegin);
+    for (int i = 1; i < 4; ++i) {
+        sendWheelEvent({-50, 0}, Qt::ScrollUpdate);
+        if (movingAtIndex < 0 && outerListView->isMoving())
+            movingAtIndex = i;
+    }
+    QVERIFY(outerListView->isDragging());
+    sendWheelEvent({}, Qt::ScrollEnd);
+    QCOMPARE(outerListView->isDragging(), false);
+    QTRY_COMPARE(outerListView->isMoving(), false); // wait until it stops
+    qCDebug(lcTests) << "outer got moving after" << movingAtIndex
+                     << "horizontal events; stopped at" << outerListView->contentX() << outerListView->currentIndex();
+    QCOMPARE_GT(movingAtIndex, 0);
+    QCOMPARE(outerListView->contentX(), 300);
+    QCOMPARE(outerCurrentIndexSpy.size(), 1);
+
+    movingAtIndex = -1;
+    QQuickListView *innerListView = qobject_cast<QQuickListView *>(outerListView->currentItem());
+    QTRY_VERIFY(innerListView);
+    QSignalSpy innerCurrentIndexSpy(innerListView, &QQuickListView::currentIndexChanged);
+
+    // send vertical pixel-delta wheel events with phases; confirm that ListView hits the next item boundary
+    sendWheelEvent({}, Qt::ScrollBegin);
+    for (int i = 1; i < 4; ++i) {
+        sendWheelEvent({0, -50}, Qt::ScrollUpdate);
+        if (movingAtIndex < 0 && innerListView->isMoving())
+            movingAtIndex = i;
+    }
+    QVERIFY(innerListView->isDragging());
+    sendWheelEvent({}, Qt::ScrollEnd);
+    QCOMPARE(innerListView->isDragging(), false);
+    QTRY_COMPARE(innerListView->isMoving(), false); // wait until it stops
+    qCDebug(lcTests) << "inner got moving after" << movingAtIndex
+                     << "vertical events; stopped at" << innerListView->contentY() << innerListView->currentIndex();
+    QCOMPARE_GT(movingAtIndex, 0);
+    QCOMPARE(innerListView->contentY(), 300);
+    QCOMPARE(innerCurrentIndexSpy.size(), 1);
+}
+
 class FriendlyItemView : public QQuickItemView
 {
     friend class ItemViewAccessor;
@@ -1250,6 +1336,111 @@ void tst_QQuickListView2::clearObjectListModel()
     QVERIFY(!list.itemAtIndex(0));
 }
 
+void tst_QQuickListView2::gadgetModelSections()
+{
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl("gadgetlist.qml")));
+    QQuickListView *listview = qobject_cast<QQuickListView*>(window.rootObject());
+    QTRY_VERIFY(listview);
+    QVERIFY(QQuickTest::qWaitForPolish(listview));
+    QCOMPARE(listview->count(), 4);
+    QCOMPARE(listview->currentSection(), "small");
+}
+
+void tst_QQuickListView2::visibleBoundToCountGreaterThanZero()
+{
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl("visibleBoundToCountGreaterThanZero.qml")));
+
+    auto *listView = window.rootObject()->property("listView").value<QQuickListView *>();
+    QVERIFY(listView);
+
+    QSignalSpy countChangedSpy(listView, SIGNAL(countChanged()));
+    QVERIFY(countChangedSpy.isValid());
+
+    QTRY_COMPARE_GT_WITH_TIMEOUT(listView->count(), 1, oneSecondInMs);
+    // Using the TRY variant here as well is necessary.
+    QTRY_COMPARE_GT_WITH_TIMEOUT(countChangedSpy.count(), 1, oneSecondInMs);
+    QVERIFY(listView->isVisible());
+}
+
+void tst_QQuickListView2::setDelegateAfterModel()
+{
+    QQmlEngine engine;
+    const QUrl url = testFileUrl("setDelegateAfterModel.qml");
+    QQmlComponent component(&engine, url);
+    QVERIFY2(component.isReady(), qPrintable(component.errorString()));
+
+    QScopedPointer<QObject> object(component.create());
+    QVERIFY(!object.isNull());
+
+    // If the model was lost by setting the delegate, the count would be 0.
+    QCOMPARE(object->property("count").toInt(), 3);
+
+    QTest::ignoreMessage(
+            QtWarningMsg,
+            qPrintable(url.toString() + ":17:5: QML ListView: Cannot retain explicitly set "
+                                        "delegate on non-DelegateModel"));
+    object->setProperty("useObjectModel", QVariant::fromValue<bool>(true));
+    QCOMPARE(object->property("count").toInt(), 2);
+
+    // The old model must not mess with the view anymore.
+    QTest::failOnWarning(qPrintable(url.toString() + ":17:5: QML ListView: Explicitly set delegate "
+                                                     "is externally overridden"));
+    QMetaObject::invokeMethod(object.data(), "plantDelegate");
+    QCOMPARE(object->property("count").toInt(), 4);
+}
+
+enum RemovalPolicy {
+    RemoveAndDestroy,
+    OnlyDestroy
+};
+
+void tst_QQuickListView2::removeAndDestroyObjectModelItem_data()
+{
+    QTest::addColumn<QString>("qmlFilePath");
+    QTest::addColumn<RemovalPolicy>("removalPolicy");
+
+    QTest::addRow("remove and destroy, no transitions")
+        << "removeAndDestroyObjectModelItem.qml" << RemoveAndDestroy;
+    QTest::addRow("destroy, no transitions")
+        << "removeAndDestroyObjectModelItem.qml" << OnlyDestroy;
+    QTest::addRow("remove and destroy, transitions")
+        << "removeAndDestroyObjectModelItemWithTransitions.qml" << RemoveAndDestroy;
+    QTest::addRow("destroy, transitions")
+        << "removeAndDestroyObjectModelItemWithTransitions.qml" << OnlyDestroy;
+}
+
+// QTBUG-46798, QTBUG-133256
+void tst_QQuickListView2::removeAndDestroyObjectModelItem()
+{
+    QFETCH(QString, qmlFilePath);
+    QFETCH(RemovalPolicy, removalPolicy);
+
+    QQuickView window;
+    QVERIFY(QQuickTest::showView(window, testFileUrl(qmlFilePath)));
+
+    QQuickListView *listView = qobject_cast<QQuickListView*>(window.rootObject());
+    QVERIFY(listView);
+
+    auto *objectModel = listView->model().value<QQmlObjectModel *>();
+    QVERIFY(objectModel);
+    QPointer<QObject> firstItem = objectModel->get(0);
+    QVERIFY(firstItem);
+    // Shouldn't crash.
+    if (removalPolicy == RemoveAndDestroy)
+        objectModel->remove(0);
+    firstItem->deleteLater();
+    QTRY_VERIFY(!firstItem);
+
+    // Now try moving the view. It also shouldn't crash.
+    if (removalPolicy == RemoveAndDestroy) {
+        if (QQuickTest::qIsPolishScheduled(listView))
+            QVERIFY(QQuickTest::qWaitForPolish(listView));
+    }
+    listView->positionViewAtEnd();
+}
+
 QTEST_MAIN(tst_QQuickListView2)
 
 #include "tst_qquicklistview2.moc"
