@@ -11,6 +11,7 @@
 #include <private/qquickrectangle_p.h>
 
 #include <QtQuickTestUtils/private/qmlutils_p.h>
+#include <QtQuickTestUtils/private/visualtestutils_p.h>
 
 class tst_qquickrectangle : public QQmlDataTest
 {
@@ -42,6 +43,7 @@ void tst_qquickrectangle::color()
 #ifdef Q_OS_ANDROID
     QSKIP("Test does not work on Android because of QTBUG-102345");
 #endif
+    SKIP_IF_NO_WINDOW_GRAB;
 
     if (QGuiApplication::primaryScreen()->depth() < 24)
         QSKIP("This test does not work at display depths < 24");
@@ -52,9 +54,6 @@ void tst_qquickrectangle::color()
 
     QVERIFY(QTest::qWaitForWindowExposed(&view));
 
-    if (QGuiApplication::platformName() == QLatin1String("minimal"))
-        QSKIP("Skipping due to grabWindow not functional on offscreen/minimal platforms");
-
     QImage image = view.grabWindow();
     QVERIFY(image.pixel(0,0) == QColor("#020202").rgba());
 }
