@@ -36,11 +36,14 @@ public:
     bool setData(const QModelIndex &index, const QVariant &value, int role = Qt::EditRole) override;
     QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const override;
     QModelIndex parent(const QModelIndex &index) const override;
+    Qt::ItemFlags flags(const QModelIndex &index) const override;
     int maxDepth() { return 4; }
 
     bool insertRows(int position, int rows, const QModelIndex &parent) override;
     bool insertColumns(int position, int cols, const QModelIndex &parent) override;
 
+    QList<QModelIndex> m_editableIndices;
+
 private:
     QScopedPointer<TreeItem> m_rootItem;
     int m_columnCount = 5;
