@@ -42,6 +42,7 @@ public:
     }
 
 private slots:
+    void defaults();
     void changingWritingSystem();
     void clickAroundInTheFamilyListView();
     void settingUnderlineAndStrikeoutEffects();
@@ -49,7 +50,6 @@ private slots:
     void changeDialogTitle();
     void searchFamily();
     void setCurrentFontFromApi();
-    void sensibleDefaults();
 
 private:
     QQuickAbstractButton *findDialogButton(QQuickDialogButtonBox *box, const QString &buttonText)
@@ -116,6 +116,45 @@ tst_QQuickFontDialogImpl::tst_QQuickFontDialogImpl()
 {
 }
 
+void tst_QQuickFontDialogImpl::defaults()
+{
+    QTest::failOnWarning(QRegularExpression(".*"));
+    CREATE_DIALOG_TEST_HELPER
+
+    // Open the dialog.
+    QVERIFY(dialogHelper.openDialog());
+    QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+
+    QQuickListView *fontFamilyListView = dialogHelper.quickDialog->findChild<QQuickListView *>("familyListView");
+    QVERIFY(fontFamilyListView);
+    QQuickListView *fontStyleListView = dialogHelper.quickDialog->findChild<QQuickListView *>("styleListView");
+    QVERIFY(fontStyleListView);
+    QQuickListView *fontSizeListView = dialogHelper.quickDialog->findChild<QQuickListView *>("sizeListView");
+    QVERIFY(fontSizeListView);
+    QQuickTextField *fontFamilyTextField = dialogHelper.quickDialog->findChild<QQuickTextField *>("familyEdit");
+    QVERIFY(fontFamilyTextField);
+    QQuickTextField *fontStyleTextField = dialogHelper.quickDialog->findChild<QQuickTextField *>("styleEdit");
+    QVERIFY(fontStyleTextField);
+
+    // The indexes should initially be 0.
+    QCOMPARE(fontFamilyListView->currentIndex(), 0);
+    QCOMPARE(fontStyleListView->currentIndex(), 0);
+    QCOMPARE(fontSizeListView->currentIndex(), 0);
+
+    // Setting the family to an empty model
+    fontFamilyListView->setModel(QStringList());
+
+    // The listviews for font family and style should now have an empty model with indexes set to -1
+    QVERIFY(fontFamilyListView->model().toStringList().isEmpty());
+    QVERIFY(fontStyleListView->model().toStringList().isEmpty());
+    QCOMPARE(fontFamilyListView->currentIndex(), -1);
+    QCOMPARE(fontStyleListView->currentIndex(), -1);
+    QVERIFY(fontFamilyTextField->text().isEmpty());
+    QVERIFY(fontStyleTextField->text().isEmpty());
+
+    CLOSE_DIALOG("Ok");
+}
+
 void tst_QQuickFontDialogImpl::changingWritingSystem()
 {
     CREATE_DIALOG_TEST_HELPER
@@ -123,6 +162,7 @@ void tst_QQuickFontDialogImpl::changingWritingSystem()
     // Open the dialog.
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     const QQuickTextEdit *sampleEdit =
             dialogHelper.quickDialog->findChild<QQuickTextEdit *>("sampleEdit");
@@ -140,7 +180,7 @@ void tst_QQuickFontDialogImpl::changingWritingSystem()
     const QQuickComboBox *writingSystemComboBox = dialogHelper.quickDialog->findChild<QQuickComboBox*>();
     QVERIFY(writingSystemComboBox);
     const QPoint comboBoxCenterPos = writingSystemComboBox->mapToScene({ writingSystemComboBox->width() / 2, writingSystemComboBox->height() / 2 }).toPoint();
-    QTest::mouseClick(dialogHelper.window(), Qt::LeftButton, Qt::NoModifier, comboBoxCenterPos);
+    QTest::mouseClick(dialogHelper.popupWindow(), Qt::LeftButton, Qt::NoModifier, comboBoxCenterPos);
     QTRY_VERIFY(writingSystemComboBox->popup()->isOpened());
 
     // "Any" should be selected by default.
@@ -184,6 +224,7 @@ void tst_QQuickFontDialogImpl::clickAroundInTheFamilyListView()
     // Open the dialog.
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     QQuickListView *fontFamilyListView =
             dialogHelper.quickDialog->findChild<QQuickListView *>("familyListView");
@@ -253,6 +294,7 @@ void tst_QQuickFontDialogImpl::settingUnderlineAndStrikeoutEffects()
     // Open the dialog.
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     QSignalSpy selectedFontSpy(dialogHelper.dialog, SIGNAL(selectedFontChanged()));
 
@@ -273,18 +315,21 @@ void tst_QQuickFontDialogImpl::settingUnderlineAndStrikeoutEffects()
     QVERIFY(dialogHelper.dialog->selectedFont().underline());
     QVERIFY(!dialogHelper.dialog->selectedFont().strikeOut());
 
+    QVERIFY(QQuickTest::qWaitForPolish(dialogHelper.popupWindow()));
     QVERIFY(clickButton(underlineCheckBox));
 
     QCOMPARE(selectedFontSpy.size(), 2);
     QVERIFY(!dialogHelper.dialog->selectedFont().underline());
     QVERIFY(!dialogHelper.dialog->selectedFont().strikeOut());
 
+    QVERIFY(QQuickTest::qWaitForPolish(dialogHelper.popupWindow()));
     QVERIFY(clickButton(strikeoutCheckBox));
 
     QCOMPARE(selectedFontSpy.size(), 3);
     QVERIFY(!dialogHelper.dialog->selectedFont().underline());
     QVERIFY(dialogHelper.dialog->selectedFont().strikeOut());
 
+    QVERIFY(QQuickTest::qWaitForPolish(dialogHelper.popupWindow()));
     QVERIFY(clickButton(strikeoutCheckBox));
 
     QCOMPARE(selectedFontSpy.size(), 4);
@@ -302,6 +347,7 @@ void tst_QQuickFontDialogImpl::changeFontSize()
     // Open the dialog.
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     QQuickTextField *sizeEdit = dialogHelper.quickDialog->findChild<QQuickTextField *>("sizeEdit");
     QVERIFY(sizeEdit);
@@ -426,6 +472,7 @@ void tst_QQuickFontDialogImpl::searchFamily()
     // Open the dialog.
     QVERIFY(dialogHelper.openDialog());
     QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
+    QVERIFY(dialogHelper.waitForPopupWindowActiveAndPolished());
 
     const QQuickTextField *familyEdit =
             dialogHelper.quickDialog->findChild<QQuickTextField *>("familyEdit");
@@ -437,7 +484,7 @@ void tst_QQuickFontDialogImpl::searchFamily()
 
     const QPoint familyEditCenterPos =
             familyEdit->mapToScene({ familyEdit->width() / 2, familyEdit->height() / 2 }).toPoint();
-    QTest::mouseClick(dialogHelper.window(), Qt::LeftButton, Qt::NoModifier, familyEditCenterPos);
+    QTest::mouseClick(dialogHelper.popupWindow(), Qt::LeftButton, Qt::NoModifier, familyEditCenterPos);
     QVERIFY(familyEdit->hasActiveFocus());
 
     QSignalSpy familyListViewCurrentIndexSpy(fontFamilyListView, SIGNAL(currentIndexChanged()));
@@ -455,7 +502,7 @@ void tst_QQuickFontDialogImpl::searchFamily()
         const int previousIndex = fontFamilyListView->currentIndex();
 
         const char keyEntered = alphabet_it->toLatin1();
-        QTest::keyClick(dialogHelper.window(), keyEntered);
+        QTest::keyClick(dialogHelper.popupWindow(), keyEntered);
 
         int expectedListViewIndex = listSearchHelper.expectedCurrentIndexForSearch(QString(keyEntered));
         if (expectedListViewIndex == -1) {
@@ -554,49 +601,6 @@ void tst_QQuickFontDialogImpl::setCurrentFontFromApi()
     CLOSE_DIALOG("Ok");
 }
 
-void tst_QQuickFontDialogImpl::sensibleDefaults()
-{
-    CREATE_DIALOG_TEST_HELPER
-
-    // Open the dialog.
-    QVERIFY(dialogHelper.openDialog());
-    QTRY_VERIFY(dialogHelper.isQuickDialogOpen());
-
-    QQuickListView *fontFamilyListView =
-            dialogHelper.quickDialog->findChild<QQuickListView *>("familyListView");
-    QVERIFY(fontFamilyListView);
-    QQuickListView *fontStyleListView =
-            dialogHelper.quickDialog->findChild<QQuickListView *>("styleListView");
-    QVERIFY(fontStyleListView);
-    QQuickListView *fontSizeListView =
-            dialogHelper.quickDialog->findChild<QQuickListView *>("sizeListView");
-    QVERIFY(fontSizeListView);
-    QQuickTextField *fontFamilyTextField =
-            dialogHelper.quickDialog->findChild<QQuickTextField *>("familyEdit");
-    QVERIFY(fontFamilyTextField);
-    QQuickTextField *fontStyleTextField =
-            dialogHelper.quickDialog->findChild<QQuickTextField *>("styleEdit");
-    QVERIFY(fontStyleTextField);
-
-    // The indexes should initially be 0.
-    QCOMPARE(fontFamilyListView->currentIndex(), 0);
-    QCOMPARE(fontStyleListView->currentIndex(), 0);
-    QCOMPARE(fontSizeListView->currentIndex(), 0);
-
-    // Setting the family to an empty model
-    fontFamilyListView->setModel(QStringList());
-
-    // The listviews for font family and style should now have an empty model with indexes set to -1
-    QVERIFY(fontFamilyListView->model().toStringList().isEmpty());
-    QVERIFY(fontStyleListView->model().toStringList().isEmpty());
-    QCOMPARE(fontFamilyListView->currentIndex(), -1);
-    QCOMPARE(fontStyleListView->currentIndex(), -1);
-    QVERIFY(fontFamilyTextField->text().isEmpty());
-    QVERIFY(fontStyleTextField->text().isEmpty());
-
-    CLOSE_DIALOG("Ok");
-}
-
 QTEST_MAIN(tst_QQuickFontDialogImpl)
 
 #include "tst_qquickfontdialogimpl.moc"
