@@ -4,6 +4,7 @@
 #include <qtest.h>
 #include <qtesttouch.h>
 #include <QtTest/QSignalSpy>
+#include <QtTest/QTestAccessibility>
 #include <QtTest/private/qtesthelpers_p.h>
 #include <QtQml/qqmlcomponent.h>
 #include <QtQml/qqmlcontext.h>
@@ -19,7 +20,9 @@
 #include <QtQuickTestUtils/private/visualtestutils_p.h>
 #include <QtGui/QWindow>
 #include <QtGui/QScreen>
+#include <QtGui/QAccessible>
 #include <QtGui/QImage>
+#include <QtGui/qpa/qplatformaccessibility.h>
 #include <QtCore/QDebug>
 #include <QtQml/qqmlengine.h>
 
@@ -31,6 +34,8 @@
 
 #include <QtQuickWidgets/QQuickWidget>
 
+using namespace Qt::StringLiterals;
+
 #if QT_CONFIG(graphicsview)
 # include <QtWidgets/QGraphicsView>
 # include <QtWidgets/QGraphicsProxyWidget>
@@ -110,6 +115,11 @@ class tst_qquickwidget : public QQmlDataTest
 public:
     tst_qquickwidget();
 
+#if QT_CONFIG(accessibility)
+private:
+    bool initAccessibility();
+#endif
+
 private slots:
     void showHide();
     void reparentAfterShow();
@@ -147,9 +157,16 @@ private slots:
     void focusOnClickInProxyWidget();
 #endif
     void focusPreserved();
+#if QT_CONFIG(accessibility)
     void accessibilityHandlesViewChange();
+    void accessibleParentOfQuickItems();
+#endif
     void cleanupRhi();
     void dontRecreateRootElementOnWindowChange();
+    void setInitialProperties();
+    void fromModuleCtor();
+    void loadFromModule_data();
+    void loadFromModule();
 
 private:
     QPointingDevice *device = QTest::createTouchDevice();
@@ -161,6 +178,19 @@ tst_qquickwidget::tst_qquickwidget()
 {
 }
 
+#if QT_CONFIG(accessibility)
+bool tst_qquickwidget::initAccessibility()
+{
+    // Copied from tst_QQuickAccessible::initTestCase()
+    QTestAccessibility::initialize();
+    QPlatformIntegration *pfIntegration = QGuiApplicationPrivate::platformIntegration();
+    if (!pfIntegration->accessibility())
+        return false;
+    pfIntegration->accessibility()->setActive(true);
+    return true;
+}
+#endif
+
 void tst_qquickwidget::showHide()
 {
     QWidget window;
@@ -955,7 +985,7 @@ void tst_qquickwidget::resizeOverlay()
     auto contentVerticalLayout = new QVBoxLayout(&widget);
     contentVerticalLayout->setContentsMargins(0, 0, 0, 0);
 
-    qmlRegisterType<Overlay>("Test", 1, 0, "Overlay");
+    qmlRegisterType<Overlay>("TestModule", 1, 0, "Overlay");
 
     auto quickWidget = new QQuickWidget(testFileUrl("resizeOverlay.qml"), &widget);
     QCOMPARE(quickWidget->status(), QQuickWidget::Ready);
@@ -1151,7 +1181,7 @@ void tst_qquickwidget::focusOnClickInProxyWidget()
 
 void tst_qquickwidget::focusPreserved()
 {
-    SKIP_IF_NO_WINDOW_ACTIVATION
+    SKIP_IF_NO_WINDOW_ACTIVATION;
     if (QGuiApplication::platformName() == "android")
         QSKIP("Test doesn't exit cleanly on Android and generates many warnings - QTBUG-112696");
 
@@ -1205,6 +1235,7 @@ void tst_qquickwidget::focusPreserved()
     QTRY_VERIFY(content2->hasActiveFocus());
 }
 
+#if QT_CONFIG(accessibility)
 /*
     Reparenting the QQuickWidget recreates the offscreen QQuickWindow.
     Since the accessible interface that is cached for the QQuickWidget dispatches
@@ -1247,6 +1278,48 @@ void tst_qquickwidget::accessibilityHandlesViewChange()
     (void)iface->child(0);
 }
 
+/*
+QTBUG-130116
+Since the quick widget reports the quick items as its children, then
+the quick items should report the quick widget as their parent.
+*/
+void tst_qquickwidget::accessibleParentOfQuickItems()
+{
+    if (!initAccessibility())
+        QSKIP("This platform does not support accessibility");
+
+    // These lines are copied from accessibilityHandlesViewChange()
+    if (QGuiApplication::platformName() == "offscreen")
+        QSKIP("Doesn't test anything on offscreen platform.");
+
+    QWidget window;
+    window.resize(300, 300);
+
+    QQuickWidget *quickWidget = new QQuickWidget(&window);
+    quickWidget->setSource(testFileUrl("button.qml"));
+
+    window.show();
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    auto *rootItem = quickWidget->rootObject();
+    QVERIFY(rootItem);
+
+    QQuickButton *button = rootItem->findChild<QQuickButton *>("button");
+    QVERIFY(button);
+
+    QAccessibleInterface *iface_quickWidget = QAccessible::queryAccessibleInterface(quickWidget);
+    QVERIFY(iface_quickWidget);
+
+    QAccessibleInterface *iface_button = QAccessible::queryAccessibleInterface(button);
+    QVERIFY(iface_button);
+
+    QAccessibleInterface *iface_popup = iface_button->parent();
+    QVERIFY(iface_popup);
+    QVERIFY(iface_popup->parent());
+    QCOMPARE(iface_popup->parent(), iface_quickWidget);
+}
+#endif
+
 class CreateDestroyWidget : public QWidget
 {
 public:
@@ -1281,6 +1354,62 @@ void tst_qquickwidget::dontRecreateRootElementOnWindowChange()
     QVERIFY(!wasDestroyed);
 }
 
+void tst_qquickwidget::setInitialProperties()
+{
+    QQuickWidget widget;
+    widget.setInitialProperties({{"z", 4}, {"width", 100}});
+    widget.setSource(testFileUrl("resizemodeitem.qml"));
+    widget.show();
+    QObject *rootObject = widget.rootObject();
+    QVERIFY(rootObject);
+    QCOMPARE(rootObject->property("z").toInt(), 4);
+    QCOMPARE(rootObject->property("width").toInt(), 100);
+}
+
+void tst_qquickwidget::fromModuleCtor()
+{
+    QQuickWidget widget("QtQuick", "Rectangle");
+    // creation is always synchronous for C++ defined types, so we don't need _TRY
+    QObject *rootObject = widget.rootObject();
+    QVERIFY(rootObject);
+    QCOMPARE(rootObject->metaObject()->className(), "QQuickRectangle");
+}
+
+void tst_qquickwidget::loadFromModule_data()
+{
+    QTest::addColumn<QString>("module");
+    QTest::addColumn<QString>("typeName");
+    QTest::addColumn<QUrl>("url");
+    QTest::addColumn<QQuickWidget::Status>("status");
+
+    QTest::addRow("Item") << u"QtQuick"_s << u"Item"_s << QUrl() << QQuickWidget::Ready;
+    QTest::addRow("composite") << u"test"_s << u"TestQml"_s << QUrl("qrc:/qt/qml/test/data/TestQml.qml") << QQuickWidget::Ready;
+    QTest::addRow("nonexistent") << u"missing"_s << u"Type"_s << QUrl() << QQuickWidget::Error;
+}
+
+void tst_qquickwidget::loadFromModule()
+{
+    QFETCH(QString, module);
+    QFETCH(QString, typeName);
+    QFETCH(QUrl, url);
+    QFETCH(QQuickWidget::Status, status);
+
+    QQuickWidget widget;
+    widget.loadFromModule(module, typeName);
+    QTRY_COMPARE(widget.status(), status);
+    QCOMPARE(widget.source(), url);
+    if (status == QQuickWidget::Ready) {
+        QPointer<QObject> rootObject = widget.rootObject();
+        QVERIFY(rootObject);
+        // loadFromModule sets the source and deletes
+        // any object that was previously created
+        widget.loadFromModule("QtTest", "SignalSpy");
+        QVERIFY(rootObject.isNull());
+        QCOMPARE(widget.status(), QQuickWidget::Ready);
+        QVERIFY(widget.source().toString().endsWith("SignalSpy.qml"));
+    }
+}
+
 QTEST_MAIN(tst_qquickwidget)
 
 #include "tst_qquickwidget.moc"
