@@ -1,31 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 Research In Motion.
-** Copyright (C) 2019 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the tools applications of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 Research In Motion.
+// Copyright (C) 2019 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "conf.h"
 
@@ -61,6 +36,7 @@
 #include <qqmlfileselector.h>
 
 #include <private/qtqmlglobal_p.h>
+#include <private/qqmlimport_p.h>
 #if QT_CONFIG(qml_animation)
 #include <private/qabstractanimation_p.h>
 #endif
@@ -97,12 +73,18 @@ static QQmlApplicationEngine *qae = nullptr;
 #if defined(Q_OS_DARWIN) || defined(QT_GUI_LIB)
 static int exitTimerId = -1;
 #endif
-static const QString iconResourcePath(QStringLiteral(":/qt-project.org/QmlRuntime/resources/qml-64.png"));
-static const QString confResourcePath(QStringLiteral(":/qt-project.org/QmlRuntime/conf/"));
+static const QString iconResourcePath(QStringLiteral(":/qt-project.org/imports/QmlRuntime/Config/resources/qml-64.png"));
+static const QString confResourcePath(QStringLiteral(":/qt-project.org/imports/QmlRuntime/Config/"));
 static const QString customConfFileName(QStringLiteral("configuration.qml"));
 static bool verboseMode = false;
 static bool quietMode = false;
 static bool glShareContexts = true;
+static bool disableShaderCache = true;
+#if defined(QT_GUI_LIB)
+static bool requestAlphaChannel = false;
+static bool requestMSAA = false;
+static bool requestCoreProfile = false;
+#endif
 
 static void loadConf(const QString &override, bool quiet) // Terminates app on failure
 {
@@ -113,23 +95,23 @@ static void loadConf(const QString &override, bool quiet) // Terminates app on f
         QFileInfo fi;
         fi.setFile(QStandardPaths::locate(QStandardPaths::AppDataLocation, defaultFileName));
         if (fi.exists()) {
-            settingsUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
+            settingsUrl = QQmlImports::urlFromLocalFileOrQrcOrUrl(fi.absoluteFilePath());
         } else {
             // ### If different built-in configs are needed per-platform, just apply QFileSelector to the qrc conf.qml path
             fi.setFile(confResourcePath + defaultFileName);
-            settingsUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
+            settingsUrl = QQmlImports::urlFromLocalFileOrQrcOrUrl(fi.absoluteFilePath());
             builtIn = true;
         }
     } else {
         QFileInfo fi;
         fi.setFile(confResourcePath + override + QLatin1String(".qml"));
         if (fi.exists()) {
-            settingsUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
+            settingsUrl = QQmlImports::urlFromLocalFileOrQrcOrUrl(fi.absoluteFilePath());
             builtIn = true;
         } else {
             fi.setFile(QDir(QStandardPaths::locate(QStandardPaths::AppConfigLocation, override, QStandardPaths::LocateDirectory)), customConfFileName);
             if (fi.exists())
-                settingsUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
+                settingsUrl = QQmlImports::urlFromLocalFileOrQrcOrUrl(fi.absoluteFilePath());
             else
                 fi.setFile(override);
             if (!fi.exists()) {
@@ -137,7 +119,7 @@ static void loadConf(const QString &override, bool quiet) // Terminates app on f
                        qPrintable(QDir::toNativeSeparators(fi.absoluteFilePath())));
                 exit(1);
             }
-            settingsUrl = QUrl::fromLocalFile(fi.absoluteFilePath());
+            settingsUrl = QQmlImports::urlFromLocalFileOrQrcOrUrl(fi.absoluteFilePath());
         }
     }
 
@@ -176,8 +158,16 @@ static void listConfFiles()
 {
     const QDir confResourceDir(confResourcePath);
     printf("%s\n", qPrintable(QCoreApplication::translate("main", "Built-in configurations:")));
-    for (const QFileInfo &fi : confResourceDir.entryInfoList(QDir::Files))
-        printf("  %s\n", qPrintable(fi.baseName()));
+    for (const QFileInfo &fi : confResourceDir.entryInfoList(QDir::Files)) {
+        if (fi.completeSuffix() != QLatin1String("qml"))
+            continue;
+
+        const QString baseName = fi.baseName();
+        if (baseName.isEmpty() || baseName[0].isUpper())
+            continue;
+
+        printf("  %s\n", qPrintable(baseName));
+    }
     printf("%s\n", qPrintable(QCoreApplication::translate("main", "Other configurations:")));
     bool foundOther = false;
     const QStringList otherLocations = QStandardPaths::standardLocations(QStandardPaths::AppConfigLocation);
@@ -258,16 +248,14 @@ public Q_SLOTS:
     {
         Q_UNUSED(url);
         if (o) {
-            checkForWindow(o);
+            ++createdObjects;
             if (conf && qae)
-                for (PartialScene *ps : qAsConst(conf->completers))
+                for (PartialScene *ps : std::as_const(conf->completers))
                     if (o->inherits(ps->itemType().toUtf8().constData()))
                         contain(o, ps->container());
         }
-        if (haveWindow)
-            return;
 
-        if (! --expectedFileCount) {
+        if (!--expectedFileCount && !createdObjects) {
             printf("qml: Did not load any objects, exiting.\n");
             exit(2);
             QCoreApplication::exit(2);
@@ -286,11 +274,10 @@ public Q_SLOTS:
 
 private:
     void contain(QObject *o, const QUrl &containPath);
-    void checkForWindow(QObject *o);
 
 private:
-    bool haveWindow = false;
     int expectedFileCount;
+    int createdObjects = 0;
 };
 
 void LoadWatcher::contain(QObject *o, const QUrl &containPath)
@@ -300,7 +287,6 @@ void LoadWatcher::contain(QObject *o, const QUrl &containPath)
     if (!o2)
         return;
     o2->setParent(this);
-    checkForWindow(o2);
     bool success = false;
     int idx;
     if ((idx = o2->metaObject()->indexOfProperty("containedObject")) != -1)
@@ -309,16 +295,6 @@ void LoadWatcher::contain(QObject *o, const QUrl &containPath)
         o->setParent(o2); // Set QObject parent, and assume container will react as needed
 }
 
-void LoadWatcher::checkForWindow(QObject *o)
-{
-#if defined(QT_GUI_LIB)
-    if (o->isWindowType() && o->inherits("QQuickWindow"))
-        haveWindow = true;
-#else
-    Q_UNUSED(o);
-#endif // QT_GUI_LIB
-}
-
 void quietMessageHandler(QtMsgType type, const QMessageLogContext &ctxt, const QString &msg)
 {
     Q_UNUSED(ctxt);
@@ -340,6 +316,9 @@ static void getAppFlags(int argc, char **argv)
 {
 #ifdef QT_GUI_LIB
     for (int i=0; i<argc; i++) {
+        if (strcmp(argv[i], "--"))
+            return; // After "--", arguments are interpreted as positional and not as options.
+
         if (!strcmp(argv[i], "--apptype") || !strcmp(argv[i], "-a") || !strcmp(argv[i], "-apptype")) {
             applicationType = QmlApplicationTypeUnknown;
             if (i+1 < argc) {
@@ -362,6 +341,14 @@ static void getAppFlags(int argc, char **argv)
             QCoreApplication::setAttribute(Qt::AA_UseSoftwareOpenGL);
         } else if (!strcmp(argv[i], "-disable-context-sharing") || !strcmp(argv[i], "--disable-context-sharing")) {
             glShareContexts = false;
+        } else if (!strcmp(argv[i], "-enable-shader-cache") || !strcmp(argv[i], "--enable-shader-cache")) {
+            disableShaderCache = false;
+        } else if (!strcmp(argv[i], "-transparent") || !strcmp(argv[i], "--transparent")) {
+            requestAlphaChannel = true;
+        } else if (!strcmp(argv[i], "-multisample") || !strcmp(argv[i], "--multisample")) {
+            requestMSAA = true;
+        } else if (!strcmp(argv[i], "-core-profile") || !strcmp(argv[i], "--core-profile")) {
+            requestCoreProfile = true;
         }
     }
 #else
@@ -388,7 +375,7 @@ static void loadDummyDataFiles(QQmlEngine &engine, const QString& directory)
 
         if (dummyData && !quietMode) {
             printf("qml: Loaded dummy data: %s\n",  qPrintable(dir.filePath(qml)));
-            qml.truncate(qml.length()-4);
+            qml.truncate(qml.size()-4);
             engine.rootContext()->setContextProperty(qml, dummyData);
             dummyData->setParent(&engine);
         }
@@ -400,8 +387,31 @@ int main(int argc, char *argv[])
 {
     getAppFlags(argc, argv);
 
+    // Must set the default QSurfaceFormat before creating the app object if
+    // AA_ShareOpenGLContexts is going to be set.
+#if defined(QT_GUI_LIB)
+    QSurfaceFormat surfaceFormat;
+    surfaceFormat.setDepthBufferSize(24);
+    surfaceFormat.setStencilBufferSize(8);
+    if (requestMSAA)
+        surfaceFormat.setSamples(4);
+    if (requestAlphaChannel)
+        surfaceFormat.setAlphaBufferSize(8);
+    if (qEnvironmentVariableIsSet("QSG_CORE_PROFILE")
+            || qEnvironmentVariableIsSet("QML_CORE_PROFILE")
+            || requestCoreProfile)
+    {
+        // intentionally requesting 4.1 core to play nice with macOS
+        surfaceFormat.setVersion(4, 1);
+        surfaceFormat.setProfile(QSurfaceFormat::CoreProfile);
+    }
+    QSurfaceFormat::setDefaultFormat(surfaceFormat);
+#endif
+
     if (glShareContexts)
         QCoreApplication::setAttribute(Qt::AA_ShareOpenGLContexts);
+    if (disableShaderCache)
+        QCoreApplication::setAttribute(Qt::AA_DisableShaderDiskCache);
 
     std::unique_ptr<QCoreApplication> app;
     switch (applicationType) {
@@ -428,7 +438,6 @@ int main(int argc, char *argv[])
     app->setOrganizationDomain("qt-project.org");
     QCoreApplication::setApplicationVersion(QLatin1String(QT_VERSION_STR));
 
-    QQmlApplicationEngine e;
     QStringList files;
     QString confFile;
     QString translationFile;
@@ -437,8 +446,8 @@ int main(int argc, char *argv[])
     QCommandLineParser parser;
     parser.setSingleDashWordOptionMode(QCommandLineParser::ParseAsLongOptions);
     parser.setOptionsAfterPositionalArgumentsMode(QCommandLineParser::ParseAsPositionalArguments);
-    const QCommandLineOption helpOption = parser.addHelpOption();
-    const QCommandLineOption versionOption = parser.addVersionOption();
+    parser.addHelpOption();
+    parser.addVersionOption();
 #ifdef QT_GUI_LIB
     QCommandLineOption apptypeOption(QStringList() << QStringLiteral("a") << QStringLiteral("apptype"),
         QCoreApplication::translate("main", "Select which application class to use. Default is gui."),
@@ -482,11 +491,22 @@ int main(int argc, char *argv[])
     parser.addOption(glSoftwareOption); // Just for the help text... we've already handled this argument above
     QCommandLineOption glCoreProfile(QStringLiteral("core-profile"),
         QCoreApplication::translate("main", "Force use of OpenGL Core Profile."));
-    parser.addOption(glCoreProfile);
+    parser.addOption(glCoreProfile); // Just for the help text... we've already handled this argument above
     QCommandLineOption glContextSharing(QStringLiteral("disable-context-sharing"),
         QCoreApplication::translate("main", "Disable the use of a shared GL context for QtQuick Windows"));
     parser.addOption(glContextSharing); // Just for the help text... we've already handled this argument above
+    // Options relevant for other 3D APIs as well
+    QCommandLineOption shaderCaching(QStringLiteral("enable-shader-cache"),
+        QCoreApplication::translate("main", "Enable persistent caching of generated shaders"));
+    parser.addOption(shaderCaching); // Just for the help text... we've already handled this argument above
+    QCommandLineOption transparentOption(QStringLiteral("transparent"),
+        QCoreApplication::translate("main", "Requests an alpha channel in order to enable semi-transparent windows."));
+    parser.addOption(transparentOption); // Just for the help text... we've already handled this argument above
+    QCommandLineOption multisampleOption(QStringLiteral("multisample"),
+        QCoreApplication::translate("main", "Requests 4x multisample antialiasing."));
+    parser.addOption(multisampleOption); // Just for the help text... we've already handled this argument above
 #endif // QT_GUI_LIB
+
     // Debugging and verbosity options
     QCommandLineOption quietOption(QStringLiteral("quiet"),
         QCoreApplication::translate("main", "Suppress all output."));
@@ -502,7 +522,7 @@ int main(int argc, char *argv[])
     parser.addOption(fixedAnimationsOption);
     QCommandLineOption rhiOption(QStringList() << QStringLiteral("r") << QStringLiteral("rhi"),
         QCoreApplication::translate("main", "Set the backend for the Qt graphics abstraction (RHI). "
-                                    "Backend is one of: default, vulkan, metal, d3d11, gl"),
+                                    "Backend is one of: default, vulkan, metal, d3d11, d3d12, opengl"),
                                  QStringLiteral("backend"));
     parser.addOption(rhiOption);
     QCommandLineOption selectorOption(QStringLiteral("S"), QCoreApplication::translate("main",
@@ -515,14 +535,7 @@ int main(int argc, char *argv[])
     parser.addPositionalArgument("args",
         QCoreApplication::translate("main", "Arguments after '--' are ignored, but passed through to the application.arguments variable in QML."), "[-- args...]");
 
-    if (!parser.parse(QCoreApplication::arguments())) {
-        qWarning() << parser.errorText();
-        exit(1);
-    }
-    if (parser.isSet(versionOption))
-        parser.showVersion();
-    if (parser.isSet(helpOption))
-        parser.showHelp();
+    parser.process(*app);
     if (parser.isSet(verboseOption))
         verboseMode = true;
     if (parser.isSet(quietOption)) {
@@ -545,6 +558,9 @@ int main(int argc, char *argv[])
     if (parser.isSet(fixedAnimationsOption))
         QUnifiedTimer::instance()->setConsistentTiming(true);
 #endif
+
+    QQmlApplicationEngine e;
+
     for (const QString &importPath : parser.values(importOption))
         e.addImportPath(importPath);
 
@@ -555,17 +571,6 @@ int main(int argc, char *argv[])
     if (!customSelectors.isEmpty())
         e.setExtraFileSelectors(customSelectors);
 
-#if defined(QT_GUI_LIB)
-    if (qEnvironmentVariableIsSet("QSG_CORE_PROFILE") || qEnvironmentVariableIsSet("QML_CORE_PROFILE") || parser.isSet(glCoreProfile)) {
-        QSurfaceFormat surfaceFormat;
-        surfaceFormat.setStencilBufferSize(8);
-        surfaceFormat.setDepthBufferSize(24);
-        surfaceFormat.setVersion(4, 1);
-        surfaceFormat.setProfile(QSurfaceFormat::CoreProfile);
-        QSurfaceFormat::setDefaultFormat(surfaceFormat);
-    }
-#endif
-
     files << parser.values(qmlFileOption);
     if (parser.isSet(configOption))
         confFile = parser.value(configOption);
@@ -588,9 +593,8 @@ int main(int argc, char *argv[])
 #if QT_CONFIG(translation)
     // Need to be installed before QQmlApplicationEngine's automatic translation loading
     // (qt_ translations are loaded there)
+    QTranslator translator;
     if (!translationFile.isEmpty()) {
-        QTranslator translator;
-
         if (translator.load(translationFile)) {
             app->installTranslator(&translator);
             if (verboseMode)
@@ -610,7 +614,7 @@ int main(int argc, char *argv[])
         QLoggingCategory::setFilterRules(QStringLiteral("*=false"));
     }
 
-    if (files.count() <= 0) {
+    if (files.size() <= 0) {
 #if defined(Q_OS_DARWIN) && defined(QT_GUI_LIB)
         if (applicationType == QmlApplicationTypeGui)
             exitTimerId = static_cast<LoaderApplication *>(app.get())->startTimer(FILE_OPEN_EVENT_WAIT_TIME);
@@ -623,7 +627,7 @@ int main(int argc, char *argv[])
     loadConf(confFile, !verboseMode);
 
     // Load files
-    QScopedPointer<LoadWatcher> lw(new LoadWatcher(&e, files.count()));
+    QScopedPointer<LoadWatcher> lw(new LoadWatcher(&e, files.size()));
 
 #if QT_DEPRECATED_SINCE(6, 3)
     QString dummyDir;
@@ -636,7 +640,7 @@ int main(int argc, char *argv[])
     }
 #endif
 
-    for (const QString &path : qAsConst(files)) {
+    for (const QString &path : std::as_const(files)) {
         QUrl url = QUrl::fromUserInput(path, QDir::currentPath(), QUrl::AssumeLocalFile);
         if (verboseMode)
             printf("qml: loading %s\n", qPrintable(url.toString()));
